#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.Utils;
#endregion
namespace Volpe.Cafe.EIS.Data
{
    [Serializable]
    public class EISAnalysisData
    {
        #region 
        public EISAnalysisData(int minYear, int maxYear)
        {
            this._minYear  = minYear;
            this._maxYear  = maxYear;
            this._numYears = maxYear - minYear + 1;
            this._ratedFuelEconomy      = new FuelValue[this._numYears];
            this._onRoadFuelEconomy     = new FuelValue[this._numYears];
            this._fuelShare             = new FuelValue[this._numYears];
            this._fleet                 = new double   [this._numYears, this._numYears + Interaction.CY];
            this._vmt                   = new FuelValue[this._numYears, this._numYears + Interaction.CY];
            this._gallons               = new FuelValue[this._numYears, this._numYears + Interaction.CY];
            this._emissionsCO           = new EISEmissionsData(this._numYears);
            this._emissionsVOC          = new EISEmissionsData(this._numYears);
            this._emissionsNOX          = new EISEmissionsData(this._numYears);
            this._emissionsPM           = new EISEmissionsData(this._numYears);
            this._emissionsSOX          = new EISEmissionsData(this._numYears);
            this._emissionsCO2          = new EISEmissionsData(this._numYears);
            this._emissionsAcetaldehyde = new EISEmissionsData(this._numYears);
            this._emissionsAcrolein     = new EISEmissionsData(this._numYears);
            this._emissionsBenzene      = new EISEmissionsData(this._numYears);
            this._emissionsButadiene    = new EISEmissionsData(this._numYears);
            this._emissionsFormaldehyde = new EISEmissionsData(this._numYears);
            this._emissionsCH4          = new EISEmissionsData(this._numYears);
            this._emissionsDPM          = new EISEmissionsData(this._numYears);
            this._emissionsMTBE         = new EISEmissionsData(this._numYears);
            this._emissionsN2O          = new EISEmissionsData(this._numYears);
        }
        #endregion
        #region 
        public EISEmissionsData GetEmissions(EmissionType emission)
        {
            switch (emission)
            {
                case EmissionType.Acetaldehyde: return this._emissionsAcetaldehyde;
                case EmissionType.Acrolein    : return this._emissionsAcrolein;
                case EmissionType.Benzene     : return this._emissionsBenzene;
                case EmissionType.Butadiene   : return this._emissionsButadiene;
                case EmissionType.CH4         : return this._emissionsCH4;
                case EmissionType.CO          : return this._emissionsCO;
                case EmissionType.CO2         : return this._emissionsCO2;
                case EmissionType.DPM         : return this._emissionsDPM;
                case EmissionType.Formaldehyde: return this._emissionsFormaldehyde;
                case EmissionType.MTBE        : return this._emissionsMTBE;
                case EmissionType.N2O         : return this._emissionsN2O;
                case EmissionType.NOX         : return this._emissionsNOX;
                case EmissionType.PM          : return this._emissionsPM;
                case EmissionType.SOX         : return this._emissionsSOX;
                case EmissionType.VOC         : return this._emissionsVOC;
            }
            throw new ArgumentException("The specified emission type is not valid.", "emission");
        }
        #endregion
        #region 
        public int MinYear { get { return this._minYear; } }
        public int MaxYear { get { return this._maxYear; } }
        public int NumYears { get { return this._numYears; } }
        public FuelValue[] RatedFuelEconomy { get { return this._ratedFuelEconomy; } }
        public FuelValue[] OnRoadFuelEconomy { get { return this._onRoadFuelEconomy; } }
        public FuelValue[] FuelShare { get { return this._fuelShare; } }
        public double[,] Fleet { get { return this._fleet; } }
        public FuelValue[,] VMT { get { return this._vmt; } }
        public FuelValue[,] Gallons { get { return this._gallons; } }
        public EISEmissionsData EmissionsCO { get { return this._emissionsCO; } }
        public EISEmissionsData EmissionsVOC { get { return this._emissionsVOC; } }
        public EISEmissionsData EmissionsNOX { get { return this._emissionsNOX; } }
        public EISEmissionsData EmissionsPM { get { return this._emissionsPM; } }
        public EISEmissionsData EmissionsSOX { get { return this._emissionsSOX; } }
        public EISEmissionsData EmissionsCO2 { get { return this._emissionsCO2; } }
        public EISEmissionsData EmissionsAcetaldehyde { get { return this._emissionsAcetaldehyde; } }
        public EISEmissionsData EmissionsAcrolein { get { return this._emissionsAcrolein; } }
        public EISEmissionsData EmissionsBenzene { get { return this._emissionsBenzene; } }
        public EISEmissionsData EmissionsButadiene { get { return this._emissionsButadiene; } }
        public EISEmissionsData EmissionsFormaldehyde { get { return this._emissionsFormaldehyde; } }
        public EISEmissionsData EmissionsCH4 { get { return this._emissionsCH4; } }
        public EISEmissionsData EmissionsDPM { get { return this._emissionsDPM; } }
        public EISEmissionsData EmissionsMTBE { get { return this._emissionsMTBE; } }
        public EISEmissionsData EmissionsN2O { get { return this._emissionsN2O; } }
        #endregion
        #region 
        int _minYear, _maxYear, _numYears;
        FuelValue[ ] _ratedFuelEconomy;
        FuelValue[ ] _onRoadFuelEconomy;
        FuelValue[ ] _fuelShare;
        double   [,] _fleet;
        FuelValue[,] _vmt;
        FuelValue[,] _gallons;
        EISEmissionsData _emissionsCO;
        EISEmissionsData _emissionsVOC;
        EISEmissionsData _emissionsNOX;
        EISEmissionsData _emissionsPM;
        EISEmissionsData _emissionsSOX;
        EISEmissionsData _emissionsCO2;
        EISEmissionsData _emissionsAcetaldehyde;
        EISEmissionsData _emissionsAcrolein;
        EISEmissionsData _emissionsBenzene;
        EISEmissionsData _emissionsButadiene;
        EISEmissionsData _emissionsFormaldehyde;
        EISEmissionsData _emissionsCH4;
        EISEmissionsData _emissionsDPM;
        EISEmissionsData _emissionsMTBE;
        EISEmissionsData _emissionsN2O;
        #endregion
    }
}

