#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.EIS.Settings;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.XlLib;
#endregion
namespace Volpe.Cafe.EIS.IO.InputParsers
{
    sealed class XlEISParametersParser : XlEISParser
    {
        #region 
        internal XlEISParametersParser(string path, string password) : base(path, password, true) { }
        #endregion
        #region 
        protected override void ParseInternal()
        {
            this._value = new EISParameters();
            string[] sheets = this.SheetNames;
            string[] names = new string[] {"Fleet Data", "Sales Data", "No CAFE Data", "Overcompliance Data"};
            int[] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < sheets.Length; i++)
            {
                for (int j = 0; j < names.Length; j++)
                {
                    if (Interaction.StringCompare(sheets[i], names[j], true)) { indexes[j] = i; break; }
                }
            }
            if (!this.VerifyIndexes("EIS Parameters workbook", indexes, names, 0, 3)) { return; }
            this.ParseFleet         (indexes[0]);
            this.ParseSales         (indexes[1]);
            this.ParseNoCAFE        (indexes[2]);
            this.ParseOvercompliance(indexes[3]);
        }
        void ParseFleet(int index)
        {
            this.ActivateWorksheet(index);
            int minYear = EISParameters.FleetData.MinYear;
            int maxYear = EISParameters.FleetData.MaxYear;
            int yrLen = maxYear - minYear + 2;
            for (int i = 0; i < Interaction.CY; i++)
            {
                for (int j = 1; j < yrLen; j++)
                {
                    this._value._fleet.CarFleet[j - 1][i] = this.GetDouble(i +                  4, j);
                    this._value._fleet.TrkFleet[j - 1][i] = this.GetDouble(i + Interaction.CY + 7, j);
                } 
            } 
        }
        void ParseSales(int index)
        {
            int minYear = EISParameters.SalesData.MinYear;
            int maxYear = EISParameters.SalesData.MaxYear;
            this.ActivateWorksheet(index, new XlCell(6, 2), new XlCell(6 + maxYear - minYear, 3));
            int length = maxYear - minYear + 1;
            for (int i = 0; i < length; i++)
            {
                this._value._sales.CarSales[i] = this.GetDouble(i, 0);
                this._value._sales.TrkSales[i] = this.GetDouble(i, 1);
            }
        }
        void ParseNoCAFE(int index)
        {
            int minYear = EISParameters.NoCAFEData.MinYear;
            int maxYear = EISParameters.NoCAFEData.MaxYear;
            this.ActivateWorksheet(index, new XlCell(6, 2), new XlCell(6 + maxYear - minYear, 21));
            int length = maxYear - minYear + 1;
            for (int i = 0; i < length; i++)
            {   
                this._value._noCAFE.CarFuelEconomy[i][FuelType.Gasoline   ] = this.GetDouble(i,  0);
                this._value._noCAFE.CarFuelEconomy[i][FuelType.Diesel     ] = this.GetDouble(i,  1);
                this._value._noCAFE.CarFuelEconomy[i][FuelType.Ethanol85  ] = this.GetDouble(i,  2);
                this._value._noCAFE.CarFuelEconomy[i][FuelType.Electricity] = this.GetDouble(i,  3);
                this._value._noCAFE.CarFuelEconomy[i][FuelType.Hydrogen   ] = this.GetDouble(i,  4);
                this._value._noCAFE.CarFuelShare  [i][FuelType.Gasoline   ] = this.GetDouble(i,  5);
                this._value._noCAFE.CarFuelShare  [i][FuelType.Diesel     ] = this.GetDouble(i,  6);
                this._value._noCAFE.CarFuelShare  [i][FuelType.Ethanol85  ] = this.GetDouble(i,  7);
                this._value._noCAFE.CarFuelShare  [i][FuelType.Electricity] = this.GetDouble(i,  8);
                this._value._noCAFE.CarFuelShare  [i][FuelType.Hydrogen   ] = this.GetDouble(i,  9);
                this._value._noCAFE.TrkFuelEconomy[i][FuelType.Gasoline   ] = this.GetDouble(i, 10);
                this._value._noCAFE.TrkFuelEconomy[i][FuelType.Diesel     ] = this.GetDouble(i, 11);
                this._value._noCAFE.TrkFuelEconomy[i][FuelType.Ethanol85  ] = this.GetDouble(i, 12);
                this._value._noCAFE.TrkFuelEconomy[i][FuelType.Electricity] = this.GetDouble(i, 13);
                this._value._noCAFE.TrkFuelEconomy[i][FuelType.Hydrogen   ] = this.GetDouble(i, 14);
                this._value._noCAFE.TrkFuelShare  [i][FuelType.Gasoline   ] = this.GetDouble(i, 15);
                this._value._noCAFE.TrkFuelShare  [i][FuelType.Diesel     ] = this.GetDouble(i, 16);
                this._value._noCAFE.TrkFuelShare  [i][FuelType.Ethanol85  ] = this.GetDouble(i, 17);
                this._value._noCAFE.TrkFuelShare  [i][FuelType.Electricity] = this.GetDouble(i, 18);
                this._value._noCAFE.TrkFuelShare  [i][FuelType.Hydrogen   ] = this.GetDouble(i, 19);
            }
        }
        void ParseOvercompliance(int index)
        {
            int minYear = EISParameters.OvercomplianceData.MinYear;
            int maxYear = EISParameters.OvercomplianceData.MaxYear;
            this.ActivateWorksheet(index, new XlCell(6, 2), new XlCell(6 + maxYear - minYear, 21));
            int length = maxYear - minYear + 1;
            for (int i = 0; i < length; i++)
            {   
                this._value._overcompliance.CarBaseGrowth[i][FuelType.Gasoline   ] = this.GetDouble(i,  0);
                this._value._overcompliance.CarBaseGrowth[i][FuelType.Diesel     ] = this.GetDouble(i,  1);
                this._value._overcompliance.CarBaseGrowth[i][FuelType.Ethanol85  ] = this.GetDouble(i,  2);
                this._value._overcompliance.CarBaseGrowth[i][FuelType.Electricity] = this.GetDouble(i,  3);
                this._value._overcompliance.CarBaseGrowth[i][FuelType.Hydrogen   ] = this.GetDouble(i,  4);
                this._value._overcompliance.TrkBaseGrowth[i][FuelType.Gasoline   ] = this.GetDouble(i,  5);
                this._value._overcompliance.TrkBaseGrowth[i][FuelType.Diesel     ] = this.GetDouble(i,  6);
                this._value._overcompliance.TrkBaseGrowth[i][FuelType.Ethanol85  ] = this.GetDouble(i,  7);
                this._value._overcompliance.TrkBaseGrowth[i][FuelType.Electricity] = this.GetDouble(i,  8);
                this._value._overcompliance.TrkBaseGrowth[i][FuelType.Hydrogen   ] = this.GetDouble(i,  9);
                this._value._overcompliance.CarScenGrowth[i][FuelType.Gasoline   ] = this.GetDouble(i, 10);
                this._value._overcompliance.CarScenGrowth[i][FuelType.Diesel     ] = this.GetDouble(i, 11);
                this._value._overcompliance.CarScenGrowth[i][FuelType.Ethanol85  ] = this.GetDouble(i, 12);
                this._value._overcompliance.CarScenGrowth[i][FuelType.Electricity] = this.GetDouble(i, 13);
                this._value._overcompliance.CarScenGrowth[i][FuelType.Hydrogen   ] = this.GetDouble(i, 14);
                this._value._overcompliance.TrkScenGrowth[i][FuelType.Gasoline   ] = this.GetDouble(i, 15);
                this._value._overcompliance.TrkScenGrowth[i][FuelType.Diesel     ] = this.GetDouble(i, 16);
                this._value._overcompliance.TrkScenGrowth[i][FuelType.Ethanol85  ] = this.GetDouble(i, 17);
                this._value._overcompliance.TrkScenGrowth[i][FuelType.Electricity] = this.GetDouble(i, 18);
                this._value._overcompliance.TrkScenGrowth[i][FuelType.Hydrogen   ] = this.GetDouble(i, 19);
            }
        }
        #endregion
        #region 
        internal EISParameters _value;
        #endregion
    }
}

