#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.EIS.Settings;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
#endregion
namespace Volpe.Cafe.EIS.IO.InputParsers
{
    sealed class XlEISTailpipeEmissionRatesParser : XlEISParser
    {
        #region 
        internal XlEISTailpipeEmissionRatesParser(string path, string password) : base(path, password, true) { }
        #endregion
        #region 
        protected override void ParseInternal()
        {
            this._value = new EISTailpipeEmissions();
            string[] sheets = this.SheetNames;
            string[] names = new string[] {"CO", "VOC", "NOX", "PM", "ACT", "ACR", "BNZ", "BTD", "FRM", "CH4", "DPM", "MTBE", "N2O"};
            int[] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < sheets.Length; i++)
            {
                for (int j = 0; j < names.Length; j++)
                {
                    if (Interaction.StringCompare(sheets[i], names[j], true)) { indexes[j] = i; break; }
                }
            }
            if (!this.VerifyIndexes("EIS Tailpipe Emissions workbook", indexes, names)) { return; }
            this.ParseEmissionRatesSheet(indexes[ 0], this._value._co );
            this.ParseEmissionRatesSheet(indexes[ 1], this._value._voc);
            this.ParseEmissionRatesSheet(indexes[ 2], this._value._nox);
            this.ParseEmissionRatesSheet(indexes[ 3], this._value._pm );
            this.ParseEmissionRatesSheet(indexes[ 4], this._value._acetaldehyde);
            this.ParseEmissionRatesSheet(indexes[ 5], this._value._acrolein    );
            this.ParseEmissionRatesSheet(indexes[ 6], this._value._benzene     );
            this.ParseEmissionRatesSheet(indexes[ 7], this._value._butadiene   );
            this.ParseEmissionRatesSheet(indexes[ 8], this._value._formaldehyde);
            this.ParseEmissionRatesSheet(indexes[ 9], this._value._ch4 );
            this.ParseEmissionRatesSheet(indexes[10], this._value._dpm );
            this.ParseEmissionRatesSheet(indexes[11], this._value._mtbe);
            this.ParseEmissionRatesSheet(indexes[12], this._value._n2o );
        }
        void ParseEmissionRatesSheet(int index, EISTailpipeEmissions.Emissions em)
        {
            this.ActivateWorksheet(index);
            int minYear = EISTailpipeEmissions.RateValues.MinYear;
            int maxYear = EISTailpipeEmissions.RateValues.MaxYear;
            int yrLen = maxYear - minYear + 2;
            for (int i = 0; i < Interaction.CY; i++)
            {
                for (int j = 1; j < yrLen; j++)
                {   
                    em.Gasoline .LDV[j - 1][i] = this.GetDouble(i +                       4, j);
                    em.Diesel   .LDV[j - 1][i] = this.GetDouble(i + Interaction.CY     +  7, j);
                    em.Ethanol85.LDV[j - 1][i] = this.GetDouble(i + Interaction.CY * 2 + 10, j);
                    em.Gasoline .LDT[j - 1][i] = this.GetDouble(i                      +  4, j + yrLen + 1);
                    em.Diesel   .LDT[j - 1][i] = this.GetDouble(i + Interaction.CY     +  7, j + yrLen + 1);
                    em.Ethanol85.LDT[j - 1][i] = this.GetDouble(i + Interaction.CY * 2 + 10, j + yrLen + 1);
                } 
            } 
        }
        #endregion
        #region 
        internal EISTailpipeEmissions _value;
        #endregion
    }
}

