#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.EIS.Data;
using Volpe.Cafe.IO.Reporting.XL;
using Volpe.Cafe.Model;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.XlLib;
#endregion
namespace Volpe.Cafe.EIS.IO.Reporting
{
    [Serializable]
    class XlEISCafe : XlReportingBase
    {
        #region 
        public XlEISCafe(string path, OutputSettings outputSettings, XlUtilities templateFile)
            : base(path, outputSettings, templateFile, "eis_cafe") { }
        #endregion
        #region 
        public void ParseData(Scenario scen, EISAnalysisData eisData, bool isCar)
        {
            try
            {
                this.ParseData_Internal(scen, eisData, (isCar) ? "Car" : "Truck");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
                if (this.WSOpen) { this.CloseBuffer(); }
                this.Close(false);
            }
        }
        void ParseData_Internal(Scenario scen, EISAnalysisData eisData, string carOrTrk)
        {
            string bufName = ((scen == null) ? "NoCAFE" : "Sn" + scen.Index);
            this.OpenBuffer(bufName);
            this._buffer[0, 0] = carOrTrk + " Fuel Economy Projections (" + ((scen == null) ? "Without CAFE Program" : "Scenario " + scen.Index) + ")";
            this.WriteData(scen, eisData);
            this.CloseBuffer();
        }
        void WriteData(Scenario scen, EISAnalysisData eisData)
        {
            for (int i = MinYear; i <= MaxYear; i++)
            {
                this._buffer[4 + i - MinYear,  1] = eisData.RatedFuelEconomy[i - MinYear].Gasoline;
                this._buffer[4 + i - MinYear,  2] = eisData.FuelShare       [i - MinYear].Gasoline;
                this._buffer[4 + i - MinYear,  3] = eisData.RatedFuelEconomy[i - MinYear].Diesel;
                this._buffer[4 + i - MinYear,  4] = eisData.FuelShare       [i - MinYear].Diesel;
                this._buffer[4 + i - MinYear,  5] = eisData.RatedFuelEconomy[i - MinYear].Ethanol85;
                this._buffer[4 + i - MinYear,  6] = eisData.FuelShare       [i - MinYear].Ethanol85;
                this._buffer[4 + i - MinYear,  7] = eisData.RatedFuelEconomy[i - MinYear].Electricity;
                this._buffer[4 + i - MinYear,  8] = eisData.FuelShare       [i - MinYear].Electricity;
                this._buffer[4 + i - MinYear,  9] = eisData.RatedFuelEconomy[i - MinYear].Hydrogen;
                this._buffer[4 + i - MinYear, 10] = eisData.FuelShare       [i - MinYear].Hydrogen;
                FuelValue vmt = FuelValue.Zero;
                for (int j = 0; j < eisData.NumYears; j++)
                {
                    vmt += eisData.VMT[i - MinYear, j];
                }
                this._buffer[4 + i - MinYear, 11] = eisData.RatedFuelEconomy[i - MinYear].HarmonicMean(vmt);
                this._buffer[4 + i - MinYear, 12] = eisData.FuelShare       [i - MinYear].Total;
            }
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, Industry scenData, Industry baseData, Industry ppData) { }
        #endregion
        #region 
        const int MinYear = 1975;
        const int MaxYear = 2064;
        #endregion
    }
}

