#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.EIS.Data;
using Volpe.Cafe.IO.Reporting.XL;
using Volpe.Cafe.Model;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.XlLib;
#endregion
namespace Volpe.Cafe.EIS.IO.Reporting
{
    [Serializable]
    class XlEISEmissions : XlReportingBase
    {
        #region 
        public XlEISEmissions(string path, OutputSettings outputSettings, XlUtilities templateFile)
            : base(path, outputSettings, templateFile, "eis_emissions") { }
        #endregion
        #region 
        public void ParseData(Scenario scen, EISEmissionsData emissionsData, string emission, bool isCar)
        {
            try
            {
                this.ParseData_Internal(scen, emissionsData, emission, (isCar) ? "Car" : "Truck");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
                if (this.WSOpen) { this.CloseBuffer(); }
                this.Close(false);
            }
        }
        void ParseData_Internal(Scenario scen, EISEmissionsData emissionsData, string emission, string carOrTrk)
        {
            string bufName = ((scen == null) ? "NoCAFE" : "Sn" + scen.Index);
            this.OpenBuffer(bufName);
            this._buffer[0, 0] = carOrTrk + " " + emission + " Emissions (" + ((scen == null) ? "Without CAFE Program" : "Scenario " + scen.Index) + ")";
            this.WriteData(scen, emissionsData);
            this.CloseBuffer();
        }
        void WriteData(Scenario scen, EISEmissionsData emissionsData)
        {
            int numYears = MaxYear - MinYear + 1;
            for (int i = MinYear; i <= MaxYear; i++)
            {
                for (int j = MinYear; j <= MaxYear; j++)
                {
                    this._buffer[ 4                + i - MinYear, 1 + j - MinYear] = emissionsData.Tailpipe[i - MinYear, j - MinYear];
                    this._buffer[ 8 +     numYears + i - MinYear, 1 + j - MinYear] = emissionsData.Upstream[i - MinYear, j - MinYear];
                    this._buffer[12 + 2 * numYears + i - MinYear, 1 + j - MinYear] = emissionsData.Total   [i - MinYear, j - MinYear];
                } 
            } 
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, Industry scenData, Industry baseData, Industry ppData) { }
        #endregion
        #region 
        const int MinYear = 1975;
        const int MaxYear = 2064;
        #endregion
    }
}

