#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.EIS.Data;
using Volpe.Cafe.IO.Reporting.XL;
using Volpe.Cafe.Model;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.XlLib;
#endregion
namespace Volpe.Cafe.EIS.IO.Reporting
{
    [Serializable]
    class XlEISFleet : XlReportingBase
    {
        #region 
        public XlEISFleet(string path, OutputSettings outputSettings, XlUtilities templateFile)
            : base(path, outputSettings, templateFile, "eis_fleet") { }
        #endregion
        #region 
        public void ParseData(Scenario scen, EISAnalysisData eisData, bool isCar)
        {
            try
            {
                this.ParseData_Internal(scen, eisData, (isCar) ? "Car" : "Truck");
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.ToString());
                if (this.WSOpen) { this.CloseBuffer(); }
                this.Close(false);
            }
        }
        void ParseData_Internal(Scenario scen, EISAnalysisData eisData, string carOrTrk)
        {
            string bufName = ((scen == null) ? "NoCAFE" : "Sn" + scen.Index);
            this.OpenBuffer(bufName);
            this._buffer[0, 0] = carOrTrk + " Fleet (" + ((scen == null) ? "Without CAFE Program" : "Scenario " + scen.Index) + ")";
            this.WriteData(scen, eisData);
            this.CloseBuffer();
        }
        void WriteData(Scenario scen, EISAnalysisData eisData)
        {
            for (int i = MinYear; i <= MaxYear; i++)
            {
                for (int j = MinYear; j <= MaxYear; j++)
                {
                    this._buffer[4 + i - MinYear, 1 + j - MinYear] = eisData.Fleet[i - MinYear, j - MinYear];
                } 
            } 
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, Industry scenData, Industry baseData, Industry ppData) { }
        #endregion
        #region 
        const int MinYear = 1975;
        const int MaxYear = 2064;
        #endregion
    }
}

