#region << Using Directives >>
using System;
using System.IO;
using Volpe.Cafe.Data;
using Volpe.Cafe.Model;
using Volpe.Cafe.IO.Reporting.XL;
using Volpe.Cafe.EIS.Data;
using Volpe.Cafe.EIS.Model;
using Volpe.Cafe.Settings;
#endregion
namespace Volpe.Cafe.EIS.IO.Reporting
{
    [Serializable]
    class XlEISReportGenerator : XlReportGenerator
    {
        #region 
        static XlEISReportGenerator()
        {
            EmTitles = Enum.GetNames(typeof(EmissionType));
        }
        public XlEISReportGenerator(ICompliance compliance) : base(compliance) { }
        #endregion
        #region 
        protected override void OpenTemplate()
        {
            if (this.TemplateFile == null)
            {
                base.OpenTemplate();
            }
        }
        protected override void CloseTemplate()
        {
            if (!this._active)
            {
                base.CloseTemplate();
            }
        }
        protected override void OpenReports(int snIndex)
        {
            base.OpenReports(snIndex);
            if (snIndex == 0)
            {
                this._active = true;
                OutputSettings oSets = this.Compliance.Settings.OutputSettings;
                string path = oSets.OutputPath + "\\reports-eis";
                if (!Directory.Exists(path)) { Directory.CreateDirectory(path); }
                this.UpdateProgress("Opening EIS CAFE reports.");
                this._carFE = new XlEISCafe(path + "\\Car_CAFE.xls", oSets, this.TemplateFile);
                this._trkFE = new XlEISCafe(path + "\\Truck_CAFE.xls", oSets, this.TemplateFile);
                this.UpdateProgress("Opening EIS Gallons reports.");
                this._carGal = new XlEISGallons(path + "\\Car_Gallons.xls", oSets, this.TemplateFile);
                this._trkGal = new XlEISGallons(path + "\\Truck_Gallons.xls", oSets, this.TemplateFile);
                this.UpdateProgress("Opening EIS VMT reports.");
                this._carVmt = new XlEISVmt(path + "\\Car_VMT.xls", oSets, this.TemplateFile);
                this._trkVmt = new XlEISVmt(path + "\\Truck_VMT.xls", oSets, this.TemplateFile);
                this.UpdateProgress("Opening EIS Fleet reports.");
                this._carFleet = new XlEISFleet(path + "\\Car_Fleet.xls", oSets, this.TemplateFile);
                this._trkFleet = new XlEISFleet(path + "\\Truck_Fleet.xls", oSets, this.TemplateFile);
                this.UpdateProgress("Opening EIS Emissions reports.");
                this._carEm = new XlEISEmissions[EmTitles.Length];
                this._trkEm = new XlEISEmissions[EmTitles.Length];
                for (int i = 0; i < EmTitles.Length; i++)
                {
                    this._carEm[i] = new XlEISEmissions(path + "\\Car_Emissions_"   + EmTitles[i] + ".xls", oSets, this.TemplateFile);
                    this._trkEm[i] = new XlEISEmissions(path + "\\Truck_Emissions_" + EmTitles[i] + ".xls", oSets, this.TemplateFile);
                }
            }
        }
        protected override void WriteReportData(Scenario scen, ModelYear year, Industry scenData, Industry baseData, Industry ppData)
        {
            base.WriteReportData(scen, year, scenData, baseData, ppData);
            if (year.Year < this.Compliance.ModelYears[this.Compliance.ModelYears.Length - 1].Year) { return; }
            EISModel eisModel = (EISModel)this.Compliance;
            if (scen.IsBaseline)
            {   
                this.UpdateProgress("Writing \"No CAFE\" data for the EIS CAFE reports.");
                this._carFE.ParseData(null, eisModel.EISData.CarsNoCAFE, true);
                this._trkFE.ParseData(null, eisModel.EISData.TrksNoCAFE, false);
                this.UpdateProgress("Writing \"No CAFE\" data for the EIS Gallons reports.");
                this._carGal.ParseData(null, eisModel.EISData.CarsNoCAFE, true);
                this._trkGal.ParseData(null, eisModel.EISData.TrksNoCAFE, false);
                this.UpdateProgress("Writing \"No CAFE\" data for the EIS VMT reports.");
                this._carVmt.ParseData(null, eisModel.EISData.CarsNoCAFE, true);
                this._trkVmt.ParseData(null, eisModel.EISData.TrksNoCAFE, false);
                this.UpdateProgress("Writing \"No CAFE\" data for the EIS Fleet reports.");
                this._carFleet.ParseData(null, eisModel.EISData.CarsNoCAFE, true);
                this._trkFleet.ParseData(null, eisModel.EISData.TrksNoCAFE, false);
                for (int i = 0; i < EmTitles.Length; i++)
                {
                    this.UpdateProgress("Writing \"No CAFE\" data for the EIS " + EmTitles[i] + " Emissions reports.");
                    this._carEm[i].ParseData(null, eisModel.EISData.CarsNoCAFE.GetEmissions((EmissionType)i), EmTitles[i], true);
                    this._trkEm[i].ParseData(null, eisModel.EISData.TrksNoCAFE.GetEmissions((EmissionType)i), EmTitles[i], false);
                }
            }
            this.UpdateProgress("Writing scenario " + scen.Index + " data for the EIS CAFE reports.");
            this._carFE.ParseData(scen, eisModel.EISData.Cars[scen.Index], true);
            this._trkFE.ParseData(scen, eisModel.EISData.Trks[scen.Index], false);
            this.UpdateProgress("Writing scenario " + scen.Index + " data for the EIS Gallons reports.");
            this._carGal.ParseData(scen, eisModel.EISData.Cars[scen.Index], true);
            this._trkGal.ParseData(scen, eisModel.EISData.Trks[scen.Index], false);
            this.UpdateProgress("Writing scenario " + scen.Index + " data for the EIS VMT reports.");
            this._carVmt.ParseData(scen, eisModel.EISData.Cars[scen.Index], true);
            this._trkVmt.ParseData(scen, eisModel.EISData.Trks[scen.Index], false);
            this.UpdateProgress("Writing scenario " + scen.Index + " data for the EIS Fleet reports.");
            this._carFleet.ParseData(scen, eisModel.EISData.Cars[scen.Index], true);
            this._trkFleet.ParseData(scen, eisModel.EISData.Trks[scen.Index], false);
            for (int i = 0; i < EmTitles.Length; i++)
            {
                this.UpdateProgress("Writing scenario " + scen.Index + " data for the EIS " + EmTitles[i] + " Emissions reports.");
                this._carEm[i].ParseData(scen, eisModel.EISData.Cars[scen.Index].GetEmissions((EmissionType)i), EmTitles[i], true);
                this._trkEm[i].ParseData(scen, eisModel.EISData.Trks[scen.Index].GetEmissions((EmissionType)i), EmTitles[i], false);
            }
        }
        protected override void CloseReports(int snIndex)
        {
            base.CloseReports(snIndex);
            if (snIndex + 1 == this.Compliance.Settings.Scenarios.Count)
            {
                this.CloseReport(this._carFE); this._carFE = null;
                this.CloseReport(this._trkFE); this._trkFE = null;
                this.CloseReport(this._carGal); this._carGal = null;
                this.CloseReport(this._trkGal); this._trkGal = null;
                this.CloseReport(this._carVmt); this._carVmt = null;
                this.CloseReport(this._trkVmt); this._trkVmt = null;
                this.CloseReport(this._carFleet); this._carFleet = null;
                this.CloseReport(this._trkFleet); this._trkFleet = null;
                if (this._carEm != null)
                {
                    for (int i = 0; i < this._carEm.Length; i++)
                    {
                        this.CloseReport(this._carEm[i]); this._carEm[i] = null;
                    }
                }
                if (this._trkEm != null)
                {
                    for (int i = 0; i < this._trkEm.Length; i++)
                    {
                        this.CloseReport(this._trkEm[i]); this._trkEm[i] = null;
                    }
                }
                this._active = false;
            }
        }
        #endregion
        #region 
        static readonly string[] EmTitles;
        bool _active;   
        XlEISCafe _carFE, _trkFE;
        XlEISGallons _carGal, _trkGal;
        XlEISVmt _carVmt, _trkVmt;
        XlEISFleet _carFleet, _trkFleet;
        XlEISEmissions[] _carEm, _trkEm;
        #endregion
    }
}

