using System;
using System.Runtime.Serialization;
using Volpe.Cafe.Data;
using Volpe.Cafe.IO;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.XlLib;
namespace Volpe.Cafe.EIS.Settings
{
    [Serializable]
    public sealed class EISParameters : ICloneable
    {
        #region 
        [Serializable]
        public sealed class FleetData
        {
            #region 
            internal FleetData()
            {
                int length = MaxYear - MinYear + 1;
                this.CarFleet = new double[length][];
                this.TrkFleet = new double[length][];
                for (int i = 0; i < length; i++)
                {
                    this.CarFleet[i] = new double[Interaction.CY];
                    this.TrkFleet[i] = new double[Interaction.CY];
                }
            }
            #endregion
            #region 
            internal void CopyTo(FleetData value)
            {
                int length = MaxYear - MinYear + 1;
                for (int i = 0; i < length; i++)
                {
                    Array.Copy(this.CarFleet[i], value.CarFleet[i], Interaction.CY);
                    Array.Copy(this.TrkFleet[i], value.TrkFleet[i], Interaction.CY);
                }
            }
            #endregion
            #region 
            public const int MinYear = 1975;
            public const int MaxYear = 2010;
            public double[][] CarFleet;
            public double[][] TrkFleet;
            #endregion
        }
        [Serializable]
        public sealed class SalesData
        {
            #region 
            internal SalesData()
            {
                int length = MaxYear - MinYear + 1;
                this.CarSales = new double[length];
                this.TrkSales = new double[length];
            }
            #endregion
            #region 
            internal void CopyTo(SalesData value)
            {
                int length = MaxYear - MinYear + 1;
                Array.Copy(this.CarSales, value.CarSales, length);
                Array.Copy(this.TrkSales, value.TrkSales, length);
            }
            #endregion
            #region 
            public const int MinYear = 2005;
            public const int MaxYear = 2064;
            public double[] CarSales;
            public double[] TrkSales;
            #endregion
        }
        [Serializable]
        public sealed class NoCAFEData
        {
            #region 
            internal NoCAFEData()
            {
                int length = MaxYear - MinYear + 1;
                this.CarFuelEconomy = new FuelValue[length];
                this.CarFuelShare   = new FuelValue[length];
                this.TrkFuelEconomy = new FuelValue[length];
                this.TrkFuelShare   = new FuelValue[length];
            }
            #endregion
            #region 
            internal void CopyTo(NoCAFEData value)
            {
                int length = MaxYear - MinYear + 1;
                Array.Copy(this.CarFuelEconomy, value.CarFuelEconomy, length);
                Array.Copy(this.CarFuelShare  , value.CarFuelShare  , length);
                Array.Copy(this.TrkFuelEconomy, value.TrkFuelEconomy, length);
                Array.Copy(this.TrkFuelShare  , value.TrkFuelShare  , length);
            }
            #endregion
            #region 
            public const int MinYear = 1975;
            public const int MaxYear = 2064;
            public FuelValue[] CarFuelEconomy;
            public FuelValue[] CarFuelShare;
            public FuelValue[] TrkFuelEconomy;
            public FuelValue[] TrkFuelShare;
            #endregion
        }
        [Serializable]
        public sealed class OvercomplianceData
        {
            #region 
            internal OvercomplianceData()
            {
                int length = MaxYear - MinYear + 1;
                this.CarBaseGrowth = new FuelValue[length];
                this.TrkBaseGrowth = new FuelValue[length];
                this.CarScenGrowth = new FuelValue[length];
                this.TrkScenGrowth = new FuelValue[length];
            }
            #endregion
            #region 
            internal void CopyTo(OvercomplianceData value)
            {
                int length = MaxYear - MinYear + 1;
                Array.Copy(this.CarBaseGrowth, value.CarBaseGrowth, length);
                Array.Copy(this.TrkBaseGrowth, value.TrkBaseGrowth, length);
                Array.Copy(this.CarScenGrowth, value.CarScenGrowth, length);
                Array.Copy(this.TrkScenGrowth, value.TrkScenGrowth, length);
            }
            #endregion
            #region 
            public const int MinYear = 1975;
            public const int MaxYear = 2064;
            public FuelValue[] CarBaseGrowth;
            public FuelValue[] TrkBaseGrowth;
            public FuelValue[] CarScenGrowth;
            public FuelValue[] TrkScenGrowth;
            #endregion
        }
        #endregion
        #region 
        public EISParameters()
        {
            this._sales          = new SalesData ();
            this._fleet          = new FleetData ();
            this._noCAFE         = new NoCAFEData();
            this._overcompliance = new OvercomplianceData();
        }
        #endregion
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        public EISParameters Clone()
        {
            EISParameters value = new EISParameters();
            this._fleet         .CopyTo(value._fleet         );
            this._sales         .CopyTo(value._sales         );
            this._noCAFE        .CopyTo(value._noCAFE        );
            this._overcompliance.CopyTo(value._overcompliance);
            return value;
        }
        #endregion
        #region 
        public EISParameters.FleetData Fleet { get { return this._fleet; } }
        public EISParameters.SalesData Sales { get { return this._sales; } }
        public EISParameters.NoCAFEData NoCAFE { get { return this._noCAFE; } }
        public EISParameters.OvercomplianceData Overcompliance { get { return this._overcompliance; } }
        #endregion
        #region 
        internal EISParameters.FleetData          _fleet;
        internal EISParameters.SalesData          _sales;
        internal EISParameters.NoCAFEData         _noCAFE;
        internal EISParameters.OvercomplianceData _overcompliance;
        #endregion
    }
}

