using System;
using Volpe.Cafe.IO;
using Volpe.Cafe.Settings;
namespace Volpe.Cafe.EIS.Settings
{
    [Serializable]
    public class EISSettings : ISettings, ICloneable
    {
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        ISettings ISettings.Clone()
        {
            return this.Clone();
        }
        EISSettings Clone()
        {
            EISSettings value = new EISSettings();
            Clone(this, value);
            return value;
        }
        static void Clone(EISSettings src, EISSettings dst)
        {
            dst.EISParametersFile             = src.EISParametersFile;
            dst.EISTailpipeEmissionsFile      = src.EISTailpipeEmissionsFile;
            dst.EISParametersTimestamp        = src.EISParametersTimestamp;
            dst.EISTailpipeEmissionsTimestamp = src.EISTailpipeEmissionsTimestamp;
            dst._eisParameters                = (src._eisParameters        == null) ? null : src._eisParameters       .Clone();
            dst._eisTailpipeEmissions         = (src._eisTailpipeEmissions == null) ? null : src._eisTailpipeEmissions.Clone();
        }
        #endregion
        #region 
        void ISettings.SetDefaultSettings()
        {
            this.EISParametersFile        = string.Empty;
            this.EISTailpipeEmissionsFile = string.Empty;
            this._eisParameters           = null;
            this._eisTailpipeEmissions    = null;
        }
        void ISettings.UpdateSettings(ISettings newSettings)
        {
            this.UpdateSettings((EISSettings)newSettings);
        }
        public void UpdateSettings(EISSettings newSettings)
        {
            Clone(newSettings, this);
        }
        void ISettings.WriteSummaryLog(LogWriter writer)
        {
            string format = "M/dd/yy H:mm:ss";
            if (writer.Summary == null) { return; }
            string outStr = "--- Input Settings ---" +
                "\n  EISParametersFile="        + this.EISParametersFile        + " (" + this.EISParametersTimestamp       .ToString(format) + ")" +
                "\n  EISTailpipeEmissionsFile=" + this.EISTailpipeEmissionsFile + " (" + this.EISTailpipeEmissionsTimestamp.ToString(format) + ")";
            writer.Summary.WriteLine(outStr + "\n");
        }
        #endregion
        #endregion
        #region 
        public EISParameters EISParameters
        {
            get { return this._eisParameters; }
            set
            {
                if (value == null) { throw new ArgumentNullException("EISParameters", "The specified value cannot be null."); }
                this._eisParameters = value;
            }
        }
        public EISTailpipeEmissions EISTailpipeEmissions
        {
            get { return this._eisTailpipeEmissions; }
            set
            {
                if (value == null) { throw new ArgumentNullException("EISTailpipeEmissions", "The specified value cannot be null."); }
                this._eisTailpipeEmissions = value;
            }
        }
        #endregion
        #region 
        public string EISParametersFile;
        public string EISTailpipeEmissionsFile;
        public DateTime EISParametersTimestamp;
        public DateTime EISTailpipeEmissionsTimestamp;
        EISParameters _eisParameters;
        EISTailpipeEmissions _eisTailpipeEmissions;
        #endregion
    }
}

