using System;
using Volpe.Cafe.IO;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.EIS.Settings
{
    [Serializable]
    public sealed class EISTailpipeEmissions : ICloneable
    {
        #region 
        [Serializable]
        public sealed class RateValues
        {
            #region 
            internal RateValues()
            {
                int length = MaxYear - MinYear + 1;
                this.LDV = new double[length][];
                this.LDT = new double[length][];
                for (int i = 0; i < length; i++)
                {
                    this.LDV[i] = new double[Interaction.CY];
                    this.LDT[i] = new double[Interaction.CY];
                }
            }
            #endregion
            #region 
            internal void CopyTo(RateValues value)
            {
                int length = MaxYear - MinYear + 1;
                for (int i = 0; i < length; i++)
                {
                    Array.Copy(this.LDV[i], value.LDV[i], Interaction.CY);
                    Array.Copy(this.LDT[i], value.LDT[i], Interaction.CY);
                }
            }
            #endregion
            #region 
            public double[] this[ModelYear year, Mobile6Class value] { get { return this[year.Year, value]; } }
            public double[] this[int modelYear, Mobile6Class value]
            {
                get
                {
                    int index = (modelYear < MinYear) ? 0 : (modelYear > MaxYear) ? MaxYear - MinYear : modelYear - MinYear;
                    return
                        (value == Mobile6Class.LDV                                                            ) ? this.LDV[index] :
                        (value == Mobile6Class.LDT1 || value == Mobile6Class.LDT2 || value == Mobile6Class.HDV) ? this.LDT[index] : null;
                }
            }
            #endregion
            #region 
            public const int MinYear = 1975;
            public const int MaxYear = 2011;
            public double[][] LDV;
            public double[][] LDT;
            #endregion
        }
        [Serializable]
        public sealed class Emissions
        {
            #region 
            internal Emissions()
            {
                this.Gasoline  = new RateValues();
                this.Diesel    = new RateValues();
                this.Ethanol85 = new RateValues();
            }
            #endregion
            #region 
            internal void CopyTo(Emissions value)
            {
                this.Gasoline .CopyTo(value.Gasoline );
                this.Diesel   .CopyTo(value.Diesel   );
                this.Ethanol85.CopyTo(value.Ethanol85);
            }
            #endregion
            #region 
            public double[] this[ModelYear year, FuelType fuel, Mobile6Class value] { get { return this[year.Year, fuel, value]; } }
            public double[] this[int modelYear, FuelType fuel, Mobile6Class value]
            {
                get
                {
                    return
                        (fuel == FuelType.Gasoline ) ? this.Gasoline [modelYear, value] :
                        (fuel == FuelType.Diesel   ) ? this.Diesel   [modelYear, value] :
                        (fuel == FuelType.Ethanol85) ? this.Ethanol85[modelYear, value] : null;
                }
            }
            #endregion
            #region 
            public RateValues Gasoline;
            public RateValues Diesel;
            public RateValues Ethanol85;
            #endregion
        }
        #endregion
        #region 
        public EISTailpipeEmissions()
        {
            this._co  = new Emissions();
            this._voc = new Emissions();
            this._nox = new Emissions();
            this._pm  = new Emissions();
            this._acetaldehyde = new Emissions();
            this._acrolein     = new Emissions();
            this._benzene      = new Emissions();
            this._butadiene    = new Emissions();
            this._formaldehyde = new Emissions();
            this._ch4  = new Emissions();
            this._dpm  = new Emissions();
            this._mtbe = new Emissions();
            this._n2o  = new Emissions();
        }
        #endregion
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        public EISTailpipeEmissions Clone()
        {
            EISTailpipeEmissions value = new EISTailpipeEmissions();
            this._co .CopyTo(value._co );
            this._voc.CopyTo(value._voc);
            this._nox.CopyTo(value._nox);
            this._pm .CopyTo(value._pm );
            this._acetaldehyde.CopyTo(value._acetaldehyde);
            this._acrolein    .CopyTo(value._acrolein    );
            this._benzene     .CopyTo(value._benzene     );
            this._butadiene   .CopyTo(value._butadiene   );
            this._formaldehyde.CopyTo(value._formaldehyde);
            this._ch4 .CopyTo(value._ch4 );
            this._dpm .CopyTo(value._dpm );
            this._mtbe.CopyTo(value._mtbe);
            this._n2o .CopyTo(value._n2o );
            return value;
        }
        #endregion
        #region 
        public EISTailpipeEmissions.Emissions CO { get { return this._co; } }
        public EISTailpipeEmissions.Emissions VOC { get { return this._voc; } }
        public EISTailpipeEmissions.Emissions NOX { get { return this._nox; } }
        public EISTailpipeEmissions.Emissions PM { get { return this._pm; } }
        public EISTailpipeEmissions.Emissions Acetaldehyde { get { return this._acetaldehyde; } }
        public EISTailpipeEmissions.Emissions Acrolein { get { return this._acrolein; } }
        public EISTailpipeEmissions.Emissions Benzene { get { return this._benzene; } }
        public EISTailpipeEmissions.Emissions Butadiene { get { return this._butadiene; } }
        public EISTailpipeEmissions.Emissions Formaldehyde { get { return this._formaldehyde; } }
        public EISTailpipeEmissions.Emissions CH4 { get { return this._ch4; } }
        public EISTailpipeEmissions.Emissions DieselPM { get { return this._dpm; } }
        public EISTailpipeEmissions.Emissions MTBE { get { return this._mtbe; } }
        public EISTailpipeEmissions.Emissions N2O { get { return this._n2o; } }
        #endregion
        #region 
        internal EISTailpipeEmissions.Emissions _co, _voc, _nox, _pm;
        internal EISTailpipeEmissions.Emissions _acetaldehyde, _acrolein, _benzene, _butadiene, _formaldehyde;
        internal EISTailpipeEmissions.Emissions _ch4, _dpm, _mtbe, _n2o;
        #endregion
    }
}

