#region << Using Directives >>
using System;
using System.Collections.Generic;
using Volpe.Cafe.Data;
using Volpe.Cafe.IO;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using RC = Volpe.Cafe.RegulatoryClass;
using System.Text;
#endregion
namespace Volpe.Cafe.MonteCarlo.Data
{
    [Serializable]
    public class TrialData : ICloneable
    {
        #region 
        TrialData() { }
        public TrialData(TrialInfo info)
        {
            this.TrialInfo      = info;
            this._isBaseline    = false;
            this.HasData        = false;
            this.Sales          = null;
            this.PCShare        = null;
            this.TechCosts      = null;
            this.Fines          = null;
            this.SocialCosts    = null;
            this.SocialBenefits = null;
            this.NetBenefits    = null;
            this.AdditionalVMT  = null;
            this.FuelSavings    = null;
            this.FuelSavingsGal = null;
        }
        #endregion
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        public TrialData Clone()
        {
            TrialData td = new TrialData();
            td._isBaseline = this._isBaseline;
            td.HasData     = this.HasData;
            td.TrialInfo   = this.TrialInfo.Clone();
            td.Sales          = (RCDouble[])Interaction.CloneArray(this.Sales         , typeof(RCDouble));
            td.PCShare        = (double  [])Interaction.CloneArray(this.PCShare       , typeof(double  ));
            td.TechCosts      = (RCDouble[])Interaction.CloneArray(this.TechCosts     , typeof(RCDouble));
            td.Fines          = (RCDouble[])Interaction.CloneArray(this.Fines         , typeof(RCDouble));
            td.SocialCosts    = (RCDouble[])Interaction.CloneArray(this.SocialCosts   , typeof(RCDouble));
            td.SocialBenefits = (RCDouble[])Interaction.CloneArray(this.SocialBenefits, typeof(RCDouble));
            td.NetBenefits    = (RCDouble[])Interaction.CloneArray(this.NetBenefits   , typeof(RCDouble));
            td.AdditionalVMT  = (RCDouble[])Interaction.CloneArray(this.AdditionalVMT , typeof(RCDouble));
            td.FuelSavings    = (RCDouble[])Interaction.CloneArray(this.FuelSavings   , typeof(RCDouble));
            td.FuelSavingsGal = (RCDouble[])Interaction.CloneArray(this.FuelSavingsGal, typeof(RCDouble));
            return td;
        }
        #endregion
        public void SaveTrialData(Scenario scen, int minYearIndex, int yearCount, TrialData baseTd, LogWriter writer,
            Industry[] modelData)
        {
            this._isBaseline    = scen.IsBaseline;
            this.Sales          = new RCDouble[yearCount];
            this.PCShare        = new double  [yearCount];
            this.TechCosts      = new RCDouble[yearCount];
            this.Fines          = new RCDouble[yearCount];
            this.SocialCosts    = new RCDouble[yearCount];
            this.SocialBenefits = new RCDouble[yearCount];
            this.NetBenefits    = new RCDouble[yearCount];
            this.AdditionalVMT  = new RCDouble[yearCount];
            this.FuelSavings    = new RCDouble[yearCount];
            this.FuelSavingsGal = new RCDouble[yearCount];
            for (int j = 0; j < yearCount; j++)
            {
                Industry ind = modelData[minYearIndex + j];
                for (int k = 0, mfrCount = ind.ManufacturerCount; k < mfrCount; k++)
                {
                    Manufacturer.CModelData mmd = ind.Manufacturers[k].ModelData;
                    this.Sales      [j] += mmd.Sales;
                    this.TechCosts  [j] += mmd.TechCost         / 1e6;  
                    this.Fines      [j] += mmd.Fines            / 1e6;
                    this.SocialCosts[j] += mmd.TotalSocialCosts / 1e6;
                }
                this.PCShare[j] = this.Sales[j].PassengerAuto / this.Sales[j].TotalAll;
                EffectsData ied = ind.EffectsData;
                this.SocialBenefits[j] = ied.DiscountedOwnerAndSocietalCosts[0].TotalSocialCosts / 1e3;    
                this.NetBenefits   [j] = RCDouble.Zero;
                this.AdditionalVMT [j] = ied.VMT.ToRCDouble();
                this.FuelSavings   [j] = ied.DiscountedOwnerAndSocietalCosts[0].RetailFuelCosts / 1e3;
                this.FuelSavingsGal[j] = ied.Gallons.ToRCDouble();
                if (!this._isBaseline)
                {
                    this.TechCosts     [j] = this  .TechCosts     [j] - baseTd.TechCosts     [j];
                    this.Fines         [j] = this  .Fines         [j] - baseTd.Fines         [j];
                    this.SocialCosts   [j] = this  .SocialCosts   [j] - baseTd.SocialCosts   [j];
                    this.SocialBenefits[j] = baseTd.SocialBenefits[j] - this  .SocialBenefits[j];
                    this.NetBenefits   [j] = this  .SocialBenefits[j] - this  .SocialCosts   [j];
                    this.AdditionalVMT [j] = this  .AdditionalVMT [j] - baseTd.AdditionalVMT [j];
                    this.FuelSavings   [j] = baseTd.FuelSavings   [j] - this  .FuelSavings   [j];
                    this.FuelSavingsGal[j] = baseTd.FuelSavingsGal[j] - this  .FuelSavingsGal[j];
                }
            }
            this.HasData = true;
        }
        public static string CreateCsvHeader(Industry data, bool includeTrialInfo)
        {
            int minYear = data.MinYear.Year;
            int maxYear = data.MaxYear.Year;
            string sales = "", pcShare = "", techCosts = "", fines = "", socialCosts = "", socialBenefits = "", netBenefits = "", vmt = "",
                fuelSavings = "", fuelSavingsGal = "", bcr = "";
            for (int i = minYear; i <= maxYear; i++)
            {
                sales          += "Sales_PC_"          + i + ",Sales_LT_"          + i + ",Sales_"          + i + ",";
                pcShare        += "PCShare_"           + i +                                                      ",";
                techCosts      += "TechCosts_PC_"      + i + ",TechCosts_LT_"      + i + ",TechCosts_"      + i + ",";
                fines          += "Fines_PC_"          + i + ",Fines_LT_"          + i + ",Fines_"          + i + ",";
                socialCosts    += "SocialCosts_PC_"    + i + ",SocialCosts_LT_"    + i + ",SocialCosts_"    + i + ",";
                socialBenefits += "SocialBenefits_PC_" + i + ",SocialBenefits_LT_" + i + ",SocialBenefits_" + i + ",";
                netBenefits    += "NetBenefits_PC_"    + i + ",NetBenefits_LT_"    + i + ",NetBenefits_"    + i + ",";
                vmt            += "AdditionalVMT_PC_"  + i + ",AdditionalVMT_LT_"  + i + ",AdditionalVMT_"  + i + ",";
                fuelSavings    += "FuelSavings_PC_"    + i + ",FuelSavings_LT_"    + i + ",FuelSavings_"    + i + ",";
                fuelSavingsGal += "FuelSavingsGal_PC_" + i + ",FuelSavingsGal_LT_" + i + ",FuelSavingsGal_" + i + ",";
                bcr            += "BCRatio_PC_"        + i + ",BCRatio_LT_"        + i + ",BCRatio_"        + i + ",";
            }
            return ((includeTrialInfo) ? TrialInfo.CreateCsvHeader() + "," : "Index,") +
                sales + pcShare + techCosts + fines + socialCosts + socialBenefits + netBenefits + vmt + fuelSavings + fuelSavingsGal +
                bcr.Remove(bcr.Length - 1, 1);
        }
        public string ToCsvString(bool includeTrialInfo)
        {
            StringBuilder sb = new StringBuilder();
            sb.Append(((includeTrialInfo) ? this.TrialInfo.ToCsvString() : this.TrialInfo.Index.ToString()));
            sb.Append(',');
            this.ToCsvStringHelper(sb, this.Sales);
            this.ToCsvStringHelper(sb, this.PCShare);
            this.ToCsvStringHelper(sb, this.TechCosts);
            this.ToCsvStringHelper(sb, this.Fines);
            this.ToCsvStringHelper(sb, this.SocialCosts);
            this.ToCsvStringHelper(sb, this.SocialBenefits);
            this.ToCsvStringHelper(sb, this.NetBenefits);
            this.ToCsvStringHelper(sb, this.AdditionalVMT);
            this.ToCsvStringHelper(sb, this.FuelSavings);
            this.ToCsvStringHelper(sb, this.FuelSavingsGal);
            for (int i = 0, count = this.TechCosts.Length; i < count; i++)
            {
                if (this._isBaseline) { sb.Append("0,0,0,"); }
                else
                {
                    if (this.SocialCosts[i].PassengerAuto == 0) { sb.Append("0,"); } else { sb.Append(this.SocialBenefits[i].PassengerAuto / this.SocialCosts[i].PassengerAuto); sb.Append(','); }
                    if (this.SocialCosts[i].LTUnregulated == 0) { sb.Append("0,"); } else { sb.Append(this.SocialBenefits[i].LTUnregulated / this.SocialCosts[i].LTUnregulated); sb.Append(','); }
                    if (this.SocialCosts[i].TotalAll      == 0) { sb.Append("0,"); } else { sb.Append(this.SocialBenefits[i].TotalAll      / this.SocialCosts[i].TotalAll     ); sb.Append(','); }
                }
            }
            return sb.ToString(0, sb.Length - 1);
        }
        void ToCsvStringHelper(StringBuilder sb, RCDouble[] arr)
        {
            for (int i = 0, count = arr.Length; i < count; i++)
            {
                sb.Append(arr[i].PassengerAuto); sb.Append(',');
                sb.Append(arr[i].LTUnregulated); sb.Append(',');
                sb.Append(arr[i].TotalAll     ); sb.Append(',');
            }
        }
        void ToCsvStringHelper(StringBuilder sb, double[] arr)
        {
            for (int i = 0, count = arr.Length; i < count; i++)
            {
                sb.Append(arr[i]); sb.Append(',');
            }
        }
        #endregion
        #region 
        bool _isBaseline;
        public bool HasData;
        public TrialInfo TrialInfo;
        public RCDouble[] Sales;
        public double[] PCShare;
        public RCDouble[] TechCosts;
        public RCDouble[] Fines;
        public RCDouble[] SocialCosts;
        public RCDouble[] SocialBenefits;
        public RCDouble[] NetBenefits;
        public RCDouble[] AdditionalVMT;
        public RCDouble[] FuelSavings;
        public RCDouble[] FuelSavingsGal;
        #endregion
    }
}

