#region << Using Directives >>
using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
#endregion
namespace Volpe.Cafe.MonteCarlo.Data
{
    [Serializable]
    public class TrialInfo : ICloneable
    {
        #region 
        public TrialInfo() { }
        #endregion
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        public TrialInfo Clone()
        {
            TrialInfo ti = new TrialInfo();
            ti.Index               = this.Index;
            ti.FleetShare          = this.FleetShare;
            ti.FuelPriceEstimates  = this.FuelPriceEstimates;
            ti.PaybackPeriod_OC    = this.PaybackPeriod_OC;
            ti.ReboundEffect       = this.ReboundEffect;
            ti.PriceShockCost      = this.PriceShockCost;
            ti.MilesCars0          = this.MilesCars0;
            ti.MilesAllLTs0        = this.MilesAllLTs0;
            ti.TechCostScaleFactor = (double[])Interaction.CloneArray(this.TechCostScaleFactor, typeof(double));
            ti.TechFCScaleFactor   = (double[])Interaction.CloneArray(this.TechFCScaleFactor  , typeof(double));
            return ti;
        }
        #endregion
        public void InitializeTrial(Industry data, ModelingSettings settings)
        {
            List<Manufacturer> mfrs = data.Manufacturers;
            for (int j = 0, mfrCount = data.ManufacturerCount; j < mfrCount; j++)
            {
                mfrs[j].Description.PaybackPeriod_OC = this.PaybackPeriod_OC;
            }
            settings.Parameters.UpdateForMonteCarlo(this.FuelPriceEstimates, this.ReboundEffect, this.PriceShockCost,
                this.MilesCars0, this.MilesAllLTs0);
            settings.OperatingModes.DFS_Epsilon = this.FleetShare;
            List<Technology> techs = settings.Technologies;
            double[] costScale = this.TechCostScaleFactor;
            double[] fcScale   = this.TechFCScaleFactor;
            for (int i = 0, techCount = techs.Count; i < techCount; i++)
            {
                Technology tech = techs[i];
                TechnologyAttributesTable.TechnologyAttributes[] techAttr = tech.Attributes.ToArray();
                int techAttrCount = techAttr.Length;
                for (int j = 0; j < techAttrCount; j++)
                {
                    for (int k = 0; k < techAttr[j].CostTable.Length; k++)
                    {
                        techAttr[j].CostTable[k] *= costScale[i];
                    }
                    techAttr[j].FC *= fcScale[i];
                    if (tech.Index == TechnologyIndexes.PHEV1 && techAttr[j].FC > 0.42)
                    {
                        techAttr[j].FC = 0.42;  
                    }
                } 
            } 
        }
        public static string CreateCsvHeader()
        {
            string cost = "", fc = "";
            for (int i = 0; i < TechnologyIndexes.TechnologyCount; i++)
            {
                cost += "Cost_" + TechnologyIndexes.TechAbbr[i] + ",";
                fc   += "FC_"   + TechnologyIndexes.TechAbbr[i] + ",";
            }
            return "Index,FleetShare,FuelPriceEstimates,PaybackPeriod_OC,ReboundEffect,PriceShockCost," +
                "MilesCars0,MilesAllLTs0" + cost + fc.Remove(fc.Length - 1, 1);
        }
        public string ToCsvString()
        {
            return
                this.Index                + "," +
                this.FleetShare           + "," +
                this.FuelPriceEstimates   + "," +
                this.PaybackPeriod_OC     + "," +
                this.ReboundEffect        + "," +
                this.PriceShockCost       + "," +
                this.MilesCars0           + "," +
                this.MilesAllLTs0         + "," +
                this.ArrToCsv(this.TechCostScaleFactor) + "," +
                this.ArrToCsv(this.TechFCScaleFactor  );
        }
        public void FromCsvString(string value)
        {
            string[] parts = value.Split(',');
            this.Index                = int   .Parse(parts[0]);
            this.FleetShare           = double.Parse(parts[1]);
            this.FuelPriceEstimates   = double.Parse(parts[2]);
            this.PaybackPeriod_OC     = double.Parse(parts[3]);
            this.ReboundEffect        = double.Parse(parts[4]);
            this.PriceShockCost       = double.Parse(parts[5]);
            this.MilesCars0           = double.Parse(parts[6]);
            this.MilesAllLTs0         = double.Parse(parts[7]);
            this.TechCostScaleFactor  = new double[TechnologyIndexes.TechnologyCount];
            this.TechFCScaleFactor    = new double[TechnologyIndexes.TechnologyCount];
            for (int i = 0; i < TechnologyIndexes.TechnologyCount; i++)
            {   
                this.TechCostScaleFactor[i] = double.Parse(parts[8 +                                     i]);
                this.TechFCScaleFactor  [i] = double.Parse(parts[8 + TechnologyIndexes.TechnologyCount + i]);
            }
        }
        string ArrToCsv(double[] arr)
        {
            string result = "";
            for (int i = 0, count = arr.Length; i < count; i++)
            {
                result += arr[i] + ",";
            }
            return result.Remove(result.Length - 1, 1);
        }
        #endregion
        #region 
        public int Index;
        public double FleetShare;
        public double FuelPriceEstimates;
        public double PaybackPeriod_OC;
        public double ReboundEffect;
        public double PriceShockCost;
        public double MilesCars0;
        public double MilesAllLTs0;
        public double[] TechCostScaleFactor;
        public double[] TechFCScaleFactor;
        #endregion
    }
}

