using System;
namespace Volpe.Cafe.Optimization.Data
{
    [Serializable]
    public class IterationsData
    {
        #region 
        public IterationsData(int iterationCount)
        {
            this.Iterations = new IterationData[iterationCount];
            for (int i = 0; i < iterationCount; i++)
            {
                this.Iterations[i] = new IterationData();
            }
        }
        #endregion
        #region 
        public void Clear()
        {
            for (int i = 0; i < this.Iterations.Length; i++)
            {
                this.Iterations[i].Reset();
            }
            this.Optimized = 0;
        }
        public double GetStandard()
        {
            return (this.Optimized < 0) ? double.NaN : this.Iterations[this.Optimized].Standard;
        }
        public double[] GetCoefficients()
        {
            return (this.Optimized < 0) ? null : this.Iterations[this.Optimized].Coefficients;
        }
        public void SetOptimizedIndexFromStnd(double standard)
        {
            int i = 0, count = this.Iterations.Length;
            for (; i < count; i++)
            {
                if      (!this.Iterations[i].Valid              ) { i--; break; }    
                else if (this.Iterations[i].Standard >= standard) {      break; }    
            }
            if (i == count) { i = count - 1; }
            this.Optimized = i;
        }
        public void SetOptimizedIndexFromCoefs(double a, double b)
        {
            int i = 0, count = this.Iterations.Length;
            for (; i < count; i++)
            {
                if      (!this.Iterations[i].Valid              ) { i--; break; }    
                else if (this.Iterations[i].Coefficients[0] >= a &&
                         this.Iterations[i].Coefficients[1] >= b) {      break; }    
            }
            if (i == count) { i = count - 1; }
            this.Optimized = i;
        }
        #endregion
        #region 
        public IterationData[] Iterations;
        public int Optimized;
        #endregion
    }
}

