#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.IO.Reporting.CSV;
using Volpe.Cafe.Model;
using Volpe.Cafe.Optimization.Data;
using Volpe.Cafe.Optimization.Model;
using Volpe.Cafe.Settings;
#endregion
namespace Volpe.Cafe.Optimization.IO.Reporting
{
    [Serializable]
    public class CsvOptimizationReport : CsvReportingBase
    {
        #region 
        public CsvOptimizationReport(string path, bool append) : base(path, append) { }
        #endregion
        #region 
        protected override string GetHeader()
        {
            return Header;
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, ICompliance compliance)
        {
            if (scen.IsBaseline) { return; }
            int yrIndex = year.Index - compliance.Data.MinYear.Index;
            IterationsData itersData = ((IOptimizationModel)compliance).OptimizationData.IndustryData[yrIndex];
            int iterCount = itersData.Iterations.Length;
            int optIndex = itersData.Optimized;
            for (int i = 0; i < iterCount; i++)
            {
                if (itersData.Iterations[i].Valid)
                {
                    this.WriteIterationData(year, i, itersData.Iterations[i], optIndex);
                }
            }
        }
        void WriteIterationData(ModelYear year, int iteration, IterationData iterData, int optIndex)
        {
            this.Write(year.ToString());
            this.Write(iteration.ToString());
            this.Write((iteration == optIndex) ? "Y" : "N");
            this.Write(iterData.Coefficients);
            this.Write(iterData.Standard);
            this.Write(iterData.CAFE);
            this.Write(iterData.TotalSocialCosts);
            this.Write(iterData.TotalSocialBenefits);
            this.Write(iterData.TotalSocialBenefitsDisc);
            this.Write(iterData.NetBenefits);
            this.Write(iterData.FuelSavings);
            this.WriteLine();
        }
        void Write(double[] coefs)
        {
            for (int i = 0; i < coefs.Length; i++)
            {
                this.Write(coefs[i]);
            }
        }
        #endregion
        #region 
        public override string ReportName { get { return CsvReportName; } }
        #endregion
        #region 
        public const string CsvReportName = "Optimization CSV Report";
        const string Header =
            "Model Year,Iteration,Is Optimal," +
            "Coef-A,Coef-B,Coef-C,Coef-D,Coef-E,Coef-F,Coef-G,Coef-H,Standard,CAFE," +
            "Total Social Costs,Total Social Benefits,Disc Social Benefits,Net Benefits (m$),Fuel Savings (m-gal)";
        #endregion
    }
}

