using System;
using Volpe.Cafe.IO;
using Volpe.Cafe.Settings;
namespace Volpe.Cafe.Optimization.Settings
{
    [Serializable]
    public class OptimizationSettings : ISettings, ICloneable
    {
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        ISettings ISettings.Clone()
        {
            return this.Clone();
        }
        OptimizationSettings Clone()
        {
            OptimizationSettings value = new OptimizationSettings();
            Clone(this, value);
            return value;
        }
        static void Clone(OptimizationSettings src, OptimizationSettings dst)
        {
            dst.Type                   = src.Type;
            dst.Mode                   = src.Mode;
            dst.AboveOptimum           = src.AboveOptimum;
            dst.BelowOptimum           = src.BelowOptimum;
            dst.Increment              = src.Increment;
            dst.BackstopStandard       = src.BackstopStandard;
            dst.BackstopAsymptotes     = src.BackstopAsymptotes;
            dst.BackstopTargetFunction = src.BackstopTargetFunction;
            dst.StartYear              = src.StartYear;
        }
        #endregion
        #region 
        void ISettings.SetDefaultSettings()
        {
            this.Type                   = OptimizationType.Default;
            this.Mode                   = OptimizationMode.MaximumNetBenefits;
            this.AboveOptimum           = 75;
            this.BelowOptimum           = 75;
            this.Increment              = 0.0001;
            this.BackstopStandard       = true;
            this.BackstopAsymptotes     = false;
            this.BackstopTargetFunction = false;
            this.StartYear              = 0;
        }
        void ISettings.UpdateSettings(ISettings newSettings)
        {
            this.UpdateSettings((OptimizationSettings)newSettings);
        }
        public void UpdateSettings(OptimizationSettings newSettings)
        {
            Clone(newSettings, this);
        }
        public void WriteSummaryLog(LogWriter writer)
        {
            if (writer.Summary == null) { return; }
            string outStr = "--- Optimization Settings ---"                 +
                "\n  Type="                   + this.Type                   +
                "\n  Mode="                   + this.Mode                   +
                "\n  AboveOptimum="           + this.AboveOptimum           +
                "\n  BelowOptimum="           + this.BelowOptimum           +
                "\n  Increment="              + this.Increment              +
                "\n  BackstopStandard="       + this.BackstopStandard       +
                "\n  BackstopAsymptotes="     + this.BackstopAsymptotes     +
                "\n  BackstopTargetFunction=" + this.BackstopTargetFunction +
                "\n  StartYear="              + this.StartYear;
            writer.Summary.WriteLine(outStr + "\n");
        }
        #endregion
        #endregion
        #region 
        public OptimizationType Type;
        public OptimizationMode Mode;
        public int AboveOptimum;
        public int BelowOptimum;
        public double Increment;
        public bool BackstopStandard;
        public bool BackstopAsymptotes;
        public bool BackstopTargetFunction;
        public int StartYear;
        #endregion
    }
}

