using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using Volpe.Cafe;
using Volpe.Cafe.MonteCarlo.Settings;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.UI.Panels
{
    public class MonteCarloPanel : CafeUserControl
    {
        #region 
        public MonteCarloPanel()
        {
            this.InitializeComponent();
            this.PostInitialize();
        }
        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);
        }
        #endregion
        #region 
        #region 
        void InitializeComponent()
        {
            this.ux_trialsFile = new System.Windows.Forms.TextBox();
            this.ux_trialsFileLabel = new System.Windows.Forms.Label();
            this.ux_browseTrialsFile = new System.Windows.Forms.Button();
            this.SuspendLayout();
            this.ux_trialsFile.Anchor = ((System.Windows.Forms.AnchorStyles)(((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Left)
                        | System.Windows.Forms.AnchorStyles.Right)));
            this.ux_trialsFile.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle;
            this.ux_trialsFile.Location = new System.Drawing.Point(8, 24);
            this.ux_trialsFile.Name = "ux_trialsFile";
            this.ux_trialsFile.Size = new System.Drawing.Size(343, 20);
            this.ux_trialsFile.TabIndex = 4;
            this.ux_trialsFile.WordWrap = false;
            this.ux_trialsFileLabel.AutoSize = true;
            this.ux_trialsFileLabel.Location = new System.Drawing.Point(8, 8);
            this.ux_trialsFileLabel.Name = "ux_trialsFileLabel";
            this.ux_trialsFileLabel.Size = new System.Drawing.Size(266, 13);
            this.ux_trialsFileLabel.TabIndex = 0;
            this.ux_trialsFileLabel.Text = "Please select an input file containing Monte-Carlo trials:";
            this.ux_browseTrialsFile.Anchor = ((System.Windows.Forms.AnchorStyles)((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Right)));
            this.ux_browseTrialsFile.FlatStyle = System.Windows.Forms.FlatStyle.Popup;
            this.ux_browseTrialsFile.Location = new System.Drawing.Point(352, 24);
            this.ux_browseTrialsFile.Name = "ux_browseTrialsFile";
            this.ux_browseTrialsFile.Size = new System.Drawing.Size(80, 21);
            this.ux_browseTrialsFile.TabIndex = 5;
            this.ux_browseTrialsFile.Text = "Browse";
            this.Controls.Add(this.ux_browseTrialsFile);
            this.Controls.Add(this.ux_trialsFile);
            this.Controls.Add(this.ux_trialsFileLabel);
            this.MinimumSize = new System.Drawing.Size(440, 0);
            this.Name = "MonteCarloPanel";
            this.Size = new System.Drawing.Size(440, 56);
            this.ResumeLayout(false);
            this.PerformLayout();
        }
        #endregion
        void PostInitialize()
        {
            this.InitializeTextBoxDragDrop(this.ux_trialsFile);
            this.AutoInitializeCheckChangesDetector(this.Controls);
            this.ux_trialsFile           .TextChanged    += new EventHandler(this.TrialsFile_TextChanged);
            this.ux_trialsFile           .EnabledChanged += new EventHandler(this.TrialsFile_EnabledChanged);
            this.ux_browseTrialsFile     .Click          += new EventHandler(this.BrowseTrialsFile_Click);
        }
        void TrialsFile_TextChanged(object sender, EventArgs e)
        {
            TextBox tb = (TextBox)sender;
            if (File.Exists(tb.Text))
            {
                bool trialsFileChanged = !this.CompareFiles(this._mcSettings.TrialsFile, tb.Text);
                tb.ForeColor = (this._visualTracking && trialsFileChanged) ? this._visualTrackingColor : Control.DefaultForeColor;
                tb.BackColor = Color.Empty;
                this._messages.RemoveMessage(tb);
            }
            else
            {
                this._messages.AddMessage("The specified location for Monte-Carlo trials file does not exist on disk.", null, Color.Red, tb);
                tb.BackColor = Color.Honeydew;
                tb.ForeColor = Color.Red;
            }
        }
        void TrialsFile_EnabledChanged(object sender, EventArgs e)
        {
            TextBox tb = (TextBox)sender;
            if (tb.Enabled) { this.TrialsFile_TextChanged(sender, e); }
            else            { this._messages.RemoveMessage(sender); tb.BackColor = Color.Empty; }
        }
        bool CompareFiles(string file1, string file2)
        {
            if (File.Exists(file1)) { file1 = Path.GetFullPath(file1.TrimEnd('\\')); }
            if (File.Exists(file2)) { file2 = Path.GetFullPath(file2.TrimEnd('\\')); }
            return (file1 == file2);
        }
        void BrowseTrialsFile_Click(object sender, EventArgs e)
        {
            TextBox tb = this.ux_trialsFile;
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.Filter = "All Files (*.*)|*.*|CSV Files(*.*)|*.*";
            ofd.FilterIndex = 1;
            ofd.Multiselect = false;
            ofd.FileName = File.Exists(tb.Text) ? tb.Text : "";
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                tb.Text = ofd.FileName;
            }
        }
        void InitializeTextBoxDragDrop(params TextBox[] values)
        {
            for (int i = 0; i < values.Length; i++)
            {
                values[i].AllowDrop = true;
                values[i].DragEnter += new DragEventHandler(this.MonteCarloPanel_DragEnter);
                values[i].DragDrop  += new DragEventHandler(this.MonteCarloPanel_DragDrop);
            }
        }
        void MonteCarloPanel_DragEnter(object sender, DragEventArgs e)
        {
            if (e.Data.GetDataPresent(DataFormats.FileDrop, false))
            {
                e.Effect = DragDropEffects.All;
            }
        }
        void MonteCarloPanel_DragDrop(object sender, DragEventArgs e)
        {
            TextBox tb = (TextBox)sender;
            string[] files = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            for (int i = 0; i < files.Length; i++)
            {
                if (File.Exists(files[i]))
                {
                    tb.Text = files[i];
                    break;
                }
            }
        }
        public void SetInitialState(MonteCarloSettings monteCarloSettings)
        {
            this._mcSettings = monteCarloSettings;
            this.ux_trialsFile.Text = this._mcSettings.TrialsFile;
            this.SaveChanges();
        }
        public override void SaveChanges()
        {
            if (this.SettingsChanged)
            {   
                this._mcSettings.TrialsFile = this.ux_trialsFile.Text;
                base.SaveChanges();
                this.PostSave();
            }
        }
        void PostSave()
        {
            if (this._visualTracking)
            {   
                this.ux_trialsFile.ForeColor = Control.DefaultForeColor;
                this.ux_trialsFile.BackColor = Color  .Empty;
            }
        }
        #endregion
        #region 
        [Browsable(false)]
        public override bool SettingsChanged
        {
            get
            {
                if (!File.Exists(this.ux_trialsFile.Text)) { return false; }
                return base.SettingsChanged ||
                    (this.ux_trialsFile.Enabled && !this.CompareFiles(this._mcSettings.TrialsFile, this.ux_trialsFile.Text));
            }
        }
        #endregion
        #region 
        #region 
        Label    ux_trialsFileLabel;
        TextBox  ux_trialsFile;
        Button ux_browseTrialsFile;
        #endregion
        MonteCarloSettings _mcSettings;
        #endregion
    }
}

