using System;
using System.Collections.Generic;
using Volpe.Cafe;
namespace Volpe.Cafe.Data
{
    [Serializable]
    public abstract class Component
    {
        #region 
        protected Component()
        {
            this._vehicles     = new List<Vehicle>();
            this._manufacturer = null;
            this._parent       = null;
        }
        #endregion
        #region 
        internal virtual void Initialize(Manufacturer manufacturer)
        {
            if (this._vehicles == null) { this._vehicles = new List<Vehicle>(); } else { this._vehicles.Clear(); }
            this._manufacturer = manufacturer;
            this._parent = null;
        }
        protected void CopyTo(Component value)
        {
            if (this._vehicles != null) { value._vehicles = new List<Vehicle>(this._vehicles); }
            value._manufacturer = this._manufacturer;
            value._parent       = this._parent;
        }
        protected void SplitTo(List<Vehicle> vehs, Component value)
        {
            for (int i = 0, vehCount = vehs.Count; i < vehCount; i++)
            {
                this._vehicles.Remove(vehs[i]);
            }
            for (int i = 0, vehCount = this._vehicles.Count; i < vehCount; i++)
            {
                value._vehicles.Remove(this._vehicles[i]);
            }
            value._parent = this;
        }
        internal virtual void RemoveVehicle(Vehicle veh)
        {
            this._vehicles.Remove(veh);
        }
        #endregion
        #region 
        public List<Vehicle> Vehicles { get { return this._vehicles; } }
        public Manufacturer Manufacturer { get { return this._manufacturer; } }
        #endregion
        #region 
        protected List<Vehicle> _vehicles;
        protected Manufacturer _manufacturer;
        protected Component _parent;        
        #endregion
    }
}

