#region << Using Directives >>
using System;
using System.Collections.Generic;
using Volpe.Cafe.Utils;
#endregion
namespace Volpe.Cafe.Data
{
    [Serializable]
    public sealed class EffectsData
    {
        #region 
        [Serializable]
        public struct FuelValueByRc
        {
            #region 
            FuelValueByRc(FuelValue unregulated, FuelValue domesticAuto, FuelValue importedAuto, FuelValue lightTruck)
            {
                this.Unregulated  = unregulated;
                this.DomesticAuto = domesticAuto;
                this.ImportedAuto = importedAuto;
                this.LightTruck   = lightTruck;
            }
            #endregion
            #region 
            #region 
            public static FuelValueByRc operator +(FuelValueByRc value1, FuelValueByRc value2)
            {
                return new FuelValueByRc(value1.Unregulated + value2.Unregulated, value1.DomesticAuto + value2.DomesticAuto,
                    value1.ImportedAuto + value2.ImportedAuto, value1.LightTruck + value2.LightTruck);
            }
            public static FuelValueByRc operator -(FuelValueByRc value1, FuelValueByRc value2)
            {
                return new FuelValueByRc(value1.Unregulated - value2.Unregulated, value1.DomesticAuto - value2.DomesticAuto,
                    value1.ImportedAuto - value2.ImportedAuto, value1.LightTruck - value2.LightTruck);
            }
            public static FuelValueByRc operator *(FuelValueByRc value1, FuelValueByRc value2)
            {
                return new FuelValueByRc(value1.Unregulated * value2.Unregulated, value1.DomesticAuto * value2.DomesticAuto,
                    value1.ImportedAuto * value2.ImportedAuto, value1.LightTruck * value2.LightTruck);
            }
            public static FuelValueByRc operator /(FuelValueByRc value1, FuelValueByRc value2)
            {
                return new FuelValueByRc(value1.Unregulated / value2.Unregulated, value1.DomesticAuto / value2.DomesticAuto,
                    value1.ImportedAuto / value2.ImportedAuto, value1.LightTruck / value2.LightTruck);
            }
            public static FuelValueByRc operator /(FuelValueByRc value1, double value2)
            {
                return new FuelValueByRc(value1.Unregulated / value2, value1.DomesticAuto / value2,
                    value1.ImportedAuto / value2, value1.LightTruck / value2);
            }
            public static FuelValueByRc operator /(FuelValueByRc value1, int value2)
            {
                return new FuelValueByRc(value1.Unregulated / value2, value1.DomesticAuto / value2,
                    value1.ImportedAuto / value2, value1.LightTruck / value2);
            }
            public static FuelValueByRc operator +(FuelValueByRc value)
            {
                return FuelValueByRc.Zero + value;
            }
            public static FuelValueByRc operator -(FuelValueByRc value)
            {
                return FuelValueByRc.Zero - value;
            }
            public static FuelValueByRc operator ++(FuelValueByRc value)
            {
                return value + FuelValueByRc.One;
            }
            public static FuelValueByRc operator --(FuelValueByRc value)
            {
                return value - FuelValueByRc.One;
            }
            public static bool operator ==(FuelValueByRc value1, FuelValueByRc value2)
            {
                return value1.Equals(value2);
            }
            public static bool operator !=(FuelValueByRc value1, FuelValueByRc value2)
            {
                return !value1.Equals(value2);
            }
            #endregion
            public override int GetHashCode()
            {
                return this.Unregulated.GetHashCode() ^ this.DomesticAuto.GetHashCode() ^ this.ImportedAuto.GetHashCode() ^
                    this.LightTruck.GetHashCode();
            }
            public override bool Equals(object obj)
            {
                if (obj is FuelValueByRc) { return this.Equals((FuelValueByRc)obj); }
                else                      { return base.Equals(obj); }
            }
            public bool Equals(FuelValueByRc value)
            {
                return (this.Unregulated == value.Unregulated && this.DomesticAuto == value.DomesticAuto &&
                    this.ImportedAuto == value.ImportedAuto && this.LightTruck == value.LightTruck);
            }
            public void Add(RegulatoryClass regClass, FuelValue value)
            {
                if      (regClass == RegulatoryClass.DomesticAuto) { this.DomesticAuto += value; }
                else if (regClass == RegulatoryClass.ImportedAuto) { this.ImportedAuto += value; }
                else if (regClass == RegulatoryClass.LightTruck  ) { this.LightTruck   += value; }
                else if (regClass == RegulatoryClass.Unregulated ) { this.Unregulated  += value; }
                else
                {
                    throw new ArgumentException("The value specified does not represent a valid regulatory class.");
                }
            }
            public void Clear()
            {
                this.Unregulated .Clear();
                this.DomesticAuto.Clear();
                this.ImportedAuto.Clear();
                this.LightTruck  .Clear();
            }
            public RCDouble ToRCDouble()
            {
                RCDouble value = new RCDouble();
                value.Unregulated  = this.Unregulated .Total;
                value.DomesticAuto = this.DomesticAuto.Total;
                value.ImportedAuto = this.ImportedAuto.Total;
                value.LightTruck   = this.LightTruck  .Total;
                return value;
            }
            #endregion
            #region 
            public FuelValue this[RegulatoryClass regClass]
            {
                get
                {
                    if      (regClass ==  RegulatoryClass.DomesticAuto) { return this.DomesticAuto; }
                    else if (regClass ==  RegulatoryClass.ImportedAuto) { return this.ImportedAuto; }
                    else if (regClass ==  RegulatoryClass.LightTruck  ) { return this.LightTruck  ; }
                    else if (regClass ==  RegulatoryClass.Unregulated ) { return this.Unregulated ; }
                    else if (regClass == (RegulatoryClass)(-1)        ) { return this.Total       ; }
                    else if (regClass == (RegulatoryClass)(-2)        ) { return this.TotalAll    ; }
                    else
                    {
                        throw new ArgumentException("The value specified does not represent a valid regulatory class.");
                    }
                }
            }
            public double this[RegulatoryClass regClass, FuelType fuelType]
            {
                get { return this[regClass][fuelType]; }
                set
                {
                    if      (regClass == RegulatoryClass.DomesticAuto) { this.DomesticAuto[fuelType] = value; }
                    else if (regClass == RegulatoryClass.ImportedAuto) { this.ImportedAuto[fuelType] = value; }
                    else if (regClass == RegulatoryClass.LightTruck  ) { this.LightTruck  [fuelType] = value; }
                    else if (regClass == RegulatoryClass.Unregulated ) { this.Unregulated [fuelType] = value; }
                    else
                    {
                        throw new ArgumentException("The value specified does not represent a valid regulatory class.");
                    }
                }
            }
            public FuelValue PassengerAuto { get { return FuelValue.Sum(this.DomesticAuto, this.ImportedAuto); } }
            public FuelValue Total { get { return FuelValue.Sum(this.DomesticAuto, this.ImportedAuto, this.LightTruck); } }
            public FuelValue TotalAll { get { return FuelValue.Sum(this.DomesticAuto, this.ImportedAuto, this.LightTruck, this.Unregulated); } }
            #endregion
            #region 
            static readonly FuelValueByRc Zero = new FuelValueByRc(FuelValue.Zero, FuelValue.Zero, FuelValue.Zero, FuelValue.Zero);
            static readonly FuelValueByRc One = new FuelValueByRc(FuelValue.One, FuelValue.One, FuelValue.One, FuelValue.One);
            public FuelValue Unregulated;
            public FuelValue DomesticAuto;
            public FuelValue ImportedAuto;
            public FuelValue LightTruck;
            #endregion
        }
        [Serializable]
        public struct SafetyValueByRc
        {
            #region 
            SafetyValueByRc(SFTDouble unregulated, SFTDouble domesticAuto, SFTDouble importedAuto, SFTDouble lightTruck)
            {
                this.Unregulated  = unregulated;
                this.DomesticAuto = domesticAuto;
                this.ImportedAuto = importedAuto;
                this.LightTruck   = lightTruck;
            }
            #endregion
            #region 
            #region 
            public static SafetyValueByRc operator +(SafetyValueByRc value1, SafetyValueByRc value2)
            {
                return new SafetyValueByRc(value1.Unregulated + value2.Unregulated, value1.DomesticAuto + value2.DomesticAuto,
                    value1.ImportedAuto + value2.ImportedAuto, value1.LightTruck + value2.LightTruck);
            }
            public static SafetyValueByRc operator -(SafetyValueByRc value1, SafetyValueByRc value2)
            {
                return new SafetyValueByRc(value1.Unregulated - value2.Unregulated, value1.DomesticAuto - value2.DomesticAuto,
                    value1.ImportedAuto - value2.ImportedAuto, value1.LightTruck - value2.LightTruck);
            }
            public static SafetyValueByRc operator *(SafetyValueByRc value1, SafetyValueByRc value2)
            {
                return new SafetyValueByRc(value1.Unregulated * value2.Unregulated, value1.DomesticAuto * value2.DomesticAuto,
                    value1.ImportedAuto * value2.ImportedAuto, value1.LightTruck * value2.LightTruck);
            }
            public static SafetyValueByRc operator /(SafetyValueByRc value1, SafetyValueByRc value2)
            {
                return new SafetyValueByRc(value1.Unregulated / value2.Unregulated, value1.DomesticAuto / value2.DomesticAuto,
                    value1.ImportedAuto / value2.ImportedAuto, value1.LightTruck / value2.LightTruck);
            }
            public static SafetyValueByRc operator +(SafetyValueByRc value)
            {
                return SafetyValueByRc.Zero + value;
            }
            public static SafetyValueByRc operator -(SafetyValueByRc value)
            {
                return SafetyValueByRc.Zero - value;
            }
            public static SafetyValueByRc operator ++(SafetyValueByRc value)
            {
                return value + SafetyValueByRc.One;
            }
            public static SafetyValueByRc operator --(SafetyValueByRc value)
            {
                return value - SafetyValueByRc.One;
            }
            public static bool operator ==(SafetyValueByRc value1, SafetyValueByRc value2)
            {
                return value1.Equals(value2);
            }
            public static bool operator !=(SafetyValueByRc value1, SafetyValueByRc value2)
            {
                return !value1.Equals(value2);
            }
            #endregion
            public override int GetHashCode()
            {
                return this.Unregulated.GetHashCode() ^ this.DomesticAuto.GetHashCode() ^ this.ImportedAuto.GetHashCode() ^
                    this.LightTruck.GetHashCode();
            }
            public override bool Equals(object obj)
            {
                if (obj is SafetyValueByRc) { return this.Equals((SafetyValueByRc)obj); }
                else                        { return base.Equals(obj); }
            }
            public bool Equals(SafetyValueByRc value)
            {
                return (this.Unregulated == value.Unregulated && this.DomesticAuto == value.DomesticAuto &&
                    this.ImportedAuto == value.ImportedAuto && this.LightTruck == value.LightTruck);
            }
            public void Clear()
            {
                this.Unregulated .Clear();
                this.DomesticAuto.Clear();
                this.ImportedAuto.Clear();
                this.LightTruck  .Clear();
            }
            public SFTDouble ArithmeticMean(SafetyValueByRc sales)
            {
                SFTDouble mean = (this * sales).TotalAll / sales.TotalAll;
                mean.ZeroNaNs();
                return mean;
            }
            #endregion
            #region 
            public SFTDouble this[RegulatoryClass regClass]
            {
                get
                {
                    if      (regClass ==  RegulatoryClass.DomesticAuto) { return this.DomesticAuto; }
                    else if (regClass ==  RegulatoryClass.ImportedAuto) { return this.ImportedAuto; }
                    else if (regClass ==  RegulatoryClass.LightTruck  ) { return this.LightTruck  ; }
                    else if (regClass ==  RegulatoryClass.Unregulated ) { return this.Unregulated ; }
                    else if (regClass == (RegulatoryClass)(-1)        ) { return this.Total       ; }
                    else if (regClass == (RegulatoryClass)(-2)        ) { return this.TotalAll    ; }
                    else
                    {
                        throw new ArgumentException("The value specified does not represent a valid regulatory class.");
                    }
                }
            }
            public double this[RegulatoryClass regClass, SafetyClass safetyClass]
            {
                get { return this[regClass][safetyClass]; }
                set
                {
                    if      (regClass == RegulatoryClass.DomesticAuto) { this.DomesticAuto[safetyClass] = value; }
                    else if (regClass == RegulatoryClass.ImportedAuto) { this.ImportedAuto[safetyClass] = value; }
                    else if (regClass == RegulatoryClass.LightTruck  ) { this.LightTruck  [safetyClass] = value; }
                    else if (regClass == RegulatoryClass.Unregulated ) { this.Unregulated [safetyClass] = value; }
                    else
                    {
                        throw new ArgumentException("The value specified does not represent a valid regulatory class.");
                    }
                }
            }
            public SFTDouble Total { get { return SFTDouble.Sum(this.DomesticAuto, this.ImportedAuto, this.LightTruck); } }
            public SFTDouble TotalAll { get { return SFTDouble.Sum(this.DomesticAuto, this.ImportedAuto, this.LightTruck, this.Unregulated); } }
            #endregion
            #region 
            static readonly SafetyValueByRc Zero = new SafetyValueByRc(SFTDouble.Zero, SFTDouble.Zero, SFTDouble.Zero, SFTDouble.Zero);
            static readonly SafetyValueByRc One = new SafetyValueByRc(SFTDouble.One, SFTDouble.One, SFTDouble.One, SFTDouble.One);
            public SFTDouble Unregulated;
            public SFTDouble DomesticAuto;
            public SFTDouble ImportedAuto;
            public SFTDouble LightTruck;
            #endregion
        }
        [Serializable]
        public class OwnerAndSocietalCosts
        {
            #region 
            internal OwnerAndSocietalCosts()
            {
                this.DiscountRate             = 0;
                this.ConsumerPreTaxFuelCost   = new RCDouble();
                this.ConsumerFuelTaxCost      = new RCDouble();
                this.ConsumerDriveSurplus     = new RCDouble();
                this.ConsumerRefuelSurplus    = new RCDouble();
                this.ConsumerRetailFuelCosts  = new RCDouble();
                this.ConsumerTotalCosts       = new RCDouble();
                this.PreTaxFuelCost           = new RCDouble();
                this.FuelTaxCost              = new RCDouble();
                this.DriveSurplus             = new RCDouble();
                this.RefuelSurplus            = new RCDouble();
                this.EconomicCosts            = new RCDouble();
                this.CongestionCosts          = new RCDouble();
                this.AccidentCosts            = new RCDouble();
                this.NoiseCosts               = new RCDouble();
                this.FatalityCosts            = new RCDouble();
                this.EmissionsCostsCO2        = new RCDouble();
                this.EmissionsCostsCO         = new RCDouble();
                this.EmissionsCostsVOC        = new RCDouble();
                this.EmissionsCostsNOX        = new RCDouble();
                this.EmissionsCostsPM         = new RCDouble();
                this.EmissionsCostsSOX        = new RCDouble();
                this.EmissionsCostsCH4        = new RCDouble();
                this.EmissionsCostsN2O        = new RCDouble();
                this.RetailFuelCosts          = new RCDouble();
                this.TotalConsumerCosts       = new RCDouble();
                this.TotalSocialCosts         = new RCDouble();
                this.AnnualPreTaxFuelCost     = new RCDouble[Interaction.CY];
                this.AnnualFuelTaxCost        = new RCDouble[Interaction.CY];
                this.AnnualDriveSurplus       = new RCDouble[Interaction.CY];
                this.AnnualRefuelSurplus      = new RCDouble[Interaction.CY];
                this.AnnualEconomicCosts      = new RCDouble[Interaction.CY];
                this.AnnualCongestionCosts    = new RCDouble[Interaction.CY];
                this.AnnualAccidentCosts      = new RCDouble[Interaction.CY];
                this.AnnualNoiseCosts         = new RCDouble[Interaction.CY];
                this.AnnualFatalityCosts      = new RCDouble[Interaction.CY];
                this.AnnualEmissionsCostsCO2  = new RCDouble[Interaction.CY];
                this.AnnualEmissionsCostsCO   = new RCDouble[Interaction.CY];
                this.AnnualEmissionsCostsVOC  = new RCDouble[Interaction.CY];
                this.AnnualEmissionsCostsNOX  = new RCDouble[Interaction.CY];
                this.AnnualEmissionsCostsPM   = new RCDouble[Interaction.CY];
                this.AnnualEmissionsCostsSOX  = new RCDouble[Interaction.CY];
                this.AnnualEmissionsCostsCH4  = new RCDouble[Interaction.CY];
                this.AnnualEmissionsCostsN2O  = new RCDouble[Interaction.CY];
                this.AnnualRetailFuelCosts    = new RCDouble[Interaction.CY];
                this.AnnualTotalConsumerCosts = new RCDouble[Interaction.CY];
                this.AnnualTotalSocialCosts   = new RCDouble[Interaction.CY];
            }
            internal OwnerAndSocietalCosts(OwnerAndSocietalCosts prototype)
            {
                this.DiscountRate             = prototype.DiscountRate;
                this.ConsumerPreTaxFuelCost   = prototype.ConsumerPreTaxFuelCost;
                this.ConsumerFuelTaxCost      = prototype.ConsumerFuelTaxCost;
                this.ConsumerDriveSurplus     = prototype.ConsumerDriveSurplus;
                this.ConsumerRefuelSurplus    = prototype.ConsumerRefuelSurplus;
                this.ConsumerRetailFuelCosts  = prototype.ConsumerRetailFuelCosts;
                this.ConsumerTotalCosts       = prototype.ConsumerTotalCosts;
                this.PreTaxFuelCost           = prototype.PreTaxFuelCost;
                this.FuelTaxCost              = prototype.FuelTaxCost;
                this.DriveSurplus             = prototype.DriveSurplus;
                this.RefuelSurplus            = prototype.RefuelSurplus;
                this.EconomicCosts            = prototype.EconomicCosts;
                this.CongestionCosts          = prototype.CongestionCosts;
                this.AccidentCosts            = prototype.AccidentCosts;
                this.NoiseCosts               = prototype.NoiseCosts;
                this.FatalityCosts            = prototype.FatalityCosts;
                this.EmissionsCostsCO2        = prototype.EmissionsCostsCO2;
                this.EmissionsCostsCO         = prototype.EmissionsCostsCO;
                this.EmissionsCostsVOC        = prototype.EmissionsCostsVOC;
                this.EmissionsCostsNOX        = prototype.EmissionsCostsNOX;
                this.EmissionsCostsPM         = prototype.EmissionsCostsPM;
                this.EmissionsCostsSOX        = prototype.EmissionsCostsSOX;
                this.EmissionsCostsCH4        = prototype.EmissionsCostsN2O;
                this.EmissionsCostsN2O        = prototype.EmissionsCostsCH4;
                this.RetailFuelCosts          = prototype.RetailFuelCosts;
                this.TotalConsumerCosts       = prototype.TotalConsumerCosts;
                this.TotalSocialCosts         = prototype.TotalSocialCosts;
                this.AnnualPreTaxFuelCost     = (RCDouble[])prototype.AnnualPreTaxFuelCost    .Clone();
                this.AnnualFuelTaxCost        = (RCDouble[])prototype.AnnualFuelTaxCost       .Clone();
                this.AnnualDriveSurplus       = (RCDouble[])prototype.AnnualDriveSurplus      .Clone();
                this.AnnualRefuelSurplus      = (RCDouble[])prototype.AnnualRefuelSurplus     .Clone();
                this.AnnualEconomicCosts      = (RCDouble[])prototype.AnnualEconomicCosts     .Clone();
                this.AnnualCongestionCosts    = (RCDouble[])prototype.AnnualCongestionCosts   .Clone();
                this.AnnualAccidentCosts      = (RCDouble[])prototype.AnnualAccidentCosts     .Clone();
                this.AnnualNoiseCosts         = (RCDouble[])prototype.AnnualNoiseCosts        .Clone();
                this.AnnualFatalityCosts      = (RCDouble[])prototype.AnnualFatalityCosts     .Clone();
                this.AnnualEmissionsCostsCO2  = (RCDouble[])prototype.AnnualEmissionsCostsCO2 .Clone();
                this.AnnualEmissionsCostsCO   = (RCDouble[])prototype.AnnualEmissionsCostsCO  .Clone();
                this.AnnualEmissionsCostsVOC  = (RCDouble[])prototype.AnnualEmissionsCostsVOC .Clone();
                this.AnnualEmissionsCostsNOX  = (RCDouble[])prototype.AnnualEmissionsCostsNOX .Clone();
                this.AnnualEmissionsCostsPM   = (RCDouble[])prototype.AnnualEmissionsCostsPM  .Clone();
                this.AnnualEmissionsCostsSOX  = (RCDouble[])prototype.AnnualEmissionsCostsSOX .Clone();
                this.AnnualEmissionsCostsCH4  = (RCDouble[])prototype.AnnualEmissionsCostsCH4 .Clone();
                this.AnnualEmissionsCostsN2O  = (RCDouble[])prototype.AnnualEmissionsCostsN2O .Clone();
                this.AnnualRetailFuelCosts    = (RCDouble[])prototype.AnnualRetailFuelCosts   .Clone();
                this.AnnualTotalConsumerCosts = (RCDouble[])prototype.AnnualTotalConsumerCosts.Clone();
                this.AnnualTotalSocialCosts   = (RCDouble[])prototype.AnnualTotalSocialCosts  .Clone();
            }
            #endregion
            #region 
            internal void Clear()
            {
                this.DiscountRate = 0;
                this.ConsumerPreTaxFuelCost .Clear();
                this.ConsumerFuelTaxCost    .Clear();
                this.ConsumerDriveSurplus   .Clear();
                this.ConsumerRefuelSurplus  .Clear();
                this.ConsumerRetailFuelCosts.Clear();
                this.ConsumerTotalCosts     .Clear();
                this.PreTaxFuelCost         .Clear();
                this.FuelTaxCost            .Clear();
                this.DriveSurplus           .Clear();
                this.RefuelSurplus          .Clear();
                this.EconomicCosts          .Clear();
                this.CongestionCosts        .Clear();
                this.AccidentCosts          .Clear();
                this.NoiseCosts             .Clear();
                this.FatalityCosts          .Clear();
                this.EmissionsCostsCO2      .Clear();
                this.EmissionsCostsCO       .Clear();
                this.EmissionsCostsVOC      .Clear();
                this.EmissionsCostsNOX      .Clear();
                this.EmissionsCostsPM       .Clear();
                this.EmissionsCostsSOX      .Clear();
                this.EmissionsCostsCH4      .Clear();
                this.EmissionsCostsN2O      .Clear();
                this.RetailFuelCosts        .Clear();
                this.TotalConsumerCosts     .Clear();
                this.TotalSocialCosts       .Clear();
                Array.Clear(this.AnnualPreTaxFuelCost    , 0, this.AnnualPreTaxFuelCost    .Length);
                Array.Clear(this.AnnualFuelTaxCost       , 0, this.AnnualFuelTaxCost       .Length);
                Array.Clear(this.AnnualDriveSurplus      , 0, this.AnnualDriveSurplus      .Length);
                Array.Clear(this.AnnualRefuelSurplus     , 0, this.AnnualRefuelSurplus     .Length);
                Array.Clear(this.AnnualEconomicCosts     , 0, this.AnnualEconomicCosts     .Length);
                Array.Clear(this.AnnualCongestionCosts   , 0, this.AnnualCongestionCosts   .Length);
                Array.Clear(this.AnnualAccidentCosts     , 0, this.AnnualAccidentCosts     .Length);
                Array.Clear(this.AnnualNoiseCosts        , 0, this.AnnualNoiseCosts        .Length);
                Array.Clear(this.AnnualFatalityCosts     , 0, this.AnnualFatalityCosts     .Length);
                Array.Clear(this.AnnualEmissionsCostsCO2 , 0, this.AnnualEmissionsCostsCO2 .Length);
                Array.Clear(this.AnnualEmissionsCostsCO  , 0, this.AnnualEmissionsCostsCO  .Length);
                Array.Clear(this.AnnualEmissionsCostsVOC , 0, this.AnnualEmissionsCostsVOC .Length);
                Array.Clear(this.AnnualEmissionsCostsNOX , 0, this.AnnualEmissionsCostsNOX .Length);
                Array.Clear(this.AnnualEmissionsCostsPM  , 0, this.AnnualEmissionsCostsPM  .Length);
                Array.Clear(this.AnnualEmissionsCostsSOX , 0, this.AnnualEmissionsCostsSOX .Length);
                Array.Clear(this.AnnualEmissionsCostsCH4 , 0, this.AnnualEmissionsCostsCH4 .Length);
                Array.Clear(this.AnnualEmissionsCostsN2O , 0, this.AnnualEmissionsCostsN2O .Length);
                Array.Clear(this.AnnualRetailFuelCosts   , 0, this.AnnualRetailFuelCosts   .Length);
                Array.Clear(this.AnnualTotalConsumerCosts, 0, this.AnnualTotalConsumerCosts.Length);
                Array.Clear(this.AnnualTotalSocialCosts  , 0, this.AnnualTotalSocialCosts  .Length);
            }
            internal void CalcSubtotals()
            {
                this.ConsumerRetailFuelCosts = this.ConsumerPreTaxFuelCost  + this.ConsumerFuelTaxCost;
                this.ConsumerTotalCosts      = this.ConsumerRetailFuelCosts + this.ConsumerDriveSurplus + this.ConsumerRefuelSurplus;
                this.RetailFuelCosts    = this.PreTaxFuelCost  + this.FuelTaxCost;
                this.TotalConsumerCosts = this.RetailFuelCosts + this.DriveSurplus + this.RefuelSurplus;
                this.TotalSocialCosts   =
                    this.PreTaxFuelCost    + this.DriveSurplus     + this.RefuelSurplus     +
                    this.EconomicCosts     + this.CongestionCosts  + this.AccidentCosts     + this.NoiseCosts + this.FatalityCosts +
                    this.EmissionsCostsCO2 + this.EmissionsCostsCO + this.EmissionsCostsVOC +
                    this.EmissionsCostsNOX + this.EmissionsCostsPM + this.EmissionsCostsSOX +
                    this.EmissionsCostsCH4 + this.EmissionsCostsN2O;
                for (int i = 0; i < this.AnnualTotalSocialCosts.Length; i++)
                {
                    this.AnnualRetailFuelCosts   [i] = this.AnnualPreTaxFuelCost [i] + this.AnnualFuelTaxCost [i];
                    this.AnnualTotalConsumerCosts[i] = this.AnnualRetailFuelCosts[i] + this.AnnualDriveSurplus[i] + this.AnnualRefuelSurplus[i];
                    this.AnnualTotalSocialCosts  [i] =
                        this.AnnualPreTaxFuelCost   [i] + this.AnnualDriveSurplus     [i] + this.AnnualRefuelSurplus    [i] +
                        this.AnnualEconomicCosts    [i] + this.AnnualCongestionCosts  [i] + this.AnnualAccidentCosts    [i] + this.AnnualNoiseCosts[i] + this.AnnualFatalityCosts[i] +
                        this.AnnualEmissionsCostsCO2[i] + this.AnnualEmissionsCostsCO [i] + this.AnnualEmissionsCostsVOC[i] +
                        this.AnnualEmissionsCostsNOX[i] + this.AnnualEmissionsCostsPM [i] + this.AnnualEmissionsCostsSOX[i] +
                        this.AnnualEmissionsCostsCH4[i] + this.AnnualEmissionsCostsN2O[i];
                }
            }
            internal void Add(OwnerAndSocietalCosts value)
            {
                this.DiscountRate = value.DiscountRate;
                this.ConsumerPreTaxFuelCost  += value.ConsumerPreTaxFuelCost;
                this.ConsumerFuelTaxCost     += value.ConsumerFuelTaxCost;
                this.ConsumerDriveSurplus    += value.ConsumerDriveSurplus;
                this.ConsumerRefuelSurplus   += value.ConsumerRefuelSurplus;
                this.ConsumerRetailFuelCosts += value.ConsumerRetailFuelCosts;
                this.ConsumerTotalCosts      += value.ConsumerTotalCosts;
                this.PreTaxFuelCost          += value.PreTaxFuelCost;
                this.FuelTaxCost             += value.FuelTaxCost;
                this.DriveSurplus            += value.DriveSurplus;
                this.RefuelSurplus           += value.RefuelSurplus;
                this.EconomicCosts           += value.EconomicCosts;
                this.CongestionCosts         += value.CongestionCosts;
                this.AccidentCosts           += value.AccidentCosts;
                this.NoiseCosts              += value.NoiseCosts;
                this.FatalityCosts           += value.FatalityCosts;
                this.EmissionsCostsCO2       += value.EmissionsCostsCO2;
                this.EmissionsCostsCO        += value.EmissionsCostsCO;
                this.EmissionsCostsVOC       += value.EmissionsCostsVOC;
                this.EmissionsCostsNOX       += value.EmissionsCostsNOX;
                this.EmissionsCostsPM        += value.EmissionsCostsPM;
                this.EmissionsCostsSOX       += value.EmissionsCostsSOX;
                this.EmissionsCostsCH4       += value.EmissionsCostsCH4;
                this.EmissionsCostsN2O       += value.EmissionsCostsN2O;
                this.RetailFuelCosts         += value.RetailFuelCosts;
                this.TotalConsumerCosts      += value.TotalConsumerCosts;
                this.TotalSocialCosts        += value.TotalSocialCosts;
                for (int i = 0; i < this.AnnualTotalSocialCosts.Length; i++)
                {
                    this.AnnualPreTaxFuelCost    [i] += value.AnnualPreTaxFuelCost    [i];
                    this.AnnualFuelTaxCost       [i] += value.AnnualFuelTaxCost       [i];
                    this.AnnualDriveSurplus      [i] += value.AnnualDriveSurplus      [i];
                    this.AnnualRefuelSurplus     [i] += value.AnnualRefuelSurplus     [i];
                    this.AnnualEconomicCosts     [i] += value.AnnualEconomicCosts     [i];
                    this.AnnualCongestionCosts   [i] += value.AnnualCongestionCosts   [i];
                    this.AnnualAccidentCosts     [i] += value.AnnualAccidentCosts     [i];
                    this.AnnualNoiseCosts        [i] += value.AnnualNoiseCosts        [i];
                    this.AnnualFatalityCosts     [i] += value.AnnualFatalityCosts     [i];
                    this.AnnualEmissionsCostsCO2 [i] += value.AnnualEmissionsCostsCO2 [i];
                    this.AnnualEmissionsCostsCO  [i] += value.AnnualEmissionsCostsCO  [i];
                    this.AnnualEmissionsCostsVOC [i] += value.AnnualEmissionsCostsVOC [i];
                    this.AnnualEmissionsCostsNOX [i] += value.AnnualEmissionsCostsNOX [i];
                    this.AnnualEmissionsCostsPM  [i] += value.AnnualEmissionsCostsPM  [i];
                    this.AnnualEmissionsCostsSOX [i] += value.AnnualEmissionsCostsSOX [i];
                    this.AnnualEmissionsCostsCH4 [i] += value.AnnualEmissionsCostsCH4 [i];
                    this.AnnualEmissionsCostsN2O [i] += value.AnnualEmissionsCostsN2O [i];
                    this.AnnualRetailFuelCosts   [i] += value.AnnualRetailFuelCosts   [i];
                    this.AnnualTotalConsumerCosts[i] += value.AnnualTotalConsumerCosts[i];
                    this.AnnualTotalSocialCosts  [i] += value.AnnualTotalSocialCosts  [i];
                }
            }
            #endregion
            #region 
            public double DiscountRate;
            public RCDouble ConsumerPreTaxFuelCost;
            public RCDouble ConsumerFuelTaxCost;
            public RCDouble ConsumerDriveSurplus;
            public RCDouble ConsumerRefuelSurplus;
            public RCDouble ConsumerRetailFuelCosts;
            public RCDouble ConsumerTotalCosts;
            public RCDouble PreTaxFuelCost;
            public RCDouble FuelTaxCost;
            public RCDouble DriveSurplus;
            public RCDouble RefuelSurplus;
            public RCDouble EconomicCosts;
            public RCDouble CongestionCosts;
            public RCDouble AccidentCosts;
            public RCDouble NoiseCosts;
            public RCDouble FatalityCosts;
            public RCDouble EmissionsCostsCO2;
            public RCDouble EmissionsCostsCO;
            public RCDouble EmissionsCostsVOC;
            public RCDouble EmissionsCostsNOX;
            public RCDouble EmissionsCostsPM;
            public RCDouble EmissionsCostsSOX;
            public RCDouble EmissionsCostsCH4;
            public RCDouble EmissionsCostsN2O;
            public RCDouble RetailFuelCosts;
            public RCDouble TotalConsumerCosts;
            public RCDouble TotalSocialCosts;
            public RCDouble[] AnnualPreTaxFuelCost;
            public RCDouble[] AnnualFuelTaxCost;
            public RCDouble[] AnnualDriveSurplus;
            public RCDouble[] AnnualRefuelSurplus;
            public RCDouble[] AnnualEconomicCosts;
            public RCDouble[] AnnualCongestionCosts;
            public RCDouble[] AnnualAccidentCosts;
            public RCDouble[] AnnualNoiseCosts;
            public RCDouble[] AnnualFatalityCosts;
            public RCDouble[] AnnualEmissionsCostsCO2;
            public RCDouble[] AnnualEmissionsCostsCO;
            public RCDouble[] AnnualEmissionsCostsVOC;
            public RCDouble[] AnnualEmissionsCostsNOX;
            public RCDouble[] AnnualEmissionsCostsPM;
            public RCDouble[] AnnualEmissionsCostsSOX;
            public RCDouble[] AnnualEmissionsCostsCH4;
            public RCDouble[] AnnualEmissionsCostsN2O;
            public RCDouble[] AnnualRetailFuelCosts;
            public RCDouble[] AnnualTotalConsumerCosts;
            public RCDouble[] AnnualTotalSocialCosts;
            #endregion
        }
        #endregion
        #region 
        internal EffectsData()
        {
            this.Sales                             = new FuelValueByRc();
            this.VMT                               = new FuelValueByRc();
            this.RatedFuelEconomy                  = new FuelValueByRc();
            this.OnRoadFuelEconomy                 = new FuelValueByRc();
            this.AverageRatedFuelEconomy           = new FuelValue    ();
            this.AverageOnRoadFuelEconomy          = new FuelValue    ();
            this.BTU                               = new FuelValueByRc();
            this.Gallons                           = new FuelValueByRc();
            this.KWh                               = new FuelValueByRc();
            this.Scf                               = new FuelValueByRc();
            this.AnnualVMT                         = new RCDouble[Interaction.CY];
            this.AnnualBTU                         = new RCDouble[Interaction.CY];
            this.AnnualGallons                     = new RCDouble[Interaction.CY];
            this.AnnualKWh                         = new RCDouble[Interaction.CY];
            this.AnnualScf                         = new RCDouble[Interaction.CY];
            this.EmissionsCO2                      = new FuelValueByRc();
            this.EmissionsCO                       = new FuelValueByRc();
            this.EmissionsVOC                      = new FuelValueByRc();
            this.EmissionsNOX                      = new FuelValueByRc();
            this.EmissionsPM                       = new FuelValueByRc();
            this.EmissionsSOX                      = new FuelValueByRc();
            this.EmissionsCH4                      = new FuelValueByRc();
            this.EmissionsN2O                      = new FuelValueByRc();
            this.AnnualEmissionsCO2                = new RCDouble[Interaction.CY];
            this.AnnualEmissionsCO                 = new RCDouble[Interaction.CY];
            this.AnnualEmissionsVOC                = new RCDouble[Interaction.CY];
            this.AnnualEmissionsNOX                = new RCDouble[Interaction.CY];
            this.AnnualEmissionsPM                 = new RCDouble[Interaction.CY];
            this.AnnualEmissionsSOX                = new RCDouble[Interaction.CY];
            this.AnnualEmissionsCH4                = new RCDouble[Interaction.CY];
            this.AnnualEmissionsN2O                = new RCDouble[Interaction.CY];
            this.SftSales                          = new SafetyValueByRc();
            this.SftVMT                            = new SafetyValueByRc();
            this.SftCurbWeight                     = new SafetyValueByRc();
            this.SftFatalities                     = new SafetyValueByRc();
            this.AnnualSftFatalities               = new RCDouble[Interaction.CY];
            this.UndiscountedOwnerAndSocietalCosts = new OwnerAndSocietalCosts();
            this.DiscountedOwnerAndSocietalCosts   = new OwnerAndSocietalCosts[0];
        }
        EffectsData(EffectsData prototype)
        {
            this.Sales                             = prototype.Sales;
            this.VMT                               = prototype.VMT;
            this.RatedFuelEconomy                  = prototype.RatedFuelEconomy;
            this.OnRoadFuelEconomy                 = prototype.OnRoadFuelEconomy;
            this.AverageRatedFuelEconomy           = prototype.AverageRatedFuelEconomy;
            this.AverageOnRoadFuelEconomy          = prototype.AverageOnRoadFuelEconomy;
            this.BTU                               = prototype.BTU;
            this.Gallons                           = prototype.Gallons;
            this.KWh                               = prototype.KWh;
            this.Scf                               = prototype.Scf;
            this.AnnualVMT                         = (RCDouble[])prototype.AnnualVMT    .Clone();
            this.AnnualBTU                         = (RCDouble[])prototype.AnnualBTU    .Clone();
            this.AnnualGallons                     = (RCDouble[])prototype.AnnualGallons.Clone();
            this.AnnualKWh                         = (RCDouble[])prototype.AnnualKWh    .Clone();
            this.AnnualScf                         = (RCDouble[])prototype.AnnualScf    .Clone();
            this.EmissionsCO2                      = prototype.EmissionsCO2;
            this.EmissionsCO                       = prototype.EmissionsCO;
            this.EmissionsVOC                      = prototype.EmissionsVOC;
            this.EmissionsNOX                      = prototype.EmissionsNOX;
            this.EmissionsPM                       = prototype.EmissionsPM;
            this.EmissionsSOX                      = prototype.EmissionsSOX;
            this.EmissionsCH4                      = prototype.EmissionsCH4;
            this.EmissionsN2O                      = prototype.EmissionsN2O;
            this.AnnualEmissionsCO2                = (RCDouble[])prototype.AnnualEmissionsCO2.Clone();
            this.AnnualEmissionsCO                 = (RCDouble[])prototype.AnnualEmissionsCO .Clone();
            this.AnnualEmissionsVOC                = (RCDouble[])prototype.AnnualEmissionsVOC.Clone();
            this.AnnualEmissionsNOX                = (RCDouble[])prototype.AnnualEmissionsNOX.Clone();
            this.AnnualEmissionsPM                 = (RCDouble[])prototype.AnnualEmissionsPM .Clone();
            this.AnnualEmissionsSOX                = (RCDouble[])prototype.AnnualEmissionsSOX.Clone();
            this.AnnualEmissionsCH4                = (RCDouble[])prototype.AnnualEmissionsCH4.Clone();
            this.AnnualEmissionsN2O                = (RCDouble[])prototype.AnnualEmissionsN2O.Clone();
            this.SftSales                          = prototype.SftSales;
            this.SftVMT                            = prototype.SftVMT;
            this.SftCurbWeight                     = prototype.SftCurbWeight;
            this.SftFatalities                     = prototype.SftFatalities;
            this.AnnualSftFatalities               = (RCDouble[])prototype.AnnualSftFatalities.Clone();
            this.UndiscountedOwnerAndSocietalCosts = new OwnerAndSocietalCosts(prototype.UndiscountedOwnerAndSocietalCosts);
            this.DiscountedOwnerAndSocietalCosts   = new OwnerAndSocietalCosts[prototype.DiscountedOwnerAndSocietalCosts.Length];
            for (int i = 0; i < this.DiscountedOwnerAndSocietalCosts.Length; i++)
            {
                this.DiscountedOwnerAndSocietalCosts[i] = new OwnerAndSocietalCosts(prototype.DiscountedOwnerAndSocietalCosts[i]);
            }
        }
        #endregion
        #region 
        internal EffectsData Clone()
        {
            return new EffectsData(this);
        }
        public void Reset(int drCount)
        {
            this.Sales                   .Clear();
            this.VMT                     .Clear();
            this.RatedFuelEconomy        .Clear();
            this.OnRoadFuelEconomy       .Clear();
            this.AverageRatedFuelEconomy .Clear();
            this.AverageOnRoadFuelEconomy.Clear();
            this.BTU                     .Clear();
            this.Gallons                 .Clear();
            this.KWh                     .Clear();
            this.Scf                     .Clear();
            Array.Clear(this.AnnualVMT    , 0, this.AnnualVMT    .Length);
            Array.Clear(this.AnnualBTU    , 0, this.AnnualBTU    .Length);
            Array.Clear(this.AnnualGallons, 0, this.AnnualGallons.Length);
            Array.Clear(this.AnnualKWh    , 0, this.AnnualKWh    .Length);
            Array.Clear(this.AnnualScf    , 0, this.AnnualScf    .Length);
            this.EmissionsCO2            .Clear();
            this.EmissionsCO             .Clear();
            this.EmissionsVOC            .Clear();
            this.EmissionsNOX            .Clear();
            this.EmissionsPM             .Clear();
            this.EmissionsSOX            .Clear();
            this.EmissionsCH4            .Clear();
            this.EmissionsN2O            .Clear();
            Array.Clear(this.AnnualEmissionsCO2, 0, this.AnnualEmissionsCO2.Length);
            Array.Clear(this.AnnualEmissionsCO , 0, this.AnnualEmissionsCO .Length);
            Array.Clear(this.AnnualEmissionsVOC, 0, this.AnnualEmissionsVOC.Length);
            Array.Clear(this.AnnualEmissionsNOX, 0, this.AnnualEmissionsNOX.Length);
            Array.Clear(this.AnnualEmissionsPM , 0, this.AnnualEmissionsPM .Length);
            Array.Clear(this.AnnualEmissionsSOX, 0, this.AnnualEmissionsSOX.Length);
            Array.Clear(this.AnnualEmissionsCH4, 0, this.AnnualEmissionsCH4.Length);
            Array.Clear(this.AnnualEmissionsN2O, 0, this.AnnualEmissionsN2O.Length);
            this.SftSales                .Clear();
            this.SftVMT                  .Clear();
            this.SftCurbWeight           .Clear();
            this.SftFatalities           .Clear();
            Array.Clear(this.AnnualSftFatalities, 0, this.AnnualSftFatalities.Length);
            this.UndiscountedOwnerAndSocietalCosts.Clear();
            if (this.DiscountedOwnerAndSocietalCosts.Length < drCount)
            {
                this.DiscountedOwnerAndSocietalCosts = new OwnerAndSocietalCosts[drCount];
                for (int i = 0; i < this.DiscountedOwnerAndSocietalCosts.Length; i++)
                {
                    this.DiscountedOwnerAndSocietalCosts[i] = new OwnerAndSocietalCosts();
                }
            }
            else
            {
                for (int i = 0; i < this.DiscountedOwnerAndSocietalCosts.Length; i++)
                {
                    this.DiscountedOwnerAndSocietalCosts[i].Clear();
                }
            }
        }
        #endregion
        #region 
        public FuelValueByRc Sales;
        public FuelValueByRc VMT;
        public FuelValueByRc RatedFuelEconomy;
        public FuelValueByRc OnRoadFuelEconomy;
        public FuelValue AverageRatedFuelEconomy;
        public FuelValue AverageOnRoadFuelEconomy;
        public FuelValueByRc BTU;
        public FuelValueByRc Gallons;
        public FuelValueByRc KWh;
        public FuelValueByRc Scf;
        public RCDouble[] AnnualVMT;
        public RCDouble[] AnnualBTU;
        public RCDouble[] AnnualGallons;
        public RCDouble[] AnnualKWh;
        public RCDouble[] AnnualScf;
        public FuelValueByRc EmissionsCO2;
        public FuelValueByRc EmissionsCO;
        public FuelValueByRc EmissionsVOC;
        public FuelValueByRc EmissionsNOX;
        public FuelValueByRc EmissionsPM;
        public FuelValueByRc EmissionsSOX;
        public FuelValueByRc EmissionsCH4;
        public FuelValueByRc EmissionsN2O;
        public RCDouble[] AnnualEmissionsCO2;
        public RCDouble[] AnnualEmissionsCO;
        public RCDouble[] AnnualEmissionsVOC;
        public RCDouble[] AnnualEmissionsNOX;
        public RCDouble[] AnnualEmissionsPM;
        public RCDouble[] AnnualEmissionsSOX;
        public RCDouble[] AnnualEmissionsCH4;
        public RCDouble[] AnnualEmissionsN2O;
        public SafetyValueByRc SftSales;
        public SafetyValueByRc SftVMT;
        public SafetyValueByRc SftCurbWeight;
        public SafetyValueByRc SftFatalities;
        public RCDouble[] AnnualSftFatalities;
        public OwnerAndSocietalCosts UndiscountedOwnerAndSocietalCosts;
        public OwnerAndSocietalCosts[] DiscountedOwnerAndSocietalCosts;
        #endregion
    }
}

