using System;
namespace Volpe.Cafe.Data
{
    [Serializable]
    public struct FuelValue
	{
        #region 
        static FuelValue()
        {
            FuelValue.NaN  = new FuelValue(double.NaN);
            FuelValue.Zero = new FuelValue(0);
            FuelValue.One  = new FuelValue(1);
        }
        FuelValue(double value) : this(value, value, value, value, value) { }
        FuelValue(double gasoline, double diesel, double ethanol85, double electricity, double hydrogen)
        {
            this._gas = gasoline;
            this._dsl = diesel;
            this._e85 = ethanol85;
            this._elc = electricity;
            this._hgn = hydrogen;
        }
        #endregion
        #region 
        #region 
        public static FuelValue operator +(FuelValue value1, FuelValue value2)
        {
            return new FuelValue(value1._gas + value2._gas, value1._dsl + value2._dsl, value1._e85 + value2._e85,
                value1._elc + value2._elc, value1._hgn + value2._hgn);
        }
        public static FuelValue operator +(FuelValue value1, double value2)
        {
            return value1 + new FuelValue(value2);
        }
        public static FuelValue operator +(double value1, FuelValue value2)
        {
            return new FuelValue(value1) + value2;
        }
        public static FuelValue operator +(FuelValue value1, int value2)
        {
            return value1 + new FuelValue(value2);
        }
        public static FuelValue operator +(int value1, FuelValue value2)
        {
            return new FuelValue(value1) + value2;
        }
        public static FuelValue operator -(FuelValue value1, FuelValue value2)
        {
            return new FuelValue(value1._gas - value2._gas, value1._dsl - value2._dsl, value1._e85 - value2._e85,
                value1._elc - value2._elc, value1._hgn - value2._hgn);
        }
        public static FuelValue operator -(FuelValue value1, double value2)
        {
            return value1 - new FuelValue(value2);
        }
        public static FuelValue operator -(double value1, FuelValue value2)
        {
            return new FuelValue(value1) - value2;
        }
        public static FuelValue operator -(FuelValue value1, int value2)
        {
            return value1 - new FuelValue(value2);
        }
        public static FuelValue operator -(int value1, FuelValue value2)
        {
            return new FuelValue(value1) - value2;
        }
        public static FuelValue operator *(FuelValue value1, FuelValue value2)
        {
            return new FuelValue(value1._gas * value2._gas, value1._dsl * value2._dsl, value1._e85 * value2._e85,
                value1._elc * value2._elc, value1._hgn * value2._hgn);
        }
        public static FuelValue operator *(FuelValue value1, double value2)
        {
            return value1 * new FuelValue(value2);
        }
        public static FuelValue operator *(double value1, FuelValue value2)
        {
            return new FuelValue(value1) * value2;
        }
        public static FuelValue operator *(FuelValue value1, int value2)
        {
            return value1 * new FuelValue(value2);
        }
        public static FuelValue operator *(int value1, FuelValue value2)
        {
            return new FuelValue(value1) * value2;
        }
        public static FuelValue operator /(FuelValue value1, FuelValue value2)
        {
            return new FuelValue(value1._gas / value2._gas, value1._dsl / value2._dsl, value1._e85 / value2._e85,
                value1._elc / value2._elc, value1._hgn / value2._hgn);
        }
        public static FuelValue operator /(FuelValue value1, double value2)
        {
            return value1 / new FuelValue(value2);
        }
        public static FuelValue operator /(double value1, FuelValue value2)
        {
            return new FuelValue(value1) / value2;
        }
        public static FuelValue operator /(FuelValue value1, int value2)
        {
            return value1 / new FuelValue(value2);
        }
        public static FuelValue operator /(int value1, FuelValue value2)
        {
            return new FuelValue(value1) / value2;
        }
        public static FuelValue operator +(FuelValue value)
        {
            return FuelValue.Zero + value;
        }
        public static FuelValue operator -(FuelValue value)
        {
            return FuelValue.Zero - value;
        }
        public static FuelValue operator ++(FuelValue value)
        {
            return value + FuelValue.One;
        }
        public static FuelValue operator --(FuelValue value)
        {
            return value - FuelValue.One;
        }
        public static bool operator ==(FuelValue value1, FuelValue value2)
        {
            return value1.Equals(value2);
        }
        public static bool operator !=(FuelValue value1, FuelValue value2)
        {
            return !value1.Equals(value2);
        }
        #endregion
        public override string ToString()
        {
            return "{Gasoline="  + this._gas.ToString() +
                ", Diesel="      + this._dsl.ToString() +
                ", Ethanol-85="  + this._e85.ToString() +
                ", Electricity=" + this._elc.ToString() +
                ", Hydrogen="    + this._hgn.ToString() + "}";
        }
        public string ToString(string format)
        {
            return "{Gasoline="  + this._gas.ToString(format) +
                ", Diesel="      + this._dsl.ToString(format) +
                ", Ethanol-85="  + this._e85.ToString(format) +
                ", Electricity=" + this._elc.ToString(format) +
                ", Hydrogen="    + this._hgn.ToString(format) + "}";
        }
        public string ToString(int digits)
        {
            return "{Gasoline="  + Math.Round(this._gas, digits).ToString() +
                ", Diesel="      + Math.Round(this._dsl, digits).ToString() +
                ", Ethanol-85="  + Math.Round(this._e85, digits).ToString() +
                ", Electricity=" + Math.Round(this._elc, digits).ToString() +
                ", Hydrogen="    + Math.Round(this._hgn, digits).ToString() + "}";
        }
        public override int GetHashCode()
        {
            return this._gas.GetHashCode() ^ this._dsl.GetHashCode() ^ this._e85.GetHashCode() ^ this._elc.GetHashCode() ^ this._hgn.GetHashCode();
        }
        public override bool Equals(object obj)
        {
            if (obj is FuelValue) { return this.Equals((FuelValue)obj); }
            else                  { return base.Equals(           obj); }
        }
        public bool Equals(FuelValue value)
        {
            return (this._gas == value._gas && this._dsl == value._dsl && this._e85 == value._e85 && this._elc == value._elc &&
                this._hgn == value._hgn);
        }
        public static FuelValue Sum(params FuelValue[] values)
        {
            FuelValue sum = new FuelValue();
            for (int i = 0; i < values.Length; i++)
            {
                sum._gas += values[i]._gas;
                sum._dsl += values[i]._dsl;
                sum._e85 += values[i]._e85;
                sum._elc += values[i]._elc;
                sum._hgn += values[i]._hgn;
            }
            return sum;
        }
        public void Clear()
        {
            this._gas = this._dsl = this._e85 = this._elc = this._hgn = 0.0;
        }
        public double ArithmeticMean()
        {
            return this.ArithmeticMean(FuelValue.One);
        }
        public double ArithmeticMean(FuelValue weight)
        {
            return (this * weight).Total / weight.Total;
        }
        public double GeometricMean()
        {
            return this.GeometricMean(FuelValue.One);
        }
        public double GeometricMean(FuelValue weight)
        {
            double d = 0;
            if (this._gas != 0 && !double.IsNaN(this._gas) && !double.IsInfinity(this._gas)) { d += weight._gas * Math.Log(this._gas); }
            if (this._dsl != 0 && !double.IsNaN(this._dsl) && !double.IsInfinity(this._dsl)) { d += weight._dsl * Math.Log(this._dsl); }
            if (this._e85 != 0 && !double.IsNaN(this._e85) && !double.IsInfinity(this._e85)) { d += weight._e85 * Math.Log(this._e85); }
            if (this._elc != 0 && !double.IsNaN(this._elc) && !double.IsInfinity(this._elc)) { d += weight._elc * Math.Log(this._elc); }
            if (this._hgn != 0 && !double.IsNaN(this._hgn) && !double.IsInfinity(this._hgn)) { d += weight._hgn * Math.Log(this._hgn); }
            return Math.Exp(d / weight.Total);
        }
        public double HarmonicMean()
        {
            return this.HarmonicMean(FuelValue.One);
        }
        public double HarmonicMean(FuelValue weight)
        {
            double d = 0;
            if (this._gas != 0 && !double.IsNaN(this._gas) && !double.IsInfinity(this._gas)) { d += weight._gas / this._gas; }
            if (this._dsl != 0 && !double.IsNaN(this._dsl) && !double.IsInfinity(this._dsl)) { d += weight._dsl / this._dsl; }
            if (this._e85 != 0 && !double.IsNaN(this._e85) && !double.IsInfinity(this._e85)) { d += weight._e85 / this._e85; }
            if (this._elc != 0 && !double.IsNaN(this._elc) && !double.IsInfinity(this._elc)) { d += weight._elc / this._elc; }
            if (this._hgn != 0 && !double.IsNaN(this._hgn) && !double.IsInfinity(this._hgn)) { d += weight._hgn / this._hgn; }
            return (d == 0) ? 0 : weight.Total / d;
        }
        #endregion
        #region 
        public double this[FuelType fuelType]
        {
            get
            {
                return
                    (fuelType == FuelType.Gasoline   ) ? this._gas :
                    (fuelType == FuelType.Diesel     ) ? this._dsl :
                    (fuelType == FuelType.Ethanol85  ) ? this._e85 :
                    (fuelType == FuelType.Electricity) ? this._elc :
                    (fuelType == FuelType.Hydrogen   ) ? this._hgn : 0;
            }
            set
            {
                if      (fuelType == FuelType.Gasoline   ) { this._gas = value; }
                else if (fuelType == FuelType.Diesel     ) { this._dsl = value; }
                else if (fuelType == FuelType.Ethanol85  ) { this._e85 = value; }
                else if (fuelType == FuelType.Electricity) { this._elc = value; }
                else if (fuelType == FuelType.Hydrogen   ) { this._hgn = value; }
            }
        }
        public double Gasoline { get { return this._gas; } }
        public double Diesel { get { return this._dsl; } }
        public double Ethanol85 { get { return this._e85; } }
        public double Electricity { get { return this._elc; } }
        public double Hydrogen { get { return this._hgn; } }
        public double Total
        {
            get
            {
                if (this.IsNaN             ) { return double.NaN;              }
                if (this.IsPositiveInfinity) { return double.PositiveInfinity; }
                if (this.IsNegativeInfinity) { return double.NegativeInfinity; }
                double total = 0;
                if (!double.IsNaN(this._gas) && !double.IsInfinity(this._gas)) { total += this._gas; }
                if (!double.IsNaN(this._dsl) && !double.IsInfinity(this._dsl)) { total += this._dsl; }
                if (!double.IsNaN(this._e85) && !double.IsInfinity(this._e85)) { total += this._e85; }
                if (!double.IsNaN(this._elc) && !double.IsInfinity(this._elc)) { total += this._elc; }
                if (!double.IsNaN(this._hgn) && !double.IsInfinity(this._hgn)) { total += this._hgn; }
                return total;
            }
        }
        public bool IsValid { get { return !(this._gas < 0 || this._dsl < 0 || this._e85 < 0 || this._elc < 0 || this._hgn < 0); } }
        public bool IsEmpty { get { return this.Equals(FuelValue.Zero); } }
        public bool IsNaN
        {
            get
            {
                return (double.IsNaN(this._gas) && double.IsNaN(this._dsl) && double.IsNaN(this._e85) && double.IsNaN(this._elc) &&
                        double.IsNaN(this._hgn));
            }
        }
        public bool IsPositiveInfinity
        {
            get
            {
                return (double.IsPositiveInfinity(this._gas) && double.IsPositiveInfinity(this._dsl) && double.IsPositiveInfinity(this._e85) &&
                        double.IsPositiveInfinity(this._elc) && double.IsPositiveInfinity(this._hgn));
            }
        }
        public bool IsNegativeInfinity
        {
            get
            {
                return (double.IsNegativeInfinity(this._gas) && double.IsNegativeInfinity(this._dsl) && double.IsNegativeInfinity(this._e85) &&
                        double.IsNegativeInfinity(this._elc) && double.IsNegativeInfinity(this._hgn));
            }
        }
        public bool IsMultiFuel
        {
            get
            {
                int fuelTypes = 0;
                if (this._gas > 0) { fuelTypes++; }
                if (this._dsl > 0) { fuelTypes++; }
                if (this._e85 > 0) { fuelTypes++; }
                if (this._elc > 0) { fuelTypes++; }
                if (this._hgn > 0) { fuelTypes++; }
                return (fuelTypes > 1);
            }
        }
        public FuelType FuelType
        {
            get
            {
                FuelType fuelType = FuelType.None;
                if (this._gas > 0) { fuelType |= FuelType.Gasoline;    }
                if (this._dsl > 0) { fuelType |= FuelType.Diesel;      }
                if (this._e85 > 0) { fuelType |= FuelType.Ethanol85;   }
                if (this._elc > 0) { fuelType |= FuelType.Electricity; }
                if (this._hgn > 0) { fuelType |= FuelType.Hydrogen;    }
                return fuelType;
            }
        }
        public double PrimaryValue
        {
            get
            {
                return (this._gas > 0) ? this._gas :
                       (this._dsl > 0) ? this._dsl :
                       (this._e85 > 0) ? this._e85 :
                       (this._elc > 0) ? this._elc :
                       (this._hgn > 0) ? this._hgn : 0;
            }
        }
        public FuelType PrimaryFuel
        {
            get
            {
                return (this._gas > 0) ? FuelType.Gasoline    :
                       (this._dsl > 0) ? FuelType.Diesel      :
                       (this._e85 > 0) ? FuelType.Ethanol85   :
                       (this._elc > 0) ? FuelType.Electricity :
                       (this._hgn > 0) ? FuelType.Hydrogen    : FuelType.None;
            }
        }
        #endregion
        #region 
        public static readonly FuelValue NaN;
        public static readonly FuelValue Zero;
        public static readonly FuelValue One;
        double _gas, _dsl, _e85, _elc, _hgn;
        #endregion
	}
}

