using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using Volpe.Cafe.IO;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.Data
{
    [Serializable]
    public sealed class Industry : ICloneable
    {
        #region 
        [Serializable]
        public sealed class CModelData
        {
            #region 
            internal CModelData() { }
            #endregion
            #region 
            internal Industry.CModelData Clone()
            {
                Industry.CModelData imd = new Industry.CModelData();
                imd.TechUsedSales = (RCDouble[])Interaction.CloneArray(this.TechUsedSales, typeof(RCDouble));
                imd.MinFP = this.MinFP;
                imd.MaxFP = this.MaxFP;
                imd.MinCW = this.MinCW;
                imd.MaxCW = this.MaxCW;
                return imd;
            }
            internal void UpdateFPCW(List<Manufacturer> mfrs)
            {
                double minFP = 99999; double minCW = 99999;
                double maxFP = -1;    double maxCW = -1;
                for (int i = 0, mfrCount = mfrs.Count; i < mfrCount; i++)
                {   
                    Manufacturer mfr = mfrs[i];
                    for (int j = 0, vehCount = mfr.VehicleCount; j < vehCount; j++)
                    {
                        Vehicle.CDescription vd = mfr.Vehicles[j].Description;
                        double fp = vd.Footprint, cw = vd.CurbWeight;
                        if (fp < minFP) { minFP = fp; } else if (fp > maxFP) { maxFP = fp; }
                        if (cw < minCW) { minCW = cw; } else if (cw > maxCW) { maxCW = cw; }
                    } 
                } 
                this.MinFP = minFP; this.MaxFP = maxFP;
                this.MinCW = minCW; this.MaxCW = maxCW;
            }
            #endregion
            #region 
            public RCDouble[] TechUsedSales;
            public double MinFP;
            public double MaxFP;
            public double MinCW;
            public double MaxCW;
            #endregion
        }
        #endregion
        #region 
        Industry() { }
        internal Industry(List<Manufacturer> mfrs, int minYear, int maxYear)
        {
            this._modelData = new CModelData();
            this._effectsData = new EffectsData();
            this._manufacturers = mfrs;
            this._minYear = new ModelYear(minYear);
            this._maxYear = new ModelYear(maxYear);
        }
        #endregion
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        public Industry Clone()
        {
            Industry ind = new Industry();
            ind._modelData     = this._modelData.Clone();
            ind._effectsData   = this._effectsData.Clone();
            ind._manufacturers = this.CloneManufacturers(this._manufacturers);
            ind._minYear       = new ModelYear(this._minYear.Year);
            ind._maxYear       = new ModelYear(this._maxYear.Year);
            return ind;
        }
        List<Manufacturer> CloneManufacturers(List<Manufacturer> list)
        {
            List<Manufacturer> newList = new List<Manufacturer>(list.Count);
            for (int i = 0, count = list.Count; i < count; i++)
            {
                newList.Add((Manufacturer)(list[i].Clone()));
            }
            return newList;
        }
        #endregion
        public void WriteSummaryLog(LogWriter writer)
        {
            if (writer.Summary == null) { return; }
            writer.Summary.WriteLine("---------- Modeling Data ----------");
            writer.Summary.WriteLine("--- Model Years Analyzed ---\n  "
                + this._minYear.ToString() + " to " + this._maxYear.ToString() + "\n");
            writer.Summary.WriteLine("--- Valid Manufacturers ---");
            for (int i = 0, mfrCount = this._manufacturers.Count; i < mfrCount; i++)
            {
                Manufacturer mfr = this._manufacturers[i];
                writer.Summary.WriteLine(
                    "  " + mfr.Index + ". Code=" + mfr.Description.Code + ", Name=" + mfr.Description.Name +
                        "\n      Vehicles=" + mfr.VehicleCount +
                            " (Dom Autos=" + this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Domestic | VehicleType.PassengerAuto) +
                            ", Imp Autos=" + this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Imported | VehicleType.PassengerAuto) +
                            ", Dom LTs  =" + this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Domestic | VehicleType.LightTruck   ) +
                            ", Imp LTs  =" + this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Imported | VehicleType.LightTruck   ) +
                            ", Dom HLTs =" + this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Domestic | VehicleType.HeavyLT      ) +
                            ", Imp HLTs =" + this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Imported | VehicleType.HeavyLT      ) + ")" +
                        "\n      Engines=" + mfr.Engines.Count +
                        "\n      Transmissions=" + mfr.Transmissions.Count);
            }
            writer.Summary.WriteLine();
            writer.Summary.WriteLine("--- Industry Summary ---");
            int engs = 0, trns = 0, vehs = 0, da = 0, ia = 0, dlt = 0, ilt = 0, dhlt = 0, ihlt = 0;
            for (int i = 0, mfrCount = this._manufacturers.Count; i < mfrCount; i++)
            {
                Manufacturer mfr = this._manufacturers[i];
                engs += mfr.Engines.Count;
                trns += mfr.Transmissions.Count;
                vehs += mfr.VehicleCount;
                da   += this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Domestic | VehicleType.PassengerAuto);
                ia   += this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Imported | VehicleType.PassengerAuto);
                dlt  += this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Domestic | VehicleType.LightTruck   );
                ilt  += this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Imported | VehicleType.LightTruck   );
                dhlt += this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Domestic | VehicleType.HeavyLT      );
                ihlt += this.Log_GetVehCountByType(mfr.Vehicles, VehicleType.Imported | VehicleType.HeavyLT      );
            }
            writer.Summary.WriteLine("  Total Manufacturers Analyzed:  " + this._manufacturers.Count);
            writer.Summary.WriteLine("  Total Vehicles Analyzed:  " + vehs);
            writer.Summary.WriteLine("    Domestic Autos: " + da);
            writer.Summary.WriteLine("    Imported Autos: " + ia);
            writer.Summary.WriteLine("    Domestic LTs:   " + dlt);
            writer.Summary.WriteLine("    Imported LTs:   " + ilt);
            writer.Summary.WriteLine("    Domestic HLTs:  " + dhlt);
            writer.Summary.WriteLine("    Imported HLTs:  " + ihlt);
            writer.Summary.WriteLine("  Total Engines Analyzed:   " + engs);
            writer.Summary.WriteLine("  Total Transmissions Analyzed:  " + trns);
            writer.Summary.WriteLine();
        }
        int Log_GetVehCountByType(List<Vehicle> vehs, VehicleType type)
        {
            int count = 0;
            for (int i = 0, vehCount = vehs.Count; i < vehCount; i++)
            {
                if ((vehs[i].VehicleType & type) == type) { count++; }
            }
            return count;
        }
        public void UpdateMinMaxFpCw()
        {
            this._modelData.UpdateFPCW(this._manufacturers);
        }
        public static Industry BuildMergedFleet(Industry ind)
        {
            return BuildMergedFleet(ind, "MERGED FLEET");
        }
        public static Industry BuildMergedFleet(Industry ind, string newMfrName)
        {
            if (newMfrName == null || newMfrName.Trim().Length == 0) { throw new ArgumentException(ErrorStrings.MergedFleetNewMfrNameNull, "newMfrName"); }
            Manufacturer.CDescription baseMD = ind.Manufacturers[0].Description.Clone();
            ind = ind.Clone();
            List<Vehicle     > vehs = new List<Vehicle     >();
            List<Engine      > engs = new List<Engine      >();
            List<Transmission> trns = new List<Transmission>();
            for (int i = 0, mfrCount = ind._manufacturers.Count; i < mfrCount; i++)
            {   
                vehs.AddRange(ind._manufacturers[i].Vehicles     );
                engs.AddRange(ind._manufacturers[i].Engines      );
                trns.AddRange(ind._manufacturers[i].Transmissions);
            }
            int vehIndex = 1, engIndex = 1, trnIndex = 1;
            for (int i = 0, vehCount = vehs.Count; i < vehCount; i++)
            {   
                string mfrName = vehs[i].Description.Manufacturer;
                vehs[i].Description.Manufacturer = newMfrName;
                vehs[i].Description.Code         = vehIndex;
                vehs[i].Description.Model        = mfrName + "_" + vehs[i].Description.Model;
                vehs[i].Description.Nameplate    = mfrName + "_" + vehs[i].Description.Nameplate;
                vehs[i].Description.Origin       = 'D';
                vehIndex++;
            }
            for (int i = 0, engCount = engs.Count; i < engCount; i++)
            {   
                string mfrName = engs[i].Description.Manufacturer;
                engs[i].Description.Manufacturer = newMfrName;
                engs[i].Description.Code         = engIndex;
                for (int j = 0, vehCount = engs[i].Vehicles.Count; j < vehCount; j++)
                {   
                    engs[i].Vehicles[j].Description.EngineCode = engIndex;
                }
                engIndex++;
            }
            for (int i = 0, trnCount = trns.Count; i < trnCount; i++)
            {   
                string mfrName = trns[i].Description.Manufacturer;
                trns[i].Description.Manufacturer = newMfrName;
                trns[i].Description.Code         = trnIndex;
                for (int j = 0, vehCount = trns[i].Vehicles.Count; j < vehCount; j++)
                {   
                    trns[i].Vehicles[j].Description.TransmissionCode = trnIndex;
                }
                trnIndex++;
            }
            Manufacturer.CDescription md = new Manufacturer.CDescription();
            md.Code                      = 1;
            md.Name                      = newMfrName;
            md.CostAllocationStrategy    = 0;
            md.DiscountRate              = baseMD.DiscountRate;
            md.PaybackPeriod             = baseMD.PaybackPeriod;
            md.PaybackPeriod_OC          = baseMD.PaybackPeriod_OC;
            md.Optimize                  = true;
            md.WillingToPayFines         = new bool  [ind._maxYear.Year - ModelYear.MinYear + 1];
            md.AvailableCreditsDA        = new double[ind._maxYear.Year - ModelYear.MinYear + 1];
            md.AvailableCreditsIA        = new double[ind._maxYear.Year - ModelYear.MinYear + 1];
            md.AvailableCreditsLT        = new double[ind._maxYear.Year - ModelYear.MinYear + 1];
            md.CreditsApplyToBaseline    = false;
            int minYear, maxYear;
            Manufacturer mfr = new Manufacturer(md);
            mfr.Initialize(vehs, engs, trns, out minYear, out maxYear);
            mfr.Index = 0;
            ind._manufacturers.Clear();
            ind._manufacturers.Add(mfr);
            return ind;
        }
        public double CalcLaborHours(ModelYear year, RegulatoryClass regClass)
        {
            double hours = 0;
            for (int i = 0, mfrCount = this._manufacturers.Count; i < mfrCount; i++)
            {
                hours += this._manufacturers[i].CalcLaborHours(year, regClass);
            }
            return hours;
        }
        public double CalcLaborHours(ModelYear minYear, ModelYear maxYear, RegulatoryClass regClass)
        {
            double hours = 0;
            for (int i = 0, mfrCount = this._manufacturers.Count; i < mfrCount; i++)
            {
                hours += this._manufacturers[i].CalcLaborHours(minYear, maxYear, regClass);
            }
            return hours;
        }
        #endregion
        #region 
        public Industry.CModelData ModelData { get { return this._modelData; } }
        public EffectsData EffectsData { get { return this._effectsData; } }
        public List<Manufacturer> Manufacturers { get { return this._manufacturers; } }
        public int ManufacturerCount { get { return (this._manufacturers == null) ? 0 : this._manufacturers.Count; } }
        public List<Vehicle> Vehicles
        {
            get
            {
                List<Vehicle> vc = new List<Vehicle>(this.VehicleCount);
                for (int i = 0; i < this._manufacturers.Count; i++)
                {
                    vc.AddRange(this._manufacturers[i].Vehicles);
                }
                return vc;
            }
        }
        public int VehicleCount
        {
            get
            {
                int count = 0;
                for (int i = 0; i < this._manufacturers.Count; i++)
                {
                    count += this._manufacturers[i].VehicleCount;
                }
                return count;
            }
        }
        public int EngineCount
        {
            get
            {
                int count = 0;
                for (int i = 0; i < this._manufacturers.Count; i++)
                {
                    count += this._manufacturers[i].Engines.Count;
                }
                return count;
            }
        }
        public int TransmissionCount
        {
            get
            {
                int count = 0;
                for (int i = 0; i < this._manufacturers.Count; i++)
                {
                    count += this._manufacturers[i].Transmissions.Count;
                }
                return count;
            }
        }
        public ModelYear MinYear { get { return this._minYear; } }
        public ModelYear MaxYear { get { return this._maxYear; } }
        #endregion
        #region 
        Industry.CModelData _modelData;
        EffectsData         _effectsData;
        List<Manufacturer> _manufacturers;
        ModelYear _minYear;
        ModelYear _maxYear;
        #endregion
    }
}

