using System;
using System.Collections.Generic;
using Volpe.Cafe;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.Data
{
    [Serializable]
    public sealed class Manufacturer
    {
        #region 
        [Serializable]
        public sealed class CDescription
        {
            #region 
            internal CDescription() { }
            #endregion
            #region 
            internal CDescription Clone()
            {
                Manufacturer.CDescription md = new Manufacturer.CDescription();
                md.Code                   = this.Code;
                md.Name                   = this.Name;
                md.ProperName             = this.ProperName;
                md.CostAllocationStrategy = this.CostAllocationStrategy;
                md.DiscountRate           = this.DiscountRate;
                md.PaybackPeriod          = this.PaybackPeriod;
                md.PaybackPeriod_OC       = this.PaybackPeriod_OC;
                md.Optimize               = this.Optimize;
                md.WillingToPayFines      = (bool  [])Interaction.CloneArray(this.WillingToPayFines   , typeof(bool  ));
                md.AvailableCreditsDA     = (double[])Interaction.CloneArray(this.AvailableCreditsDA  , typeof(double));
                md.AvailableCreditsIA     = (double[])Interaction.CloneArray(this.AvailableCreditsIA  , typeof(double));
                md.AvailableCreditsLT     = (double[])Interaction.CloneArray(this.AvailableCreditsLT  , typeof(double));
                md.CreditsApplyToBaseline = this.CreditsApplyToBaseline;
                return md;
            }
            public override string ToString()
            {
                return this.ProperName;
            }
            #endregion
            #region 
            public int Code;
            public string Name;
            public string ProperName;
            public int CostAllocationStrategy;
            public double DiscountRate;
            public double PaybackPeriod;
            public double PaybackPeriod_OC;
            public bool Optimize;
            public bool[] WillingToPayFines;
            public double[] AvailableCreditsDA;
            public double[] AvailableCreditsIA;
            public double[] AvailableCreditsLT;
            public bool CreditsApplyToBaseline;
            #endregion
        }
        [Serializable]
        public sealed class CModelData
        {
            #region 
            internal CModelData() { }
            #endregion
            #region 
            internal Manufacturer.CModelData Clone()
            {
                Manufacturer.CModelData mmd = new Manufacturer.CModelData();
                mmd.KMY                 = (KMYType [])Interaction.CloneArray(this.KMY            , typeof(KMYType ));
                mmd.KMY_OC              = (KMYType [])Interaction.CloneArray(this.KMY_OC         , typeof(KMYType ));
                mmd.DefaultStandard     = (RCDouble[])Interaction.CloneArray(this.DefaultStandard, typeof(RCDouble));
                mmd.AverageStandard     = this.AverageStandard;
                mmd.PreliminaryStandard = this.PreliminaryStandard;
                mmd.Standard            = this.Standard;
                mmd.Sales               = this.Sales;
                mmd.SalesOverFE         = this.SalesOverFE;
                mmd.CAFE                = this.CAFE;
                mmd.CAFE_2Bag           = this.CAFE_2Bag;
                mmd.Credits             = this.Credits;
                mmd.Fines               = this.Fines;
                mmd.TCreditsInCapped    = this.TCreditsInCapped;
                mmd.TCreditsIn          = this.TCreditsIn;
                mmd.TCreditsOut         = this.TCreditsOut;
                mmd.TechCost            = this.TechCost;
                mmd.RegCost             = this.RegCost;
                mmd.LossOfValue         = this.LossOfValue;
                mmd.RelativeLossOfValue = this.RelativeLossOfValue;
                mmd.MaintenanceCost     = this.MaintenanceCost;
                mmd.RepairCost          = this.RepairCost;
                mmd.TotalConsumerCosts  = this.TotalConsumerCosts;
                mmd.TotalSocialCosts    = this.TotalSocialCosts;
                mmd.TaxesAndFees        = this.TaxesAndFees;
                mmd.FinancingCost       = this.FinancingCost;
                mmd.InsuranceCost       = this.InsuranceCost;
                mmd.TechUsedSales       = (RCDouble [])Interaction.CloneArray(this.TechUsedSales    , typeof(RCDouble ));
                mmd.TechAppliedSales    = (RCDouble [])Interaction.CloneArray(this.TechAppliedSales , typeof(RCDouble ));
                mmd.TechExhausted       = (RCBoolean[])Interaction.CloneArray(this.TechExhausted    , typeof(RCBoolean));
                mmd.TechPhaseInOffset   = (RCInt32  [])Interaction.CloneArray(this.TechPhaseInOffset, typeof(RCInt32  ));
                mmd.CFExhausted         = this.CFExhausted;
                return mmd;
            }
            #endregion
            #region 
            public KMYType[] KMY;
            public KMYType[] KMY_OC;
            public RCDouble[] DefaultStandard;
            public RCDouble AverageStandard;
            public RCDouble PreliminaryStandard;
            public RCDouble Standard;
            public RCDouble Sales;
            public RCDouble SalesOverFE;
            public RCDouble CAFE;
            public RCDouble CAFE_2Bag;
            public RCDouble Credits;
            public RCDouble Fines;
            public RCDouble TCreditsInCapped;
            public RCDouble TCreditsIn;
            public RCDouble TCreditsOut;
            public RCDouble TechCost;
            public RCDouble RegCost;
            public RCDouble LossOfValue;
            public RCDouble RelativeLossOfValue;
            public RCDouble MaintenanceCost;
            public RCDouble RepairCost;
            public RCDouble TotalConsumerCosts;
            public RCDouble TotalSocialCosts;
            public RCDouble TaxesAndFees;
            public RCDouble FinancingCost;
            public RCDouble InsuranceCost;
            public RCDouble[] TechUsedSales;
            public RCDouble[] TechAppliedSales;
            public RCBoolean[] TechExhausted;
            public RCInt32[] TechPhaseInOffset;
            public bool CFExhausted;
            #endregion
        }
        #endregion
        #region 
        Manufacturer() { }
        internal Manufacturer(Manufacturer.CDescription description)
        {
            this._index = -1;
            this._description   = description;
            this._modelData     = new Manufacturer.CModelData();
            this._effectsData   = new EffectsData();
            this._vehicles      = new List<Vehicle>     (8);
            this._engines       = new List<Engine>      (8);
            this._transmissions = new List<Transmission>(8);
            this._namePlates    = new List<NamePlate>   (8);
            this._lastEngCode = -1;
            this._lastTrnCode = -1;
        }
        #endregion
        #region 
        #region 
        internal Manufacturer Clone()
        {
            Manufacturer mfr = new Manufacturer();
            mfr._index         = this._index;
            mfr._lastEngCode   = this._lastEngCode;
            mfr._lastTrnCode   = this._lastTrnCode;
            mfr._description   = this._description.Clone();
            mfr._modelData     = this._modelData  .Clone();
            mfr._effectsData   = this._effectsData.Clone();
            mfr._vehicles      = this.CloneVehicles     (this._vehicles);
            mfr._engines       = this.CloneEngines      (this._engines);
            mfr._transmissions = this.CloneTransmissions(this._transmissions);
            mfr._namePlates    = this.CloneNamePlates   (this._namePlates);
            Engine      [] engs = mfr._engines.ToArray();
            Transmission[] trns = mfr._transmissions.ToArray();
            NamePlate   [] npts = mfr._namePlates.ToArray();
            Vehicle     [] vehs = mfr._vehicles.ToArray();
            for (int i = 0; i < engs.Length; i++)
            {
                int parentCode = ((engs[i].Parent == null) ? -1 : engs[i].Parent.Description.Code);
                engs[i].Initialize(mfr);
                engs[i].SetParent(parentCode);
            }
            for (int i = 0; i < trns.Length; i++)
            {
                int parentCode = ((trns[i].Parent == null) ? -1 : trns[i].Parent.Description.Code);
                trns[i].Initialize(mfr);
                trns[i].SetParent(parentCode);
            }
            for (int i = 0; i < npts.Length; i++)
            {
                npts[i].Initialize(mfr);
            }
            for (int i = 0; i < vehs.Length; i++)
            {   
                vehs[i].Initialize(mfr, vehs);
                NamePlate np = mfr.GetNamePlate(vehs[i].Description.Model);
                np.Vehicles.Add(vehs[i]);
                vehs[i].SetNamePlate(np);
            }
            return mfr;
        }
        List<Vehicle> CloneVehicles(List<Vehicle> list)
        {
            List<Vehicle> newList = new List<Vehicle>(list.Count);
            for (int i = 0, count = list.Count; i < count; i++)
            {
                newList.Add(list[i].Clone());
            }
            return newList;
        }
        List<Engine> CloneEngines(List<Engine> list)
        {
            List<Engine> newList = new List<Engine>(list.Count);
            for (int i = 0, count = list.Count; i < count; i++)
            {
                newList.Add(list[i].Clone());
            }
            return newList;
        }
        List<Transmission> CloneTransmissions(List<Transmission> list)
        {
            List<Transmission> newList = new List<Transmission>(list.Count);
            for (int i = 0, count = list.Count; i < count; i++)
            {
                newList.Add(list[i].Clone());
            }
            return newList;
        }
        List<NamePlate> CloneNamePlates(List<NamePlate> list)
        {
            List<NamePlate> newList = new List<NamePlate>(list.Count);
            for (int i = 0, count = list.Count; i < count; i++)
            {
                newList.Add(list[i].Clone());
            }
            return newList;
        }
        #endregion
        public override string ToString()
        {
            return this._description.Name;
        }
        internal void Initialize(List<Vehicle> vehicles, List<Engine> engines, List<Transmission> transmissions, out int minYear, out int maxYear)
        {
            Engine      [] engs = engines      .ToArray();
            Transmission[] trns = transmissions.ToArray();
            Vehicle     [] vehs = vehicles     .ToArray();
            for (int i = 0; i < engs.Length; i++)
            {
                engs[i].Initialize(this);
                this._engines.Add(engs[i]);
                if (this._lastEngCode < engs[i].Description.Code)
                {
                    this._lastEngCode = engs[i].Description.Code;
                }
            }
            for (int i = 0; i < trns.Length; i++)
            {
                trns[i].Initialize(this);
                this._transmissions.Add(trns[i]);
                if (this._lastTrnCode < trns[i].Description.Code)
                {
                    this._lastTrnCode = trns[i].Description.Code;
                }
            }
            for (int i = 0; i < vehs.Length; i++)
            {
                vehs[i].Initialize(this, vehs);
                NamePlate np = this.GetNamePlate(vehs[i].Description.Model);
                np.Vehicles.Add(vehs[i]);
                vehs[i].SetNamePlate(np);
                this._vehicles.Add(vehs[i]);
            }
            List<Vehicle> vc = this.Vehicles;
            minYear = int.MaxValue;
            maxYear = -1;
            for (int i = 0, vehCount = this.VehicleCount; i < vehCount; i++)
            {
                Vehicle v = vc[i];
                if (v.ValidYears != null)
                {
                    minYear = Math.Min(v.MinYear.Year, minYear);
                    maxYear = Math.Max(v.MaxYear.Year, maxYear);
                }
            }
        }
        NamePlate GetNamePlate(string name)
        {
            for (int i = 0; i < this._namePlates.Count; i++)
            {
                if (this._namePlates[i].Name.Equals(name.Trim().ToUpper()))
                {    
                    return this._namePlates[i];
                }
            }
            NamePlate np = new NamePlate(name);
            np.Initialize(this);
            this._namePlates.Add(np);
            return np;
        }
        internal int GetNextEngineCode()
        {
            return ++this._lastEngCode;
        }
        internal int GetNextTransmissionCode()
        {
            return ++this._lastTrnCode;
        }
        public double CalcLaborHours(ModelYear year, RegulatoryClass regClass)
        {
            double hours = 0;
            for (int i = 0, vehCount = this._vehicles.Count; i < vehCount; i++)
            {
                if (regClass == (RegulatoryClass)(-2) ||
                    (regClass == (RegulatoryClass)(-1) && this._vehicles[i].RegClass != RegulatoryClass.Unregulated) ||
                    regClass == this._vehicles[i].RegClass)
                {
                    hours += this._vehicles[i].CalcLaborHours(year);
                }
            }
            return hours;
        }
        public double CalcLaborHours(ModelYear minYear, ModelYear maxYear, RegulatoryClass regClass)
        {
            double hours = 0;
            for (int i = 0, vehCount = this._vehicles.Count; i < vehCount; i++)
            {
                if (regClass == (RegulatoryClass)(-2) ||
                    (regClass == (RegulatoryClass)(-1) && this._vehicles[i].RegClass != RegulatoryClass.Unregulated) ||
                    regClass == this._vehicles[i].RegClass)
                {
                    hours += this._vehicles[i].CalcLaborHours(minYear, maxYear);
                }
            }
            return hours;
        }
        #endregion
        #region 
        public int Index { get { return this._index; } internal set { this._index = value; } }
        public Manufacturer.CDescription Description { get { return this._description; } }
        public Manufacturer.CModelData ModelData { get { return this._modelData; } }
        public EffectsData EffectsData { get { return this._effectsData; } }
        public List<Vehicle> Vehicles { get { return this._vehicles; } }
        public int VehicleCount { get { return this._vehicles.Count; } }
        public List<Engine> Engines { get { return this._engines; } }
        public List<Transmission> Transmissions { get { return this._transmissions; } }
        public List<NamePlate> NamePlates { get { return this._namePlates; } }
        internal int LastEngCode { get { return this._lastEngCode; } set { this._lastEngCode = value; } }
        internal int LastTrnCode { get { return this._lastTrnCode; } set { this._lastTrnCode = value; } }
        #endregion
        #region 
        int _index; 
        Manufacturer.CDescription _description;
        Manufacturer.CModelData   _modelData;
        EffectsData               _effectsData;
        List<Vehicle>      _vehicles;
        List<Engine>       _engines;
        List<Transmission> _transmissions;
        List<NamePlate>    _namePlates;
        int _lastEngCode;
        int _lastTrnCode;
        #endregion
    }
}

