using System;
namespace Volpe.Cafe.Data
{
    [Serializable]
    public struct RCBoolean
    {
        #region 
        static RCBoolean()
        {
            RCBoolean.False = new RCBoolean(false);
            RCBoolean.True  = new RCBoolean(true);
        }
        RCBoolean(bool value)
            : this(value, value, value, value)
        {
        }
        RCBoolean(bool unregulated, bool domesticAuto, bool importedAuto, bool lightTruck)
        {
            this.Unregulated  = unregulated;
            this.DomesticAuto = domesticAuto;
            this.ImportedAuto = importedAuto;
            this.LightTruck   = lightTruck;
        }
        #endregion
        #region 
        #region 
        public static bool operator ==(RCBoolean value1, RCBoolean value2)
        {
            return value1.Equals(value2);
        }
        public static bool operator !=(RCBoolean value1, RCBoolean value2)
        {
            return !value1.Equals(value2);
        }
        #endregion
        public override string ToString()
        {
            return "{Dom Auto="  + this.DomesticAuto.ToString() +
                ", Imp Auto="    + this.ImportedAuto.ToString() +
                ", Light Truck=" + this.LightTruck  .ToString() +
                ", Unregulated=" + this.Unregulated .ToString() + "}";
        }
        public override int GetHashCode()
        {
            return this.DomesticAuto.GetHashCode() ^ this.ImportedAuto.GetHashCode() ^ this.LightTruck.GetHashCode() ^
                this.Unregulated.GetHashCode();
        }
        public override bool Equals(object obj)
        {
            if (obj is RCBoolean) { return this.Equals((RCBoolean)obj); }
            else                  { return base.Equals(obj); }
        }
        public bool Equals(RCBoolean value)
        {
            return (this.DomesticAuto == value.DomesticAuto && this.ImportedAuto == value.ImportedAuto &&
                    this.LightTruck   == value.LightTruck   && this.Unregulated  == value.Unregulated);
        }
        public bool Value(RegulatoryClass regClass)
        {
            return
                (regClass == RegulatoryClass.DomesticAuto) ? this.DomesticAuto :
                (regClass == RegulatoryClass.ImportedAuto) ? this.ImportedAuto :
                (regClass == RegulatoryClass.LightTruck  ) ? this.LightTruck   : this.Unregulated;
        }
        public void Clear()
        {
            this.Unregulated  = false;
            this.DomesticAuto = false;
            this.ImportedAuto = false;
            this.LightTruck   = false;
        }
        #endregion
        #region 
        public bool this[RegulatoryClass regClass]
        {
            get { return this.Value(regClass); }
            set
            {
                if      (regClass == RegulatoryClass.DomesticAuto) { this.DomesticAuto = value; }
                else if (regClass == RegulatoryClass.ImportedAuto) { this.ImportedAuto = value; }
                else if (regClass == RegulatoryClass.LightTruck  ) { this.LightTruck   = value; }
                else if (regClass == RegulatoryClass.Unregulated ) { this.Unregulated  = value; }
                else { throw new ArgumentException("The value specified does not represent a valid regulatory class."); }
            }
        }
        public bool IsTrue { get { return this.DomesticAuto && this.ImportedAuto && this.LightTruck; } }
        public bool IsTrueAll { get { return this.Unregulated && this.IsTrue; } }
        public bool IsFalse { get { return !(this.DomesticAuto || this.ImportedAuto || this.LightTruck); } }
        public bool IsFalseAll { get { return this.IsFalse && !this.Unregulated; } }
        #endregion
        #region 
        public static readonly RCBoolean False;
        public static readonly RCBoolean True;
        public bool Unregulated;
        public bool DomesticAuto;
        public bool ImportedAuto;
        public bool LightTruck;
        #endregion
    }
}

