using System;
namespace Volpe.Cafe.Data
{
    [Serializable]
    public struct RCDouble
    {
        #region 
        static RCDouble()
        {
            RCDouble.NaN  = new RCDouble(double.NaN);
            RCDouble.Zero = new RCDouble(0);
            RCDouble.One  = new RCDouble(1);
        }
        RCDouble(double value) : this(value, value, value, value) { }
        RCDouble(double unregulated, double domesticAuto, double importedAuto, double lightTruck)
        {
            this._un = unregulated;
            this._da = domesticAuto;
            this._ia = importedAuto;
            this._lt = lightTruck;
        }
        #endregion
        #region 
        #region 
        public static RCDouble operator +(RCDouble value1, RCDouble value2)
        {
            return new RCDouble(value1._un + value2._un, value1._da + value2._da, value1._ia + value2._ia, value1._lt + value2._lt);
        }
        public static RCDouble operator +(RCDouble value1, double value2)
        {
            return value1 + new RCDouble(value2);
        }
        public static RCDouble operator +(double value1, RCDouble value2)
        {
            return new RCDouble(value1) + value2;
        }
        public static RCDouble operator +(RCDouble value1, int value2)
        {
            return value1 + new RCDouble(value2);
        }
        public static RCDouble operator +(int value1, RCDouble value2)
        {
            return new RCDouble(value1) + value2;
        }
        public static RCDouble operator -(RCDouble value1, RCDouble value2)
        {
            return new RCDouble(value1._un - value2._un, value1._da - value2._da, value1._ia - value2._ia, value1._lt - value2._lt);
        }
        public static RCDouble operator -(RCDouble value1, double value2)
        {
            return value1 - new RCDouble(value2);
        }
        public static RCDouble operator -(double value1, RCDouble value2)
        {
            return new RCDouble(value1) - value2;
        }
        public static RCDouble operator -(RCDouble value1, int value2)
        {
            return value1 - new RCDouble(value2);
        }
        public static RCDouble operator -(int value1, RCDouble value2)
        {
            return new RCDouble(value1) - value2;
        }
        public static RCDouble operator *(RCDouble value1, RCDouble value2)
        {
            return new RCDouble(value1._un * value2._un, value1._da * value2._da, value1._ia * value2._ia, value1._lt * value2._lt);
        }
        public static RCDouble operator *(RCDouble value1, double value2)
        {
            return value1 * new RCDouble(value2);
        }
        public static RCDouble operator *(double value1, RCDouble value2)
        {
            return new RCDouble(value1) * value2;
        }
        public static RCDouble operator *(RCDouble value1, int value2)
        {
            return value1 * new RCDouble(value2);
        }
        public static RCDouble operator *(int value1, RCDouble value2)
        {
            return new RCDouble(value1) * value2;
        }
        public static RCDouble operator /(RCDouble value1, RCDouble value2)
        {
            return new RCDouble(value1._un / value2._un, value1._da / value2._da, value1._ia / value2._ia, value1._lt / value2._lt);
        }
        public static RCDouble operator /(RCDouble value1, double value2)
        {
            return value1 / new RCDouble(value2);
        }
        public static RCDouble operator /(double value1, RCDouble value2)
        {
            return new RCDouble(value1) / value2;
        }
        public static RCDouble operator /(RCDouble value1, int value2)
        {
            return value1 / new RCDouble(value2);
        }
        public static RCDouble operator /(int value1, RCDouble value2)
        {
            return new RCDouble(value1) / value2;
        }
        public static RCDouble operator +(RCDouble value)
        {
            return RCDouble.Zero + value;
        }
        public static RCDouble operator -(RCDouble value)
        {
            return RCDouble.Zero - value;
        }
        public static RCDouble operator ++(RCDouble value)
        {
            return value + RCDouble.One;
        }
        public static RCDouble operator --(RCDouble value)
        {
            return value - RCDouble.One;
        }
        public static bool operator ==(RCDouble value1, RCDouble value2)
        {
            return value1.Equals(value2);
        }
        public static bool operator !=(RCDouble value1, RCDouble value2)
        {
            return !value1.Equals(value2);
        }
        #endregion
        public override string ToString()
        {
            return "{Dom Auto="  + this._da.ToString() +
                ", Imp Auto="    + this._ia.ToString() +
                ", Light Truck=" + this._lt.ToString() +
                ", Unregulated=" + this._un.ToString() + "}";
        }
        public string ToString(string format)
        {
            return "{Dom Auto="  + this._da.ToString(format) +
                ", Imp Auto="    + this._ia.ToString(format) +
                ", Light Truck=" + this._lt.ToString(format) +
                ", Unregulated=" + this._un.ToString(format) + "}";
        }
        public string ToString(int digits)
        {
            return "{Dom Auto="  + Math.Round(this._da, digits).ToString() +
                ", Imp Auto="    + Math.Round(this._ia, digits).ToString() +
                ", Light Truck=" + Math.Round(this._lt, digits).ToString() +
                ", Unregulated=" + Math.Round(this._un, digits).ToString() + "}";
        }
        public override int GetHashCode()
        {
            return this._da.GetHashCode() ^ this._ia.GetHashCode() ^ this._lt.GetHashCode() ^ this._un.GetHashCode();
        }
        public override bool Equals(object obj)
        {
            return (obj is RCDouble) ? this.Equals((RCDouble)obj) : base.Equals(obj);
        }
        public bool Equals(RCDouble value)
        {
            return (this._da == value._da && this._ia == value._ia && this._lt == value._lt && this._un == value._un);
        }
        public double Value(RegulatoryClass regClass)
        {
            return
                (regClass ==  RegulatoryClass.DomesticAuto) ? this._da :
                (regClass ==  RegulatoryClass.ImportedAuto) ? this._ia :
                (regClass ==  RegulatoryClass.LightTruck  ) ? this._lt :
                (regClass ==  RegulatoryClass.Unregulated ) ? this._un :
                (regClass == (RegulatoryClass)(-1)        ) ? this.Total    :
                (regClass == (RegulatoryClass)(-2)        ) ? this.TotalAll : 0;
        }
        public void Clear()
        {
            this._da = 0.0;
            this._ia = 0.0;
            this._lt = 0.0;
            this._un = 0.0;
        }
        public void ZeroNaNs()
        {
            if (double.IsNaN(this._da)) { this._da = 0; }
            if (double.IsNaN(this._ia)) { this._ia = 0; }
            if (double.IsNaN(this._lt)) { this._lt = 0; }
            if (double.IsNaN(this._un)) { this._un = 0; }
        }
        public static RCDouble ZeroNaNs(RCDouble value)
        {
            value.ZeroNaNs();
            return value;
        }
        public RCDouble CreateRounded(int digits)
        {
            RCDouble value = new RCDouble();
            value._da = Math.Round(this._da, digits);
            value._ia = Math.Round(this._ia, digits);
            value._lt = Math.Round(this._lt, digits);
            value._un = Math.Round(this._un, digits);
            return value;
        }
        public double ArithmeticMean()
        {
            return this.ArithmeticMean(RCDouble.One);
        }
        public double ArithmeticMean(RCDouble weight)
        {
            return (this * weight).Total / weight.Total;
        }
        public double GeometricMean()
        {
            return this.GeometricMean(RCDouble.One);
        }
        public double GeometricMean(RCDouble weight)
        {
            double d = 0;
            if (this._da != 0 && !double.IsNaN(this._da) && !double.IsInfinity(this._da)) { d += weight._da * Math.Log(this._da); }
            if (this._ia != 0 && !double.IsNaN(this._ia) && !double.IsInfinity(this._ia)) { d += weight._ia * Math.Log(this._ia); }
            if (this._lt != 0 && !double.IsNaN(this._lt) && !double.IsInfinity(this._lt)) { d += weight._lt * Math.Log(this._lt); }
            if (this._un != 0 && !double.IsNaN(this._un) && !double.IsInfinity(this._un)) { d += weight._un * Math.Log(this._un); }
            return Math.Exp(d / weight.Total);
        }
        public double HarmonicMean()
        {
            return this.HarmonicMean(RCDouble.One);
        }
        public double HarmonicMean(RCDouble weight)
        {
            double d = 0;
            if (this._da != 0 && !double.IsNaN(this._da) && !double.IsInfinity(this._da)) { d += weight._da / this._da; }
            if (this._ia != 0 && !double.IsNaN(this._ia) && !double.IsInfinity(this._ia)) { d += weight._ia / this._ia; }
            if (this._lt != 0 && !double.IsNaN(this._lt) && !double.IsInfinity(this._lt)) { d += weight._lt / this._lt; }
            if (this._un != 0 && !double.IsNaN(this._un) && !double.IsInfinity(this._un)) { d += weight._un / this._un; }
            return (d == 0) ? 0 : weight.Total / d;
        }
        #endregion
        #region 
        public double this[RegulatoryClass regClass]
        {
            get { return this.Value(regClass); }
            set
            {
                if      (regClass == RegulatoryClass.DomesticAuto) { this._da = value; }
                else if (regClass == RegulatoryClass.ImportedAuto) { this._ia = value; }
                else if (regClass == RegulatoryClass.LightTruck  ) { this._lt = value; }
                else if (regClass == RegulatoryClass.Unregulated ) { this._un = value; }
                else
                {
                    throw new ArgumentException("The value specified does not represent a valid regulatory class.");
                }
            }
        }
        public double Unregulated { get { return this._un; } set { this._un = value; } }
        public double DomesticAuto { get { return this._da; } set { this._da = value; } }
        public double ImportedAuto { get { return this._ia; } set { this._ia = value; } }
        public double LightTruck { get { return this._lt; } set { this._lt = value; } }
        public double PassengerAuto
        {
            get
            {
                if (double.IsNaN(this._da) && double.IsNaN(this._ia)) { return double.NaN; }
                else { return (double.IsNaN(this._da) ? 0 : this._da) + (double.IsNaN(this._ia) ? 0 : this._ia); }
            }
        }
        public double LTUnregulated
        {
            get
            {
                if (double.IsNaN(this._lt) && double.IsNaN(this._un)) { return double.NaN; }
                else { return (double.IsNaN(this._lt) ? 0 : this._lt) + (double.IsNaN(this._un) ? 0 : this._un); }
            }
        }
        public double Total
        {
            get
            {
                if (double.IsNaN(this._da) && double.IsNaN(this._ia) && double.IsNaN(this._lt)) { return double.NaN; }
                else
                {
                    return (double.IsNaN(this._da) ? 0 : this._da) +
                           (double.IsNaN(this._ia) ? 0 : this._ia) +
                           (double.IsNaN(this._lt) ? 0 : this._lt);
                }
            }
        }
        public double TotalAll
        {
            get
            {
                double total = this.Total;
                if (double.IsNaN(total) && double.IsNaN(this._un)) { return double.NaN; }
                else { return (double.IsNaN(total) ? 0 : total) + (double.IsNaN(this._un) ? 0 : this._un); }
            }
        }
        public bool IsNaN
        {
            get
            {
                return (double.IsNaN(this._da) && double.IsNaN(this._ia) && double.IsNaN(this._lt) && double.IsNaN(this._un));
            }
        }
        #endregion
        #region 
        public static readonly RCDouble NaN;
        public static readonly RCDouble Zero;
        public static readonly RCDouble One;
        double _un, _da, _ia, _lt;
        #endregion
    }
}

