using System;
namespace Volpe.Cafe.Data
{
    [Serializable]
    public struct SFTDouble
    {
        #region 
        static SFTDouble()
        {
            SFTDouble.Zero = new SFTDouble(0);
            SFTDouble.One  = new SFTDouble(1);
        }
        SFTDouble(double value)
            : this(value, value, value)
        {
        }
        SFTDouble(double pc, double lt, double cm)
        {
            this.PC = pc;
            this.LT = lt;
            this.CM = cm;
        }
        #endregion
        #region 
        #region 
        public static SFTDouble operator +(SFTDouble value1, SFTDouble value2)
        {
            return new SFTDouble(value1.PC + value2.PC, value1.LT + value2.LT, value1.CM + value2.CM);
        }
        public static SFTDouble operator +(SFTDouble value1, double value2)
        {
            return value1 + new SFTDouble(value2);
        }
        public static SFTDouble operator +(double value1, SFTDouble value2)
        {
            return new SFTDouble(value1) + value2;
        }
        public static SFTDouble operator +(SFTDouble value1, int value2)
        {
            return value1 + new SFTDouble(value2);
        }
        public static SFTDouble operator +(int value1, SFTDouble value2)
        {
            return new SFTDouble(value1) + value2;
        }
        public static SFTDouble operator -(SFTDouble value1, SFTDouble value2)
        {
            return new SFTDouble(value1.PC - value2.PC, value1.LT - value2.LT, value1.CM - value2.CM);
        }
        public static SFTDouble operator -(SFTDouble value1, double value2)
        {
            return value1 - new SFTDouble(value2);
        }
        public static SFTDouble operator -(double value1, SFTDouble value2)
        {
            return new SFTDouble(value1) - value2;
        }
        public static SFTDouble operator -(SFTDouble value1, int value2)
        {
            return value1 - new SFTDouble(value2);
        }
        public static SFTDouble operator -(int value1, SFTDouble value2)
        {
            return new SFTDouble(value1) - value2;
        }
        public static SFTDouble operator *(SFTDouble value1, SFTDouble value2)
        {
            return new SFTDouble(value1.PC * value2.PC, value1.LT * value2.LT, value1.CM * value2.CM);
        }
        public static SFTDouble operator *(SFTDouble value1, double value2)
        {
            return value1 * new SFTDouble(value2);
        }
        public static SFTDouble operator *(double value1, SFTDouble value2)
        {
            return new SFTDouble(value1) * value2;
        }
        public static SFTDouble operator *(SFTDouble value1, int value2)
        {
            return value1 * new SFTDouble(value2);
        }
        public static SFTDouble operator *(int value1, SFTDouble value2)
        {
            return new SFTDouble(value1) * value2;
        }
        public static SFTDouble operator /(SFTDouble value1, SFTDouble value2)
        {
            return new SFTDouble(value1.PC / value2.PC, value1.LT / value2.LT, value1.CM / value2.CM);
        }
        public static SFTDouble operator /(SFTDouble value1, double value2)
        {
            return value1 / new SFTDouble(value2);
        }
        public static SFTDouble operator /(double value1, SFTDouble value2)
        {
            return new SFTDouble(value1) / value2;
        }
        public static SFTDouble operator /(SFTDouble value1, int value2)
        {
            return value1 / new SFTDouble(value2);
        }
        public static SFTDouble operator /(int value1, SFTDouble value2)
        {
            return new SFTDouble(value1) / value2;
        }
        public static SFTDouble operator +(SFTDouble value)
        {
            return SFTDouble.Zero + value;
        }
        public static SFTDouble operator -(SFTDouble value)
        {
            return SFTDouble.Zero - value;
        }
        public static SFTDouble operator ++(SFTDouble value)
        {
            return value + SFTDouble.One;
        }
        public static SFTDouble operator --(SFTDouble value)
        {
            return value - SFTDouble.One;
        }
        public static bool operator ==(SFTDouble value1, SFTDouble value2)
        {
            return value1.Equals(value2);
        }
        public static bool operator !=(SFTDouble value1, SFTDouble value2)
        {
            return !value1.Equals(value2);
        }
        #endregion
        public override string ToString()
        {
            return
                "{PC="     + this.PC   .ToString() +
                ", LT="    + this.LT   .ToString() +
                ", CM="    + this.CM   .ToString() +
                ", Total=" + this.Total.ToString() + "}";
        }
        public override int GetHashCode()
        {
            return (this.PC.GetHashCode() ^ this.PC.GetHashCode() ^
                    this.LT.GetHashCode() ^ this.LT.GetHashCode() ^
                    this.CM.GetHashCode() ^ this.CM.GetHashCode());
        }
        public override bool Equals(object obj)
        {
            if (obj is SFTDouble) { return this.Equals((SFTDouble)obj); }
            else                  { return base.Equals(obj); }
        }
        public bool Equals(SFTDouble value)
        {
            return (this.PC == value.PC && this.LT == value.LT && this.CM == value.CM);
        }
        public double Value(SafetyClass safetyClass)
        {
            return (safetyClass == SafetyClass.PC) ? this.PC :
                   (safetyClass == SafetyClass.LT) ? this.LT :
                   (safetyClass == SafetyClass.CM) ? this.CM : 0;
        }
        public void ZeroNaNs()
        {
            if (double.IsNaN(this.PC)) { this.PC = 0; }
            if (double.IsNaN(this.LT)) { this.LT = 0; }
            if (double.IsNaN(this.CM)) { this.CM = 0; }
        }
        public void Clear()
        {
            this.PC = 0.0;
            this.LT = 0.0;
            this.CM = 0.0;
        }
        public static SFTDouble Sum(params SFTDouble[] values)
        {
            SFTDouble sum = new SFTDouble();
            for (int i = 0; i < values.Length; i++)
            {
                sum.PC += values[i].PC;
                sum.LT += values[i].LT;
                sum.CM += values[i].CM;
            }
            return sum;
        }
        #endregion
        #region 
        public double this[SafetyClass safetyClass]
        {
            get { return this.Value(safetyClass); }
            set
            {
                if      (safetyClass == SafetyClass.PC) { this.PC = value; }
                else if (safetyClass == SafetyClass.LT) { this.LT = value; }
                else if (safetyClass == SafetyClass.CM) { this.CM = value; }
            }
        }
        public double Total
        {
            get
            {
                if (double.IsNaN(this.PC) && double.IsNaN(this.LT) && double.IsNaN(this.CM))
                {
                    return double.NaN;
                }
                else
                {
                    return (double.IsNaN(this.PC) ? 0 : this.PC) +
                           (double.IsNaN(this.LT) ? 0 : this.LT) +
                           (double.IsNaN(this.CM) ? 0 : this.CM);
                }
            }
        }
        #endregion
        #region 
        public static readonly SFTDouble Zero;
        public static readonly SFTDouble One;
        public double PC;
        public double LT;
        public double CM;
        #endregion
    }
}

