#region << Using Directives >>
using System;
using System.Collections.Generic;
using Volpe.Cafe.Data;
using Volpe.Cafe.Utils;
#endregion
namespace Volpe.Cafe.IO.InputParsers
{
    sealed class XlMarketDataParser : XlParser
    {
        #region 
        internal XlMarketDataParser(string path, string password) : base(path, password, true) { }
        #endregion
        #region 
        protected override void ParseInternal()
        {
            this._ind = null;
            List<Manufacturer> mfrs = new List<Manufacturer>(32);
            string[] sheets  = this.SheetNames;
            string[] names   = {"Manufacturers", "Vehicles", "Engines", "Transmissions"};
            int   [] indexes = {-1, -1, -1, -1};
            for (int i = 0; i < sheets.Length; i++)
            {
                for (int j = 0; j < names.Length; j++)
                {
                    if (Interaction.StringCompare(sheets[i], names[j], false)) { indexes[j] = i; break; }
                }
            }
            if (!this.VerifyIndexes("Market Data workbook", indexes, names)) { return; }
            int mfrWsMinYear, mfrWsMaxYear;         
            List<Manufacturer>   mc = this.LoadManufacturers(indexes[0], out mfrWsMinYear, out mfrWsMaxYear);
            List<Vehicle     >[] vc = this.LoadVehicles     (indexes[1], mc);
            List<Engine      >[] ec = this.LoadEngines      (indexes[2], mc);
            List<Transmission>[] tc = this.LoadTransmissions(indexes[3], mc);
            if (this.HasErrors) { return; }
            int mfrIndex = 0;
            int indMinYear = int.MaxValue, indMaxYear = -1;
            for (int i = 0; i < mc.Count; i++)
            {
                int minYear, maxYear;
                mc[i].Initialize(vc[i], ec[i], tc[i], out minYear, out maxYear);
                if (mc[i].VehicleCount == 0)
                {
                    this.LogError("Manufacturer Code " + mc[i].Description.Code + " does not contain any valid vehicles.");
                    return;
                }
                else
                {   
                    for (int j = 0, vehCount = vc[i].Count; j < vehCount; j++)
                    {
                        Vehicle veh = vc[i][j];
                        Vehicle.CDescription vd = veh.Description;
                        FuelType fuel = (veh.Engine == null) ? FuelType.None : veh.Engine.Description.Fuel;
                        bool isGas = ((fuel & FuelType.Gasoline) == FuelType.Gasoline);
                        bool isDsl = ((fuel & FuelType.Diesel) == FuelType.Diesel);
                        bool isE85 = ((fuel & FuelType.Ethanol85) == FuelType.Ethanol85);
                        string hdr = "Vehicle (Code:" + vd.Code + ",Mfr:" + vd.Manufacturer + "):  ";
                        if ( isGas && vd.FuelShare  [FuelType.Gasoline ] <= 0) { this.LogError(hdr + "Engine is listed as using Gasoline Fuel, but Vehicle Gasoline Fuel Share is not specified or not valid."); }
                        if ( isDsl && vd.FuelShare  [FuelType.Diesel   ] <= 0) { this.LogError(hdr + "Engine is listed as using Diesel Fuel, but Vehicle Diesel Fuel Share is not specified or not valid."); }
                        if ( isE85 && vd.FuelShare  [FuelType.Ethanol85] <= 0) { this.LogError(hdr + "Engine is listed as using Ethanol-85 Fuel, but Vehicle Ethanol-85 Fuel Share is not specified or not valid."); }
                        if ( isGas && vd.FuelEconomy[FuelType.Gasoline ] <= 0) { this.LogError(hdr + "Engine is listed as using Gasoline Fuel, but Vehicle Gasoline Fuel Economy is not specified or not valid."); }
                        if ( isDsl && vd.FuelEconomy[FuelType.Diesel   ] <= 0) { this.LogError(hdr + "Engine is listed as using Diesel Fuel, but Vehicle Diesel Fuel Economy is not specified or not valid."); }
                        if ( isE85 && vd.FuelEconomy[FuelType.Ethanol85] <= 0) { this.LogError(hdr + "Engine is listed as using Ethanol-85 Fuel, but Vehicle Ethanol-85 Fuel Economy is not specified or not valid."); }
                        if (!isGas && vd.FuelShare  [FuelType.Gasoline ] > 0) { this.LogError(hdr + "Vehicle specifies a Gasoline Fuel Share, but the Engine is not listed as using Gasoline Fuel."); }
                        if (!isDsl && vd.FuelShare  [FuelType.Diesel   ] > 0) { this.LogError(hdr + "Vehicle specifies a Diesel Fuel Share, but the Engine is not listed as using Diesel Fuel."); }
                        if (!isE85 && vd.FuelShare  [FuelType.Ethanol85] > 0) { this.LogError(hdr + "Vehicle specifies an Ethanol-85 Fuel Share, but the Engine is not listed as using Ethanol-85 Fuel."); }
                        if (!isGas && vd.FuelEconomy[FuelType.Gasoline ] > 0) { this.LogError(hdr + "Vehicle specifies a Gasoline Fuel Economy, but the Engine is not listed as using Gasoline Fuel."); }
                        if (!isDsl && vd.FuelEconomy[FuelType.Diesel   ] > 0) { this.LogError(hdr + "Vehicle specifies a Diesel Fuel Economy, but the Engine is not listed as using Diesel Fuel."); }
                        if (!isE85 && vd.FuelEconomy[FuelType.Ethanol85] > 0) { this.LogError(hdr + "Vehicle specifies an Ethanol-85 Fuel Economy, but the Engine is not listed as using Ethanol-85 Fuel."); }
                    }
                    if (this.HasErrors) { return; }
                    mfrs.Add(mc[i]);
                    mc[i].Index = mfrIndex++;
                    indMinYear = Math.Min(minYear, indMinYear);
                    indMaxYear = Math.Max(maxYear, indMaxYear);
                }
            }
            indMinYear = Math.Max(mfrWsMinYear, indMinYear);
            indMaxYear = Math.Min(mfrWsMaxYear, indMaxYear);
            this._ind = new Industry(mfrs, indMinYear, indMaxYear);
            this._ind.UpdateMinMaxFpCw();
        }
        List<Manufacturer> LoadManufacturers(int index, out int mfrMinYear, out int mfrMaxYear)
        {
            this.ActivateWorksheet(index);
            int rows = this.Rows;
            int cols = this.Columns;
            string[] names = {   
                                 "Manufacturer Code",                       
                                 "Manufacturer Name",                       
                                 "Cost Allocation Strategy",                
                                 "Discount Rate",                           
                                 "Payback Period",                          
                                 "Payback Period (After Compliance)",       
                                 "Optimize",                                
                                 "Willingness to Pay CAFE Fines",           
                                 "Available Domestic Auto Credits (mpg)",   
                                 "Available Imported Auto Credits (mpg)",   
                                 "Available Light Truck Credits (mpg)",     
                                 "Credits Apply to Baseline"                
                             };
            int[] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < cols; i++)
            {
                string column0 = this.GetString(0, i);
                string column1 = this.GetString(1, i);
                int column1_int = Interaction.GetInt32(column1);
                if      (Interaction.StringCompare(column1, names[ 0], true)) { indexes[0] = i; }
                else if (Interaction.StringCompare(column1, names[ 1], true)) { indexes[1] = i; }
                else if (Interaction.StringCompare(column1, names[ 2], true)) { indexes[2] = i; }
                else if (Interaction.StringCompare(column1, names[ 3], true)) { indexes[3] = i; }
                else if (Interaction.StringCompare(column1, names[ 4], true)) { indexes[4] = i; }
                else if (Interaction.StringCompare(column1, names[ 5], true)) { indexes[5] = i; }
                else if (Interaction.StringCompare(column1, names[ 6], true)) { indexes[6] = i; }
                else if (Interaction.StringCompare(column0, names[ 7], true) && ModelYear.IsValid(column1_int)) { indexes[ 7] = i; }
                else if (Interaction.StringCompare(column0, names[ 8], true) && ModelYear.IsValid(column1_int)) { indexes[ 8] = i; }
                else if (Interaction.StringCompare(column0, names[ 9], true) && ModelYear.IsValid(column1_int)) { indexes[ 9] = i; }
                else if (Interaction.StringCompare(column0, names[10], true) && ModelYear.IsValid(column1_int)) { indexes[10] = i; }
                else if (Interaction.StringCompare(column0, names[11], true) || Interaction.StringCompare(column1, names[11], true)) { indexes[11] = i; }
            }
            mfrMinYear = mfrMaxYear = 0;
            if (!this.VerifyIndexes(this.SheetNames[index] + " worksheet", indexes, names)) { return null; }
            this.GetMinMaxYears(1, indexes[7], out mfrMinYear, out mfrMaxYear);
            List<Manufacturer> mc = new List<Manufacturer>(32);
            for (int i = 2; i < rows; i++)
            {
                Manufacturer.CDescription md = this.LoadManufacturer(i, indexes, mfrMinYear, mfrMaxYear);
                if (md != null) { mc.Add(new Manufacturer(md)); }
            }
            return mc;
        }
        Manufacturer.CDescription LoadManufacturer(int row, int[] indexes, int minYear, int maxYear)
        {
            int    mfrCode = this.GetInt32 (row, indexes[0]);
            string mfrName = this.GetString(row, indexes[1]);
            if (mfrCode <= 0 || mfrName == string.Empty) { return null; }
            Manufacturer.CDescription md = new Manufacturer.CDescription();
            md.Code = mfrCode;
            md.Name = mfrName;
            md.ProperName = Interaction.GetTitleCase(mfrName, 4);
            md.CostAllocationStrategy =  this.GetInt32 (row, indexes[ 2]);
            md.DiscountRate           =  this.GetDouble(row, indexes[ 3]);
            md.PaybackPeriod          =  this.GetDouble(row, indexes[ 4]);
            md.PaybackPeriod_OC       =  this.GetDouble(row, indexes[ 5]);
            md.Optimize               = (this.GetChar  (row, indexes[ 6]) == 'Y');
            md.CreditsApplyToBaseline = (this.GetChar  (row, indexes[11]) == 'Y');
            md.WillingToPayFines      = new bool  [maxYear - ModelYear.MinYear + 1];
            md.AvailableCreditsDA     = new double[maxYear - ModelYear.MinYear + 1];
            md.AvailableCreditsIA     = new double[maxYear - ModelYear.MinYear + 1];
            md.AvailableCreditsLT     = new double[maxYear - ModelYear.MinYear + 1];
            int offset = minYear - ModelYear.MinYear;
            for (int i = 0; i < maxYear - minYear + 1; i++)
            {
                md.WillingToPayFines [offset + i] = this.GetString(row, indexes[ 7] + i).ToUpper().Equals("Y");
                md.AvailableCreditsDA[offset + i] = this.GetDouble(row, indexes[ 8] + i);
                md.AvailableCreditsIA[offset + i] = this.GetDouble(row, indexes[ 9] + i);
                md.AvailableCreditsLT[offset + i] = this.GetDouble(row, indexes[10] + i);
            }
            return md;
        }
        List<Vehicle>[] LoadVehicles(int index, List<Manufacturer> mfrs)
        {
            this.ActivateWorksheet(index);
            int rows = this.Rows;
            int cols = this.Columns;
            string[] namesMain = new string[] {
                "Sales", "MSRP", "Price",           
                "Vehicle Code"                ,     
                "Manufacturer"                ,     
                "Model"                       ,     
                "Nameplate"                   ,     
                "Engine Code"                 ,     
                "Transmission Code"           ,     
                "Origin"                      ,     
                "Fuel Economy (Gasoline)"     ,     
                "Fuel Economy (Diesel)"       ,     
                "Fuel Economy (Ethanol-85)"   ,     
                "Fuel Economy (Electricity)"  ,     
                "Fuel Economy (Hydrogen)"     ,     
                "Fuel Share (Gasoline)"       ,     
                "Fuel Share (Diesel)"         ,     
                "Fuel Share (Ethanol-85)"     ,     
                "Fuel Share (Electricity)"    ,     
                "Fuel Share (Hydrogen)"       ,     
                "Regulatory Class"            ,     
                "Technology Class"            ,     
                "Safety Class"                ,     
                "Market Segment"              ,     
                "SmallCar"                    ,     
                "MLCar"                       ,     
                "Truck"                       ,     
                "Van"                         ,     
                "SUV"                         ,     
                "AWD"                         ,     
                "Prestige"                    ,     
                "MM"                          ,     
                "ASC1"                        ,     
                "ASC2"                        ,     
                "Class"                       ,     
                "Style"                       ,     
                "Structure"                   ,     
                "Drive"                       ,     
                "Wheelbase"                   ,     
                "Footprint"                   ,     
                "Curb Weight"                 ,     
                "GVWR"                        ,     
                "Seating (max)"               ,     
                "Fuel Capacity"               ,     
                "Type of Hybrid/Electric Vehicle",  
                "Electric Power"              ,     
                "Electric Range"              ,     
                "Refresh Years"               ,     
                "Redesign Years"              ,     
                "Percent of 2 DR Cars"        ,     
                "Employment Hours per Vehicle"};    
            string[] namesTech = new string[] {
                "EPS"  , "IACC1", "IACC2", "MHEV", "ISG", "SHEV1", "SHEV1_2", "SHEV2", "PHEV1", "PHEV2",
                "EV1"  , "EV2"  , "EV3"  , "EV4" , "FCV", "MR1"  , "MR2"    , "MR3"  , "MR4"  , "MR5"  ,
                "ROLL1", "ROLL2", "ROLL3", "LDB" , "SAX", "AERO1", "AERO2"};
            int[] indexesMain = new int[namesMain.Length];
            int[] indexesTech = new int[namesTech.Length];
            for (int i = 0; i < indexesMain.Length; i++) { indexesMain[i] = -1; }
            for (int i = 0; i < indexesTech.Length; i++) { indexesTech[i] = -1; }
            for (int i = 0; i < cols; i++)
            {
                string column = this.GetString(1, i);
                string smpCol = this.GetString(0, i);      
                for (int j = 0; j < namesMain.Length; j++)
                {
                    if (indexesMain[j] == -1)
                    {
                        if      (Interaction.StringCompare(smpCol, namesMain[j], true)) { indexesMain[j] = i; break; }
                        else if (Interaction.StringCompare(column, namesMain[j], true)) { indexesMain[j] = i; break; }
                    }
                }
                for (int j = 0; j < namesTech.Length; j++)
                {
                    if (indexesTech[j] == -1 && Interaction.StringCompare(column, namesTech[j], true)) { indexesTech[j] = i; break; }
                }
            }
            if      (indexesMain[1] == -1 && indexesMain[2] != -1) { indexesMain[1] = indexesMain[2]; }
            else if (indexesMain[1] != -1 && indexesMain[2] == -1) { indexesMain[2] = indexesMain[1]; }
            if (!this.VerifyIndexes(this.SheetNames[index] + " worksheet", indexesMain, namesMain)) { return null; }
            if (!this.VerifyIndexes(this.SheetNames[index] + " worksheet", indexesTech, namesTech)) { return null; }
            int salesMinYear, salesMaxYear, msrpMinYear, msrpMaxYear, priceMinYear, priceMaxYear;
            this.GetMinMaxYears(1, indexesMain[0], out salesMinYear, out salesMaxYear);
            this.GetMinMaxYears(1, indexesMain[1], out msrpMinYear , out msrpMaxYear );
            this.GetMinMaxYears(1, indexesMain[2], out priceMinYear, out priceMaxYear);
            List<Vehicle>[] vc = new List<Vehicle>[mfrs.Count];
            for (int i = 0; i < mfrs.Count; i++)
            {
                vc[i] = new List<Vehicle>(32);
            }
            for (int i = 2; i < rows; i++)
            {   
                int mfrIndex;
                Vehicle.CDescription vd = this.LoadVehicle(i, indexesMain, indexesTech, salesMinYear, salesMaxYear, msrpMinYear,
                    msrpMaxYear, priceMinYear, priceMaxYear, mfrs, out mfrIndex);
                if (vd != null && mfrIndex != -1) { vc[mfrIndex].Add(new Vehicle(vd)); }
            }
            return vc;
        }
        Vehicle.CDescription LoadVehicle(int row, int[] indexesMain, int[] indexesTech, int salesMinYear, int salesMaxYear,
            int msrpMinYear, int msrpMaxYear, int priceMinYear, int priceMaxYear, List<Manufacturer> mfrs, out int mfrIndex)
        {
            mfrIndex = -1;
            double curbWeigth = this.GetDouble(row, indexesMain[40]);   
            Vehicle.CDescription vd = new Vehicle.CDescription(curbWeigth);
            string vehCode = this.GetString(row, indexesMain[3]);
            string mfrName = this.GetString(row, indexesMain[4]);
            string model   = this.GetString(row, indexesMain[5]);
            if (vehCode == string.Empty && mfrName == string.Empty && model == string.Empty) { return null; }
            if (vehCode == "X") { return null; }
            vd.Code                              = Interaction.GetInt32(vehCode); 
            vd.Manufacturer                      = mfrName;                     
            vd.Model                             = model;                       
            vd.Nameplate                         = this.GetString(row, indexesMain[ 6]);
            vd.EngineCode                        = this.GetInt32 (row, indexesMain[ 7]);
            vd.TransmissionCode                  = this.GetInt32 (row, indexesMain[ 8]);
            vd.Origin                            = this.GetChar  (row, indexesMain[ 9]);
            vd.FuelEconomy[FuelType.Gasoline   ] = this.GetDouble(row, indexesMain[10]);
            vd.FuelEconomy[FuelType.Diesel     ] = this.GetDouble(row, indexesMain[11]);
            vd.FuelEconomy[FuelType.Ethanol85  ] = this.GetDouble(row, indexesMain[12]);
            vd.FuelEconomy[FuelType.Electricity] = this.GetDouble(row, indexesMain[13]);
            vd.FuelEconomy[FuelType.Hydrogen   ] = this.GetDouble(row, indexesMain[14]);
            vd.FuelEconomy2                      = vd.FuelEconomy;                      
            vd.FuelShare  [FuelType.Gasoline   ] = this.GetDouble(row, indexesMain[15]);
            vd.FuelShare  [FuelType.Diesel     ] = this.GetDouble(row, indexesMain[16]);
            vd.FuelShare  [FuelType.Ethanol85  ] = this.GetDouble(row, indexesMain[17]);
            vd.FuelShare  [FuelType.Electricity] = this.GetDouble(row, indexesMain[18]);
            vd.FuelShare  [FuelType.Hydrogen   ] = this.GetDouble(row, indexesMain[19]);
            vd.RegulatoryIndicator               = this.GetString(row, indexesMain[20]);
            vd.TechnologyClass                   = this.GetString(row, indexesMain[21]);
            vd.SafetyClass                       = this.GetString(row, indexesMain[22]);
            vd.MarketSegment                     = this.GetInt32 (row, indexesMain[23]);
            vd.SmallCar                          = this.GetInt32 (row, indexesMain[24]);
            vd.MLCar                             = this.GetInt32 (row, indexesMain[25]);
            vd.Truck                             = this.GetInt32 (row, indexesMain[26]);
            vd.Van                               = this.GetInt32 (row, indexesMain[27]);
            vd.SUV                               = this.GetInt32 (row, indexesMain[28]);
            vd.AWD                               = this.GetInt32 (row, indexesMain[29]);
            vd.Prestige                          = this.GetInt32 (row, indexesMain[30]);
            vd.MM                                = this.GetDouble(row, indexesMain[31]);
            vd.ASC1                              = this.GetDouble(row, indexesMain[32]);
            vd.ASC2                              = this.GetDouble(row, indexesMain[33]);
            vd.Class                             = this.GetString(row, indexesMain[34]);
            vd.Style                             = this.GetString(row, indexesMain[35]);
            vd.Structure                         = this.GetString(row, indexesMain[36]);
            vd.Drive                             = this.GetChar  (row, indexesMain[37]);
            vd.Wheelbase                         = this.GetDouble(row, indexesMain[38]);
            vd.Footprint                         = this.GetDouble(row, indexesMain[39]);
            vd.CurbWeight                        = curbWeigth;                  
            vd.GVWR                              = this.GetDouble(row, indexesMain[41]);
            vd.Seating                           = this.GetInt32 (row, indexesMain[42]);
            vd.FuelCapacity                      = this.GetDouble(row, indexesMain[43]);
            vd.HybridType                        = this.GetString(row, indexesMain[44]);
            vd.ElectricPower                     = this.GetDouble(row, indexesMain[45]);
            vd.ElectricRange                     = this.GetDouble(row, indexesMain[46]);
            vd.RefreshYears                      = this.LoadVehicle_RefreshRedesign(this.GetString(row, indexesMain[47]).Split(','));
            vd.RedesignYears                     = this.LoadVehicle_RefreshRedesign(this.GetString(row, indexesMain[48]).Split(','));
            vd.Percent2Dr                        = this.GetDouble(row, indexesMain[49]);
            vd.EmploymentHours                   = this.GetDouble(row, indexesMain[50]);
            vd.Sales = this.LoadVehicleSMP(row, indexesMain[0], salesMinYear, salesMaxYear);
            vd.Msrp  = this.LoadVehicleSMP(row, indexesMain[1], msrpMinYear , msrpMaxYear );
            vd.Price = this.LoadVehicleSMP(row, indexesMain[2], priceMinYear, priceMaxYear);
            int maxYr = (int)Math.Max(Math.Max(priceMaxYear, msrpMaxYear), salesMaxYear) - ModelYear.MinYear + 1;
            if (vd.Sales.Length < maxYr) { double[] sales = new double[maxYr]; vd.Sales.CopyTo(sales, 0); vd.Sales = sales; }
            if (vd.Msrp .Length < maxYr) { double[] msrp  = new double[maxYr]; vd.Msrp .CopyTo(msrp , 0); vd.Msrp  = msrp ; }
            if (vd.Price.Length < maxYr) { double[] price = new double[maxYr]; vd.Price.CopyTo(price, 0); vd.Price = price; }
            for (int i = 0; i < maxYr; i++)
            {
                if      (vd.Msrp[i] == 0 && vd.Price[i] != 0) { vd.Msrp[i] = vd.Price[i]; }
                else if (vd.Msrp[i] != 0 && vd.Price[i] == 0) { vd.Price[i] = vd.Msrp[i]; }
            }
            const int techCount = 27;   
            this.LoadTechApplicability(row, indexesTech, 0, techCount, out vd.UsedTechnologies, out vd.AvailableTechnologies, TechnologyIndexes.EPS);
            bool isStrongHEV = false;
            bool isElcOrHgnOnly = (vd.FuelEconomy.PrimaryFuel == FuelType.Electricity || vd.FuelEconomy.PrimaryFuel == FuelType.Hydrogen);
            for (int i = 0; i < vd.UsedTechnologies.Length; i++)
            {
                if (TechnologyIndexes.IsHybridization(vd.UsedTechnologies[i])) { isStrongHEV = true; }
                if (TechnologyIndexes.IsPureElectric (vd.UsedTechnologies[i]) ||
                    TechnologyIndexes.IsHydrogenBased(vd.UsedTechnologies[i])) { isElcOrHgnOnly = true; }
            }
            string hdr = "Vehicle (R:" + (row + 1).ToString("0000") + "):  ";
            mfrIndex = this.GetManufacturerIndex(mfrs, vd.Manufacturer);
            if (vd.Code < 1) { this.LogError(hdr + "Vehicle Code is not valid."); }
            if (mfrIndex == -1) { this.LogError(hdr + "Manufacturer is not specified or not valid."); }
            if (vd.Model == string.Empty) { this.LogError(hdr + "Model is not specified."); }
            if (vd.Nameplate == string.Empty) { this.LogError(hdr + "Nameplate is not specified."); }
            if (vd.EngineCode < 1 && !isElcOrHgnOnly) { this.LogError(hdr + "Engine Code is not valid."); }
            if (vd.EngineCode > 0 && isElcOrHgnOnly) { this.LogError(hdr + "Engine Code cannot be specified for pure-Electric or Hydrogen-based vehicles."); }
            if (vd.TransmissionCode < 1 && !isElcOrHgnOnly && !isStrongHEV) { this.LogError(hdr + "Transmission Code is not valid."); }
            if (vd.TransmissionCode > 0 && isElcOrHgnOnly) { this.LogError(hdr + "Transmission Code cannot be specified for pure-Electric or Hydrogen-based vehicles."); }
            if (vd.TransmissionCode > 0 && isStrongHEV) { this.LogError(hdr + "Transmission Code cannot be specified for Strong Hybrid or Plug-In Hybrid vehicles."); }
            if (vd.Origin != 'I' && vd.Origin != 'D') { this.LogError(hdr + "Origin is not valid."); }
            if (!vd.FuelEconomy.IsValid) { this.LogError(hdr + "Fuel Economy is not valid (the Fuel Economy value cannot be negative)."); }
            if ( vd.FuelEconomy.IsEmpty) { this.LogError(hdr + "Fuel Economy is not valid (at least one Fuel Economy value must be specified)."); }
            if (!vd.FuelShare  .IsValid) { this.LogError(hdr + "Fuel Share is not valid (the Fuel Share value cannot be negative)."); }
            if ( vd.FuelShare  .IsEmpty) { this.LogError(hdr + "Fuel Share is not valid (at least one Fuel Share value must be specified)."); }
            if ( vd.FuelShare.Gasoline + vd.FuelShare.Diesel + vd.FuelShare.Ethanol85 + vd.FuelShare.Electricity +
                 vd.FuelShare.Hydrogen != 1) { this.LogError(hdr + "Fuel Share is not valid (Fuel Shares for all fuel types must add up to 100%)."); }
            if ((vd.FuelEconomy.Gasoline    == 0 && vd.FuelShare.Gasoline    != 0) ||
                (vd.FuelEconomy.Gasoline    != 0 && vd.FuelShare.Gasoline    == 0)) { this.LogError(hdr + "Gasoline Fuel Economy and Fuel Share do not correspond."); }
            if ((vd.FuelEconomy.Diesel      == 0 && vd.FuelShare.Diesel      != 0) ||
                (vd.FuelEconomy.Diesel      != 0 && vd.FuelShare.Diesel      == 0)) { this.LogError(hdr + "Diesel Fuel Economy and Fuel Share do not correspond."); }
            if ((vd.FuelEconomy.Ethanol85   == 0 && vd.FuelShare.Ethanol85   != 0) ||
                (vd.FuelEconomy.Ethanol85   != 0 && vd.FuelShare.Ethanol85   == 0)) { this.LogError(hdr + "Ethanol85 Fuel Economy and Fuel Share do not correspond."); }
            if ((vd.FuelEconomy.Electricity == 0 && vd.FuelShare.Electricity != 0) ||
                (vd.FuelEconomy.Electricity != 0 && vd.FuelShare.Electricity == 0)) { this.LogError(hdr + "Electricity Fuel Economy and Fuel Share do not correspond."); }
            if ((vd.FuelEconomy.Hydrogen    == 0 && vd.FuelShare.Hydrogen    != 0) ||
                (vd.FuelEconomy.Hydrogen    != 0 && vd.FuelShare.Hydrogen    == 0)) { this.LogError(hdr + "Hydrogen Fuel Economy and Fuel Share do not correspond."); }
            if (vd.RegulatoryIndicator != "PC" && vd.RegulatoryIndicator != "LT") { this.LogError(hdr + "Regulatory Indicator is not valid."); }
            if (!Interaction.StringCompareAny(vd.TechnologyClass, new string[] {"Subcompact PC", "Subcompact Perf. PC", "Compact PC",
                "Compact Perf. PC", "Midsize PC", "Midsize Perf. PC", "Large PC", "Large Perf. PC", "Minivan LT", "Small LT",
                "Midsize LT", "Large LT"}, true)) { this.LogError(hdr + "Technology Class is not valid."); }
            if (!Interaction.StringCompareAny(vd.SafetyClass, new string[] { "PC", "LT", "CM" }, true)) { this.LogError(hdr + "Safety Class is not valid."); }
            if (vd.MarketSegment < 0 || vd.MarketSegment > 21) { this.LogError(hdr + "Market Segment is not valid."); }
            if (vd.SmallCar != 0 && vd.SmallCar != 1) { this.LogError(hdr + "SmallCar must be 0 or 1."); }
            if (vd.MLCar != 0 && vd.MLCar != 1) { this.LogError(hdr + "MLCar must be 0 or 1."); }
            if (vd.Truck != 0 && vd.Truck != 1) { this.LogError(hdr + "Truck must be 0 or 1."); }
            if (vd.Van != 0 && vd.Van != 1) { this.LogError(hdr + "Van must be 0 or 1."); }
            if (vd.SUV != 0 && vd.SUV != 1) { this.LogError(hdr + "SUV must be 0 or 1."); }
            if (vd.AWD != 0 && vd.AWD != 1) { this.LogError(hdr + "AWD must be 0 or 1."); }
            if (vd.Prestige != 0 && vd.Prestige != 1) { this.LogError(hdr + "Prestige must be 0 or 1."); }
            if (vd.Style == string.Empty) { this.LogError(hdr + "Style is not specified."); }
            else if (!Interaction.StringCompareAny(vd.Style, new string[] { "Convertible", "Coupe", "Hatchback", "Sedan", "Wagon", "Minivan", "Van", "Sport Utility", "Pickup" }, false)) { this.LogError(hdr + "Style is not valid."); }
            if (vd.Structure == string.Empty) { this.LogError(hdr + "Structure is not specified."); }
            if (!Interaction.StringCompareAny(vd.Drive.ToString(), new string[] {"F", "R", "2", "4", "A"}, false)) { this.LogError(hdr + "Drive is not specified."); }
            if (vd.Wheelbase    < 1) { this.LogError(hdr + "Wheelbase is not valid."); }
            if (vd.Footprint    < 1) { this.LogError(hdr + "Footprint is not valid."); }
            if (vd.CurbWeight   < 1) { this.LogError(hdr + "Curb Weight is not valid."); }
            if (vd.GVWR         < 1) { this.LogError(hdr + "GVWR is not valid."); }
            if (vd.Seating      < 1) { this.LogError(hdr + "Seating (Max) is not valid."); }
            if (vd.FuelCapacity < 1 && (vd.FuelShare.Gasoline > 0 || vd.FuelShare.Diesel > 0 || vd.FuelShare.Ethanol85 > 0)) { this.LogError(hdr + "Fuel Capacity is not valid."); }
            if (vd.FuelShare.Electricity > 0 && vd.ElectricPower < 1) { this.LogError(hdr + "Electric Power has to be specified for electric vehicles."); }
            if (vd.FuelShare.Electricity > 0 && vd.ElectricRange < 1) { this.LogError(hdr + "Electric Range has to be specified for electric vehicles."); }
            bool foundSlMsrp = false;
            for (int i = 0; i < maxYr; i++)
            {
                if (vd.Sales[i] > 0 && vd.Msrp[i] > 0) { foundSlMsrp = true; break; }
            }
            if (!foundSlMsrp) { this.LogError(hdr + "No Sales and MSRP found for the same year."); }
            return vd;
        }
        int[] LoadVehicle_RefreshRedesign(string[] years)
        {
            int count = years.Length;
            int[] iYears = new int[count];
            int iCount = 0;
            for (int i = 0; i < count; i++)
            {
                int year = Interaction.GetInt32(years[i]);
                if (year > 1900)
                {   
                    bool duplicate = false;
                    for (int j = 0; j < iCount; j++)
                    {
                        if (iYears[j] == year) { duplicate = true; break; }
                    }
                    if (!duplicate) { iYears[iCount++] = year; }
                }
            }
            int[] actualYears = new int[iCount];
            Array.Copy(iYears, actualYears, iCount);
            Array.Sort(actualYears);
            return actualYears;
        }
        double[] LoadVehicleSMP(int row, int column, int minYear, int maxYear)
        {
            double[] values = new double[maxYear - ModelYear.MinYear + 1];
            int years = maxYear - minYear + 1;
            int offset = minYear - ModelYear.MinYear;
            for (int i = 0; i < years; i++)
            {
                values[i + offset] = this.GetDouble(row, i + column);
            }
            return values;
        }
        List<Engine>[] LoadEngines(int index, List<Manufacturer> mfrs)
        {
            this.ActivateWorksheet(index);
            int rows = this.Rows;
            int cols = this.Columns;
            string[] names = {   
                                 "Engine Code"            ,     
                                 "Manufacturer"           ,     
                                 "Configuration"          ,     
                                 "Fuel"                   ,     
                                 "Engine Oil Viscosity"   ,     
                                 "Cycle"                  ,     
                                 "Fuel Delivery System"   ,     
                                 "Aspiration"             ,     
                                 "Valvetrain Design"      ,     
                                 "Valve Actuation/Timing" ,     
                                 "Valve Lift"             ,     
                                 "Cylinders"              ,     
                                 "Valves/Cylinder"        ,     
                                 "Deactivation"           ,     
                                 "Displacement"           ,     
                                 "Max. Horsepower"        ,     
                                 "Max. Torque"            ,     
                                 "LUB1"     , "EFR1"     , "LUB2_EFR2", "CCPS"     , "DVVLS"    ,
                                 "DEACS"    , "ICP"      , "DCP"      , "DVVLD"    , "CVVL"     ,
                                 "DEACD"    , "SGDI"     , "DEACO"    , "VVA"      , "SGDIO"    ,
                                 "TRBDS1_SD", "TRBDS1_MD", "TRBDS1_LD", "TRBDS2_SD", "TRBDS2_MD",
                                 "TRBDS2_LD", "CEGR1_SD" , "CEGR1_MD" , "CEGR1_LD" , "CEGR2_SD" ,
                                 "CEGR2_MD" , "CEGR2_LD" , "ADSL_SD"  , "ADSL_MD"  , "ADSL_LD"
                             };
            int[] indexes = new int[names.Length];
            for (int i = 0; i < names.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < cols; i++)
            {
                string column = this.GetString(1, i);
                for (int j = 0; j < names.Length; j++)
                {
                    if (indexes[j] == -1 && Interaction.StringCompare(column, names[j], true)) { indexes[j] = i; break; }
                }
            }
            if (!this.VerifyIndexes(this.SheetNames[index] + " worksheet", indexes, names)) { return null; }
            List<Engine>[] ec = new List<Engine>[mfrs.Count];
            for (int i = 0; i < mfrs.Count; i++)
            {
                ec[i] = new List<Engine>(32);
            }
            for (int i = 2; i < rows; i++)
            {   
                int mfrIndex;
                Engine.CDescription ed = this.LoadEngine(i, indexes, mfrs, out mfrIndex);
                if (ed != null && mfrIndex != -1) { ec[mfrIndex].Add(new Engine(ed)); }
            }
            return ec;
        }
        Engine.CDescription LoadEngine(int row, int[] indexes, List<Manufacturer> mfrs, out int mfrIndex)
        {
            mfrIndex = -1;
            Engine.CDescription ed = new Engine.CDescription();
            string engCode = this.GetString(row, indexes[0]);
            string mfrName = this.GetString(row, indexes[1]);
            if (engCode == string.Empty && mfrName == string.Empty) { return null; }
            if (engCode == "X") { return null; }
            ed.Code                 = Interaction.GetInt32(engCode); 
            ed.Manufacturer         = mfrName;                 
            ed.Configuration        = this.GetString(row, indexes[ 2]);
            string fuel             = this.GetString(row, indexes[ 3]);
            ed.Fuel                 = (fuel == "G") ? FuelType.Gasoline : (fuel == "D") ? FuelType.Diesel : (fuel == "E85") ? FuelType.Ethanol85 :
                                      (fuel == "G+E85") ? FuelType.Gasoline | FuelType.Ethanol85 : FuelType.None;
            ed.EngineOilViscosity   = this.GetString(row, indexes[ 4]);
            ed.Cycle                = this.GetChar  (row, indexes[ 5]);
            ed.FuelSystem           = this.GetString(row, indexes[ 6]);
            ed.Aspiration           = this.GetString(row, indexes[ 7]);
            ed.ValvetrainDesign     = this.GetString(row, indexes[ 8]);
            ed.ValveActuationTiming = this.GetString(row, indexes[ 9]);
            ed.ValveLift            = this.GetString(row, indexes[10]);
            ed.Cylinders            = this.GetInt32 (row, indexes[11]);
            ed.ValvesPerCylinder    = this.GetInt32 (row, indexes[12]);
            ed.Deactivation         = this.GetDouble(row, indexes[13]); if (ed.Deactivation == 0 && this.GetString(row, indexes[13]) == "Y") { ed.Deactivation = 0.3; }
            ed.Displacement         = this.GetDouble(row, indexes[14]);
            ed.Horsepower           = this.GetDouble(row, indexes[15]);
            ed.Torque               = this.GetDouble(row, indexes[16]);
            const int techCount = 30;   
            this.LoadTechApplicability(row, indexes, 17, techCount, out ed.UsedTechnologies, out ed.AvailableTechnologies, TechnologyIndexes.LUB1);
            string hdr = "Engine (R:" + (row + 1).ToString("0000") + "):  ";
            mfrIndex = this.GetManufacturerIndex(mfrs, ed.Manufacturer);
            if (ed.Code < 1) { this.LogError(hdr + "Engine Code is not valid."); }
            if (mfrIndex == -1) { this.LogError(hdr + "Manufacturer is not specified or not valid."); }
            if (ed.Configuration == string.Empty) { this.LogError(hdr + "Configuration is not specified."); }
            if (ed.Fuel == FuelType.None) { this.LogError(hdr + "Engine Fuel is not specified or not valid."); }
            if (ed.EngineOilViscosity == string.Empty) { this.LogError(hdr + "Engine Oil Viscosity is not specified."); }
            if (ed.Cycle == '\0') { this.LogError(hdr + "Cycle is not specified."); }
            if (ed.FuelSystem == string.Empty) { this.LogError(hdr + "Fuel Delivery System is not specified."); }
            if (!Interaction.StringCompareAny(ed.Aspiration, new string[] { "NA", "S", "T", "T1", "T2", "T3", "T4", "T5", "T6" }, false)) { this.LogError(hdr + "Aspiration is not valid."); }
            if (ed.Configuration != "R")
            {
                if (!Interaction.StringCompareAny(ed.ValvetrainDesign, new string[] { "OHV", "SOHC", "DOHC" }, false)) { this.LogError(hdr + "Valvetrain Design is not valid."); }
                if (ed.ValveActuationTiming == string.Empty) { this.LogError(hdr + "Valve Actuation/Timing is not specified."); }
                if (ed.ValveLift == string.Empty) { this.LogError(hdr + "Valve Lift is not specified."); }
                if (ed.Cylinders < 1) { this.LogError(hdr + "Number of Cylinders is too small."); }
                if (ed.ValvesPerCylinder < 1) { this.LogError(hdr + "Number of Valves/Cylinders is too small."); }
            }
            if (ed.Deactivation < 0) { this.LogError(hdr + "Deactivation is not valid."); }
            if (ed.Displacement < 0) { this.LogError(hdr + "Displacement is not valid."); }
            if (ed.Horsepower < 0) { this.LogError(hdr + "Horsepower is not valid."); }
            if (ed.Torque < 0) { this.LogError(hdr + "Torque is not valid."); }
            return ed;
        }
        double GetCompressionRatio(string value)
        {
            double ratio = 0;
            if (value != "")
            {
                int index = value.IndexOf(":");
                if (index != -1)
                {   
                    ratio = Interaction.GetDouble(value.Substring(0, index)) / Interaction.GetDouble(value.Substring(index + 1));
                }
                else
                {   
                    ratio = Interaction.GetDouble(value);
                }
            }
            return ratio;
        }
        List<Transmission>[] LoadTransmissions(int index, List<Manufacturer> mfrs)
        {
            this.ActivateWorksheet(index);
            int rows = this.Rows;
            int cols = this.Columns;
            string[] names = {   
                                 "Transmission Code"      ,     
                                 "Manufacturer"           ,     
                                 "Type"                   ,     
                                 "Number of Forward Gears",     
                                 "6MAN", "HETRANSM", "IATC", "NAUTO", "DCT", "8SPD", "HETRANS", "SHFTOPT"
                             };
            int[] indexes = new int[names.Length];
            for (int i = 0; i < names.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < cols; i++)
            {
                string column = this.GetString(1, i);
                for (int j = 0; j < names.Length; j++)
                {
                    if (indexes[j] == -1 && Interaction.StringCompare(column, names[j], true)) { indexes[j] = i; break; }
                }
            }
            if (!this.VerifyIndexes(this.SheetNames[index] + " worksheet", indexes, names)) { return null; }
            List<Transmission>[] tc = new List<Transmission>[mfrs.Count];
            for (int i = 0; i < mfrs.Count; i++)
            {
                tc[i] = new List<Transmission>(32);
            }
            for (int i = 2; i < rows; i++)
            {   
                int mfrIndex;
                Transmission.CDescription td = this.LoadTransmission(i, indexes, mfrs, out mfrIndex);
                if (td != null && mfrIndex != -1) { tc[mfrIndex].Add(new Transmission(td)); }
            }
            return tc;
        }
        Transmission.CDescription LoadTransmission(int row, int[] indexes, List<Manufacturer> mfrs, out int mfrIndex)
        {
            mfrIndex = -1;
            Transmission.CDescription td = new Transmission.CDescription();
            string trnCode = this.GetString(row, indexes[0]);
            string mfrName = this.GetString(row, indexes[1]);
            if (trnCode == string.Empty && mfrName == string.Empty) { return null; }
            if (trnCode == "X") { return null; }
            td.Code            = Interaction.GetInt32(trnCode); 
            td.Manufacturer    = mfrName;                 
            td.Type            = this.GetString(row, indexes[2]);
            td.NumForwardGears = this.GetInt32 (row, indexes[3]);
            const int techCount = 8;    
            this.LoadTechApplicability(row, indexes, 4, techCount, out td.UsedTechnologies, out td.AvailableTechnologies, TechnologyIndexes.MAN6);
            string hdr = "Transmission (R:" + (row + 1).ToString("0000") + "):  ";
            mfrIndex = this.GetManufacturerIndex(mfrs, td.Manufacturer);
            if (td.Code < 1) { this.LogError(hdr + "Transmission Code is not valid."); }
            if (mfrIndex == -1) { this.LogError(hdr + "Manufacturer is not specified or not valid."); }
            if (!Interaction.StringCompareAny(td.Type, new string[] { "A", "M", "CVT", "AMT", "DCT" }, false)) { this.LogError(hdr + "Type is not valid."); }
            if (td.Type != "CVT" && td.NumForwardGears < 3) { this.LogError(hdr + "Number of Forward Gears is too small."); }
            return td;
        }
        void LoadTechApplicability(int row, int[] indexes, int colIndex,
            int techCount, out int[] usedTechnologies, out int[] availableTechnologies, int firstTechIndex)
        {
            int[] techAp = new int[techCount];
            int techUsedCount = 0, techAvailCount = 0;
            for (int i = 0; i < techCount; i++)
            {
                techAp[i] = this.ParseTechApplicability(this.GetString(row, indexes[colIndex + i]));
                if      (techAp[i] == 1) { techUsedCount++; }
                else if (techAp[i] == 0) { techAvailCount++; }
            }
            usedTechnologies      = new int[techUsedCount];
            availableTechnologies = new int[techAvailCount];
            for (int i = 0, ut = 0, at = 0; i < techCount; i++)
            {
                if      (techAp[i] == 1) { usedTechnologies     [ut++] = firstTechIndex + i; }
                else if (techAp[i] == 0) { availableTechnologies[at++] = firstTechIndex + i; }
            }
        }
        int ParseTechApplicability(string value)
        {
            return (value == "SKIP" || value == "_SKIP") ? -1 : (value == "USED" || value == "_USED") ? 1 : 0;
        }
        int GetManufacturerIndex(List<Manufacturer> mfrs, string mfrName)
        {
            for (int i = 0; i < mfrs.Count; i++)
            {
                if (Interaction.StringCompare(mfrs[i].Description.Name, mfrName, false)) { return i; }
            }
            return -1;
        }
        #endregion
        #region 
        internal Industry _ind;
        #endregion
    }
}

