#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.XlLib;
using System.Collections.Generic;
#endregion
namespace Volpe.Cafe.IO.InputParsers
{
    sealed class XlParametersParser : XlParser
    {
        #region 
        internal XlParametersParser(string path, string password) : base(path, password, true) { }
        #endregion
        #region 
        protected override void ParseInternal()
        {
            this._value = new Parameters();
            string[] sheets = this.SheetNames;
            string[] names = new string[] {
                "Vehicle Age Data"  , "Forecast Data", "Fuel Economy Data", "Economic Values", "Fuel Properties",       
                "Upstream Emissions", "Safety Values", "Vehicle Choice"   , "HH Characteristics"};                      
            int[] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < sheets.Length; i++)
            {
                for (int j = 0; j < names.Length; j++)
                {
                    if (indexes[j] == -1 && Interaction.StringCompare(sheets[i], names[j], true)) { indexes[j] = i; break; }
                }
            }
            if (indexes[7] == -1) { indexes[7] = -2; }  
            if (indexes[8] == -1) { indexes[8] = -2; }  
            if (!this.VerifyIndexes("Parameters workbook", indexes, names)) { return; }
            double[] fuelMix;
            this.ParseVehicleAgeData   (indexes[0]);                            
            this.ParseForecastData     (indexes[1]);                            
            this.ParseFuelEconomyData  (indexes[2]);                            
            this.ParseEconomicValues   (indexes[3]);                            
            this.ParseFuelProperties   (indexes[4], out fuelMix);               
            this.ParseUpstreamEmissions(indexes[5],     fuelMix);               
            this.ParseSafetyValues     (indexes[6]);                            
            if (indexes[7] != -2) { this.ParseVehicleChoice    (indexes[7]); }  
            if (indexes[8] != -2) { this.ParseHHCharacteristics(indexes[8]); }  
        }
        void ParseVehicleAgeData(int sheetIndex)
        {
            this.ActivateWorksheet(sheetIndex, new XlCell(6, 2), new XlCell(5 + Interaction.CY, 11));
            for (int i = 0; i < Interaction.CY; i++)
            {   
                this._value._vehicleAgeData.SurvivalCars   [i] = this.GetDouble(i, 0);
                this._value._vehicleAgeData.SurvivalVans   [i] = this.GetDouble(i, 1);
                this._value._vehicleAgeData.SurvivalSUVs   [i] = this.GetDouble(i, 2);
                this._value._vehicleAgeData.SurvivalPickups[i] = this.GetDouble(i, 3);
                this._value._vehicleAgeData.SurvivalAllLTs [i] = this.GetDouble(i, 4);
                this._value._vehicleAgeData.MilesCars      [i] = this.GetDouble(i, 5);
                this._value._vehicleAgeData.MilesVans      [i] = this.GetDouble(i, 6);
                this._value._vehicleAgeData.MilesSUVs      [i] = this.GetDouble(i, 7);
                this._value._vehicleAgeData.MilesPickups   [i] = this.GetDouble(i, 8);
                this._value._vehicleAgeData.MilesAllLTs    [i] = this.GetDouble(i, 9);
            }
        }
        void ParseForecastData(int sheetIndex)
        {
            int minYear = Parameters.ForecastDataValues.MinYear;
            int maxYear = Parameters.ForecastDataValues.MaxYear;
            this.ActivateWorksheet(sheetIndex, new XlCell(6, 2), new XlCell(6 + maxYear - minYear, 31));
            int length = maxYear - minYear + 1;
            for (int i = 0; i < length; i++)
            {   
                this._value._forecastData.SetData(minYear + i, FuelType.Gasoline   , this.GetDouble(i,  0), this.GetDouble(i,  1), this.GetDouble(i,  2), this.GetDouble(i,  3), this.GetDouble(i, 20), this.GetDouble(i, 21));
                this._value._forecastData.SetData(minYear + i, FuelType.Diesel     , this.GetDouble(i,  4), this.GetDouble(i,  5), this.GetDouble(i,  6), this.GetDouble(i,  7), this.GetDouble(i, 22), this.GetDouble(i, 23));
                this._value._forecastData.SetData(minYear + i, FuelType.Ethanol85  , this.GetDouble(i,  8), this.GetDouble(i,  9), this.GetDouble(i, 10), this.GetDouble(i, 11), this.GetDouble(i, 24), this.GetDouble(i, 25));
                this._value._forecastData.SetData(minYear + i, FuelType.Electricity, this.GetDouble(i, 12), this.GetDouble(i, 13), this.GetDouble(i, 14), this.GetDouble(i, 15), this.GetDouble(i, 26), this.GetDouble(i, 27));
                this._value._forecastData.SetData(minYear + i, FuelType.Hydrogen   , this.GetDouble(i, 16), this.GetDouble(i, 17), this.GetDouble(i, 18), this.GetDouble(i, 19), this.GetDouble(i, 28), this.GetDouble(i, 29));
            }
        }
        void ParseFuelEconomyData(int sheetIndex)
        {
            int minYear = Parameters.FuelEconomyDataValues.MinYear;
            int maxYear = Parameters.FuelEconomyDataValues.MaxYear;
            this.ActivateWorksheet(sheetIndex, new XlCell(6, 2), new XlCell(6 + maxYear - minYear, 21));
            int length = maxYear - minYear + 1;
            for (int i = 0; i < length; i++)
            {   
                this._value._fuelEconomyData.CarsFuelEconomy[i][FuelType.Gasoline   ] = this.GetDouble(i,  0);
                this._value._fuelEconomyData.CarsFuelEconomy[i][FuelType.Diesel     ] = this.GetDouble(i,  1);
                this._value._fuelEconomyData.CarsFuelEconomy[i][FuelType.Ethanol85  ] = this.GetDouble(i,  2);
                this._value._fuelEconomyData.CarsFuelEconomy[i][FuelType.Electricity] = this.GetDouble(i,  3);
                this._value._fuelEconomyData.CarsFuelEconomy[i][FuelType.Hydrogen   ] = this.GetDouble(i,  4);
                this._value._fuelEconomyData.CarsFuelShare  [i][FuelType.Gasoline   ] = this.GetDouble(i,  5);
                this._value._fuelEconomyData.CarsFuelShare  [i][FuelType.Diesel     ] = this.GetDouble(i,  6);
                this._value._fuelEconomyData.CarsFuelShare  [i][FuelType.Ethanol85  ] = this.GetDouble(i,  7);
                this._value._fuelEconomyData.CarsFuelShare  [i][FuelType.Electricity] = this.GetDouble(i,  8);
                this._value._fuelEconomyData.CarsFuelShare  [i][FuelType.Hydrogen   ] = this.GetDouble(i,  9);
                this._value._fuelEconomyData.TrksFuelEconomy[i][FuelType.Gasoline   ] = this.GetDouble(i, 10);
                this._value._fuelEconomyData.TrksFuelEconomy[i][FuelType.Diesel     ] = this.GetDouble(i, 11);
                this._value._fuelEconomyData.TrksFuelEconomy[i][FuelType.Ethanol85  ] = this.GetDouble(i, 12);
                this._value._fuelEconomyData.TrksFuelEconomy[i][FuelType.Electricity] = this.GetDouble(i, 13);
                this._value._fuelEconomyData.TrksFuelEconomy[i][FuelType.Hydrogen   ] = this.GetDouble(i, 14);
                this._value._fuelEconomyData.TrksFuelShare  [i][FuelType.Gasoline   ] = this.GetDouble(i, 15);
                this._value._fuelEconomyData.TrksFuelShare  [i][FuelType.Diesel     ] = this.GetDouble(i, 16);
                this._value._fuelEconomyData.TrksFuelShare  [i][FuelType.Ethanol85  ] = this.GetDouble(i, 17);
                this._value._fuelEconomyData.TrksFuelShare  [i][FuelType.Electricity] = this.GetDouble(i, 18);
                this._value._fuelEconomyData.TrksFuelShare  [i][FuelType.Hydrogen   ] = this.GetDouble(i, 19);
            }
        }
        void ParseEconomicValues(int sheetIndex)
        {
            this.ActivateWorksheet(sheetIndex);
            this._value._economicData.ReboundEffect          = this.GetDouble( 2, 1);
            string[] discRates = this.GetString(3, 1).Split(',');
            int discRatesCount = discRates.Length;
            this._value._economicData.DiscountRates = new double[discRatesCount];
            for (int i = 0; i < discRatesCount; i++)
            {
                this._value._economicData.DiscountRates[i] = Interaction.GetDouble(discRates[i]);
            }
            this._value._economicData.PrivateDiscountRate    = this.GetDouble( 4, 1);
            this._value._economicData.Kf                     = this.GetDouble( 5, 1);
            this._value._economicData.VehicleTravelTimeValue = this.GetDouble( 6, 1);
            this._value._economicData.MonopsonyCosts         = this.GetDouble( 8, 1);
            this._value._economicData.PriceShockCosts        = this.GetDouble( 9, 1);
            this._value._economicData.MilitarySecurityCosts  = this.GetDouble(10, 1);
            this._value._economicData.TotalEconomicCosts     = this.GetDouble(11, 1);
            this._value._economicData.TotalEconomicCostsBBL  = this.GetDouble(12, 1);
            this._value._economicData.CarCongestionCosts     = this.GetDouble(15, 1);
            this._value._economicData.CarAccidentCosts       = this.GetDouble(16, 1);
            this._value._economicData.CarNoiseCosts          = this.GetDouble(17, 1);
            this._value._economicData.TrkCongestionCosts     = this.GetDouble(19, 1);
            this._value._economicData.TrkAccidentCosts       = this.GetDouble(20, 1);
            this._value._economicData.TrkNoiseCosts          = this.GetDouble(21, 1);
            this._value._economicData.CODamageCosts          = this.GetDouble(23, 1);
            this._value._economicData.VOCDamageCosts         = this.GetDouble(24, 1);
            this._value._economicData.NOXDamageCosts         = this.GetDouble(25, 1);
            this._value._economicData.PMDamageCosts          = this.GetDouble(26, 1);
            this._value._economicData.SOXDamageCosts         = this.GetDouble(27, 1);
            this._value._economicData.CH4DamageCostScalar    = this.GetDouble(28, 1);
            this._value._economicData.N2ODamageCostScalar    = this.GetDouble(29, 1);
            this._value._economicData.VMTBaseYear            = this.GetInt32 (31, 1);
            this._value._economicData.VMTGrowthRateLow       = this.GetDouble(32, 1);
            this._value._economicData.VMTGrowthRateAverage   = this.GetDouble(33, 1);
            this._value._economicData.VMTGrowthRateHigh      = this.GetDouble(34, 1);
            this._value._economicData.CO2DiscRateLow         = this.GetDouble(36, 1);
            this._value._economicData.CO2DiscRateAverage     = this.GetDouble(37, 1);
            this._value._economicData.CO2DiscRateHigh        = this.GetDouble(38, 1);
            this._value._economicData.CO2DiscRateVeryHigh    = this.GetDouble(39, 1);
            const int co2Len = 71;      
            this._value._economicData.CO2DamageCostsLow      = new double[co2Len];
            this._value._economicData.CO2DamageCostsAverage  = new double[co2Len];
            this._value._economicData.CO2DamageCostsHigh     = new double[co2Len];
            this._value._economicData.CO2DamageCostsVeryHigh = new double[co2Len];
            for (int i = 3, rows = this.Rows; i < rows; i++)
            {
                int co2YearIndex = this.GetInt32(i, 3) - ModelYear.MinYear;
                if (co2YearIndex < 0 || co2YearIndex > 70) { continue; }    
                this._value._economicData.CO2DamageCostsLow     [co2YearIndex] = this.GetDouble(i, 4);
                this._value._economicData.CO2DamageCostsAverage [co2YearIndex] = this.GetDouble(i, 5);
                this._value._economicData.CO2DamageCostsHigh    [co2YearIndex] = this.GetDouble(i, 6);
                this._value._economicData.CO2DamageCostsVeryHigh[co2YearIndex] = this.GetDouble(i, 7);
            }
            this._value._economicData.TestVsOnRoadGap[FuelType.Gasoline   ] = this.GetDouble(41, 1);
            this._value._economicData.TestVsOnRoadGap[FuelType.Diesel     ] = this.GetDouble(42, 1);
            this._value._economicData.TestVsOnRoadGap[FuelType.Ethanol85  ] = this.GetDouble(43, 1);
            this._value._economicData.TestVsOnRoadGap[FuelType.Electricity] = this.GetDouble(44, 1);
            this._value._economicData.TestVsOnRoadGap[FuelType.Hydrogen   ] = this.GetDouble(45, 1);
            this._value._economicData.RefuelTime     [FuelType.Gasoline   ] = this.GetDouble(47, 1);
            this._value._economicData.RefuelTime     [FuelType.Diesel     ] = this.GetDouble(48, 1);
            this._value._economicData.RefuelTime     [FuelType.Ethanol85  ] = this.GetDouble(49, 1);
            this._value._economicData.RefuelTime     [FuelType.Electricity] = this.GetDouble(50, 1);
            this._value._economicData.RefuelTime     [FuelType.Hydrogen   ] = this.GetDouble(51, 1);
            this._value._economicData.RefuelTankVolume                      = this.GetDouble(52, 1);
            this._value._economicData.TaxesAndFees                          = this.GetDouble(54, 1);
            this._value._economicData.Financing                             = this.GetDouble(55, 1);
            this._value._economicData.Insurance                             = this.GetDouble(56, 1);
            this._value._economicData.RelativeValueLoss                     = this.GetDouble(57, 1);
            this._value._economicData.ResaleValue                           = this.GetDouble(58, 1);
        }
        void ParseFuelProperties(int sheetIndex, out double[] fuelMix)
        {
            this.ActivateWorksheet(sheetIndex, new XlCell(4, 2), new XlCell(19, 6));
            fuelMix = new double[5];
            fuelMix[0] = this.GetDouble(0, 0);
            fuelMix[1] = this.GetDouble(1, 0);
            fuelMix[2] = this.GetDouble(2, 0);
            fuelMix[3] = this.GetDouble(3, 0);
            fuelMix[4] = this.GetDouble(4, 0);
            this._value._fuelProperties.ConventionalGasolineShare = fuelMix[0];
            this.ParseFuelValue(0, 1, fuelMix, ref this._value._fuelProperties.EnergyDensity);
            this.ParseFuelValue(0, 2, fuelMix, ref this._value._fuelProperties.MassDensity  );
            this.ParseFuelValue(0, 3, fuelMix, ref this._value._fuelProperties.CarbonContent);
            this.ParseFuelValue(0, 4, fuelMix, ref this._value._fuelProperties.SOXRate      );
            for (int i = 0; i < 5; i++)
            {
                FuelType fuelType = (FuelType)(Math.Pow(2, i));
                this._value._fuelProperties.ShareOfFuelSavingsLeadingToLowerImports           [fuelType] = this.GetDouble(12, i);
                this._value._fuelProperties.ShareOfFuelSavingsLeadingToReducedDomesticRefining[fuelType] = this.GetDouble(13, i);
                this._value._fuelProperties.ShareOfReducedDomesticRefiningFromDomesticCrude   [fuelType] = this.GetDouble(14, i);
                this._value._fuelProperties.ShareOfReducedDomesticRefiningFromImportedCrude   [fuelType] = this.GetDouble(15, i);
            }
        }
        void ParseUpstreamEmissions(int sheetIndex, double[] fuelMix)
        {
            this.ActivateWorksheet(sheetIndex, new XlCell(5, 7), new XlCell(124, 7));
            this.ParseFuelValue(  0, 0, fuelMix, ref this._value._upstreamEmissions.CO );
            this.ParseFuelValue(  8, 0, fuelMix, ref this._value._upstreamEmissions.VOC);
            this.ParseFuelValue( 16, 0, fuelMix, ref this._value._upstreamEmissions.NOX);
            this.ParseFuelValue( 24, 0, fuelMix, ref this._value._upstreamEmissions.SOX);
            this.ParseFuelValue( 32, 0, fuelMix, ref this._value._upstreamEmissions.PM );
            this.ParseFuelValue( 40, 0, fuelMix, ref this._value._upstreamEmissions.CO2);
            this.ParseFuelValue( 48, 0, fuelMix, ref this._value._upstreamEmissions.Acetaldehyde);
            this.ParseFuelValue( 56, 0, fuelMix, ref this._value._upstreamEmissions.Acrolein);
            this.ParseFuelValue( 64, 0, fuelMix, ref this._value._upstreamEmissions.Benzene);
            this.ParseFuelValue( 72, 0, fuelMix, ref this._value._upstreamEmissions.Butadiene);
            this.ParseFuelValue( 80, 0, fuelMix, ref this._value._upstreamEmissions.Formaldehyde);
            this.ParseFuelValue( 88, 0, fuelMix, ref this._value._upstreamEmissions.CH4);
            this.ParseFuelValue( 96, 0, fuelMix, ref this._value._upstreamEmissions.DieselPM);
            this.ParseFuelValue(104, 0, fuelMix, ref this._value._upstreamEmissions.MTBE);
            this.ParseFuelValue(112, 0, fuelMix, ref this._value._upstreamEmissions.N2O);
        }
        void ParseFuelValue(int startRow, int col, double[] fuelMix, ref FuelValue value)
        {
            value[FuelType.Gasoline   ] = this.GetDouble(startRow + 0, col) * fuelMix[0] +
                this.GetDouble(startRow + 1, col) * fuelMix[1] +
                this.GetDouble(startRow + 2, col) * fuelMix[2];
            value[FuelType.Diesel     ] = this.GetDouble(startRow + 3, col) * fuelMix[3] +
                this.GetDouble(startRow + 4, col) * fuelMix[4];
            value[FuelType.Ethanol85  ] = this.GetDouble(startRow + 5, col);
            value[FuelType.Electricity] = this.GetDouble(startRow + 6, col);
            value[FuelType.Hydrogen   ] = this.GetDouble(startRow + 7, col);
        }
        void ParseSafetyValues(int sheetIndex)
        {
            this.ActivateWorksheet(sheetIndex);
            this._value._safetyData.PC.Threshold   = this.GetDouble( 2, 1);
            this._value._safetyData.LT.Threshold   = this.GetDouble( 3, 1);
            this._value._safetyData.CM.Threshold   = this.GetDouble( 4, 1);
            this._value._safetyData.PC.SmallEffect = this.GetDouble( 7, 1);
            this._value._safetyData.PC.SmallBase   = this.GetDouble( 7, 2);
            this._value._safetyData.PC.SmallFMVSS  = this.GetDouble( 7, 3);
            this._value._safetyData.PC.LargeEffect = this.GetDouble( 8, 1);
            this._value._safetyData.PC.LargeBase   = this.GetDouble( 8, 2);
            this._value._safetyData.PC.LargeFMVSS  = this.GetDouble( 8, 3);
            this._value._safetyData.LT.SmallEffect = this.GetDouble( 9, 1);
            this._value._safetyData.LT.SmallBase   = this.GetDouble( 9, 2);
            this._value._safetyData.LT.SmallFMVSS  = this.GetDouble( 9, 3);
            this._value._safetyData.LT.LargeEffect = this.GetDouble(10, 1);
            this._value._safetyData.LT.LargeBase   = this.GetDouble(10, 2);
            this._value._safetyData.LT.LargeFMVSS  = this.GetDouble(10, 3);
            this._value._safetyData.CM.SmallEffect = this.GetDouble(11, 1);
            this._value._safetyData.CM.SmallBase   = this.GetDouble(11, 2);
            this._value._safetyData.CM.SmallFMVSS  = this.GetDouble(11, 3);
            this._value._safetyData.CM.LargeEffect = this.GetDouble(12, 1);
            this._value._safetyData.CM.LargeBase   = this.GetDouble(12, 2);
            this._value._safetyData.CM.LargeFMVSS  = this.GetDouble(12, 3);
            this._value._safetyData.FatalityCosts  = this.GetDouble(15, 1);
        }
        void ParseVehicleChoice(int sheetIndex)
        {
            this.ActivateWorksheet(sheetIndex);
            int rows = this.Rows;
            int row = 3;
            int numCoefs = 0;
            while (row < rows && Interaction.IsNumeric(this.GetString(row++, 1))) { numCoefs++; }
            this._value._vehicleChoiceData.Mu   = new double[numCoefs];
            this._value._vehicleChoiceData.Beta = new double[numCoefs];
            for (int i = 0; i < numCoefs; i++)
            {
                this._value._vehicleChoiceData.Mu  [i] = this.GetDouble(3 + i, 1);
                this._value._vehicleChoiceData.Beta[i] = this.GetDouble(3 + i, 2);
            }
            row = 3;
            numCoefs = 0;
            while (row < rows && Interaction.IsNumeric(this.GetString(row++, 6))) { numCoefs++; }
            this._value._vehicleChoiceData.BetaNew = new double[numCoefs];
            for (int i = 0; i < numCoefs; i++)
            {
                this._value._vehicleChoiceData.BetaNew[i] = this.GetDouble(3 + i, 6);
            }
        }
        void ParseHHCharacteristics(int sheetIndex)
        {
            this.ActivateWorksheet(sheetIndex);
            int rows = this.Rows;
            int cols = this.Columns;
            string[] headers = new string[] {
                "HH_ID"        , "Urban"      , "HHSize4Plus", "NumAd2Plus", "NumWork2Plus",    
                "YKidLE15"     , "RetiredNK"  , "NonRNK"     , "College"   , "HHIncLT25K"  ,    
                "HHInc75to100K", "HHIncGE100K", "Income"     , "BuyUsedExp"};                   
            int[] headerIndexes = new int[headers.Length];
            for (int i = 0; i < headers.Length; i++) { headerIndexes[i] = -1; }
            List<int> weightIndexes = new List<int>();  
            List<int> weightYears   = new List<int>();  
            for (int i = 0; i < cols; i++)
            {
                string column = this.GetString(0, i);
                if (Interaction.IsNumeric(column) && Interaction.GetInt32(column) >= ModelYear.MinYear)
                {   
                    weightIndexes.Add(i);
                    weightYears.Add(Interaction.GetInt32(column));
                }
                else
                {   
                    for (int j = 0; j < headers.Length; j++)
                    {
                        if (headerIndexes[j] == -1 && Interaction.StringCompare(column, headers[j], true)) { headerIndexes[j] = i; break; }
                    }
                }
            }
            if (!this.VerifyIndexes(this.SheetNames[sheetIndex] + " worksheet", headerIndexes, headers)) { return; }
            int maxYear = 0;
            for (int i = 0; i < weightYears.Count; i++)
            {
                maxYear = Math.Max(maxYear, weightYears[i]);
            }
            int yearCount = maxYear - ModelYear.MinYear + 1;
            for (int i = 1; i < rows; i++)
            {
                Parameters.HouseholdDataValues.Household h = new Parameters.HouseholdDataValues.Household();
                h.ID            = this.GetInt32 (i, headerIndexes[ 0]);
                h.Urban         = this.GetInt32 (i, headerIndexes[ 1]);
                h.Rural         = 1 - h.Urban;
                h.HHSize4Plus   = this.GetInt32 (i, headerIndexes[ 2]);
                h.NumAd2Plus    = this.GetInt32 (i, headerIndexes[ 3]);
                h.NumWork2Plus  = this.GetInt32 (i, headerIndexes[ 4]);
                h.YKidLE15      = this.GetInt32 (i, headerIndexes[ 5]);
                h.RetiredNK     = this.GetInt32 (i, headerIndexes[ 6]);
                h.NonRNK        = this.GetInt32 (i, headerIndexes[ 7]);
                h.College       = this.GetInt32 (i, headerIndexes[ 8]);
                h.HHIncLT25K    = this.GetInt32 (i, headerIndexes[ 9]);
                h.HHInc75to100K = this.GetInt32 (i, headerIndexes[10]);
                h.HHIncGE100K   = this.GetInt32 (i, headerIndexes[11]);
                h.Income        = this.GetDouble(i, headerIndexes[12]);
                h.BuyUsedExp    = this.GetDouble(i, headerIndexes[13]);
                h.Weights = new double[yearCount];
                for (int j = 0; j < weightIndexes.Count; j++)
                {
                    h.Weights[weightYears[j] - ModelYear.MinYear] = this.GetDouble(i, weightIndexes[j]);
                }
                this._value._householdData.AddHousehold(h);
            } 
        }
        #endregion
        #region 
        internal Parameters _value;
        #endregion
    }
}

