#region << Using Directives >>
using System;
using System.Collections.Generic;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
#endregion
namespace Volpe.Cafe.IO.InputParsers
{
    sealed class XlScenariosParser : XlParser
    {
        #region 
        internal XlScenariosParser(string path, string password) : base(path, password, true) { }
        #endregion
        #region 
        protected override void ParseInternal()
        {
            this._scenList = new List<Scenario>();
            string[] sheets = this.SheetNames;
            for (int i = 0; i < sheets.Length; i++)
            {
                if (sheets[i].ToUpper().StartsWith("SCEN_"))
                {   
                    this._scenList.Add(this.LoadScenario(i, this._scenList.Count));
                }
            }
        }
        Scenario LoadScenario(int wsIndex, int scenIndex)
        {
            this.ActivateWorksheet(wsIndex);
            int rows = this.Rows;
            int cols = this.Columns;
            Scenario scen = new Scenario(scenIndex);
            int minYear, maxYear;
            this.GetMinMaxYears(4, 2, out minYear, out maxYear);
            int years  = maxYear - minYear + 1;
            int offset = minYear - ModelYear.MinYear;
            int numCoef = 0;
            while (this.GetString(11 + numCoef, 1) != string.Empty) { numCoef++; }
            int arrLen = maxYear - ModelYear.MinYear + 1;
            scen._description    = this.GetString(2, 2);
            scen._multiFuelCAFE  = new int   [         arrLen];
            scen._pcFunctionType = new int   [         arrLen];
            scen._pcCoefficients = new double[numCoef, arrLen];
            scen._pcMinStndMpg   = new double[         arrLen];
            scen._pcMinStndPct   = new double[         arrLen];
            scen._ltFunctionType = new int   [         arrLen];
            scen._ltCoefficients = new double[numCoef, arrLen];
            scen._ltMinStndMpg   = new double[         arrLen];
            scen._ltMinStndPct   = new double[         arrLen];
            scen._includeAC      = new bool  [         arrLen];
            scen._pcACAdj        = new double[         arrLen];
            scen._pcACCost       = new double[         arrLen];
            scen._ltACAdj        = new double[         arrLen];
            scen._ltACCost       = new double[         arrLen];
            scen._pcOffCycleCap  = new double[         arrLen];
            scen._ltOffCycleCap  = new double[         arrLen];
            scen._taxCreditPHEV  = new double[         arrLen];
            scen._taxCreditEV    = new double[         arrLen];
            for (int i = 0; i < years; i++)
            {
                scen._multiFuelCAFE [i + offset] =             this.GetInt32 ( 5              , i + 2);
                scen._pcFunctionType[i + offset] =             this.GetInt32 ( 8              , i + 2);
                scen._pcMinStndMpg  [i + offset] =             this.GetDouble(13 +     numCoef, i + 2);
                scen._pcMinStndPct  [i + offset] =             this.GetDouble(14 +     numCoef, i + 2);
                scen._ltFunctionType[i + offset] =             this.GetInt32 (17 +     numCoef, i + 2);
                scen._ltMinStndMpg  [i + offset] =             this.GetDouble(22 + 2 * numCoef, i + 2);
                scen._ltMinStndPct  [i + offset] =             this.GetDouble(23 + 2 * numCoef, i + 2);
                scen._includeAC     [i + offset] =             this.GetString(26 + 2 * numCoef, i + 2).Equals("Y");
                scen._pcACAdj       [i + offset] = Math.Max(0, this.GetDouble(29 + 2 * numCoef, i + 2));
                scen._pcACCost      [i + offset] =             this.GetDouble(30 + 2 * numCoef, i + 2);
                scen._ltACAdj       [i + offset] = Math.Max(0, this.GetDouble(33 + 2 * numCoef, i + 2));
                scen._ltACCost      [i + offset] =             this.GetDouble(34 + 2 * numCoef, i + 2);
                scen._pcOffCycleCap [i + offset] = Math.Max(0, this.GetDouble(37 + 2 * numCoef, i + 2));
                scen._ltOffCycleCap [i + offset] = Math.Max(0, this.GetDouble(38 + 2 * numCoef, i + 2));
                scen._taxCreditPHEV [i + offset] =             this.GetDouble(41 + 2 * numCoef, i + 2);
                scen._taxCreditEV   [i + offset] =             this.GetDouble(42 + 2 * numCoef, i + 2);
                for (int j = 0; j < numCoef; j++)
                {
                    scen._pcCoefficients[j, i + offset] = this.GetDouble(11 +           j, i + 2);
                    scen._ltCoefficients[j, i + offset] = this.GetDouble(20 + numCoef + j, i + 2);
                }
            }
            scen._options = this.LoadScenarioOptions(wsIndex, scenIndex, years);
            return scen;
        }
        Scenario.ScenarioOptions LoadScenarioOptions(int wsIndex, int scenIndex, int numYears)
        {
            int colOffset = 4 + numYears;
            bool   preferredAlternative = this.GetBool  (4, colOffset);
            string regulatoryMerging    = this.GetString(5, colOffset);
            return new Scenario.ScenarioOptions(preferredAlternative, regulatoryMerging);
        }
        #endregion
        #region 
        internal List<Scenario> _scenList;
        #endregion
    }
}

