#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
#endregion
namespace Volpe.Cafe.IO.InputParsers
{
    sealed class XlTailpipeEmissionRatesParser : XlParser
    {
        #region 
        internal XlTailpipeEmissionRatesParser(string path, string password) : base(path, password, true) { }
        #endregion
        #region 
        protected override void ParseInternal()
        {
            this._value = new TailpipeEmissions();
            string[] sheets = this.SheetNames;
            string[] names = new string[] {"CO", "VOC", "NOX", "PM", "CH4", "N2O"};
            int[] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < sheets.Length; i++)
            {
                for (int j = 0; j < names.Length; j++)
                {
                    if (Interaction.StringCompare(sheets[i], names[j], true)) { indexes[j] = i; break; }
                }
            }
            if (!this.VerifyIndexes("Tailpipe Emissions workbook", indexes, names)) { return; }
            this.ParseEmissionRatesSheet(indexes[0], this._value._co );
            this.ParseEmissionRatesSheet(indexes[1], this._value._voc);
            this.ParseEmissionRatesSheet(indexes[2], this._value._nox);
            this.ParseEmissionRatesSheet(indexes[3], this._value._pm );
            this.ParseEmissionRatesSheet(indexes[4], this._value._ch4);
            this.ParseEmissionRatesSheet(indexes[5], this._value._n2o);
        }
        void ParseEmissionRatesSheet(int index, TailpipeEmissions.Emissions em)
        {
            this.ActivateWorksheet(index);
            for (int i = 4; i < 3 + Interaction.CY; i++)
            {
                em.Gasoline   .LDV [i - 4] = this.GetDouble(i,  1);
                em.Gasoline   .LDT1[i - 4] = this.GetDouble(i,  2);
                em.Gasoline   .LDT2[i - 4] = this.GetDouble(i,  3);
                em.Gasoline   .HDV [i - 4] = this.GetDouble(i,  4);
                em.GasolineRfg.LDV [i - 4] = this.GetDouble(i,  5);
                em.GasolineRfg.LDT1[i - 4] = this.GetDouble(i,  6);
                em.GasolineRfg.LDT2[i - 4] = this.GetDouble(i,  7);
                em.GasolineRfg.HDV [i - 4] = this.GetDouble(i,  8);
                em.Diesel     .LDV [i - 4] = this.GetDouble(i,  9);
                em.Diesel     .LDT1[i - 4] = this.GetDouble(i, 10);
                em.Diesel     .LDT2[i - 4] = this.GetDouble(i, 11);
                em.Diesel     .HDV [i - 4] = this.GetDouble(i, 12);
                em.Ethanol85  .LDV [i - 4] = this.GetDouble(i, 13);
                em.Ethanol85  .LDT1[i - 4] = this.GetDouble(i, 14);
                em.Ethanol85  .LDT2[i - 4] = this.GetDouble(i, 15);
                em.Ethanol85  .HDV [i - 4] = this.GetDouble(i, 16);
            }
        }
        #endregion
        #region 
        internal TailpipeEmissions _value;
        #endregion
    }
}

