#region << Using Directives >>
using System;
using System.Collections.Generic;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using TI = Volpe.Cafe.TechnologyIndexes;
#endregion
namespace Volpe.Cafe.IO.InputParsers
{
    sealed class XlTechnologiesParser : XlParser
    {
        #region 
        internal XlTechnologiesParser(string path, string password) : base(path, password, true) { }
        #endregion
        #region 
        protected override void ParseInternal()
        {
            this._techList = new List<Technology>(TechnologyIndexes.TechnologyCount + 1);
            string[] sheets = this.SheetNames;
            string[] names =
                new string[] {                                                                                          
                                 "Technologies", "SubcompactPC" , "SubcompactPerfPC", "CompactPC"  , "CompactPerfPC",   
                                 "MidsizePC"   , "MidsizePerfPC", "LargePC"         , "LargePerfPC", "MinivanLT"    ,   
                                 "SmallLT"     , "MidsizeLT"    , "LargeLT"         , "FCSynergies", "CostSynergies"    
                             };
            int[] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < sheets.Length; i++)
            {
                string sheet = sheets[i].Trim().ToUpper();
                for (int j = 0; j < names.Length; j++)
                {
                    if (Interaction.StringCompare(sheet, names[j], true)) { indexes[j] = i; break; }
                }
            }
            if (!this.VerifyIndexes("Technologies workbook", indexes, names, 0, 13)) { return; }
            this.LoadTechnologies(indexes[0]);
            for (int i = 0, techClassCount = (int)TechnologyClass.Max; i < techClassCount; i++)
            {
                this.LoadTechnologyClasses(indexes[i + 1], TI.TechClasses[i]);
            }
            TechnologySynergiesTable[] imprvSynergies = null, costSynergies = null;
            if (indexes[13] != -1) { imprvSynergies = this.LoadSynergies(indexes[13], TI.TechClasses); }
            if (indexes[14] != -1) { costSynergies  = this.LoadSynergies(indexes[14], TI.TechClasses); }
            for (int i = 0; i < TechnologyIndexes.TechnologyCount; i++)
            {
                if (indexes[13] != -1) { this._techList[i].ImprvSynergies = imprvSynergies[i]; }
                if (indexes[14] != -1) { this._techList[i].CostSynergies  = costSynergies[i]; }
            }
        }
        void LoadTechnologies(int wsIndex)
        {
            this.ActivateWorksheet(wsIndex);
            int rows = this.Rows;
            int cols = this.Columns;
            string[] names = {"Technology", "Abbr", "TechType", "Complexity", "FC Variation", "Cost Variation", "Phase-In Values",  
                              "ERC-1", "ERC-2", "ERC-3", "ERC-4", "ERC-5", "ERC-6", "ERC-7", "ERC-8", "ERC-9", "ERC-10"};           
            int   [] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            int phaseInCount = 0;   
            for (int i = 0; i < cols; i++)
            {
                string column = this.GetString(1, i);
                if (indexes[6] == -1 && Interaction.StringStartsWith(column, "PV-", false))
                {   
                    indexes[6] = i;    
                    while (Interaction.StringStartsWith(column, "PV-", false))
                    {   
                        phaseInCount++; i++;
                        if (i == cols) { break; }
                        column = this.GetString(1, i);
                    }
                    i--;
                }
                else
                {   
                    for (int j = 0; j < names.Length; j++)
                    {
                        if (Interaction.StringCompare(column, names[j], true)) { indexes[j] = i; break; }
                    }
                }
            }
            if (!this.VerifyIndexes(this.SheetNames[wsIndex] + " worksheet", indexes, names)) { return; }
            Technology[] techs = new Technology[TechnologyIndexes.TechnologyCount];
            for (int i = 2; i < rows; i++)
            {   
                Technology tech = new Technology();
                tech.Name  = this.GetString(i, indexes[0]);
                tech.Abbr  = this.GetString(i, indexes[1]);
                tech.Index = TechnologyIndexes.GetIndex(tech.Abbr);     
                if (tech.Index == -1) { this.LogError("On the main Technologies worksheet, the technology with abbreviation \"" + tech.Abbr + "\" is undefined."); return; }
                string techType = this.GetString(i, indexes[2]);
                tech.Type =
                    (techType == "ENGMOD") ? TechnologyType.Engine :
                    (techType == "TRMOD" ) ? TechnologyType.Transmission :
                    (techType == "ELEC"  ) ? TechnologyType.Electrification :
                    (techType == "MR"    ) ? TechnologyType.MR :
                    (techType == "ROLL"  ) ? TechnologyType.ROLL :
                    (techType == "DLR"   ) ? TechnologyType.DLR :
                    (techType == "AERO"  ) ? TechnologyType.Aerodynamics : TechnologyType.Other;
                char complexity    = this.GetChar(i, indexes[3]);
                tech.Complexity    = (complexity == 'L') ? TechnologyComplexity.Low    :
                                     (complexity == 'M') ? TechnologyComplexity.Medium :
                                     (complexity == 'H') ? TechnologyComplexity.High   : TechnologyComplexity.None;
                tech.FcVariation   = this.GetDouble(i, indexes[4]);
                tech.CostVariation = this.GetDouble(i, indexes[5]);
                tech.PhaseIn = new double[phaseInCount];
                for (int j = 0; j < phaseInCount; j++)
                {
                    tech.PhaseIn[j] = Math.Min(1.0, this.GetDouble(i, indexes[6] + j));
                }
                tech.ERPCTable = new double[10];
                for (int j = 0; j < 10; j++)
                {
                    tech.ERPCTable[j] = this.GetDouble(i, indexes[7 + j]);
                }
                techs[tech.Index] = tech;
            }
            this._techList.AddRange(techs);
        }
        void LoadTechnologyClasses(int wsIndex, TechnologyClass techClass)
        {
            this.ActivateWorksheet(wsIndex);
            int rows = this.Rows;
            int cols = this.Columns;
            string[] names = new string[] {                                                                         
                "Abbr"            , "Applicable"         , "Year Avail"         , "Year Retired", "Electric Range", 
                "Delta Weight (%)", "Delta Weight (lbs)" , "Loss Of Value"      , "FC"          , "FCG"           , 
                "FCG Share"       , "Off-Cycle Credit PC", "Off-Cycle Credit LT", "Cost Table"  , "Maint. Table"  , 
                "Repair Table"   };                                                                                 
            int[] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            int costTableCount = 0, maintTableCount = 0, repairTableCount = 0;  
            for (int i = 0; i < cols; i++)
            {
                string column = this.GetString(1, i);
                if (indexes[13] == -1 && Interaction.StringStartsWith(column, "COST 2", true) &&
                                         Interaction.StringCompare(this.GetString(0, i), names[13], false))
                {   
                    indexes[13] = i;    
                    while (Interaction.StringStartsWith(column, "COST 2", true))
                    {   
                        costTableCount++; i++;
                        if (i == cols) { break; }
                        column = this.GetString(1, i);
                    }
                }
                if (indexes[14] == -1 && Interaction.StringStartsWith(column, "MAINT. 2", true) &&
                                         Interaction.StringCompare(this.GetString(0, i), names[14], false))
                {   
                    indexes[14] = i;    
                    while (Interaction.StringStartsWith(column, "MAINT. 2", true))
                    {   
                        maintTableCount++; i++;
                        if (i == cols) { break; }
                        column = this.GetString(1, i);
                    }
                }
                if (indexes[15] == -1 && Interaction.StringStartsWith(column, "REPAIR 2", true) &&
                                         Interaction.StringCompare(this.GetString(0, i), names[15], false))
                {   
                    indexes[15] = i;    
                    while (Interaction.StringStartsWith(column, "REPAIR 2", true))
                    {   
                        repairTableCount++; i++;
                        if (i == cols) { break; }
                        column = this.GetString(1, i);
                    }
                }
                for (int j = 0; j < names.Length; j++)
                {
                    if (Interaction.StringCompare(column, names[j], true)) { indexes[j] = i; break; }
                }
            }
            if (!this.VerifyIndexes(this.SheetNames[wsIndex] + " worksheet", indexes, names)) { return; }
            int costTableMinYear = Interaction.GetInt32(
                Interaction.StringRemoveWhitespacePunctuationAndSpecialChars(this.GetString(1, indexes[13])).Replace("COST", ""));
            if (costTableMinYear < ModelYear.MinYear) { this.LogError("On the " + techClass.ToString() + " worksheet, the Cost Table section is not valid."); return; }
            int maintTableMinYear = Interaction.GetInt32(
                Interaction.StringRemoveWhitespacePunctuationAndSpecialChars(this.GetString(1, indexes[14])).Replace("MAINT", ""));
            if (maintTableMinYear < ModelYear.MinYear) { this.LogError("On the " + techClass.ToString() + " worksheet, the Maintance Table section is not valid."); return; }
            int repairTableMinYear = Interaction.GetInt32(
                Interaction.StringRemoveWhitespacePunctuationAndSpecialChars(this.GetString(1, indexes[15])).Replace("REPAIR", ""));
            if (repairTableMinYear < ModelYear.MinYear) { this.LogError("On the " + techClass.ToString() + " worksheet, the Repair Table section is not valid."); return; }
            for (int i = 2; i < rows; i++)
            {   
                TechnologyAttributesTable.TechnologyAttributes ta = new TechnologyAttributesTable.TechnologyAttributes();
                string abbr  = this.GetString(i, indexes[0]);
                int    index = TechnologyIndexes.GetIndex(abbr);    
                if (index == -1) { this.LogError("On the " + techClass.ToString() + " worksheet, the technology with abbreviation \"" + abbr + "\" is undefined."); return; }
                ta.TechnologyClass   = techClass;
                ta.Applicable       = this.GetBool  (i, indexes[ 1]);
                ta.YearAvailable    = this.GetInt32 (i, indexes[ 2]);
                ta.YearRetired      = this.GetInt32 (i, indexes[ 3]);
                ta.ElectricRange    = this.GetDouble(i, indexes[ 4]);
                ta.DeltaWeightPCT   = this.GetDouble(i, indexes[ 5]);
                ta.DeltaWeightLBS   = this.GetDouble(i, indexes[ 6]);
                ta.LossOfValue      = this.GetDouble(i, indexes[ 7]);
                ta.FC               = this.GetDouble(i, indexes[ 8]);
                ta.FCg              = this.GetDouble(i, indexes[ 9]);
                ta.FCgShare         = this.GetDouble(i, indexes[10]);
                ta.OffCycleCreditPC = this.GetDouble(i, indexes[11]);
                ta.OffCycleCreditLT = this.GetDouble(i, indexes[12]);
                int costTableStart = costTableMinYear - ModelYear.MinYear;
                ta.CostTable = new double[costTableStart + costTableCount];
                for (int j = 0; j < costTableCount; j++)
                {
                    ta.CostTable[costTableStart + j] = this.GetDouble(i, indexes[13] + j);
                }
                int maintTableStart = maintTableMinYear - ModelYear.MinYear;
                ta.MaintenanceCostTable = new double[maintTableStart + maintTableCount];
                for (int j = 0; j < maintTableCount; j++)
                {
                    ta.MaintenanceCostTable[maintTableStart + j] = this.GetDouble(i, indexes[14] + j);
                }
                int repairTableStart = repairTableMinYear - ModelYear.MinYear;
                ta.RepairCostTable = new double[repairTableStart + repairTableCount];
                for (int j = 0; j < repairTableCount; j++)
                {
                    ta.RepairCostTable[repairTableStart + j] = this.GetDouble(i, indexes[15] + j);
                }
                this._techList[index].Attributes[techClass] = ta;
            }
        }
        TechnologySynergiesTable[] LoadSynergies(int index, TechnologyClass[] classes)
        {
            this.ActivateWorksheet(index);
            int rows = this.Rows;
            int cols = this.Columns;
            string[] names = new string[] {                                                                         
                "Type"          , "Technology A"    , "Technology B", "Subcompact PC"   , "Subcompact Perf. PC",    
                "Compact PC"    , "Compact Perf. PC", "Midsize PC"  , "Midsize Perf. PC", "Large PC"           ,    
                "Large Perf. PC", "Minivan LT"      , "Small LT"    , "Midsize LT"      , "Large LT"           };   
            int[] indexes = new int[names.Length];
            for (int i = 0; i < indexes.Length; i++) { indexes[i] = -1; }
            for (int i = 0; i < cols; i++)
            {   
                string colName = this.GetString(1, i);
                for (int j = 0; j < names.Length; j++)
                {
                    if (Interaction.StringCompare(colName, names[j], true)) { indexes[j] = i; break; }
                }
            }
            int row = 2;    
            TechnologySynergiesTable[] synergies = new TechnologySynergiesTable[TechnologyIndexes.TechnologyCount];
            for (int i = 0; i < TechnologyIndexes.TechnologyCount; i++)
            {
                synergies[i] = new TechnologySynergiesTable();
            }
            do
            {   
                string techA = this.GetString(row, indexes[1]);
                if (techA == "") { break; }     
                string techB = this.GetString(row, indexes[2]);
                SynergyType type = SynergyType.None;
                if (indexes[0] != -1)
                {
                    string typeStr = this.GetString(row, indexes[0]).ToUpper();
                    type = (typeStr == "ACCOUNTING") ? SynergyType.Accounting :
                        (typeStr == "PHYSICAL") ? SynergyType.Physical : SynergyType.None;
                }
                int techAIndex = TechnologyIndexes.GetIndex(techA);
                int techBIndex = TechnologyIndexes.GetIndex(techB);
                if ((techAIndex >= 0) && (techBIndex >= 0))
                {   
                    for (int i = 3; i < indexes.Length; i++)
                    {
                        if (indexes[i] != -1)
                        {   
                            double synergy = this.GetDouble(row, indexes[i]);
                            synergies[techAIndex][classes[i - 3]].SetSynergy(techBIndex, synergy, type);
                            synergies[techBIndex][classes[i - 3]].SetSynergy(techAIndex, synergy, type);
                        }
                    }
                }
                row++;
            } while (row < rows);
            return synergies;
        }
        #endregion
        #region 
        internal List<Technology> _techList;
        #endregion
    }
}

