#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.Cafe.Model;
#endregion
namespace Volpe.Cafe.IO.Reporting.CSV
{
    [Serializable]
    public sealed class CsvAnnualSocietalEffectsReport : CsvReportingBase
    {
        #region 
        public CsvAnnualSocietalEffectsReport(string path, bool append) : base(path, append)
        {
            this._edSum = new EffectsData();
        }
        #endregion
        #region 
        protected override string GetHeader()
        {
            return Header;
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, ICompliance compliance)
        {
            Industry scenData = compliance.GetData(scen, year);
            EffectsData ed = scenData.EffectsData;
            this.WriteEffectsData(scen, year, ed);
            this._edSum.Sales         += ed.Sales;          
            this._edSum.VMT           += ed.VMT;
            this._edSum.BTU           += ed.BTU;
            this._edSum.Gallons       += ed.Gallons;
            this._edSum.KWh           += ed.KWh;
            this._edSum.Scf           += ed.Scf;
            this._edSum.EmissionsCO2  += ed.EmissionsCO2;   
            this._edSum.EmissionsCO   += ed.EmissionsCO;
            this._edSum.EmissionsVOC  += ed.EmissionsVOC;
            this._edSum.EmissionsNOX  += ed.EmissionsNOX;
            this._edSum.EmissionsPM   += ed.EmissionsPM;
            this._edSum.EmissionsSOX  += ed.EmissionsSOX;
            this._edSum.EmissionsCH4  += ed.EmissionsCH4;
            this._edSum.EmissionsN2O  += ed.EmissionsN2O;
            this._edSum.SftFatalities += ed.SftFatalities;  
            for (int i = 0; i < Interaction.CY; i++)
            {
                this._edSum.AnnualVMT          [i] += ed.AnnualVMT          [i];    
                this._edSum.AnnualBTU          [i] += ed.AnnualBTU          [i];
                this._edSum.AnnualGallons      [i] += ed.AnnualGallons      [i];
                this._edSum.AnnualKWh          [i] += ed.AnnualKWh          [i];
                this._edSum.AnnualScf          [i] += ed.AnnualScf          [i];
                this._edSum.AnnualEmissionsCO2 [i] += ed.AnnualEmissionsCO2 [i];    
                this._edSum.AnnualEmissionsCO  [i] += ed.AnnualEmissionsCO  [i];
                this._edSum.AnnualEmissionsVOC [i] += ed.AnnualEmissionsVOC [i];
                this._edSum.AnnualEmissionsNOX [i] += ed.AnnualEmissionsNOX [i];
                this._edSum.AnnualEmissionsPM  [i] += ed.AnnualEmissionsPM  [i];
                this._edSum.AnnualEmissionsSOX [i] += ed.AnnualEmissionsSOX [i];
                this._edSum.AnnualEmissionsCH4 [i] += ed.AnnualEmissionsCH4 [i];
                this._edSum.AnnualEmissionsN2O [i] += ed.AnnualEmissionsN2O [i];
                this._edSum.AnnualSftFatalities[i] += ed.AnnualSftFatalities[i];    
            }
            if (year.Year == scenData.MaxYear.Year)
            {
                this.WriteEffectsData(scen, null, this._edSum);
            }
        }
        void WriteEffectsData(Scenario scen, ModelYear year, EffectsData ed)
        {
            for (int i = 0; i < Interaction.CY; i++)
            {
                for (int j = 0; j < RegClasses.Length; j++)
                {
                    RegulatoryClass regClass = RegClasses[j];
                    if (regClass != RC_TotalAll && ed.Sales[regClass].Total == 0) { continue; }
                    this.WriteEffectsData(scen, year, i, regClass, ed);
                }
            } 
            for (int j = 0; j < RegClasses.Length; j++)
            {
                RegulatoryClass regClass = RegClasses[j];
                if (regClass != RC_TotalAll && ed.Sales[regClass].Total == 0) { continue; }
                this.WriteEffectsData(scen, year, -1, regClass, ed);
            }
        }
        void WriteEffectsData(Scenario scen, ModelYear year, int age, RegulatoryClass regClass, EffectsData ed)
        {
            this.Write(scen.Index);
            this.Write(Interaction.GetTitleCase(scen.Description));
            this.Write(year     == null        ? "Total" : year    .ToString());
            this.Write(age      == -1          ? "Total" : age     .ToString());
            this.Write(regClass == RC_TotalAll ? "Total" : regClass.ToString());
            if (age == -1)
            {
                this.Write(ed.VMT                [regClass].Total); 
                this.Write(ed.BTU                [regClass].Total);
                this.Write(ed.Gallons            [regClass].Total);
                this.Write(ed.KWh                [regClass].Total);
                this.Write(ed.Scf                [regClass].Total);
                this.Write(ed.EmissionsCO2       [regClass].Total); 
                this.Write(ed.EmissionsCO        [regClass].Total);
                this.Write(ed.EmissionsVOC       [regClass].Total);
                this.Write(ed.EmissionsNOX       [regClass].Total);
                this.Write(ed.EmissionsPM        [regClass].Total);
                this.Write(ed.EmissionsSOX       [regClass].Total);
                this.Write(ed.EmissionsCH4       [regClass].Total);
                this.Write(ed.EmissionsN2O       [regClass].Total);
                this.Write(ed.SftFatalities      [regClass].Total); 
            }
            else
            {
                this.Write(ed.AnnualVMT          [age][regClass]);  
                this.Write(ed.AnnualBTU          [age][regClass]);
                this.Write(ed.AnnualGallons      [age][regClass]);
                this.Write(ed.AnnualKWh          [age][regClass]);
                this.Write(ed.AnnualScf          [age][regClass]);
                this.Write(ed.AnnualEmissionsCO2 [age][regClass]);  
                this.Write(ed.AnnualEmissionsCO  [age][regClass]);
                this.Write(ed.AnnualEmissionsVOC [age][regClass]);
                this.Write(ed.AnnualEmissionsNOX [age][regClass]);
                this.Write(ed.AnnualEmissionsPM  [age][regClass]);
                this.Write(ed.AnnualEmissionsSOX [age][regClass]);
                this.Write(ed.AnnualEmissionsCH4 [age][regClass]);
                this.Write(ed.AnnualEmissionsN2O [age][regClass]);
                this.Write(ed.AnnualSftFatalities[age][regClass]);  
            }
            this.WriteLine();
        }
        #endregion
        #region 
        public override string ReportName { get { return CsvReportName; } }
        #endregion
        #region 
        public const string CsvReportName = "Annual Societal Effects CSV Report";
        const string Header = "Scenario,Scenario Name,Model Year,Calendar Year,Reg-Class," +
            "kVMT,MBTU,kGallons,mW-h,Mcf,CO-2 (mmT),CO (tons),VOC (tons),NOx (tons),PM (tons),SOx (tons),CH4 (mmT),N2O (mmT),Fatalities";
        const RegulatoryClass RC_TotalAll = (RegulatoryClass)(-2);
        static readonly RegulatoryClass[] RegClasses = {RegulatoryClass.DomesticAuto, RegulatoryClass.ImportedAuto, RegulatoryClass.LightTruck, RegulatoryClass.Unregulated, RC_TotalAll};
        EffectsData _edSum;
        #endregion
    }
}

