#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.Cafe.Model;
#endregion
namespace Volpe.Cafe.IO.Reporting.CSV
{
    [Serializable]
    public sealed class CsvComplianceReport : CsvReportingBase
    {
        #region 
        public CsvComplianceReport(string path, bool append) : base(path, append) { }
        #endregion
        #region 
        protected override string GetHeader()
        {
            return Header;
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, ICompliance compliance)
        {
            Industry scenData = compliance.GetData(scen, year);
            Manufacturer[] mfrs = scenData.Manufacturers.ToArray();
            for (int i = 0; i < mfrs.Length; i++)
            {
                this.WriteComplianceData(scen, year, scenData, mfrs[i], mfrs[i].ModelData, mfrs[i].Vehicles.ToArray());
            }
            Manufacturer.CModelData imd = this.AggregateIndustryData(mfrs);
            this.WriteComplianceData(scen, year, scenData, null, imd, scenData.Vehicles.ToArray());
            if (year.Year == scenData.MinYear.Year)
            {
                this._mmdSum = new Manufacturer.CModelData[mfrs.Length];
                for (int i = 0; i < mfrs.Length; i++)
                {
                    this._mmdSum[i] = new Manufacturer.CModelData();
                }
                this._imdSum = new Manufacturer.CModelData();
            }
            for (int i = 0; i < mfrs.Length; i++)
            {
                this.AggregateModelYearData(this._mmdSum[i], mfrs[i].ModelData);
            }
            this.AggregateModelYearData(this._imdSum, imd);
            if (year.Year == scenData.MaxYear.Year)
            {   
                for (int i = 0; i < mfrs.Length; i++)
                {
                    this.WriteComplianceData(scen, null, scenData, mfrs[i], this._mmdSum[i], null);
                }
                this.WriteComplianceData(scen, null, scenData, null, this._imdSum, null);
            }
        }
        void WriteComplianceData(Scenario scen, ModelYear year, Industry ind, Manufacturer mfr,
            Manufacturer.CModelData complianceData, Vehicle[] vehs)
        {
            for (int i = 0; i < RegClasses.Length; i++)
            {
                RegulatoryClass regClass = RegClasses[i];
                if (regClass != RC_TotalAll && complianceData.Sales[regClass] == 0) { continue; }
                this.WriteComplianceData(scen, year, regClass, ind, mfr, complianceData, vehs);
            }
        }
        void WriteComplianceData(Scenario scen, ModelYear year, RegulatoryClass regClass, Industry scenData, Manufacturer mfr,
            Manufacturer.CModelData complianceData, Vehicle[] vehs)
        {
            this.Write(scen.Index);
            this.Write(Interaction.GetTitleCase(scen.Description));
            this.Write(year == null ? "Total" : year.ToString());
            this.Write(mfr == null ? "<Industry>" : mfr.Description.ProperName);
            this.Write(regClass == RC_TotalAll ? "Total" : regClass.ToString());
            this.Write(complianceData.Sales[regClass]);
            if (mfr == null)
            {
                if (year == null) { this.Write(scenData.CalcLaborHours(scenData.MinYear, scenData.MaxYear, regClass)); }
                else              { this.Write(scenData.CalcLaborHours(year, regClass)); }
            }
            else
            {
                if (year == null) { this.Write(mfr.CalcLaborHours(scenData.MinYear, scenData.MaxYear, regClass)); }
                else              { this.Write(mfr.CalcLaborHours(year, regClass)); }
            }
            if (year == null)
            {
                this.Write();
                this.Write();
                this.Write();
                this.Write();
                this.Write();
                this.Write();
            }
            else
            {
                RCDouble avgCW, avgFP;
                this.ComputeAverageCWFP(year, vehs, out avgCW, out avgFP);
                if (regClass == RC_TotalAll)
                {
                    this.Write(complianceData.PreliminaryStandard.HarmonicMean  (complianceData.Sales));
                    this.Write(complianceData.Standard           .HarmonicMean  (complianceData.Sales));
                    this.Write(complianceData.CAFE_2Bag          .HarmonicMean  (complianceData.Sales));
                    this.Write(complianceData.CAFE               .HarmonicMean  (complianceData.Sales));
                    this.Write(               avgCW              .ArithmeticMean(complianceData.Sales));
                    this.Write(               avgFP              .ArithmeticMean(complianceData.Sales));
                }
                else
                {
                    this.Write(complianceData.PreliminaryStandard[regClass]);
                    this.Write(complianceData.Standard           [regClass]);
                    this.Write(complianceData.CAFE_2Bag          [regClass]);
                    this.Write(complianceData.CAFE               [regClass]);
                    this.Write(               avgCW              [regClass]);
                    this.Write(               avgFP              [regClass]);
                }
            }
            this.Write(complianceData.TechCost           [regClass]);                                   
            this.Write(complianceData.Fines              [regClass]);
            this.Write(complianceData.RegCost            [regClass]);
            this.Write(complianceData.LossOfValue        [regClass]);
            this.Write(complianceData.RelativeLossOfValue[regClass]);
            this.Write(complianceData.MaintenanceCost    [regClass]);
            this.Write(complianceData.RepairCost         [regClass]);
            this.Write(complianceData.TaxesAndFees       [regClass]);
            this.Write(complianceData.FinancingCost      [regClass]);
            this.Write(complianceData.InsuranceCost      [regClass]);
            this.Write(complianceData.TotalConsumerCosts [regClass]);
            this.Write(complianceData.TotalSocialCosts   [regClass]);
            this.Write(complianceData.TechCost           [regClass] / complianceData.Sales[regClass]);  
            this.Write(complianceData.Fines              [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.RegCost            [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.LossOfValue        [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.RelativeLossOfValue[regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.MaintenanceCost    [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.RepairCost         [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.TaxesAndFees       [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.FinancingCost      [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.InsuranceCost      [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.TotalConsumerCosts [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.TotalSocialCosts   [regClass] / complianceData.Sales[regClass]);
            this.Write(complianceData.Credits            [regClass] * 10);                              
            this.Write(complianceData.TCreditsOut        [regClass] * 10);
            this.Write(complianceData.TCreditsIn         [regClass] * 10);
            this.WriteLine();
        }
        void ComputeAverageCWFP(ModelYear year, Vehicle[] vehs, out RCDouble avgCW, out RCDouble avgFP)
        {
            RCDouble sales = new RCDouble();
            avgCW = new RCDouble();
            avgFP = new RCDouble();
            for (int i = 0; i < vehs.Length; i++)
            {
                Vehicle.CDescription vd       = vehs[i].Description;
                RegulatoryClass      regClass = vehs[i].RegClass;
                double               vehSales = vd.Sales[year.Index];
                sales[regClass] += vehSales;
                avgCW[regClass] += vd.CurbWeight * vehSales;
                avgFP[regClass] += vd.Footprint  * vehSales;
            }
            avgCW /= sales; avgCW.ZeroNaNs();
            avgFP /= sales; avgFP.ZeroNaNs();
        }
        Manufacturer.CModelData AggregateIndustryData(Manufacturer[] mfrs)
        {
            Manufacturer.CModelData imd = new Manufacturer.CModelData();
            for (int i = 0; i < mfrs.Length; i++)
            {
                Manufacturer.CModelData mmd = mfrs[i].ModelData;
                imd.Sales               +=                   mmd.Sales;
                imd.PreliminaryStandard += RCDouble.ZeroNaNs(mmd.Sales / mmd.PreliminaryStandard);
                imd.Standard            += RCDouble.ZeroNaNs(mmd.Sales / mmd.Standard);
                imd.CAFE_2Bag           += RCDouble.ZeroNaNs(mmd.Sales / mmd.CAFE_2Bag);
                imd.CAFE                += RCDouble.ZeroNaNs(mmd.Sales / mmd.CAFE);
                imd.TechCost            +=                   mmd.TechCost;
                imd.Fines               +=                   mmd.Fines;
                imd.RegCost             +=                   mmd.RegCost;
                imd.LossOfValue         +=                   mmd.LossOfValue;
                imd.RelativeLossOfValue +=                   mmd.RelativeLossOfValue;
                imd.MaintenanceCost     +=                   mmd.MaintenanceCost;
                imd.RepairCost          +=                   mmd.RepairCost;
                imd.TaxesAndFees        +=                   mmd.TaxesAndFees;
                imd.FinancingCost       +=                   mmd.FinancingCost;
                imd.InsuranceCost       +=                   mmd.InsuranceCost;
                imd.TotalConsumerCosts  +=                   mmd.TotalConsumerCosts;
                imd.TotalSocialCosts    +=                   mmd.TotalSocialCosts;
                imd.Credits             +=                   mmd.Credits;
                imd.TCreditsOut         +=                   mmd.TCreditsOut;
                imd.TCreditsIn          +=                   mmd.TCreditsIn;
            }
            imd.PreliminaryStandard = RCDouble.ZeroNaNs(imd.Sales / imd.PreliminaryStandard);
            imd.Standard            = RCDouble.ZeroNaNs(imd.Sales / imd.Standard           );
            imd.CAFE_2Bag           = RCDouble.ZeroNaNs(imd.Sales / imd.CAFE_2Bag          );
            imd.CAFE                = RCDouble.ZeroNaNs(imd.Sales / imd.CAFE               );
            return imd;
        }
        void AggregateModelYearData(Manufacturer.CModelData sum, Manufacturer.CModelData value)
        {
            sum.Sales               += value.Sales;
            sum.TechCost            += value.TechCost;
            sum.Fines               += value.Fines;
            sum.RegCost             += value.RegCost;
            sum.LossOfValue         += value.LossOfValue;
            sum.RelativeLossOfValue += value.RelativeLossOfValue;
            sum.MaintenanceCost     += value.MaintenanceCost;
            sum.RepairCost          += value.RepairCost;
            sum.TaxesAndFees        += value.TaxesAndFees;
            sum.FinancingCost       += value.FinancingCost;
            sum.InsuranceCost       += value.InsuranceCost;
            sum.TotalConsumerCosts  += value.TotalConsumerCosts;
            sum.TotalSocialCosts    += value.TotalSocialCosts;
            sum.Credits             += value.Credits;
            sum.TCreditsOut         += value.TCreditsOut;
            sum.TCreditsIn          += value.TCreditsIn;
        }
        #endregion
        #region 
        public override string ReportName { get { return CsvReportName; } }
        #endregion
        #region 
        public const string CsvReportName = "Compliance CSV Report";
        const string Header =
            "Scenario,Scenario Name,Model Year,Manufacturer,Reg-Class," +
            "Sales,k.Labor Hours,Prelim-Stnd,Standard,CAFE (2-cycle),CAFE,Average CW,Average FP," +
            "Tech Cost,Fines,Reg-Cost,Value Loss,Rel. Value Loss,Maint Cost,Repair Cost,Taxes/Fees,Financing,Insurance,Total Consumer Costs,Total Social Costs," +
            "Avg Tech Cost,Avg Fines,Avg Reg-Cost,Avg Value Loss,Avg Rel. Value Loss,Avg Maint Cost,Avg Repair Cost,Avg Taxes/Fees,Avg Financing,Avg Insurance,Avg Consumer Costs,Avg Social Costs," +
            "Credits Earned,Credits Out,Credits In";
        const RegulatoryClass RC_TotalAll = (RegulatoryClass)(-2);
        static readonly RegulatoryClass[] RegClasses = {RegulatoryClass.DomesticAuto, RegulatoryClass.ImportedAuto, RegulatoryClass.LightTruck, RegulatoryClass.Unregulated, RC_TotalAll};
        Manufacturer.CModelData[] _mmdSum;
        Manufacturer.CModelData   _imdSum;
        #endregion
    }
}

