#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.Cafe.Model;
#endregion
namespace Volpe.Cafe.IO.Reporting.CSV
{
    [Serializable]
    public sealed class CsvSocietalCostsReport : CsvReportingBase
    {
        #region 
        public CsvSocietalCostsReport(string path, bool append) : base(path, append) { }
        #endregion
        #region 
        protected override string GetHeader()
        {
            return Header;
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, ICompliance compliance)
        {
            Industry scenData = compliance.GetData(scen, year);
            EffectsData.OwnerAndSocietalCosts undiscCost = scenData.EffectsData.UndiscountedOwnerAndSocietalCosts;
            EffectsData.OwnerAndSocietalCosts[] discCost = scenData.EffectsData.DiscountedOwnerAndSocietalCosts;
            this.WriteCostData(scen, year, undiscCost);
            for (int i = 0; i < discCost.Length; i++)
            {
                this.WriteCostData(scen, year, discCost[i]);
            }
            if (year.Year == scenData.MinYear.Year)
            {
                this._ucSum = new EffectsData.OwnerAndSocietalCosts();
                this._dcSum = new EffectsData.OwnerAndSocietalCosts[discCost.Length];
                for (int i = 0; i < discCost.Length; i++)
                {
                    this._dcSum[i] = new EffectsData.OwnerAndSocietalCosts();
                }
            }
            this._ucSum.Add(undiscCost);
            for (int i = 0; i < discCost.Length; i++)
            {
                this._dcSum[i].Add(discCost[i]);
            }
            if (year.Year == scenData.MaxYear.Year)
            {
                this.WriteCostData(scen, null, this._ucSum);
                for (int i = 0; i < discCost.Length; i++)
                {
                    this.WriteCostData(scen, null, this._dcSum[i]);
                }
            }
        }
        void WriteCostData(Scenario scen, ModelYear year, EffectsData.OwnerAndSocietalCosts osc)
        {
            for (int i = 0; i < RegClasses.Length; i++)
            {
                RegulatoryClass regClass = RegClasses[i];
                if (regClass != RC_TotalAll && osc.PreTaxFuelCost[regClass] == 0) { continue; }
                this.WriteCostData(scen, year, regClass, osc);
            }
        }
        void WriteCostData(Scenario scen, ModelYear year, RegulatoryClass regClass, EffectsData.OwnerAndSocietalCosts osc)
        {
            this.Write(scen.Index);
            this.Write(Interaction.GetTitleCase(scen.Description));
            this.Write(year     == null        ? "Total" : year    .ToString());
            this.Write(regClass == RC_TotalAll ? "Total" : regClass.ToString());
            this.Write(osc.DiscountRate);
            this.Write(osc.PreTaxFuelCost    [regClass]);
            this.Write(osc.FuelTaxCost       [regClass]);
            this.Write(osc.DriveSurplus      [regClass]);
            this.Write(osc.RefuelSurplus     [regClass]);
            this.Write(osc.EconomicCosts     [regClass]);
            this.Write(osc.CongestionCosts   [regClass]);
            this.Write(osc.AccidentCosts     [regClass]);
            this.Write(osc.NoiseCosts        [regClass]);
            this.Write(osc.FatalityCosts     [regClass]);
            this.Write(osc.EmissionsCostsCO2 [regClass]);   
            this.Write(osc.EmissionsCostsCO  [regClass]);
            this.Write(osc.EmissionsCostsVOC [regClass]);
            this.Write(osc.EmissionsCostsNOX [regClass]);
            this.Write(osc.EmissionsCostsPM  [regClass]);
            this.Write(osc.EmissionsCostsSOX [regClass]);
            this.Write(osc.EmissionsCostsCH4 [regClass]);
            this.Write(osc.EmissionsCostsN2O [regClass]);
            this.Write(osc.RetailFuelCosts   [regClass]);   
            this.Write(osc.TotalConsumerCosts[regClass]);
            this.Write(osc.TotalSocialCosts  [regClass]);
            this.WriteLine();
        }
        #endregion
        #region 
        public override string ReportName { get { return CsvReportName; } }
        #endregion
        #region 
        public const string CsvReportName = "Societal Costs CSV Report";
        const string Header = "Scenario,Scenario Name,Model Year,Reg-Class,Disc-Rate," +
            "Pre-Tax Fuel Cost,Fuel Tax Cost,Drive Surplus,Refuel Surplus," +
            "Market Externalities,Congestion Costs,Accident Costs,Noise Costs,Fatality Costs," +
            "CO-2 Damage Costs,CO Damage Costs,VOC Damage Costs,NOx Damage Costs,PM Damage Costs,SOx Damage Costs,CH4 Damage Costs,N2O Damage Costs," +
            "Retail Fuel Costs,Total Consumer Costs,Total Social Costs";
        const RegulatoryClass RC_TotalAll = (RegulatoryClass)(-2);
        static readonly RegulatoryClass[] RegClasses = {RegulatoryClass.DomesticAuto, RegulatoryClass.ImportedAuto, RegulatoryClass.LightTruck, RegulatoryClass.Unregulated, RC_TotalAll};
        EffectsData.OwnerAndSocietalCosts   _ucSum;
        EffectsData.OwnerAndSocietalCosts[] _dcSum;
        #endregion
    }
}

