#region << Using Directives >>
using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.Cafe.Model;
#endregion
namespace Volpe.Cafe.IO.Reporting.CSV
{
    [Serializable]
    public sealed class CsvSocietalEffectsReport : CsvReportingBase
    {
        #region 
        public CsvSocietalEffectsReport(string path, bool append) : base(path, append) { }
        #endregion
        #region 
        protected override string GetHeader()
        {
            return Header;
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, ICompliance compliance)
        {
            Industry scenData = compliance.GetData(scen, year);
            EffectsData ed = scenData.EffectsData;
            this.WriteEffectsData(scen, year, ed);
            if (year.Year == scenData.MinYear.Year)
            {
                this._edSum = new EffectsData();
            }
            this._edSum.Sales         += ed.Sales;          
            this._edSum.VMT           += ed.VMT;
            this._edSum.BTU           += ed.BTU;
            this._edSum.Gallons       += ed.Gallons;
            this._edSum.KWh           += ed.KWh;
            this._edSum.Scf           += ed.Scf;
            this._edSum.EmissionsCO2  += ed.EmissionsCO2;   
            this._edSum.EmissionsCO   += ed.EmissionsCO;
            this._edSum.EmissionsVOC  += ed.EmissionsVOC;
            this._edSum.EmissionsNOX  += ed.EmissionsNOX;
            this._edSum.EmissionsPM   += ed.EmissionsPM;
            this._edSum.EmissionsSOX  += ed.EmissionsSOX;
            this._edSum.EmissionsCH4  += ed.EmissionsCH4;
            this._edSum.EmissionsN2O  += ed.EmissionsN2O;
            this._edSum.SftFatalities += ed.SftFatalities;  
            if (year.Year == scenData.MaxYear.Year)
            {
                this.WriteEffectsData(scen, null, this._edSum);
            }
        }
        void WriteEffectsData(Scenario scen, ModelYear year, EffectsData ed)
        {
            for (int i = 0; i < RegClasses.Length; i++)
            {
                RegulatoryClass regClass = RegClasses[i];
                if (regClass != RC_TotalAll && ed.Sales[regClass].Total == 0) { continue; }
                for (int j = 0; j < FuelTypes.Length; j++)
                {
                    FuelType fuelType = FuelTypes[j];
                    this.WriteEffectsData(scen, year, regClass, fuelType, ed);
                }
            }
        }
        void WriteEffectsData(Scenario scen, ModelYear year, RegulatoryClass regClass, FuelType fuelType, EffectsData ed)
        {
            this.Write(scen.Index);
            this.Write(Interaction.GetTitleCase(scen.Description));
            this.Write(year     == null          ? "Total" : year    .ToString());
            this.Write(regClass == RC_TotalAll   ? "Total" : regClass.ToString());
            this.Write(fuelType == FuelType.None ? "Total" : fuelType.ToString());
            if (fuelType == FuelType.None)
            {
                this.Write(ed.Sales        [regClass].Total);
                this.Write(ed.VMT          [regClass].Total);       
                this.Write(ed.BTU          [regClass].Total);
                this.Write(ed.Gallons      [regClass].Total);
                this.Write(ed.KWh          [regClass].Total);
                this.Write(ed.Scf          [regClass].Total);
                if (regClass == RC_TotalAll)
                {
                    this.Write(ed.AverageRatedFuelEconomy .HarmonicMean(ed.VMT[regClass]));
                    this.Write(ed.AverageOnRoadFuelEconomy.HarmonicMean(ed.VMT[regClass]));
                }
                else
                {
                    this.Write(ed.RatedFuelEconomy [regClass].HarmonicMean(ed.VMT[regClass]));
                    this.Write(ed.OnRoadFuelEconomy[regClass].HarmonicMean(ed.VMT[regClass]));
                }
                this.Write(ed.EmissionsCO2 [regClass].Total);       
                this.Write(ed.EmissionsCO  [regClass].Total);
                this.Write(ed.EmissionsVOC [regClass].Total);
                this.Write(ed.EmissionsNOX [regClass].Total);
                this.Write(ed.EmissionsPM  [regClass].Total);
                this.Write(ed.EmissionsSOX [regClass].Total);
                this.Write(ed.EmissionsCH4 [regClass].Total);
                this.Write(ed.EmissionsN2O [regClass].Total);
                this.Write(ed.SftFatalities[regClass].PC   );       
                this.Write(ed.SftFatalities[regClass].CM   );
                this.Write(ed.SftFatalities[regClass].LT   );
            }
            else
            {
                this.Write(ed.Sales        [regClass][fuelType]);
                this.Write(ed.VMT          [regClass][fuelType]);   
                this.Write(ed.BTU          [regClass][fuelType]);
                this.Write(ed.Gallons      [regClass][fuelType]);
                this.Write(ed.KWh          [regClass][fuelType]);
                this.Write(ed.Scf          [regClass][fuelType]);
                if (regClass == RC_TotalAll)
                {
                    this.Write(ed.AverageRatedFuelEconomy [fuelType]);
                    this.Write(ed.AverageOnRoadFuelEconomy[fuelType]);
                }
                else
                {
                    this.Write(ed.RatedFuelEconomy [regClass][fuelType]);
                    this.Write(ed.OnRoadFuelEconomy[regClass][fuelType]);
                }
                this.Write(ed.EmissionsCO2 [regClass][fuelType]);   
                this.Write(ed.EmissionsCO  [regClass][fuelType]);
                this.Write(ed.EmissionsVOC [regClass][fuelType]);
                this.Write(ed.EmissionsNOX [regClass][fuelType]);
                this.Write(ed.EmissionsPM  [regClass][fuelType]);
                this.Write(ed.EmissionsSOX [regClass][fuelType]);
                this.Write(ed.EmissionsCH4 [regClass][fuelType]);
                this.Write(ed.EmissionsN2O [regClass][fuelType]);
                this.Write();                                       
                this.Write();
                this.Write();
            }
            this.WriteLine();
        }
        #endregion
        #region 
        public override string ReportName { get { return CsvReportName; } }
        #endregion
        #region 
        public const string CsvReportName = "Societal Effects CSV Report";
        const string Header = "Scenario,Scenario Name,Model Year,Reg-Class,Fuel Type," +
            "Sales,kVMT,MBTU,kGallons,mW-h,Mcf,Rated FE,On-road FE," +
            "CO-2 (mmT),CO (tons),VOC (tons),NOx (tons),PM (tons),SOx (tons),CH4 (mmT),N2O (mmT)," +
            "Fatals PC,Fatals CM,Fatals LT";
        const RegulatoryClass RC_TotalAll = (RegulatoryClass)(-2);
        static readonly RegulatoryClass[] RegClasses = {RegulatoryClass.DomesticAuto, RegulatoryClass.ImportedAuto, RegulatoryClass.LightTruck, RegulatoryClass.Unregulated, RC_TotalAll};
        static readonly FuelType       [] FuelTypes  = {FuelType.Gasoline, FuelType.Diesel, FuelType.Ethanol85, FuelType.Electricity, FuelType.Hydrogen, FuelType.None};
        EffectsData _edSum;
        #endregion
    }
}

