#region << Using Directives >>
using System;
using System.Collections.Generic;
using System.Text;
using Volpe.Cafe.Data;
using Volpe.Cafe.Model;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using TI = Volpe.Cafe.TechnologyIndexes;
#endregion
namespace Volpe.Cafe.IO.Reporting.CSV
{
    [Serializable]
    public sealed class CsvTechnologyUtilizationReport : CsvReportingBase
    {
        #region 
        public CsvTechnologyUtilizationReport(string path, bool append) : base(path, append) { }
        #endregion
        #region 
        protected override string GetHeader()
        {
            return Header;
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, ICompliance compliance)
        {
            Industry           data = compliance.GetData(scen, year);
            List<Manufacturer> mfrs = data.Manufacturers;
            RCDouble   indSales        = RCDouble.Zero;
            RCDouble[] indAppliedSales = new RCDouble[TI.TechnologyCount + 2];  
            RCDouble[] indUsedSales    = new RCDouble[TI.TechnologyCount + 2];
            RCDouble[] mfrAppliedSales = new RCDouble[TI.TechnologyCount + 2];
            RCDouble[] mfrUsedSales    = new RCDouble[TI.TechnologyCount + 2];
            for (int i = 0, mfrCount = mfrs.Count; i < mfrCount; i++)
            {
                RCDouble mfrSales = mfrs[i].ModelData.Sales;
                for (int j = 0; j < mfrUsedSales.Length; j++)
                {
                    this.CalcUtilization(year, mfrs[i], j, out mfrAppliedSales[j], out mfrUsedSales[j]);
                    indAppliedSales[j] += mfrAppliedSales[j];
                    indUsedSales   [j] += mfrUsedSales   [j];
                }
                indSales += mfrSales;
                for (int j = 0; j < RegClasses.Length; j++)
                {
                    RegulatoryClass regClass = RegClasses[j];
                    if (regClass != RC_TotalAll && mfrSales[regClass] == 0) { continue; }
                    this.WriteTechUtilizationData(scen, year, mfrs[i], regClass, mfrSales, mfrAppliedSales, mfrUsedSales);
                }
            } 
            for (int i = 0; i < RegClasses.Length; i++)
            {
                RegulatoryClass regClass = RegClasses[i];
                if (regClass != RC_TotalAll && indSales[regClass] == 0) { continue; }
                this.WriteTechUtilizationData(scen, year, null, regClass, indSales, indAppliedSales, indUsedSales);
            }
        }
        void WriteTechUtilizationHeader(Scenario scen, ModelYear year, Manufacturer mfr, RegulatoryClass regClass, string techAbbr)
        {
            this.Write(scen.Index);
            this.Write(Interaction.GetTitleCase(scen.Description));
            this.Write(year.ToString());
            this.Write(mfr == null ? "<Industry>" : mfr.Description.ProperName);
            this.Write(regClass == RC_TotalAll ? "<Total>" : regClass.ToString());
            this.Write(techAbbr);
        }
        void WriteTechUtilizationData(Scenario scen, ModelYear year, Manufacturer mfr, RegulatoryClass regClass,
            RCDouble sales, RCDouble[] appliedSales, RCDouble[] usedSales)
        {
            for (int i = 0; i < TI.TechnologyCount; i++)
            {
                switch (i)
                {
                    case TI.TRBDS1_SD:
                        this.WriteTechUtilizationHeader(scen, year, mfr, regClass, "TRBDS1");
                        this.Write((appliedSales[TI.TRBDS1_SD][regClass] + appliedSales[TI.TRBDS1_MD][regClass] + appliedSales[TI.TRBDS1_LD][regClass]) / sales[regClass]);
                        this.Write((usedSales   [TI.TRBDS1_SD][regClass] + usedSales   [TI.TRBDS1_MD][regClass] + usedSales   [TI.TRBDS1_LD][regClass]) / sales[regClass]);
                        break;
                    case TI.TRBDS2_SD:
                        this.WriteTechUtilizationHeader(scen, year, mfr, regClass, "TRBDS2");
                        this.Write((appliedSales[TI.TRBDS2_SD][regClass] + appliedSales[TI.TRBDS2_MD][regClass] + appliedSales[TI.TRBDS2_LD][regClass]) / sales[regClass]);
                        this.Write((usedSales   [TI.TRBDS2_SD][regClass] + usedSales   [TI.TRBDS2_MD][regClass] + usedSales   [TI.TRBDS2_LD][regClass]) / sales[regClass]);
                        break;
                    case TI.CEGR1_SD:
                        this.WriteTechUtilizationHeader(scen, year, mfr, regClass, "CEGR1");
                        this.Write((appliedSales[TI.CEGR1_SD][regClass] + appliedSales[TI.CEGR1_MD][regClass] + appliedSales[TI.CEGR1_LD][regClass]) / sales[regClass]);
                        this.Write((usedSales   [TI.CEGR1_SD][regClass] + usedSales   [TI.CEGR1_MD][regClass] + usedSales   [TI.CEGR1_LD][regClass]) / sales[regClass]);
                        break;
                    case TI.CEGR2_SD:
                        this.WriteTechUtilizationHeader(scen, year, mfr, regClass, "CEGR2");
                        this.Write((appliedSales[TI.CEGR2_SD][regClass] + appliedSales[TI.CEGR2_MD][regClass] + appliedSales[TI.CEGR2_LD][regClass]) / sales[regClass]);
                        this.Write((usedSales   [TI.CEGR2_SD][regClass] + usedSales   [TI.CEGR2_MD][regClass] + usedSales   [TI.CEGR2_LD][regClass]) / sales[regClass]);
                        break;
                    case TI.ADSL_SD:
                        this.WriteTechUtilizationHeader(scen, year, mfr, regClass, "DSL");
                        this.Write(appliedSales[TI.TechnologyCount][regClass] / sales[regClass]);
                        this.Write(usedSales   [TI.TechnologyCount][regClass] / sales[regClass]);
                        this.WriteLine();
                        this.WriteTechUtilizationHeader(scen, year, mfr, regClass, "ADSL");
                        this.Write((appliedSales[TI.ADSL_SD][regClass] + appliedSales[TI.ADSL_MD][regClass] + appliedSales[TI.ADSL_LD][regClass]) / sales[regClass]);
                        this.Write((usedSales   [TI.ADSL_SD][regClass] + usedSales   [TI.ADSL_MD][regClass] + usedSales   [TI.ADSL_LD][regClass]) / sales[regClass]);
                        this.WriteLine();
                        this.WriteTechUtilizationHeader(scen, year, mfr, regClass, "E85 FFV");
                        this.Write(appliedSales[TI.TechnologyCount + 1][regClass] / sales[regClass]);
                        this.Write(usedSales   [TI.TechnologyCount + 1][regClass] / sales[regClass]);
                        break;
                    case TI.TRBDS1_MD: case TI.TRBDS1_LD:
                    case TI.TRBDS2_MD: case TI.TRBDS2_LD:
                    case TI.CEGR1_MD : case TI.CEGR1_LD :
                    case TI.CEGR2_MD : case TI.CEGR2_LD :
                    case TI.ADSL_MD  : case TI.ADSL_LD  :
                        continue;
                    default:    
                        this.WriteTechUtilizationHeader(scen, year, mfr, regClass, TI.GetAbbr(i));
                        this.Write(appliedSales[i][regClass] / sales[regClass]);
                        this.Write(usedSales   [i][regClass] / sales[regClass]);
                        break;
                }
                this.WriteLine();
            } 
        }
        void CalcUtilization(ModelYear year, Manufacturer mfr, int techIndex, out RCDouble appliedSales, out RCDouble usedSales)
        {
            appliedSales = RCDouble.Zero;
            usedSales    = RCDouble.Zero;
            List<Vehicle> vehs = mfr.Vehicles;
            for (int i = 0, vehCount = vehs.Count; i < vehCount; i++)
            {
                Vehicle.CModelData vmd       = vehs[i].ModelData;
                RegulatoryClass    regClass  = vehs[i].RegClass;
                FuelValue          fuelShare = vehs[i].Description.FuelShare;
                double             vehSales  = vehs[i].Description.Sales[year.Index];
                if (techIndex >= TI.TechnologyCount)
                {   
                    if (techIndex == TI.TechnologyCount)
                    {   
                        bool isADSL = vmd.TechUsed[TI.ADSL_SD] || vmd.TechUsed[TI.ADSL_MD] || vmd.TechUsed[TI.ADSL_LD];
                        if (fuelShare.Diesel > 0 && !isADSL) { usedSales[regClass] += vehSales; }
                    }
                    else if (techIndex == TI.TechnologyCount + 1)
                    {   
                        if (fuelShare.Ethanol85 > 0) { usedSales[regClass] += vehSales; }
                    }
                }
                else
                {
                    if (vmd.TechUsed[techIndex] && !vmd.TechSuperseded[techIndex] && !vmd.TechPhaseInBackfilled[techIndex])
                    {
                        if (vmd.TechApplied[techIndex]) { appliedSales[regClass] += vehSales; }
                        usedSales[regClass] += vehSales;
                    }
                }
            } 
        }
        #endregion
        #region 
        public override string ReportName { get { return CsvReportName; } }
        #endregion
        #region 
        public const string CsvReportName = "Technology Utilization CSV Report";
        const string Header = "Scenario,Scenario Name,Model Year,Manufacturer,Reg-Class,Technology,App-Rate,Pen-Rate";
        const RegulatoryClass RC_TotalAll = (RegulatoryClass)(-2);
        static readonly RegulatoryClass[] RegClasses = {RegulatoryClass.DomesticAuto, RegulatoryClass.ImportedAuto, RegulatoryClass.LightTruck, RegulatoryClass.Unregulated, RC_TotalAll};
        #endregion
    }
}

