#region << Using Directives >>
using System;
using System.Collections.Generic;
using System.Text;
using Volpe.Cafe.Data;
using Volpe.Cafe.Model;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
#endregion
namespace Volpe.Cafe.IO.Reporting.CSV
{
    [Serializable]
    public sealed class CsvVehiclesReport : CsvReportingBase
    {
        #region 
        public CsvVehiclesReport(string path, bool append) : base(path, append) { }
        #endregion
        #region 
        protected override string GetHeader()
        {
            return Header;
        }
        protected override void ParseData_Internal(Scenario scen, ModelYear year, ICompliance compliance)
        {
            Industry   ppData   = compliance.Data;
            Industry   scenData = compliance.GetData(scen, year);
            Parameters param    = compliance.Settings.Parameters;
            List<Vehicle> ppVehs  = ppData  .Vehicles;
            List<Vehicle> snVehs  = scenData.Vehicles;
            int           yrIndex = year.Index;
            int           multiFuelMode = scen.MultiFuelCAFE[yrIndex];
            for (int i = 0, vehCount = ppVehs.Count; i < vehCount; i++)
            {
                Vehicle                   veh = snVehs[i];
                Manufacturer.CModelData   mmd = veh.Manufacturer.ModelData;
                Vehicle     .CDescription vd  = veh.Description;
                Vehicle     .CModelData   vmd = veh.ModelData;
                Engine                    eng = veh.Engine;
                Engine      .CDescription ed  = (eng == null) ? null : eng.Description;
                Transmission              trn = veh.Transmission;
                Transmission.CDescription td  = (trn == null) ? null : trn.Description;
                Vehicle     .CDescription pVD = ppVehs[i].Description;
                double vehSales = vd.Sales[yrIndex];
                string mfrName  = Interaction.GetTitleCase(vd.Manufacturer, 4);
                this.Write(scen.Index);
                this.Write(Interaction.GetTitleCase(scen.Description));
                this.Write(year.Year);
                this.Write(mfrName);
                this.Write(i);
                this.Write(vd.Code);
                this.Write(Interaction.GetTitleCase(vd.Model));
                this.Write(Interaction.GetTitleCase(vd.Nameplate));
                this.Write(string.Empty);
                if (eng == null)
                {
                    this.Write();
                    this.Write();
                    this.Write();
                    this.Write();
                }
                else
                {
                    string engNo = ed.Code.ToString() + ((eng.Parent != null) ? " (" + eng.Parent.Description.Code + ")" : "");
                    this.Write(engNo);
                    this.Write(this.GetFuelTypeString(ed.Fuel));
                    this.Write(ed.Configuration + ed.Cylinders + " " + ed.Displacement + "L");
                    this.Write(ed.Horsepower);
                }
                if (trn == null)
                {
                    this.Write();
                    this.Write();
                }
                else
                {
                    string trnNo = td.Code.ToString() + ((trn.Parent != null) ? " (" + trn.Parent.Description.Code + ")" : "");
                    this.Write(trnNo);
                    this.Write(td.ToString());
                }
                this.Write(pVD.FuelEconomy);
                this.Write(Standards.GetVehicleFuelEconomy(pVD.FuelEconomy, pVD.FuelShare, multiFuelMode));
                this.Write(pVD.FuelShare);
                this.Write(this.GetFuelTypeString(pVD.FuelEconomy.FuelType));
                FuelValue feWithAC    = Standards.GetAdjustedVehicleFuelEconomy(veh, scen, year, param, true , false);
                FuelValue feWithOC    = Standards.GetAdjustedVehicleFuelEconomy(veh, scen, year, param, false, true );
                FuelValue feAdj       = Standards.GetAdjustedVehicleFuelEconomy(veh, scen, year, param, true , true );
                double    avgFE       = Standards.GetVehicleFuelEconomy(vd.FuelEconomy, vd.FuelShare, multiFuelMode);
                double    avgFEWithAC = Standards.GetVehicleFuelEconomy(feWithAC      , vd.FuelShare, multiFuelMode);
                double    avgFEWithOC = Standards.GetVehicleFuelEconomy(feWithOC      , vd.FuelShare, multiFuelMode);
                double    avgFEAdj    = Standards.GetVehicleFuelEconomy(feAdj         , vd.FuelShare, multiFuelMode);
                this.Write(vd.FuelEconomy           );  this.Write(avgFE);
                this.Write(feWithAC - vd.FuelEconomy);  this.Write(avgFEWithAC - avgFE);
                this.Write(feWithOC - vd.FuelEconomy);  this.Write(avgFEWithOC - avgFE);
                this.Write(feAdj                    );  this.Write(avgFEAdj);
                this.Write(vd.FuelShare);
                this.Write(this.GetFuelTypeString(vd.FuelEconomy.FuelType));
                this.Write(veh.RegClass       .ToString());
                this.Write(veh.TechnologyClass.ToString());
                this.Write(veh.Mobile6Class   .ToString());
                this.Write(veh.SafetyClass    .ToString());
                bool atRedesign, inShadowOfRedesign;
                bool atRefresh, inShadowOfRefresh;
                veh.GetRedesignState(year, out atRedesign, out inShadowOfRedesign);
                veh.GetRefreshState (year, out atRefresh , out inShadowOfRefresh);
                this.Write((atRedesign) ? "At Redesign" : (inShadowOfRedesign) ? "Shadow" : null);
                this.Write((atRefresh ) ? "At Refresh"  : (inShadowOfRefresh ) ? "Shadow" : null);
                this.Write(pVD.Sales[yrIndex]);
                if (vehSales == 0)
                {
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                }
                else
                {
                    this.Write(vehSales);
                    this.Write(pVD.Msrp[yrIndex]);
                    this.Write(vd .Msrp[yrIndex] + vmd.RegCost);
                    this.Write(veh.CalcLaborHours(year));
                }
                this.Write(pVD.CurbWeight);
                this.Write(vd .CurbWeight);
                this.Write(vd .Footprint );
                if (vehSales == 0)
                {
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                    this.Write(0);
                }
                else
                {
                    this.Write(vmd.TechCost           );
                    this.Write(vmd.RegCost            );
                    this.Write(vmd.TaxCredit          );
                    this.Write(vmd.LossOfValue        );
                    this.Write(vmd.RelativeLossOfValue);
                    this.Write(vmd.MaintenanceCost    );
                    this.Write(vmd.RepairCost         );
                    this.Write(vmd.TaxesAndFees       );
                    this.Write(vmd.FinancingCost      );
                    this.Write(vmd.InsuranceCost      );
                }
                for (int j = 0; j < TechnologyIndexes.TechnologyCount; j++)
                {
                    string techUsedStr = null;
                    if (vmd.TechUsed[j] && vmd.TechApplied[j] && vmd.TechPhaseInBackfilled[j] && !vmd.TechSuperseded[j])
                    {   
                        techUsedStr = "PA";
                    }
                    else
                    {   
                        techUsedStr =
                            ( vmd.TechUsed     [j] && !vmd.TechApplied[j]) ? "U" :
                            ( vmd.TechUsed     [j] &&  vmd.TechApplied[j]) ? "A" :
                            (!vmd.TechEnabled  [j]                       ) ? "X" :
                            ( mmd.TechExhausted[j][veh.RegClass]         ) ? "P" : null;
                        if (vmd.TechSuperseded[j]) { techUsedStr += "S"; }
                    }
                    this.Write(techUsedStr);
                } 
                this.WriteLine();
            } 
        }
        void Write(FuelValue value)
        {
            this.Write(value.Gasoline);
            this.Write(value.Diesel);
            this.Write(value.Ethanol85);
            this.Write(value.Electricity);
            this.Write(value.Hydrogen);
        }
        string GetFuelTypeString(FuelType fuelType)
        {
            StringBuilder sb = new StringBuilder(8);
            if ((fuelType & FuelType.Gasoline   ) == FuelType.Gasoline   ) { sb.Append("G+"  ); }
            if ((fuelType & FuelType.Diesel     ) == FuelType.Diesel     ) { sb.Append("D+"  ); }
            if ((fuelType & FuelType.Ethanol85  ) == FuelType.Ethanol85  ) { sb.Append("E85+"); }
            if ((fuelType & FuelType.Electricity) == FuelType.Electricity) { sb.Append("E+"  ); }
            if ((fuelType & FuelType.Hydrogen   ) == FuelType.Hydrogen   ) { sb.Append("H+"  ); }
            return sb.ToString(0, sb.Length - 1);
        }
        #endregion
        #region 
        public override string ReportName { get { return CsvReportName; } }
        #endregion
        #region 
        public const string CsvReportName = "Vehicles CSV Report";
        const string Header =
            "Scenario,Scenario Name,Model Year,Manufacturer,Veh Index,Veh Code,Model,Name Plate,Platform,Eng Code,Eng Fuel,Eng Type,Eng HP,Trn Code,Trn Type," +
            "FE Initial (Gas),FE Initial (Dsl),FE Initial (E85),FE Initial (Elc),FE Initial (Hgn),FE Initial," +
            "FS Initial (Gas),FS Initial (Dsl),FS Initial (E85),FS Initial (Elc),FS Initial (Hgn),Fuel Initial," +
            "FE (Gas),FE (Dsl),FE (E85),FE (Elc),FE (Hgn),FE," +
            "Delta FE AC (Gas),Delta FE AC (Dsl),Delta FE AC (E85),Delta FE AC (Elc),Delta FE AC (Hgn),Delta FE AC," +
            "Delta FE Off-Cycle (Gas),Delta FE Off-Cycle (Dsl),Delta FE Off-Cycle (E85),Delta FE Off-Cycle (Elc),Delta FE Off-Cycle (Hgn),Delta FE Off-Cycle," +
            "FE Adj (Gas),FE Adj (Dsl),FE Adj (E85),FE Adj (Elc),FE Adj (Hgn),FE Adj," +
            "FS (Gas),FS (Dsl),FS (E85),FS (Elc),FS (Hgn),Fuel," +
            "Reg Class,Tech Class,Mobile6 Class,Safety Class,Redesign State,Refresh State,Sales Initial,Sales Final,MSRP Initial,MSRP Final,k.Labor Hours,CW Initial,CW Final,Footprint," +
            "Tech Cost,Price Increase,Tax Credit,Value Loss,Rel. Value Loss,Maint Cost,Repair Cost,Taxes/Fees,Financing,Insurance," +
            "LUB1,EFR1,LUB2_EFR2,CCPS,DVVLS,DEACS,ICP,DCP,DVVLD,CVVL,DEACD,SGDI,DEACO,VVA,SGDIO,TRBDS1_SD,TRBDS1_MD,TRBDS1_LD,TRBDS2_SD,TRBDS2_MD,TRBDS2_LD,CEGR1_SD,CEGR1_MD,CEGR1_LD,CEGR2_SD,CEGR2_MD,CEGR2_LD,ADSL_SD,ADSL_MD,ADSL_LD," +
            "6MAN,HETRANSM,IATC,NAUTO,DCT,8SPD,HETRANS,SHFTOPT," +
            "EPS,IACC1,IACC2,MHEV,ISG,SHEV1,SHEV1_2,SHEV2,PHEV1,PHEV2,EV1,EV2,EV3,EV4,FCV,MR1,MR2,MR3,MR4,MR5,ROLL1,ROLL2,ROLL3,LDB,SAX,AERO1,AERO2";
        #endregion
    }
}

