#region << Using Directives >>
using System;
using System.IO;
using Volpe.Cafe.Data;
using Volpe.Cafe.Model;
using Volpe.Cafe.Settings;
#endregion
namespace Volpe.Cafe.IO.Reporting.CSV
{
    [Serializable]
    public abstract class DelimitedReportingBase
    {
        #region 
        public DelimitedReportingBase(string path, bool append, char delimiter)
        {
            this._disposed  = false;
            this._stream    = new StreamWriter(path, append);
            this._path      = path;
            this._delimiter = delimiter;
            if (!append) { this._stream.WriteLine(this.GetHeader()); }
        }
        #endregion
        #region 
        protected abstract string GetHeader();
        public void ParseData(Scenario scen, ModelYear year, ICompliance compliance)
        {
            try   { this.ParseData_Internal(scen, year, compliance); }
            catch { this.Close(); }
        }
        protected abstract void ParseData_Internal(Scenario scen, ModelYear year, ICompliance compliance);
        protected void Write(object value)
        {
            if (value == null) { this.Write(string.Empty); }
            else               { this.Write(value.ToString()); }
        }
        protected void Write(bool value)
        {
            this.Write(value.ToString());
        }
        protected void Write(decimal value)
        {
            this.Write(value.ToString());
        }
        protected void Write(double value)
        {
            this.Write(value.ToString());
        }
        protected void Write(float value)
        {
            this.Write(value.ToString());
        }
        protected void Write(long value)
        {
            this.Write(value.ToString());
        }
        protected void Write(int value)
        {
            this.Write(value.ToString());
        }
        protected void Write(short value)
        {
            this.Write(value.ToString());
        }
        protected void Write(byte value)
        {
            this.Write(value.ToString());
        }
        protected void Write(sbyte value)
        {
            this.Write(value.ToString());
        }
        protected void Write(ulong value)
        {
            this.Write(value.ToString());
        }
        protected void Write(uint value)
        {
            this.Write(value.ToString());
        }
        protected void Write(ushort value)
        {
            this.Write(value.ToString());
        }
        protected void Write(char value)
        {
            this.Write(value.ToString());
        }
        protected void Write(string value)
        {
            this._stream.Write(value);
            this._stream.Write(this._delimiter);
        }
        protected void Write()
        {
            this._stream.Write(this._delimiter);
        }
        protected void WriteLine()
        {
            this._stream.WriteLine();
        }
        public void Close()
        {
            this._stream.Close();
            this._disposed = true;
        }
        #endregion
        #region 
        public bool Disposed { get { return this._disposed; } }
        public string Path { get { return this._path; } }
        public string FileName { get { return System.IO.Path.GetFileName(this._path); } }
        public abstract string ReportName { get; }
        public char Delimiter { get { return this._delimiter; } }
        #endregion
        #region 
        bool         _disposed;     
        StreamWriter _stream;
        string       _path;
        char         _delimiter;
        #endregion
    }
}

