using System;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
namespace Volpe.Cafe.Model
{
    sealed class CreditTrading
    {
        public static void ExamineCredits(Industry[] modelData, int yrIndex, int mfrIndex, ModelingSettings settings)
        {
            CreditTradingSettings ct = settings.CreditTradingSettings;
            if (!ct.AllowTrading) { return; }
            CT_CarryForward  (modelData, yrIndex, mfrIndex, settings);  
            CT_FleetTransfers(modelData, yrIndex, mfrIndex, settings);  
            CT_CarryBackward (modelData, yrIndex, mfrIndex, settings);  
        }
        static void CT_CarryForward(Industry[] modelData, int yrIndex, int mfrIndex, ModelingSettings settings)
        {
            CT_CarryForward(RegulatoryClass.DomesticAuto, modelData, yrIndex, mfrIndex, settings);
            CT_CarryForward(RegulatoryClass.ImportedAuto, modelData, yrIndex, mfrIndex, settings);
            CT_CarryForward(RegulatoryClass.LightTruck  , modelData, yrIndex, mfrIndex, settings);
        }
        static void CT_CarryForward(RegulatoryClass rc, Industry[] modelData, int yrIndex, int mfrIndex, ModelingSettings settings)
        {
            CreditTradingSettings ct = settings.CreditTradingSettings;
            if (!ct.AllowTransferIn[rc] || !ct.AllowTransferOut[rc] || !ct.AllowCarryForward || ct.CarryForwardYears < 1) { return; }
            string                  mfrName = modelData[yrIndex].Manufacturers[mfrIndex].Description.Name;
            Manufacturer.CModelData mmd     = modelData[yrIndex].Manufacturers[mfrIndex].ModelData;
            double credits = ComputeNetCredits(mmd, rc);
            if (credits >= 0) { return; }
            int cfSYrIndex = ComputeCarryFwdStartYr(modelData, yrIndex, ct);
            for (int i = cfSYrIndex; i < yrIndex; i++)
            {
                Manufacturer.CModelData pMMD = modelData[i].Manufacturers[mfrIndex].ModelData;
                double eCredits = Math.Max(0, pMMD.Credits[rc] - pMMD.TCreditsOut[rc]);
                if (eCredits <= 0) { continue; }
                DoTransfer(mfrName, eCredits, -credits, "1:1", ct, rc, i, pMMD, rc, yrIndex, mmd);
                credits = ComputeNetCredits(mmd, rc);
                if (credits >= 0) { return; }
            } 
        }
        static void CT_FleetTransfers(Industry[] modelData, int yrIndex, int mfrIndex, ModelingSettings settings)
        {
            CT_FleetTransfers(RegulatoryClass.LightTruck  , modelData, yrIndex, mfrIndex, settings);
            CT_FleetTransfers(RegulatoryClass.ImportedAuto, modelData, yrIndex, mfrIndex, settings);
            CT_FleetTransfers(RegulatoryClass.DomesticAuto, modelData, yrIndex, mfrIndex, settings);
        }
        static void CT_FleetTransfers(RegulatoryClass rc, Industry[] modelData, int yrIndex, int mfrIndex, ModelingSettings settings)
        {
            CreditTradingSettings ct = settings.CreditTradingSettings;
            if (!ct.AllowTransferIn[rc] || !ct.AllowFleetTransfers) { return; }
            string                  mfrName = modelData[yrIndex].Manufacturers[mfrIndex].Description.Name;
            Manufacturer.CModelData mmd     = modelData[yrIndex].Manufacturers[mfrIndex].ModelData;
            double credits = ComputeNetCredits (mmd, rc);
            double cap     = ComputeTransferCap(mmd, rc, yrIndex, ct);
            if (cap <= 0 || credits >= 0) { return; }
            if (ct.AlternativeTransferInForMinStnd[rc])
            {   
                if (mmd.PreliminaryStandard[rc] < mmd.CAFE[rc]) { return; }
            }
            double rCredits = Math.Min(cap, -credits);
            for (int i = 0; i < 2; i++)
            {
                RegulatoryClass eRC = CT_FleetTransfers_GetNextRC(i, rc);
                if (eRC == RegulatoryClass.Unregulated || !ct.AllowTransferOut[eRC]) { continue; }
                double eCredits = ComputeNetCredits(mmd, eRC);
                if (eCredits <= 0) { continue; }    
                DoTransfer(mfrName, eCredits, rCredits, ct.CreditAdjustmentFactor, ct, eRC, yrIndex, mmd, rc, yrIndex, mmd);
                credits = ComputeNetCredits (mmd, rc);
                cap     = ComputeTransferCap(mmd, rc, yrIndex, ct);
                if (cap <= 0 || credits >= 0) { return; }   
                rCredits = Math.Min(cap, -credits);         
            } 
            if (!ct.AllowCarryForward || ct.CarryForwardYears < 1) { return; }
            for (int i = 0; i < 2; i++)
            {
                RegulatoryClass eRC = CT_FleetTransfers_GetNextRC(i, rc);
                if (eRC == RegulatoryClass.Unregulated || !ct.AllowTransferOut[eRC]) { continue; }
                for (int j = ComputeCarryFwdStartYr(modelData, yrIndex, ct); j < yrIndex; j++)
                {
                    Manufacturer.CModelData pMMD = modelData[j].Manufacturers[mfrIndex].ModelData;
                    double eCredits = Math.Max(0, pMMD.Credits[eRC] - pMMD.TCreditsOut[eRC]); 
                    if (eCredits <= 0) { continue; }  
                    double outECredits, outRCredits;
                    RequestCredits(eCredits, rCredits, ct.CreditAdjustmentFactor, ct, eRC, yrIndex, mmd, rc, yrIndex, mmd, out outECredits, out outRCredits);
                    DoTransfer(mfrName, outECredits, double.PositiveInfinity, "1:1", ct, eRC, j, pMMD, eRC, yrIndex, mmd);
                    eCredits = ComputeNetCredits(mmd, eRC);
                    DoTransfer(mfrName, eCredits, outRCredits, ct.CreditAdjustmentFactor, ct, eRC, yrIndex, mmd, rc, yrIndex, mmd);
                    credits = ComputeNetCredits (mmd, rc);
                    cap     = ComputeTransferCap(mmd, rc, yrIndex, ct);
                    if (cap <= 0 || credits >= 0) { return; }   
                    rCredits = Math.Min(cap, -credits);         
                }
            } 
        }
        static RegulatoryClass CT_FleetTransfers_GetNextRC(int i, RegulatoryClass uRC)
        {
            if      (i == 0 && (uRC == RegulatoryClass.LightTruck   || uRC == RegulatoryClass.ImportedAuto)) { return RegulatoryClass.DomesticAuto; }
            else if (i == 1 && (uRC == RegulatoryClass.DomesticAuto || uRC == RegulatoryClass.ImportedAuto)) { return RegulatoryClass.LightTruck; }
            else if ((i == 0 && uRC == RegulatoryClass.DomesticAuto) || (i == 1 && uRC == RegulatoryClass.LightTruck)) { return RegulatoryClass.ImportedAuto; }
            return RegulatoryClass.Unregulated;
        }
        static void CT_CarryBackward(Industry[] modelData, int yrIndex, int mfrIndex, ModelingSettings settings)
        {
            CT_CarryBackward(RegulatoryClass.DomesticAuto, modelData, yrIndex, mfrIndex, settings);
            CT_CarryBackward(RegulatoryClass.ImportedAuto, modelData, yrIndex, mfrIndex, settings);
            CT_CarryBackward(RegulatoryClass.LightTruck  , modelData, yrIndex, mfrIndex, settings);
        }
        static void CT_CarryBackward(RegulatoryClass rc, Industry[] modelData, int yrIndex, int mfrIndex, ModelingSettings settings)
        {
            CreditTradingSettings ct = settings.CreditTradingSettings;
            if (!ct.AllowTransferIn[rc] || !ct.AllowTransferOut[rc] || !ct.AllowCarryBackward || ct.CarryBackwardYears < 1) { return; }
            Manufacturer.CModelData mmd = modelData[yrIndex].Manufacturers[mfrIndex].ModelData;
            double eCredits = ComputeNetCredits(mmd, rc);
            if (eCredits <= 0) { return; }
            int cbSYrIndex = ComputeCarryBwdStartYr(modelData, yrIndex, ct);
            for (int i = cbSYrIndex; i < yrIndex; i++)
            {
                string                  mfrName = modelData[i].Manufacturers[mfrIndex].Description.Name;
                Manufacturer.CModelData pMMD    = modelData[i].Manufacturers[mfrIndex].ModelData;
                double credits = ComputeNetCredits(pMMD, rc);
                if (credits >= 0) { continue; }
                DoTransfer(mfrName, eCredits, -credits, "1:1", ct, rc, yrIndex, mmd, rc, i, pMMD);
                eCredits = ComputeNetCredits(mmd, rc);
                if (eCredits <= 0) { return; }
            } 
        }
        static int ComputeCarryFwdStartYr(Industry[] modelData, int yrIndex, CreditTradingSettings ct)
        {
            return Math.Max(modelData[yrIndex].MinYear.Index, yrIndex - ct.CarryForwardYears);
        }
        static int ComputeCarryBwdStartYr(Industry[] modelData, int yrIndex, CreditTradingSettings ct)
        {
            return Math.Max(modelData[yrIndex].MinYear.Index, yrIndex - ct.CarryBackwardYears);
        }
        static double ComputeNetCredits(Manufacturer.CModelData mmd, RegulatoryClass rc)
        {
            return Math.Round(mmd.Credits[rc] + mmd.TCreditsIn[rc] - mmd.TCreditsOut[rc], 1);
        }
        static double ComputeTransferCap(Manufacturer.CModelData mmd, RegulatoryClass rc, int yrIndex, CreditTradingSettings ct)
        {
            return Math.Round(Math.Round(ct.TransferCaps[Math.Min(yrIndex, ct.TransferCaps.Count - 1)] * mmd.Sales[rc], 1) - mmd.TCreditsInCapped[rc], 1);
        }
        static double ComputeVMT(CreditTradingSettings ct, RegulatoryClass rc, int yrIndex)
        {
            if (rc == RegulatoryClass.DomesticAuto) { return ct.CreditAdjustmentVMT_DPC[Math.Min(yrIndex, ct.CreditAdjustmentVMT_DPC.Count - 1)]; }
            if (rc == RegulatoryClass.ImportedAuto) { return ct.CreditAdjustmentVMT_IPC[Math.Min(yrIndex, ct.CreditAdjustmentVMT_IPC.Count - 1)]; }
            if (rc == RegulatoryClass.LightTruck  ) { return ct.CreditAdjustmentVMT_LT [Math.Min(yrIndex, ct.CreditAdjustmentVMT_LT .Count - 1)]; }
            return 0;
        }
        static void DoTransfer(string mfrName, double eCredits, double rCredits, string adjFactor, CreditTradingSettings ct,
            RegulatoryClass eRC, int eYrIndex, Manufacturer.CModelData eMMD,    
            RegulatoryClass uRC, int uYrIndex, Manufacturer.CModelData uMMD     
            )
        {
            double outECredits, outRCredits;
            RequestCredits(eCredits, rCredits, adjFactor, ct, eRC, eYrIndex, eMMD, uRC, uYrIndex, uMMD, out outECredits, out outRCredits);
            eMMD.TCreditsOut[eRC] = Math.Round(eMMD.TCreditsOut[eRC] + outECredits, 1);
            uMMD.TCreditsIn [uRC] = Math.Round(uMMD.TCreditsIn [uRC] + outRCredits, 1);
            if (eRC != uRC) { uMMD.TCreditsInCapped[uRC] = Math.Round(uMMD.TCreditsInCapped[uRC] + outRCredits, 1); }
        }
        static void RequestCredits(double eCredits, double rCredits, string adjFactor, CreditTradingSettings ct,
            RegulatoryClass eRC, int eYrIndex, Manufacturer.CModelData eMMD,    
            RegulatoryClass uRC, int uYrIndex, Manufacturer.CModelData uMMD,    
            out double outECredits, out double outRCredits)
        {
            double eVMT = 0, uVMT = 0;
            if (adjFactor != "1:1")
            {
                eVMT = ComputeVMT(ct, eRC, eYrIndex);
                uVMT = ComputeVMT(ct, uRC, uYrIndex);
            }
            RequestCredits(eCredits, rCredits, adjFactor, eVMT, eMMD.CAFE[eRC], eMMD.Standard[eRC], uVMT, uMMD.CAFE[uRC], uMMD.Standard[uRC],
                out outECredits, out outRCredits);
        }
        static void RequestCredits(double eCredits, double rCredits, string adjFactor,
            double eVMT, double eCAFE, double eSTND, double uVMT, double uCAFE, double uSTND,
            out double outECredits, out double outRCredits)
        {
            if (double.IsInfinity(rCredits) && double.IsInfinity(eCredits)) { outECredits = outRCredits = 0; }
            if (!double.IsInfinity(eCredits))
            {
                outRCredits = GetAdjustedTCredits(eCredits, adjFactor, eVMT, eCAFE, eSTND, uVMT, uCAFE, uSTND);
                if (double.IsInfinity(rCredits) || outRCredits <= rCredits)
                {   
                    outECredits = Math.Round(eCredits, 1);
                    return;
                }
            }
            outRCredits = Math.Round(rCredits, 1);
            outECredits = GetRequiredECredits(rCredits, adjFactor, eVMT, eCAFE, eSTND, uVMT, uCAFE, uSTND);
        }
        #region 
        static double GetAdjustedTCredits(double eCredits, string adjFactor, double eVMT, double eCAFE, double eSTND, double uVMT, double uCAFE, double uSTND)
        {
            return Math.Round(eCredits / GetCreditAdjFactor(adjFactor, eVMT, eCAFE, eSTND, uVMT, uCAFE, uSTND), 1);
        }
        static double GetRequiredECredits(double tCredits, string adjFactor, double eVMT, double eCAFE, double eSTND, double uVMT, double uCAFE, double uSTND)
        {
            return Math.Round(tCredits * GetCreditAdjFactor(adjFactor, eVMT, eCAFE, eSTND, uVMT, uCAFE, uSTND), 1);
        }
        static double GetCreditAdjFactor(string adjFactor, double eVMT, double eCAFE, double eSTND, double uVMT, double uCAFE, double uSTND)
        {
            switch (adjFactor.ToLower())
            {
                case "1:1"     : return 1;
                case "variable": return (uVMT * eCAFE * eSTND) / (eVMT * uCAFE * uSTND);
                default: throw new ArgumentException("The Credit Adjustment Factor is not valid.");
            }
        }
        #endregion
    }
}

