#region << Using Directives >>
using System;
using System.Collections.Generic;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
#endregion
namespace Volpe.Cafe.Model
{
    [Serializable]
    public class DynamicFleetShare
    {
        #region 
        #endregion
        #region 
        public static void AdjustFleetShare(Industry data, Industry prevData, ModelingSettings settings, Scenario scen, ModelYear year)
        {
            OperatingModes opModes = settings.OperatingModes;
            double pcShare, pcCAFE, ltCAFE;
            if (year.Year == data.MinYear.Year)
            {   
                pcShare = opModes.DFS_InitialPCShare;
                pcCAFE  = opModes.DFS_InitialPCCAFE;
                ltCAFE  = opModes.DFS_InitialLTCAFE;
            }
            else
            {   
                AdjustFleetShare_GetPreviousShare(prevData, year, out pcShare, out pcCAFE, out ltCAFE);
            }
            double gasPrice   = settings.Parameters.ForecastData.GetRetailFuelPrice(year, opModes.FuelPriceEstimates, FuelType.Gasoline);
            double epsilon    = opModes.DFS_Epsilon;
            double newPCShare = 0.259694 + 0.222462 * pcShare + 0.005969 * pcCAFE - 0.004116 * ltCAFE + 0.018314 * gasPrice + epsilon;
            List<Manufacturer> mfrs = data.Manufacturers;
            double totalSales = 0, pcSales = 0, ltSales = 0;
            for (int i = 0, mfrCount = mfrs.Count; i < mfrCount; i++)
            {   
                List<Vehicle> vehs = mfrs[i].Vehicles;
                for (int j = 0, vehCount = vehs.Count; j < vehCount; j++)
                {
                    double vehSales = vehs[j].Description.Sales[year.Index];
                    totalSales += vehSales;
                    if (vehs[j].IsAuto) { pcSales += vehSales; }
                    else                { ltSales += vehSales; }
                }
            }
            double newPCSales = totalSales *      newPCShare ;
            double newLTSales = totalSales * (1 - newPCShare);
            for (int i = 0, mfrCount = mfrs.Count; i < mfrCount; i++)
            {
                List<Vehicle> vehs = mfrs[i].Vehicles;
                for (int j = 0, vehCount = vehs.Count; j < vehCount; j++)
                {
                    double ratio = (vehs[j].IsAuto) ? newPCSales / pcSales : newLTSales / ltSales;
                    vehs[j].Description.Sales[year.Index] *= ratio;
                }
            }
        }
        static void AdjustFleetShare_GetPreviousShare(Industry prevData, ModelYear year,
            out double pcShare, out double pcCAFE, out double ltCAFE)
        {
            double totalSales = 0, pcSales = 0, ltSales = 0;
            pcCAFE = 0; ltCAFE = 0;
            List<Manufacturer> mfrs = prevData.Manufacturers;
            for (int i = 0, mfrCount = mfrs.Count; i < mfrCount; i++)
            {
                List<Vehicle> vehs = mfrs[i].Vehicles;
                for (int j = 0, vehCount = vehs.Count; j < vehCount; j++)
                {
                    double vehSales = vehs[j].Description.Sales[year.Index];
                    double vehFE    = Standards.GetVehicleFuelEconomy(vehs[j].Description.FuelEconomy, vehs[j].Description.FuelShare, 1);
                    totalSales += vehSales;
                    if (vehs[j].IsAuto)
                    {
                        pcSales += vehSales;
                        pcCAFE  += vehSales / vehFE;
                    }
                    else
                    {
                        ltSales += vehSales;
                        ltCAFE  += vehSales / vehFE;
                    }
                } 
            } 
            pcShare = pcSales / totalSales;
            pcCAFE  = pcSales / pcCAFE;
            ltCAFE  = ltSales / ltCAFE;
        }
        #endregion
        #region 
        public const double DefaultInitialPCShare = 0.5205;
        public const double DefaultInitialPCCAFE = 33.3;
        public const double DefaultInitialLTCAFE = 24.3;
        #endregion
    }
}

