using System;
using System.Collections.Generic;
using System.IO;
using Volpe.Cafe;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.Model
{
    [Serializable]
	public class Effects
	{
        #region 
        static Effects()
        {
            FuelTypes = new FuelType[] {FuelType.Gasoline, FuelType.Diesel, FuelType.Ethanol85, FuelType.Electricity, FuelType.Hydrogen};
            FuelTypeCount = FuelTypes.Length;
            Mobile6Classes = new Mobile6Class[] {Mobile6Class.LDV, Mobile6Class.LDT1, Mobile6Class.LDT2, Mobile6Class.HDV};
            Mobile6ClassCount = Mobile6Classes.Length;
        }
        public Effects()
        {
            this.FuelShares    = new double[FuelTypeCount][];
            this.EnergyDensity = new double[FuelTypeCount];
            this.MassDensity   = new double[FuelTypeCount];
            this.CarbonContent = new double[FuelTypeCount];
            this.SOXRate       = new double[FuelTypeCount];
            for (int i = 0; i < FuelTypeCount; i++)
            {
                this.FuelShares[i] = new double[4];
            }
            this.UE_CO2 = new double[FuelTypeCount];
            this.UE_CO  = new double[FuelTypeCount];
            this.UE_VOC = new double[FuelTypeCount];
            this.UE_NOX = new double[FuelTypeCount];
            this.UE_PM  = new double[FuelTypeCount];
            this.UE_SOX = new double[FuelTypeCount];
            this.UE_CH4 = new double[FuelTypeCount];
            this.UE_N2O = new double[FuelTypeCount];
            this.TE_CO  = new double[FuelTypeCount][][];
            this.TE_VOC = new double[FuelTypeCount][][];
            this.TE_NOX = new double[FuelTypeCount][][];
            this.TE_PM  = new double[FuelTypeCount][][];
            this.TE_CH4 = new double[FuelTypeCount][][];
            this.TE_N2O = new double[FuelTypeCount][][];
            for (int i = 0; i < FuelTypeCount; i++)
            {
                this.TE_CO [i] = new double[Mobile6ClassCount][];
                this.TE_VOC[i] = new double[Mobile6ClassCount][];
                this.TE_NOX[i] = new double[Mobile6ClassCount][];
                this.TE_PM [i] = new double[Mobile6ClassCount][];
                this.TE_CH4[i] = new double[Mobile6ClassCount][];
                this.TE_N2O[i] = new double[Mobile6ClassCount][];
                for (int j = 0; j < Mobile6ClassCount; j++)
                {
                    if (FuelTypes[i] == FuelType.Electricity || FuelTypes[i] == FuelType.Hydrogen)
                    {
                        TE_CO [i][j] = null;
                        TE_VOC[i][j] = null;
                        TE_NOX[i][j] = null;
                        TE_PM [i][j] = null;
                        TE_CH4[i][j] = null;
                        TE_N2O[i][j] = null;
                    }
                    else
                    {
                        TE_CO [i][j] = new double[Interaction.CY];
                        TE_VOC[i][j] = new double[Interaction.CY];
                        TE_NOX[i][j] = new double[Interaction.CY];
                        TE_PM [i][j] = new double[Interaction.CY];
                        TE_CH4[i][j] = new double[Interaction.CY];
                        TE_N2O[i][j] = new double[Interaction.CY];
                    }
                }
            }
            this.FuelTax         = new double[FuelTypeCount][];
            this.PreTaxFP        = new double[FuelTypeCount][];
            this.RetailFP        = new double[FuelTypeCount][];
            this.RetailFPBaseVMT = new double[FuelTypeCount];
            for (int i = 0; i < FuelTypeCount; i++)
            {
                this.FuelTax [i] = new double[Interaction.CY];
                this.PreTaxFP[i] = new double[Interaction.CY];
                this.RetailFP[i] = new double[Interaction.CY];
            }
        }
        #endregion
        #region 
        public void Begin(Scenario scen, ModelYear year, Industry bData, Industry sData, ModelingSettings settings)
        {
            this._bData      = bData;
            this._sData      = sData;
            this._settings   = settings;
            this._parameters = settings.Parameters;
            this._scen       = scen;
            this._year       = year;
            this.InitializeFuelPropertiesValues();
            this.InitializeUpstreamEmissionsArrays();
            this.InitializeTailpipeEmissionsArrays();
            this.InitializeFuelPriceArrays();
            this.CalculateIndustryEffects();
        }
        void CalculateIndustryEffects()
        {
            int drCount = this._parameters.EconomicData.DiscountRates.Length;
            EffectsData ied = this._sData.EffectsData;
            ied.Reset(drCount);
            List<Manufacturer> bMfrs = this._bData.Manufacturers;
            List<Manufacturer> sMfrs = this._sData.Manufacturers;
            for (int i = 0, mfrCount = bMfrs.Count; i < mfrCount; i++)
            {
                Manufacturer bMfr = bMfrs[i];
                Manufacturer sMfr = sMfrs[i];
                EffectsData  med  = sMfr.EffectsData;
                med.Reset(drCount);
                List<Vehicle> bVehs = bMfr.Vehicles;
                List<Vehicle> sVehs = sMfr.Vehicles;
                for (int j = 0, vehCount = bVehs.Count; j < vehCount; j++)
                {
                    this.CalculateVehicleEffects(bVehs[j], sVehs[j], med);
                } 
                med.UndiscountedOwnerAndSocietalCosts.CalcSubtotals();
                for (int j = 0; j < med.DiscountedOwnerAndSocietalCosts.Length; j++)
                {
                    med.DiscountedOwnerAndSocietalCosts[j].CalcSubtotals();
                }
                ied.Sales   += med.Sales;
                ied.VMT     += med.VMT;
                ied.BTU     += med.BTU;
                ied.Gallons += med.Gallons;
                ied.KWh     += med.KWh;
                ied.Scf     += med.Scf;
                for (int j = 0; j < Interaction.CY; j++)
                {
                    ied.AnnualVMT    [j] += med.AnnualVMT    [j];
                    ied.AnnualBTU    [j] += med.AnnualBTU    [j];
                    ied.AnnualGallons[j] += med.AnnualGallons[j];
                    ied.AnnualKWh    [j] += med.AnnualKWh    [j];
                    ied.AnnualScf    [j] += med.AnnualScf    [j];
                }
                ied.EmissionsCO2 += med.EmissionsCO2;
                ied.EmissionsCO  += med.EmissionsCO ;
                ied.EmissionsVOC += med.EmissionsVOC;
                ied.EmissionsNOX += med.EmissionsNOX;
                ied.EmissionsPM  += med.EmissionsPM ;
                ied.EmissionsSOX += med.EmissionsSOX;
                ied.EmissionsCH4 += med.EmissionsCH4;
                ied.EmissionsN2O += med.EmissionsN2O;
                for (int j = 0; j < Interaction.CY; j++)
                {
                    ied.AnnualEmissionsCO2[j] += med.AnnualEmissionsCO2[j];
                    ied.AnnualEmissionsCO [j] += med.AnnualEmissionsCO [j];
                    ied.AnnualEmissionsVOC[j] += med.AnnualEmissionsVOC[j];
                    ied.AnnualEmissionsNOX[j] += med.AnnualEmissionsNOX[j];
                    ied.AnnualEmissionsPM [j] += med.AnnualEmissionsPM [j];
                    ied.AnnualEmissionsSOX[j] += med.AnnualEmissionsSOX[j];
                    ied.AnnualEmissionsCH4[j] += med.AnnualEmissionsCH4[j];
                    ied.AnnualEmissionsN2O[j] += med.AnnualEmissionsN2O[j];
                }
                ied.SftSales       += med.SftSales;
                ied.SftVMT         += med.SftVMT;
                ied.SftCurbWeight  += med.SftCurbWeight;
                ied.SftFatalities  += med.SftFatalities;
                for (int j = 0; j < Interaction.CY; j++)
                {
                    ied.AnnualSftFatalities[j] += med.AnnualSftFatalities[j];
                }
                ied.UndiscountedOwnerAndSocietalCosts.Add(med.UndiscountedOwnerAndSocietalCosts);
                for (int j = 0; j < med.DiscountedOwnerAndSocietalCosts.Length; j++)
                {
                    ied.DiscountedOwnerAndSocietalCosts[j].Add(med.DiscountedOwnerAndSocietalCosts[j]);
                }
                FuelValue mfrTtlVMT     = med.VMT    .TotalAll;
                FuelValue mfrTtlGallons = med.Gallons.TotalAll;
                for (int j = 0; j < FuelTypeCount; j++)
                {
                    FuelType fuel = FuelTypes[j];
                    if (mfrTtlGallons[fuel] == 0) { continue; }
                    double gapMult = 1 - this._parameters.EconomicData.TestVsOnRoadGap[fuel];
                    med.AverageOnRoadFuelEconomy[fuel] = mfrTtlVMT[fuel] / mfrTtlGallons[fuel];
                    med.AverageRatedFuelEconomy [fuel] = med.AverageOnRoadFuelEconomy[fuel] / gapMult;
                    for (int k = 0; k <= 3; k++)
                    {
                        RegulatoryClass regClass = (RegulatoryClass)k;
                        if (med.Gallons[regClass, fuel] != 0)
                        {
                            med.OnRoadFuelEconomy[regClass, fuel] = med.VMT[regClass, fuel] / med.Gallons[regClass, fuel];
                            med.RatedFuelEconomy [regClass, fuel] = med.OnRoadFuelEconomy[regClass, fuel] / gapMult;
                        }
                    }
                }
                for (int j = 0; j <= 3; j++)
                {
                    RegulatoryClass regClass = (RegulatoryClass)j;
                    SFTDouble sales = med.SftSales[regClass];
                    SFTDouble cw = med.SftCurbWeight[regClass];
                    med.SftCurbWeight[regClass, SafetyClass.PC] = (sales.PC == 0) ? 0 : cw.PC / sales.PC;
                    med.SftCurbWeight[regClass, SafetyClass.LT] = (sales.LT == 0) ? 0 : cw.LT / sales.LT;
                    med.SftCurbWeight[regClass, SafetyClass.CM] = (sales.CM == 0) ? 0 : cw.CM / sales.CM;
                }
            } 
            FuelValue indTtlVMT     = ied.VMT    .TotalAll;
            FuelValue indTtlGallons = ied.Gallons.TotalAll;
            for (int j = 0; j < FuelTypeCount; j++)
            {
                FuelType fuel = FuelTypes[j];
                if (indTtlGallons[fuel] == 0) { continue; }
                double gapMult = 1 - this._parameters.EconomicData.TestVsOnRoadGap[fuel];
                ied.AverageOnRoadFuelEconomy[fuel] = indTtlVMT[fuel] / indTtlGallons[fuel];
                ied.AverageRatedFuelEconomy [fuel] = ied.AverageOnRoadFuelEconomy[fuel] / gapMult;
                for (int k = 0; k <= 3; k++)
                {
                    RegulatoryClass regClass = (RegulatoryClass)k;
                    if (ied.Gallons[regClass, fuel] != 0)
                    {
                        ied.OnRoadFuelEconomy[regClass, fuel] = ied.VMT[regClass, fuel] / ied.Gallons[regClass, fuel];
                        ied.RatedFuelEconomy [regClass, fuel] = ied.OnRoadFuelEconomy[regClass, fuel] / gapMult;
                    }
                }
            }
            for (int j = 0; j <= 3; j++)
            {
                RegulatoryClass regClass = (RegulatoryClass)j;
                SFTDouble sales = ied.SftSales     [regClass];
                SFTDouble cw    = ied.SftCurbWeight[regClass];
                ied.SftCurbWeight[regClass, SafetyClass.PC] = (sales.PC == 0) ? 0 : cw.PC / sales.PC;
                ied.SftCurbWeight[regClass, SafetyClass.LT] = (sales.LT == 0) ? 0 : cw.LT / sales.LT;
                ied.SftCurbWeight[regClass, SafetyClass.CM] = (sales.CM == 0) ? 0 : cw.CM / sales.CM;
            }
        }
        void CalculateVehicleEffects(Vehicle bVeh, Vehicle sVeh, EffectsData med)
        {
            Parameters.EconomicDataValues economicData = this._parameters.EconomicData;
            Parameters.SafetyDataValues   safetyData   = this._parameters.SafetyData;
            Estimates fuelPriceEstimates   = this._settings.OperatingModes.FuelPriceEstimates;
            double    gasEnergyDensity     = this._parameters.FuelProperties.EnergyDensity.Gasoline;
            double    consumerBenefitsMult = (this._settings.ParametersOverrides.ScaleConsumerBenefits) ?
                                              this._settings.ParametersOverrides.ConsumerBenefits : 1;
            double[] survRate = this._parameters.GetVehicleSurvivability(sVeh);
            double[] milesDrv = this._parameters.GetVehicleMilesDriven  (sVeh);
            double sVehSales = sVeh.Description.Sales[this._year.Index];
            FuelValue bFE = this.GetVehicleFE(bVeh);    FuelValue bFS = bVeh.Description.FuelShare;
            FuelValue sFE = this.GetVehicleFE(sVeh);    FuelValue sFS = sVeh.Description.FuelShare;
            FuelValue fuelCapacity = new FuelValue();
            for (int i = 0; i < FuelTypeCount; i++) { fuelCapacity[FuelTypes[i]] = sVeh.Description.FuelCapacity; }
            Mobile6Class    mblClass = sVeh.Mobile6Class;
            RegulatoryClass regClass = sVeh.RegClass;
            SafetyClass     sftClass = sVeh.SafetyClass;
            bool isCar    = (mblClass == Mobile6Class.LDV);
            int  mblIndex = (int)mblClass - 1;
            CO2Estimates co2Estimates = this._settings.OperatingModes.CO2Estimates;
            double[]     ed_CO2Cost;
            double       ed_CO2DiscRate;
            economicData.GetCO2Estimates(co2Estimates, out ed_CO2Cost, out ed_CO2DiscRate);
            double sales = sVeh.Description.Sales[this._year.Index];
            double baseWeight       = sVeh.Description.BaseWeight;
            double curbWeight       = sVeh.Description.CurbWeight;
            double deltaWeight      = curbWeight - baseWeight;
            bool   hasWeightChanged = (deltaWeight != 0);
            for (int i = 0; i < Interaction.CY; i++)
            {
                double bCPM, cpm, bCPMRate, sCPMRate;
                this.ComputeVehicleCPM(isCar, bFE, bFS, sFE, sFS, i, out bCPM, out cpm, out bCPMRate, out sCPMRate);
                double vmtGrowth = economicData.GetVMTGrowthFactor(fuelPriceEstimates, this._year.Year, i);
                FuelValue bVMT = new FuelValue();
                FuelValue bGal = new FuelValue();
                for (int j = 0; j < FuelTypeCount; j++)
                {
                    FuelType fuel = FuelTypes[j];
                    if (bFS[fuel] == 0) { continue; }
                    bVMT[fuel] = bFS[fuel] * sales * survRate[i] * milesDrv[i] * vmtGrowth / 1000 * (1 + bCPMRate);
                    bGal[fuel] = bVMT[fuel] / bFE[fuel];
                }
                FuelValue vmt = new FuelValue();
                FuelValue btu = new FuelValue(), gal = new FuelValue(), kwh = new FuelValue(), scf = new FuelValue();
                FuelValue energyUse = new FuelValue();     
                for (int j = 0; j < FuelTypeCount; j++)
                {
                    FuelType fuel = FuelTypes[j];
                    if (sFS[fuel] == 0) { continue; }
                    vmt[fuel] = sFS[fuel] * sales * survRate[i] * milesDrv[i] * vmtGrowth / 1000 * (1 + sCPMRate);
                    gal[fuel] = vmt[fuel] / sFE[fuel];
                    if (fuel == FuelType.Hydrogen || fuel == FuelType.Electricity)
                    {
                        btu      [fuel] = gal[fuel] * gasEnergyDensity;
                        energyUse[fuel] = btu[fuel] / EnergyDensity[j];
                        if (fuel == FuelType.Hydrogen) { scf[fuel] = energyUse[fuel]; }
                        else                           { kwh[fuel] = energyUse[fuel]; }
                    }
                    else
                    {
                        btu      [fuel] = gal[fuel] * EnergyDensity[j];
                        energyUse[fuel] = gal[fuel];
                    }
                }
                FuelValue
                    emCO2 = new FuelValue(), emCO  = new FuelValue(), emVOC = new FuelValue(),
                    emNOX = new FuelValue(), emPM  = new FuelValue(), emSOX = new FuelValue(),
                    emCH4 = new FuelValue(), emN2O = new FuelValue();
                for (int j = 0; j < FuelTypeCount; j++)
                {
                    FuelType fuel = FuelTypes[j];
                    if (sFS[fuel] == 0) { continue; }
                    emCO2[fuel] = this.CalculateEmissions(energyUse[fuel], btu[fuel], UE_CO2[j], j, vmt[fuel], null               , i, false, true , true );
                    emCO [fuel] = this.CalculateEmissions(energyUse[fuel], btu[fuel], UE_CO [j], j, vmt[fuel], TE_CO [j][mblIndex], i, false, false, false);
                    emVOC[fuel] = this.CalculateEmissions(energyUse[fuel], btu[fuel], UE_VOC[j], j, vmt[fuel], TE_VOC[j][mblIndex], i, false, false, false);
                    emNOX[fuel] = this.CalculateEmissions(energyUse[fuel], btu[fuel], UE_NOX[j], j, vmt[fuel], TE_NOX[j][mblIndex], i, false, false, false);
                    emPM [fuel] = this.CalculateEmissions(energyUse[fuel], btu[fuel], UE_PM [j], j, vmt[fuel], TE_PM [j][mblIndex], i, false, false, false);
                    emSOX[fuel] = this.CalculateEmissions(energyUse[fuel], btu[fuel], UE_SOX[j], j, vmt[fuel], null               , i, true , false, false);
                    emCH4[fuel] = this.CalculateEmissions(energyUse[fuel], btu[fuel], UE_CH4[j], j, vmt[fuel], TE_CH4[j][mblIndex], i, false, false, true );
                    emN2O[fuel] = this.CalculateEmissions(energyUse[fuel], btu[fuel], UE_N2O[j], j, vmt[fuel], TE_N2O[j][mblIndex], i, false, false, true );
                }
                double sftFatals = 0;
                double sftVMT = vmt.Total;
                if (hasWeightChanged)
                {   
                    Parameters.SafetyDataValues.SafetyParameters sftP;
                    if      (sftClass == SafetyClass.PC) { sftP = safetyData.PC; }
                    else if (sftClass == SafetyClass.LT) { sftP = safetyData.LT; }
                    else if (sftClass == SafetyClass.CM) { sftP = safetyData.CM; }
                    else { sftP = new Parameters.SafetyDataValues.SafetyParameters(); }
                    if (sftClass != SafetyClass.None)
                    {
                        double wrAbove = Math.Max(0, baseWeight - Math.Max(curbWeight, sftP.Threshold));
                        double wrBelow = Math.Max(0, Math.Min(baseWeight, sftP.Threshold) - curbWeight);
                        sftFatals = sftVMT / 1e6 / 100 * (
                            wrAbove * sftP.LargeEffect * sftP.LargeBase * sftP.LargeFMVSS +
                            wrBelow * sftP.SmallEffect * sftP.SmallBase * sftP.SmallFMVSS);
                    }
                }
                double preTaxCost = 0, fuelTaxCost = 0;
                for (int j = 0; j < FuelTypeCount; j++)
                {
                    FuelType fuel = FuelTypes[j];
                    if (sFS[fuel] == 0) { continue; }
                    preTaxCost  += gal[fuel] * PreTaxFP[j][i] * consumerBenefitsMult;
                    fuelTaxCost += gal[fuel] * FuelTax [j][i] * consumerBenefitsMult;
                }
                double driveSurplus  = (bVMT.Total - vmt.Total) * (bCPM + cpm) / 2;
                double refuelSurplus = 0;
                for (int j = 0; j < FuelTypeCount; j++)
                {
                    FuelType fuel = FuelTypes[j];
                    if (fuel == FuelType.Electricity || fuel == FuelType.Hydrogen || sFS[fuel] == 0) { continue; }
                    refuelSurplus = economicData.VehicleTravelTimeValue / economicData.RefuelTankVolume * economicData.RefuelTime[fuel] / 60 * (gal[fuel] - bGal[fuel]) / fuelCapacity[fuel];
                }
                double economicCosts   = 0;
                for (int j = 0; j < FuelTypeCount; j++)
                {
                    FuelType fuel = FuelTypes[j];
                    if (sFS[fuel] == 0) { continue; }
                    economicCosts += gal[fuel] * (FuelShares[j][0] + FuelShares[j][1] * FuelShares[j][3]) *
                        (economicData.MonopsonyCosts + economicData.PriceShockCosts + economicData.MilitarySecurityCosts);
                }
                double congestionCosts = vmt.Total * ((isCar) ? economicData.CarCongestionCosts : economicData.TrkCongestionCosts);
                double accidentCosts   = vmt.Total * ((isCar) ? economicData.CarAccidentCosts   : economicData.TrkAccidentCosts);
                double noiseCosts      = vmt.Total * ((isCar) ? economicData.CarNoiseCosts      : economicData.TrkNoiseCosts);
                double fatalityCosts   = sftFatals * safetyData.FatalityCosts / 1000;
                double emCostCO2 = emCO2.Total * ed_CO2Cost[this._year.Index + i] * 1000;
                double emCostCO  = emCO .Total * economicData.CODamageCosts  / 1000;
                double emCostVOC = emVOC.Total * economicData.VOCDamageCosts / 1000;
                double emCostNOX = emNOX.Total * economicData.NOXDamageCosts / 1000;
                double emCostPM  = emPM .Total * economicData.PMDamageCosts  / 1000;
                double emCostSOX = emSOX.Total * economicData.SOXDamageCosts / 1000;
                double emCostCH4 = emCH4.Total * economicData.CH4DamageCostScalar * ed_CO2Cost[this._year.Index + i] * 1000;
                double emCostN2O = emN2O.Total * economicData.N2ODamageCostScalar * ed_CO2Cost[this._year.Index + i] * 1000;
                med.VMT    .Add(regClass, vmt);
                med.BTU    .Add(regClass, btu);
                med.Gallons.Add(regClass, gal);
                med.KWh    .Add(regClass, kwh);
                med.Scf    .Add(regClass, scf);
                med.AnnualVMT    [i][regClass] += vmt.Total;
                med.AnnualBTU    [i][regClass] += btu.Total;
                med.AnnualGallons[i][regClass] += gal.Total;
                med.AnnualKWh    [i][regClass] += kwh.Total;
                med.AnnualScf    [i][regClass] += scf.Total;
                med.EmissionsCO2.Add(regClass, emCO2);
                med.EmissionsCO .Add(regClass, emCO );
                med.EmissionsVOC.Add(regClass, emVOC);
                med.EmissionsNOX.Add(regClass, emNOX);
                med.EmissionsPM .Add(regClass, emPM );
                med.EmissionsSOX.Add(regClass, emSOX);
                med.EmissionsCH4.Add(regClass, emCH4);
                med.EmissionsN2O.Add(regClass, emN2O);
                med.AnnualEmissionsCO2[i][regClass] += emCO2.Total;
                med.AnnualEmissionsCO [i][regClass] += emCO .Total;
                med.AnnualEmissionsVOC[i][regClass] += emVOC.Total;
                med.AnnualEmissionsNOX[i][regClass] += emNOX.Total;
                med.AnnualEmissionsPM [i][regClass] += emPM .Total;
                med.AnnualEmissionsSOX[i][regClass] += emSOX.Total;
                med.AnnualEmissionsCH4[i][regClass] += emCH4.Total;
                med.AnnualEmissionsN2O[i][regClass] += emN2O.Total;
                med.SftVMT       [regClass, sftClass] += sftVMT;
                med.SftFatalities[regClass, sftClass] += sftFatals;
                med.AnnualSftFatalities[i][regClass]  += sftFatals;
                med.UndiscountedOwnerAndSocietalCosts.DiscountRate = 0;
                med.UndiscountedOwnerAndSocietalCosts.ConsumerPreTaxFuelCost    [regClass] += preTaxCost;
                med.UndiscountedOwnerAndSocietalCosts.ConsumerFuelTaxCost       [regClass] += fuelTaxCost;
                med.UndiscountedOwnerAndSocietalCosts.ConsumerDriveSurplus      [regClass] += driveSurplus;
                med.UndiscountedOwnerAndSocietalCosts.ConsumerRefuelSurplus     [regClass] += refuelSurplus;
                med.UndiscountedOwnerAndSocietalCosts.PreTaxFuelCost            [regClass] += preTaxCost;
                med.UndiscountedOwnerAndSocietalCosts.FuelTaxCost               [regClass] += fuelTaxCost;
                med.UndiscountedOwnerAndSocietalCosts.DriveSurplus              [regClass] += driveSurplus;
                med.UndiscountedOwnerAndSocietalCosts.RefuelSurplus             [regClass] += refuelSurplus;
                med.UndiscountedOwnerAndSocietalCosts.EconomicCosts             [regClass] += economicCosts;
                med.UndiscountedOwnerAndSocietalCosts.CongestionCosts           [regClass] += congestionCosts;
                med.UndiscountedOwnerAndSocietalCosts.AccidentCosts             [regClass] += accidentCosts;
                med.UndiscountedOwnerAndSocietalCosts.NoiseCosts                [regClass] += noiseCosts;
                med.UndiscountedOwnerAndSocietalCosts.FatalityCosts             [regClass] += fatalityCosts;
                med.UndiscountedOwnerAndSocietalCosts.EmissionsCostsCO2         [regClass] += emCostCO2;
                med.UndiscountedOwnerAndSocietalCosts.EmissionsCostsCO          [regClass] += emCostCO;
                med.UndiscountedOwnerAndSocietalCosts.EmissionsCostsVOC         [regClass] += emCostVOC;
                med.UndiscountedOwnerAndSocietalCosts.EmissionsCostsNOX         [regClass] += emCostNOX;
                med.UndiscountedOwnerAndSocietalCosts.EmissionsCostsPM          [regClass] += emCostPM;
                med.UndiscountedOwnerAndSocietalCosts.EmissionsCostsSOX         [regClass] += emCostSOX;
                med.UndiscountedOwnerAndSocietalCosts.EmissionsCostsCH4         [regClass] += emCostCH4;
                med.UndiscountedOwnerAndSocietalCosts.EmissionsCostsN2O         [regClass] += emCostN2O;
                med.UndiscountedOwnerAndSocietalCosts.AnnualPreTaxFuelCost   [i][regClass] += preTaxCost;
                med.UndiscountedOwnerAndSocietalCosts.AnnualFuelTaxCost      [i][regClass] += fuelTaxCost;
                med.UndiscountedOwnerAndSocietalCosts.AnnualDriveSurplus     [i][regClass] += driveSurplus;
                med.UndiscountedOwnerAndSocietalCosts.AnnualRefuelSurplus    [i][regClass] += refuelSurplus;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEconomicCosts    [i][regClass] += economicCosts;
                med.UndiscountedOwnerAndSocietalCosts.AnnualCongestionCosts  [i][regClass] += congestionCosts;
                med.UndiscountedOwnerAndSocietalCosts.AnnualAccidentCosts    [i][regClass] += accidentCosts;
                med.UndiscountedOwnerAndSocietalCosts.AnnualNoiseCosts       [i][regClass] += noiseCosts;
                med.UndiscountedOwnerAndSocietalCosts.AnnualFatalityCosts    [i][regClass] += fatalityCosts;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEmissionsCostsCO2[i][regClass] += emCostCO2;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEmissionsCostsCO [i][regClass] += emCostCO;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEmissionsCostsVOC[i][regClass] += emCostVOC;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEmissionsCostsNOX[i][regClass] += emCostNOX;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEmissionsCostsPM [i][regClass] += emCostPM;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEmissionsCostsSOX[i][regClass] += emCostSOX;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEmissionsCostsCH4[i][regClass] += emCostCH4;
                med.UndiscountedOwnerAndSocietalCosts.AnnualEmissionsCostsN2O[i][regClass] += emCostN2O;
                double prvDiscRate = Math.Pow(1 + economicData.PrivateDiscountRate, -i);
                double co2DiscRate = Math.Pow(1 + ed_CO2DiscRate                  , -i);
                for (int j = 0; j < economicData.DiscountRates.Length; j++)
                {
                    double discRate = Math.Pow(1 + economicData.DiscountRates[j], -i);
                    med.DiscountedOwnerAndSocietalCosts[j].DiscountRate = economicData.DiscountRates[j];
                    med.DiscountedOwnerAndSocietalCosts[j].ConsumerPreTaxFuelCost    [regClass] += prvDiscRate * preTaxCost;
                    med.DiscountedOwnerAndSocietalCosts[j].ConsumerFuelTaxCost       [regClass] += prvDiscRate * fuelTaxCost;
                    med.DiscountedOwnerAndSocietalCosts[j].ConsumerDriveSurplus      [regClass] += prvDiscRate * driveSurplus;
                    med.DiscountedOwnerAndSocietalCosts[j].ConsumerRefuelSurplus     [regClass] += prvDiscRate * refuelSurplus;
                    med.DiscountedOwnerAndSocietalCosts[j].PreTaxFuelCost            [regClass] += discRate    * preTaxCost;
                    med.DiscountedOwnerAndSocietalCosts[j].FuelTaxCost               [regClass] += discRate    * fuelTaxCost;
                    med.DiscountedOwnerAndSocietalCosts[j].DriveSurplus              [regClass] += discRate    * driveSurplus;
                    med.DiscountedOwnerAndSocietalCosts[j].RefuelSurplus             [regClass] += discRate    * refuelSurplus;
                    med.DiscountedOwnerAndSocietalCosts[j].EconomicCosts             [regClass] += discRate    * economicCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].CongestionCosts           [regClass] += discRate    * congestionCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].AccidentCosts             [regClass] += discRate    * accidentCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].NoiseCosts                [regClass] += discRate    * noiseCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].FatalityCosts             [regClass] += discRate    * fatalityCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].EmissionsCostsCO2         [regClass] += co2DiscRate * emCostCO2;
                    med.DiscountedOwnerAndSocietalCosts[j].EmissionsCostsCO          [regClass] += discRate    * emCostCO;
                    med.DiscountedOwnerAndSocietalCosts[j].EmissionsCostsVOC         [regClass] += discRate    * emCostVOC;
                    med.DiscountedOwnerAndSocietalCosts[j].EmissionsCostsNOX         [regClass] += discRate    * emCostNOX;
                    med.DiscountedOwnerAndSocietalCosts[j].EmissionsCostsPM          [regClass] += discRate    * emCostPM;
                    med.DiscountedOwnerAndSocietalCosts[j].EmissionsCostsSOX         [regClass] += discRate    * emCostSOX;
                    med.DiscountedOwnerAndSocietalCosts[j].EmissionsCostsCH4         [regClass] += discRate    * emCostCH4;
                    med.DiscountedOwnerAndSocietalCosts[j].EmissionsCostsN2O         [regClass] += discRate    * emCostN2O;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualPreTaxFuelCost   [i][regClass] += discRate    * preTaxCost;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualFuelTaxCost      [i][regClass] += discRate    * fuelTaxCost;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualDriveSurplus     [i][regClass] += discRate    * driveSurplus;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualRefuelSurplus    [i][regClass] += discRate    * refuelSurplus;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEconomicCosts    [i][regClass] += discRate    * economicCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualCongestionCosts  [i][regClass] += discRate    * congestionCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualAccidentCosts    [i][regClass] += discRate    * accidentCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualNoiseCosts       [i][regClass] += discRate    * noiseCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualFatalityCosts    [i][regClass] += discRate    * fatalityCosts;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEmissionsCostsCO2[i][regClass] += co2DiscRate * emCostCO2;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEmissionsCostsCO [i][regClass] += discRate    * emCostCO;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEmissionsCostsVOC[i][regClass] += discRate    * emCostVOC;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEmissionsCostsNOX[i][regClass] += discRate    * emCostNOX;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEmissionsCostsPM [i][regClass] += discRate    * emCostPM;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEmissionsCostsSOX[i][regClass] += discRate    * emCostSOX;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEmissionsCostsCH4[i][regClass] += discRate    * emCostCH4;
                    med.DiscountedOwnerAndSocietalCosts[j].AnnualEmissionsCostsN2O[i][regClass] += discRate    * emCostN2O;
                }
            } 
            for (int i = 0; i < FuelTypeCount; i++)
            {
                FuelType fuel = FuelTypes[i];
                if (sFS[fuel] == 0) { continue; }
                med.Sales[regClass, fuel] += (sales * sFS[fuel]);
            }
            med.SftSales     [regClass, sftClass] += sales;
            med.SftCurbWeight[regClass, sftClass] += (sales * curbWeight);  
        }
        void InitializeFuelPropertiesValues()
        {
            Parameters.FuelPropertiesValues fp = this._parameters.FuelProperties;
            for (int i = 0; i < FuelTypeCount; i++)
            {
                FuelType f = FuelTypes[i];
                this.FuelShares   [i][0] = fp.ShareOfFuelSavingsLeadingToLowerImports           [f];
                this.FuelShares   [i][1] = fp.ShareOfFuelSavingsLeadingToReducedDomesticRefining[f];
                this.FuelShares   [i][2] = fp.ShareOfReducedDomesticRefiningFromDomesticCrude   [f];
                this.FuelShares   [i][3] = fp.ShareOfReducedDomesticRefiningFromImportedCrude   [f];
                this.EnergyDensity[i]    = fp.EnergyDensity[f];
                this.MassDensity  [i]    = fp.MassDensity  [f];
                this.CarbonContent[i]    = fp.CarbonContent[f];
                this.SOXRate      [i]    = fp.SOXRate      [f];
            }
        }
        void InitializeUpstreamEmissionsArrays()
        {
            Parameters.UpstreamEmissionsValues ue = this._parameters.UpstreamEmissions;
            for (int i = 0; i < FuelTypeCount; i++)
            {
                FuelType f = FuelTypes[i];
                this.UE_CO2[i] = ue.CO2[f];
                this.UE_CO [i] = ue.CO [f];
                this.UE_VOC[i] = ue.VOC[f];
                this.UE_NOX[i] = ue.NOX[f];
                this.UE_PM [i] = ue.PM [f];
                this.UE_SOX[i] = ue.SOX[f];
                this.UE_CH4[i] = ue.CH4[f];
                this.UE_N2O[i] = ue.N2O[f];
            }
        }
        void InitializeTailpipeEmissionsArrays()
        {
            TailpipeEmissions te = this._settings.TailpipeEmissions;
            for (int i = 0; i < FuelTypeCount; i++)
            {
                if (FuelTypes[i] == FuelType.Electricity || FuelTypes[i] == FuelType.Hydrogen) { continue; }
                for (int j = 0; j < Mobile6ClassCount; j++)
                {
                    this.InitializeTailpipeEmissionsArrays_Helper(TE_CO [i][j], te.CO , FuelTypes[i], Mobile6Classes[j]);
                    this.InitializeTailpipeEmissionsArrays_Helper(TE_VOC[i][j], te.VOC, FuelTypes[i], Mobile6Classes[j]);
                    this.InitializeTailpipeEmissionsArrays_Helper(TE_NOX[i][j], te.NOX, FuelTypes[i], Mobile6Classes[j]);
                    this.InitializeTailpipeEmissionsArrays_Helper(TE_PM [i][j], te.PM , FuelTypes[i], Mobile6Classes[j]);
                    this.InitializeTailpipeEmissionsArrays_Helper(TE_CH4[i][j], te.CH4, FuelTypes[i], Mobile6Classes[j]);
                    this.InitializeTailpipeEmissionsArrays_Helper(TE_N2O[i][j], te.N2O, FuelTypes[i], Mobile6Classes[j]);
                } 
            } 
        }
        void InitializeTailpipeEmissionsArrays_Helper(double[] teArr, TailpipeEmissions.Emissions em, FuelType fuel, Mobile6Class mbl)
        {
            double[] rate0 = null, rate1 = null;
            if      (fuel == FuelType.Gasoline ) { rate0 = em.Gasoline [mbl]; rate1 = em.GasolineRfg[mbl]; }
            else if (fuel == FuelType.Diesel   ) { rate0 = em.Diesel   [mbl]; }
            else if (fuel == FuelType.Ethanol85) { rate0 = em.Ethanol85[mbl]; }
            int length = rate0.Length;
            if (rate1 == null)
            {
                Array.Copy(rate0, teArr, length);
            }
            else
            {
                double convGasMix = this._parameters.FuelProperties.ConventionalGasolineShare;
                for (int i = 0; i < length; i++)
                {
                    teArr[i] = rate0[i] * convGasMix + rate1[i] * (1 - convGasMix);
                }
            }
            for (int i = length; i < teArr.Length; i++)
            {
                teArr[i] = teArr[i - 1];
            }
        }
        void InitializeFuelPriceArrays()
        {
            Estimates                     fpEstimates  = this._settings.OperatingModes.FuelPriceEstimates;
            Parameters.ForecastDataValues forecastData = this._parameters.ForecastData;
            FuelValue energyDensity = this._parameters.FuelProperties.EnergyDensity;
            double elcRatio = energyDensity.Gasoline / energyDensity.Electricity;
            double hgnRatio = energyDensity.Gasoline / energyDensity.Hydrogen;
            double[] fuelTax, preTaxFP, retailFP;
            for (int i = 0; i < FuelTypeCount; i++)
            {   
                fuelTax  = forecastData.GetFuelTax        (             FuelTypes[i]);
                preTaxFP = forecastData.GetPreTaxFuelPrice(fpEstimates, FuelTypes[i]);
                retailFP = forecastData.GetRetailFuelPrice(fpEstimates, FuelTypes[i]);
                double edRatio = (FuelTypes[i] == FuelType.Hydrogen) ? hgnRatio : (FuelTypes[i] == FuelType.Electricity) ? elcRatio : 1;
                for (int j = 0; j < Interaction.CY; j++)
                {
                    int index = Math.Min(this._year.Year + j, Parameters.ForecastDataValues.MaxYear) - Parameters.ForecastDataValues.MinYear;
                    this.FuelTax [i][j] = fuelTax [index] * edRatio;
                    this.PreTaxFP[i][j] = preTaxFP[index] * edRatio;
                    this.RetailFP[i][j] = retailFP[index] * edRatio;
                }
                this.RetailFPBaseVMT[i] = retailFP[this._parameters.EconomicData.VMTBaseYear - Parameters.ForecastDataValues.MinYear] * edRatio;
            } 
        }
        void ComputeVehicleCPM(bool isCar, FuelValue bFE, FuelValue bFS, FuelValue sFE, FuelValue sFS, int age,
            out double bCPM, out double sCPM, out double bCPMRate, out double sCPMRate)
        {
            double hCPM = 0;
            bCPM = 0;
            sCPM = 0;
            FuelValue[] hFE = (isCar) ? this._parameters.FuelEconomyData.CarsFuelEconomy : this._parameters.FuelEconomyData.TrksFuelEconomy;
            FuelValue[] hFS = (isCar) ? this._parameters.FuelEconomyData.CarsFuelShare   : this._parameters.FuelEconomyData.TrksFuelShare  ;
            FuelValue gap = this._parameters.EconomicData.TestVsOnRoadGap;
            for (int f = 0; f < FuelTypeCount; f++)
            {
                FuelType fuel = Effects.FuelTypes[f];
                int hIndex = this._parameters.EconomicData.VMTBaseYear - Parameters.FuelEconomyDataValues.MinYear - age;
                if (hIndex < 0) { hIndex = 0; }
                if (hFS[hIndex][fuel] > 0)
                {
                    hCPM += (this.RetailFPBaseVMT[f] / (hFE[hIndex][fuel] * (1 - gap[fuel])) * hFS[hIndex][fuel]);
                }
                if (bFS[fuel] > 0) { bCPM += (this.RetailFP[f][age] / bFE[fuel] * bFS[fuel]); }
                if (sFS[fuel] > 0) { sCPM += (this.RetailFP[f][age] / sFE[fuel] * sFS[fuel]); }
            } 
            double reboundEffect = this._settings.Parameters.EconomicData.ReboundEffect;
            bCPMRate = (bCPM / hCPM - 1) * reboundEffect;
            sCPMRate = (sCPM / hCPM - 1) * reboundEffect;
        }
        double CalculateEmissions(double energyUse, double btu, double ue, int fi, double vmt, double[] te, int cy,
            bool isSOX, bool isCO2, bool isMMT)
        {
            const double kgToTons = 0.00110231131;  
            const double kgToMMT  = 1e-9;           
            double emissions = 0;
            if (isCO2)
            {   
                emissions = ((ue                                           ) * btu / 1e6 +
                             (MassDensity[fi] * CarbonContent[fi] * 44 / 12) * energyUse);
            }
            else if (isSOX)
            {   
                emissions = ((ue         ) * btu / 1e6 +
                             (SOXRate[fi]) * energyUse);
            }
            else
            {   
                emissions = ((ue                       ) * btu / 1e6 +
                             ((te == null) ? 0 : te[cy]) * vmt);
            }
            return (isMMT) ? (emissions * kgToMMT) : (emissions * kgToTons);
        }
        FuelValue GetVehicleFE(Vehicle veh)
        {
            return GetVehcleFE(veh, this._scen, this._year, this._parameters);
        }
        public static FuelValue GetVehcleFE(Vehicle veh, Scenario scen, ModelYear year, Parameters param)
        {
            FuelValue fe = Standards.GetAdjustedVehicleFuelEconomy(veh, scen, year, param, true, false);
            if (fe.Electricity > 0)
            {
                const double pef = 82.049;
                double gasEnergyDensity = param.FuelProperties.EnergyDensity.Gasoline;
                double elcEnergyDensity = param.FuelProperties.EnergyDensity.Electricity;
                fe[FuelType.Electricity] = (fe.Electricity / pef) * (gasEnergyDensity / elcEnergyDensity);
            }
            return fe * (FuelValue.One - param.EconomicData.TestVsOnRoadGap);
        }
        #endregion
        #region 
        static readonly FuelType[] FuelTypes;
        static readonly int FuelTypeCount;
        static readonly Mobile6Class[] Mobile6Classes;
        static readonly int Mobile6ClassCount;
        Industry         _bData, _sData;
        ModelingSettings _settings;
        Parameters       _parameters;
        Scenario         _scen;
        ModelYear        _year;
        double[][] FuelShares;      
        double[]   EnergyDensity;   
        double[]   MassDensity;
        double[]   CarbonContent;
        double[]   SOXRate;
        double[] UE_CO2;        
        double[] UE_CO;
        double[] UE_VOC;
        double[] UE_NOX;
        double[] UE_PM;
        double[] UE_SOX;
        double[] UE_CH4;
        double[] UE_N2O;
        double[][][] TE_CO;     
        double[][][] TE_VOC;
        double[][][] TE_NOX;
        double[][][] TE_PM;
        double[][][] TE_CH4;
        double[][][] TE_N2O;
        double[][] FuelTax;     
        double[][] PreTaxFP;
        double[][] RetailFP;
        double[]   RetailFPBaseVMT; 
        #endregion
	}
}

