#region << Using Directives >>
using System;
using System.Collections.Generic;
using Volpe.Cafe;
using Volpe.Cafe.Data;
using Volpe.Cafe.Settings;
using TI = Volpe.Cafe.TechnologyIndexes;
#endregion
namespace Volpe.Cafe.Model
{
    public sealed class TechnologyApplicability
    {
        #region 
        static TechnologyApplicability()
        {
            TechSupersedes       = new int[TI.TechnologyCount][];
            TechSoftSupersedes   = new int[TI.TechnologyCount][];
            TechSupersedes2      = new int[TI.TechnologyCount][];
            TechDisallowBackfill = new int[TI.TechnologyCount][];
            TechKeys = new ulong[63];
            for (int i = 0; i < 63; i++) { TechKeys[i] = (ulong)Math.Pow(2, i); }
            TechTypes = new TechnologyType[] {TechnologyType.Engine, TechnologyType.Transmission, TechnologyType.Electrification,
                TechnologyType.MR, TechnologyType.ROLL, TechnologyType.DLR, TechnologyType.Aerodynamics, TechnologyType.Other};
            TechTypeCount = TechTypes.Length;
            RegClasses = new RegulatoryClass[] {RegulatoryClass.DomesticAuto, RegulatoryClass.ImportedAuto, RegulatoryClass.LightTruck};
            for (int i = 0; i < TI.TechnologyCount; i++)
            {
                TechSupersedes      [i] = new int[0];
                TechSoftSupersedes  [i] = new int[0];
                TechDisallowBackfill[i] = new int[0];
                switch (i)
                {
                    case TI.DCP:
                        TechSupersedes[i] = new int[] {TI.ICP};
                        break;
                    case TI.DVVLD:
                        TechDisallowBackfill[i] = new int[] {TI.CVVL};
                        break;
                    case TI.CVVL:
                        TechDisallowBackfill[i] = new int[] {TI.DVVLD};
                        break;
                    case TI.TRBDS1_SD:
                    case TI.TRBDS1_MD:
                    case TI.TRBDS1_LD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO};
                        break;
                    case TI.TRBDS2_SD: 
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_SD};
                        break;
                    case TI.TRBDS2_MD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_MD};
                        break;
                    case TI.TRBDS2_LD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_LD};
                        break;
                    case TI.CEGR1_SD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_SD, TI.TRBDS2_SD};
                        break;
                    case TI.CEGR1_MD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_MD, TI.TRBDS2_MD};
                        break;
                    case TI.CEGR1_LD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_LD, TI.TRBDS2_LD};
                        break;
                    case TI.CEGR2_SD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_SD, TI.TRBDS2_SD, TI.CEGR1_SD};
                        break;
                    case TI.CEGR2_MD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_MD, TI.TRBDS2_MD, TI.CEGR1_MD};
                        break;
                    case TI.CEGR2_LD:
                        TechSupersedes[i] = new int[] {TI.DEACS, TI.DEACD, TI.DEACO, TI.TRBDS1_LD, TI.TRBDS2_LD, TI.CEGR1_LD};
                        break;
                    case TI.ADSL_SD:
                        TechSupersedes[i] = new int[]
                         {TI.CCPS, TI.DVVLS, TI.DEACS, TI.ICP, TI.DCP, TI.DVVLD, TI.CVVL, TI.DEACD, TI.SGDI, TI.DEACO, TI.VVA, TI.SGDIO,
                             TI.TRBDS1_SD, TI.TRBDS2_SD, TI.CEGR1_SD, TI.CEGR2_SD};
                        break;
                    case TI.ADSL_MD:
                        TechSupersedes[i] = new int[]
                         {TI.CCPS, TI.DVVLS, TI.DEACS, TI.ICP, TI.DCP, TI.DVVLD, TI.CVVL, TI.DEACD, TI.SGDI, TI.DEACO, TI.VVA, TI.SGDIO,
                             TI.TRBDS1_MD, TI.TRBDS2_MD, TI.CEGR1_MD, TI.CEGR2_MD};
                        break;
                    case TI.ADSL_LD:
                        TechSupersedes[i] = new int[] {TI.CCPS, TI.DVVLS, TI.DEACS, TI.ICP, TI.DCP, TI.DVVLD, TI.CVVL, TI.DEACD, TI.SGDI,
                            TI.DEACO, TI.VVA, TI.SGDIO, TI.TRBDS1_LD, TI.TRBDS2_LD, TI.CEGR1_LD, TI.CEGR2_LD};
                        break;
                    case TI.HETRANSM:
                        TechSupersedes[i] = new int[] {TI.MAN6};
                        break;
                    case TI.NAUTO:
                        TechSupersedes[i] = new int[] {TI.IATC};
                        break;
                    case TI.DCT:
                        TechSupersedes[i] = new int[] {TI.IATC, TI.NAUTO};
                        break;
                    case TI.A8SPD:
                        TechSupersedes[i] = new int[] {TI.IATC, TI.NAUTO, TI.DCT};
                        break;
                    case TI.HETRANS:
                        TechSupersedes[i] = new int[] {TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD};
                        break;
                    case TI.ISG:
                        TechSupersedes[i] = new int[] {TI.MHEV};
                        break;
                    case TI.SHEV1:
                        TechSupersedes[i] = new int[] {TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD,
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD,
                            TI.HETRANS, TI.SHFTOPT, TI.MHEV, TI.ISG};
                        TechDisallowBackfill[i] = new int[] {TI.SHEV2};
                        break;
                    case TI.SHEV1_2:
                        TechSupersedes[i] = new int[] {TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD,
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD,
                            TI.HETRANS, TI.SHFTOPT, TI.MHEV, TI.ISG, TI.SHEV1};
                        TechDisallowBackfill[i] = new int[] {TI.SHEV2};
                        break;
                    case TI.SHEV2:
                        TechSupersedes[i] = new int[] {TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD,
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD,
                            TI.HETRANS, TI.SHFTOPT, TI.MHEV, TI.ISG};
                        TechDisallowBackfill[i] = new int[] {TI.SHEV1, TI.SHEV1_2};
                        break;
                    case TI.PHEV1:
                        TechSupersedes[i] = new int[] {TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD,
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD,
                            TI.HETRANS, TI.SHFTOPT, TI.MHEV, TI.ISG, TI.SHEV1, TI.SHEV1_2, TI.SHEV2};
                        break;
                    case TI.PHEV2:
                        TechSupersedes[i] = new int[] {TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD,
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD,
                            TI.HETRANS, TI.SHFTOPT, TI.MHEV, TI.ISG, TI.SHEV1, TI.SHEV1_2, TI.SHEV2, TI.PHEV1};
                        break;
                    case TI.EV1:
                        TechSupersedes[i] = new int[] {
                            TI.LUB1, TI.EFR1, TI.LUB2_EFR2,
                            TI.CCPS, TI.DVVLS, TI.DEACS, TI.ICP, TI.DCP, TI.DVVLD, TI.CVVL, TI.DEACD, TI.SGDI, TI.DEACO, TI.VVA, TI.SGDIO,
                            TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD,
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, 
                            TI.ADSL_LD, TI.ADSL_MD, TI.ADSL_SD,
                            TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD, TI.HETRANS, TI.SHFTOPT,
                            TI.MHEV, TI.ISG, TI.SHEV1, TI.SHEV1_2, TI.SHEV2, TI.PHEV1, TI.PHEV2};
                        TechDisallowBackfill[i] = new int[] {TI.FCV};
                        break;
                    case TI.EV2:
                        TechSupersedes[i] = new int[] {
                            TI.LUB1, TI.EFR1, TI.LUB2_EFR2, TI.CCPS, TI.DVVLS, TI.DEACS, TI.ICP, TI.DCP, TI.DVVLD, TI.CVVL, TI.DEACD, 
                            TI.SGDI, TI.DEACO, TI.VVA, TI.SGDIO,
                            TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD, 
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, 
                            TI.ADSL_LD, TI.ADSL_MD, TI.ADSL_SD,
                            TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD, TI.HETRANS, TI.SHFTOPT,
                            TI.MHEV, TI.ISG, TI.SHEV1, TI.SHEV1_2, TI.SHEV2, TI.PHEV1, TI.PHEV2, TI.EV1};
                        TechDisallowBackfill[i] = new int[] {TI.FCV};
                        break;
                    case TI.EV3:
                        TechSupersedes[i] = new int[] {
                            TI.LUB1, TI.EFR1, TI.LUB2_EFR2, TI.CCPS, TI.DVVLS, TI.DEACS, TI.ICP, TI.DCP, TI.DVVLD, TI.CVVL, TI.DEACD, 
                            TI.SGDI, TI.DEACO, TI.VVA, TI.SGDIO,
                            TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD, 
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, 
                            TI.ADSL_LD, TI.ADSL_MD, TI.ADSL_SD,
                            TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD, TI.HETRANS, TI.SHFTOPT,
                            TI.MHEV, TI.ISG, TI.SHEV1, TI.SHEV1_2, TI.SHEV2, TI.PHEV1, TI.PHEV2, TI.EV1, TI.EV2};
                        TechDisallowBackfill[i] = new int[] {TI.FCV};
                        break;
                    case TI.EV4:
                        TechSupersedes[i] = new int[] {
                            TI.LUB1, TI.EFR1, TI.LUB2_EFR2, TI.CCPS, TI.DVVLS, TI.DEACS, TI.ICP, TI.DCP, TI.DVVLD, TI.CVVL, TI.DEACD, 
                            TI.SGDI, TI.DEACO, TI.VVA, TI.SGDIO,
                            TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD, 
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, 
                            TI.ADSL_LD, TI.ADSL_MD, TI.ADSL_SD,
                            TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD, TI.HETRANS, TI.SHFTOPT,
                            TI.MHEV, TI.ISG, TI.SHEV1, TI.SHEV1_2, TI.SHEV2, TI.PHEV1, TI.PHEV2, TI.EV1, TI.EV2, TI.EV3};
                        TechDisallowBackfill[i] = new int[] {TI.FCV};
                        break;
                    case TI.FCV:
                        TechSupersedes[i] = new int[] {
                            TI.LUB1, TI.EFR1, TI.LUB2_EFR2,
                            TI.CCPS, TI.DVVLS, TI.DEACS, TI.ICP, TI.DCP, TI.DVVLD, TI.CVVL, TI.DEACD, TI.SGDI, TI.DEACO, TI.VVA, TI.SGDIO,
                            TI.TRBDS1_LD, TI.TRBDS1_MD, TI.TRBDS1_SD, TI.TRBDS2_LD, TI.TRBDS2_MD, TI.TRBDS2_SD,
                            TI.CEGR1_LD, TI.CEGR1_MD, TI.CEGR1_SD, TI.CEGR2_LD, TI.CEGR2_MD, TI.CEGR2_SD, 
                            TI.ADSL_LD, TI.ADSL_MD, TI.ADSL_SD,
                            TI.IATC, TI.NAUTO, TI.DCT, TI.A8SPD, TI.HETRANS, TI.SHFTOPT,
                            TI.MHEV, TI.ISG, TI.SHEV1, TI.SHEV1_2, TI.SHEV2, TI.PHEV1, TI.PHEV2};
                        TechDisallowBackfill[i] = new int[] {TI.EV1, TI.EV2, TI.EV3, TI.EV4};
                        break;
                    case TI.MR2:
                    case TI.MR3:
                    case TI.MR4:
                    case TI.MR5:
                        TechSoftSupersedes[i] = new int[] {TI.NAUTO, TI.DCT, TI.A8SPD, TI.TRBDS1_SD, TI.TRBDS1_MD, TI.TRBDS1_LD, TI.TRBDS2_SD,
                            TI.TRBDS2_MD, TI.TRBDS2_LD, TI.CEGR1_SD, TI.CEGR1_MD, TI.CEGR1_LD, TI.CEGR2_SD, TI.CEGR2_MD, TI.CEGR2_LD};
                        break;
                } 
            } 
            for (int i = 0; i < TI.TechnologyCount; i++)
            {
                TechSupersedes2[i] = new int[TechSupersedes[i].Length + TechSoftSupersedes[i].Length];
                Array.Copy(TechSupersedes    [i], 0, TechSupersedes2[i],                        0, TechSupersedes    [i].Length);
                Array.Copy(TechSoftSupersedes[i], 0, TechSupersedes2[i], TechSupersedes[i].Length, TechSoftSupersedes[i].Length);
            } 
        }
        #endregion
        #region 
        #region 
        public static void Initialize(Vehicle vehicle, ModelingSettings settings)
        {
            bool                      hasEng = (vehicle.Engine       != null);
            bool                      hasTrn = (vehicle.Transmission != null);
            Vehicle     .CDescription vd     = vehicle.Description;
            Engine      .CDescription ed     = (hasEng) ? vehicle.Engine      .Description : null;
            Transmission.CDescription td     = (hasTrn) ? vehicle.Transmission.Description : null;
            Vehicle     .CModelData   vmd    = vehicle.ModelData;   
            TechnologyClass  techClass    = vehicle.TechnologyClass;
            List<Technology> technologies = settings.Technologies;
            vmd.TechAvailable         = new bool[TI.TechnologyCount];
            vmd.TechUsed              = new bool[TI.TechnologyCount];
            vmd.TechApplied           = new bool[TI.TechnologyCount];
            vmd.TechAppliedYear       = new int [TI.TechnologyCount];
            vmd.TechSuperseded        = new bool[TI.TechnologyCount];
            vmd.TechSupersededYear    = new int [TI.TechnologyCount];
            vmd.TechBackfilled        = new bool[TI.TechnologyCount];
            vmd.TechPhaseInBackfilled = new bool[TI.TechnologyCount];
            vmd.TechEnabled           = new bool[TI.TechnologyCount];
            vmd.TechIgnored           = new bool[TI.TechnologyCount];
            vmd.TechLingering         = new bool[TI.TechnologyCount];
            vmd.TechLingerYear        = null;
            for (int i = 0; i < vd.UsedTechnologies.Length; i++) { vmd.TechUsed[vd.UsedTechnologies[i]] = true; }
            for (int i = 0; i < vd.AvailableTechnologies.Length; i++)
            {
                int index = vd.AvailableTechnologies[i];
                vmd.TechAvailable[index] = technologies[index].Attributes[techClass].Applicable;
            }
            if (hasEng)
            {
                for (int i = 0; i < ed.UsedTechnologies.Length; i++) { vmd.TechUsed[ed.UsedTechnologies[i]] = true; }
                for (int i = 0; i < ed.AvailableTechnologies.Length; i++)
                {
                    int index = ed.AvailableTechnologies[i];
                    vmd.TechAvailable[index] = technologies[index].Attributes[techClass].Applicable;
                }
            }
            if (hasTrn)
            {
                for (int i = 0; i < td.UsedTechnologies.Length; i++) { vmd.TechUsed[td.UsedTechnologies[i]] = true; }
                for (int i = 0; i < td.AvailableTechnologies.Length; i++)
                {
                    int index = td.AvailableTechnologies[i];
                    vmd.TechAvailable[index] = technologies[index].Attributes[techClass].Applicable;
                }
            }
            if (hasTrn && td.Type == "M")
            {
                TechnologyApplicability.RectifyDependencies_DisableDeacEng(vmd.TechAvailable);
            }
            if (!settings.OperatingModes.IgnoreWeightInMsApplication)
            {
                if (vd.CurbWeight < 5000)
                {   
                    vmd.TechAvailable[TI.MR1] = false;
                    vmd.TechAvailable[TI.MR2] = false;
                    vmd.TechAvailable[TI.MR3] = false;
                    vmd.TechAvailable[TI.MR4] = false;
                    vmd.TechAvailable[TI.MR5] = false;
                }
            }
            vmd.OffCycleCredit = 0;
            for (int i = 0; i < TI.TechnologyCount; i++)
            {
                if (vmd.TechUsed[i])
                {
                    if      (vehicle.IsRCAuto ) { vmd.OffCycleCredit += technologies[i].Attributes[techClass].OffCycleCreditPC; }
                    else if (vehicle.IsRCTruck) { vmd.OffCycleCredit += technologies[i].Attributes[techClass].OffCycleCreditLT; }
                }
            }
            TechnologyApplicability.RectifyDependencies(vehicle, null, -1, settings);
        }
        public static void Enable(Vehicle vehicle, ModelYear year, int techIndex, ModelingSettings settings)
        {
            Vehicle.CModelData vmd          = vehicle.ModelData;
            TechnologyClass    techClass    = vehicle.TechnologyClass;
            List<Technology>   technologies = settings.Technologies;
            if (techIndex == -1)
            {   
                for (int i = 0; i < TI.TechnologyCount; i++)
                {
                    TechnologyAttributesTable.TechnologyAttributes attr = technologies[i].Attributes[techClass];
                    vmd.TechEnabled [i] = vmd.TechAvailable[i] &&
                        (settings.OperatingModes.IgnoreYearAvailable || attr.YearAvailable <= year.Year) &&
                        (attr.YearRetired == 0 || attr.YearRetired >= year.Year);
                    vmd.TechIgnored [i] = false;
                }
            }
            else
            {   
                TechnologyAttributesTable.TechnologyAttributes attr = technologies[techIndex].Attributes[techClass];
                vmd.TechEnabled[techIndex] = vmd.TechAvailable[techIndex] &&
                    (attr.YearAvailable <= year.Year) &&
                    (attr.YearRetired == 0 || attr.YearRetired >= year.Year);
            }
            int sIdx = (techIndex == -1) ? 0                  : techIndex;
            int eIdx = (techIndex == -1) ? TI.TechnologyCount : techIndex + 1;
            if (!settings.OperatingModes.IgnoreRefreshRedesign)
            {   
                bool atRedesign, inShadowOfRedesign;
                bool atRefresh , inShadowOfRefresh ;
                vehicle.GetRedesignState(year, out atRedesign, out inShadowOfRedesign);
                vehicle.GetRefreshState (year, out atRefresh , out inShadowOfRefresh );
                for (int i = sIdx; i < eIdx; i++)
                {   
                    if (vmd.TechEnabled[i])
                    {
                        if (inShadowOfRedesign)
                        {   
                            if (TI.IsTiedToRedesign(i))
                            {
                                vmd.TechEnabled[i] = false;
                            }
                            else if (!atRefresh)
                            {   
                                if (TI.IsTiedToRefresh(i)) { vmd.TechEnabled[i] = false; }
                            }
                        }
                        else if (!atRedesign && inShadowOfRefresh)
                        {   
                            if (TI.IsTiedToRefresh(i)) { vmd.TechEnabled[i] = false; }
                        }
                    } 
                } 
            }
            for (int i = eIdx - 1; i >= sIdx; i--)
            {
                if (vmd.TechUsed[i])
                {
                    if (TI.IsHydrogenBased    (i)) { vehicle.HEVType = HEVType.FuelCell    ; break; }
                    if (TI.IsPureElectric     (i)) { vehicle.HEVType = HEVType.PureElectric; break; }
                    if (TI.IsPartialElectric  (i)) { vehicle.HEVType = HEVType.PlugInHybrid; break; }
                    if (TI.IsHybridization    (i)) { vehicle.HEVType = HEVType.StrongHybrid; break; }
                    if (TI.IsMildHybridization(i)) { vehicle.HEVType = HEVType.MildHybrid  ; break; }
                }
            }
            TechnologyApplicability.RectifyDependencies(vehicle, year, techIndex, settings);
        }
        static void RectifyDependencies(Vehicle vehicle, ModelYear year, int techIdx, ModelingSettings settings)
        {
            Vehicle.CModelData vmd = vehicle.ModelData;     
            bool updateTechEnabled = (year != null);
            bool updateSuperseded  = (techIdx != -1);
            bool[] techAE   = (updateTechEnabled) ? vmd.TechEnabled : vmd.TechAvailable;
            bool[] techLing = vmd.TechLingering;
            bool[] techUsed = vmd.TechUsed;
            int sIdx = (techIdx == -1) ? 0                  : techIdx;
            int eIdx = (techIdx == -1) ? TI.TechnologyCount : techIdx + 1;
            bool hevRectified      = false;     
            bool advTurboRectified = false;     
            for (int i = sIdx; i < eIdx; i++)
            {   
                if (techUsed[i]) { techAE[i] = false; }
                RectifyDependenciesHelper(i, techUsed, techAE, updateTechEnabled, ref hevRectified, ref advTurboRectified,
                    vehicle, year, techIdx, settings);
                if (updateTechEnabled)
                {   
                    RectifyDependenciesHelper(i, techUsed, techLing, updateTechEnabled, ref hevRectified, ref advTurboRectified,
                        vehicle, year, techIdx, settings);
                }
                if (updateSuperseded && techUsed[i] && vmd.TechApplied[i])
                {   
                    int[] techSupersedes = TechSupersedes[i];
                    for (int j = 0; j < techSupersedes.Length; j++)
                    {   
                        if (vmd.TechUsed[techSupersedes[j]] && !vmd.TechSuperseded[techSupersedes[j]])
                        {
                            vmd.TechSuperseded    [techSupersedes[j]] = true;
                            vmd.TechSupersededYear[techSupersedes[j]] = vmd.TechAppliedYear[i];
                        }
                    }
                    int[] techSoftSupersedes = TechSoftSupersedes[i];
                    for (int j = 0; j < techSoftSupersedes.Length; j++)
                    {   
                        if (vmd.TechUsed[techSoftSupersedes[j]] && vmd.TechSupersededYear[techSoftSupersedes[j]] == 0)
                        {   
                            if (vmd.TechAppliedYear[i] > vmd.TechAppliedYear[techSoftSupersedes[j]])
                            {
                                vmd.TechSupersededYear[techSoftSupersedes[j]] = vmd.TechAppliedYear[i];
                            }
                        }
                    }
                }
            } 
        }
        static void RectifyDependenciesHelper(int i, bool[] techUsed, bool[] techArr, bool updateTechEnabled,
            ref bool hevRectified, ref bool advTurboRectified, Vehicle vehicle, ModelYear year, int techIdx, ModelingSettings settings)
        {
            switch (i)
            {
                case TI.LUB1:
                case TI.EFR1:
                case TI.LUB2_EFR2:
                    if (updateTechEnabled && !hevRectified)
                    {
                        TechnologyApplicability.RectifyDependencies_EnableHev(techArr, techUsed, vehicle, year, techIdx, settings);
                        hevRectified = true;    
                    }
                    break;
                case TI.CCPS:
                case TI.DVVLS:
                case TI.DEACS:
                    break;
                case TI.ICP:
                case TI.DEACD:
                    break;
                case TI.DCP:
                    if (techUsed[i]) { techArr[TI.ICP] = false; }
                    break;
                case TI.DVVLD:
                    if (techUsed[i]) { techArr[TI.CVVL] = false; }
                    break;
                case TI.CVVL:
                    if (techUsed[i]) { techArr[TI.DVVLD] = false; }
                    break;
                case TI.SGDI:
                    break;
                case TI.DEACO:
                case TI.VVA:
                case TI.SGDIO:
                    break;
                case TI.TRBDS1_LD:
                case TI.TRBDS1_MD:
                case TI.TRBDS1_SD:
                case TI.TRBDS2_LD:
                case TI.TRBDS2_MD:
                case TI.TRBDS2_SD:
                case TI.CEGR1_LD:
                case TI.CEGR1_MD:
                case TI.CEGR1_SD:
                case TI.CEGR2_LD:
                case TI.CEGR2_MD:
                case TI.CEGR2_SD:
                    if (techUsed[i])
                    {   
                        TechnologyApplicability.RectifyDependencies_DisableDeacEng(techArr);
                    }
                    if (updateTechEnabled && !hevRectified)
                    {
                        TechnologyApplicability.RectifyDependencies_EnableHev(techArr, techUsed, vehicle, year, techIdx, settings);
                        hevRectified = true;    
                    }
                    break;
                case TI.ADSL_LD:
                case TI.ADSL_MD:
                case TI.ADSL_SD:
                    if (techUsed[i])
                    {   
                        TechnologyApplicability.RectifyDependencies_DisableAllEng(techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllHev(techArr, true, true);
                    }
                    break;
                case TI.MAN6:
                case TI.HETRANSM:
                    if (techUsed[i])
                    {   
                        TechnologyApplicability.RectifyDependencies_DisableAutoTrn(techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllHev (techArr, true, true);
                        TechnologyApplicability.RectifyDependencies_DisableDeacEng(techArr);
                    }
                    if (updateTechEnabled && !advTurboRectified && (i == TI.MAN6))
                    {
                        TechnologyApplicability.RectifyDependencies_EnableAdvancedTurbo(techArr, techUsed, vehicle, year, techIdx, settings);
                        advTurboRectified = true;   
                    }
                    break;
                case TI.IATC:
                case TI.NAUTO:
                case TI.DCT:
                case TI.A8SPD:
                case TI.HETRANS:
                case TI.SHFTOPT:
                    if (techUsed[i])
                    {   
                        TechnologyApplicability.RectifyDependencies_DisableManualTrn(techArr);
                        techArr[TI.IATC] = false;
                        if (i == TI.DCT) { techArr[TI.NAUTO] = false; }
                        if (i == TI.A8SPD) { techArr[TI.NAUTO] = false; techArr[TI.DCT] = false; }
                    }
                    if (updateTechEnabled && !advTurboRectified && (i == TI.A8SPD))
                    {
                        TechnologyApplicability.RectifyDependencies_EnableAdvancedTurbo(techArr, techUsed, vehicle, year, techIdx, settings);
                        advTurboRectified = true;   
                    }
                    if (updateTechEnabled && !hevRectified)
                    {
                        TechnologyApplicability.RectifyDependencies_EnableHev(techArr, techUsed, vehicle, year, techIdx, settings);
                        hevRectified = true;    
                    }
                    break;
                case TI.EPS:
                case TI.IACC1:
                case TI.IACC2:
                case TI.MHEV:
                    break;
                case TI.ISG:
                    if (techUsed[i])
                    {   
                        RectifyDependencies_DisableAllElec(techArr);
                    }
                    break;
                case TI.SHEV1:
                    if (techUsed[i])
                    {
                        TechnologyApplicability.RectifyDependencies_DisableAllEng (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllTrn (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllElec(techArr);
                        techArr[TI.SHEV2] = false;
                    }
                    if (updateTechEnabled && !techUsed[i] && !techArr[i]) { techArr[TI.SHEV1_2] = false; }
                    break;
                case TI.SHEV1_2:
                    if (techUsed[i])
                    {
                        TechnologyApplicability.RectifyDependencies_DisableAllEng (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllTrn (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllElec(techArr);
                        techArr[TI.SHEV1] = false;
                        techArr[TI.SHEV2] = false;
                    }
                    break;
                case TI.SHEV2:
                    if (techUsed[i])
                    {
                        TechnologyApplicability.RectifyDependencies_DisableAllEng (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllTrn (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllElec(techArr);
                        techArr[TI.SHEV1  ] = false;
                        techArr[TI.SHEV1_2] = false;
                    }
                    break;
                case TI.PHEV1:
                    if (techUsed[i])
                    {
                        TechnologyApplicability.RectifyDependencies_DisableAllEng (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllTrn (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllElec(techArr);
                        techArr[TI.SHEV1  ] = false;
                        techArr[TI.SHEV1_2] = false;
                        techArr[TI.SHEV2  ] = false;
                    }
                    break;
                case TI.PHEV2:
                    if (techUsed[i])
                    {
                        TechnologyApplicability.RectifyDependencies_DisableAllEng (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllTrn (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllElec(techArr);
                        techArr[TI.SHEV1  ] = false;
                        techArr[TI.SHEV1_2] = false;
                        techArr[TI.SHEV2  ] = false;
                        techArr[TI.PHEV1  ] = false;
                    }
                    break;
                case TI.EV1:
                case TI.EV2:
                case TI.EV3:
                case TI.EV4:
                case TI.FCV:
                    if (techUsed[i])
                    {
                        TechnologyApplicability.RectifyDependencies_DisableAllEng (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllTrn (techArr);
                        TechnologyApplicability.RectifyDependencies_DisableAllElec(techArr);
                        if (i == TI.EV4 || i == TI.FCV)
                        {   
                            TechnologyApplicability.RectifyDependencies_DisableAllHev(techArr, true, true);
                        }
                        else
                        {   
                            TechnologyApplicability.RectifyDependencies_DisableAllHev(techArr, false, true);
                            if (i == TI.EV2) { techArr[TI.EV1] = false; }
                            if (i == TI.EV3) { techArr[TI.EV1] = false; techArr[TI.EV2] = false; }
                        }
                    }
                    break;
                case TI.MR1:
                    if (updateTechEnabled)
                    {   
                        if      (!techUsed[i] ) { techArr[TI.MR2] = false; }
                        else if (techIdx != -1) { TechnologyApplicability.Enable(vehicle, year, TI.MR2, settings); }
                    }
                    break;
                case TI.MR2:
                    if (updateTechEnabled)
                    {   
                        if      (!techUsed[i] ) { techArr[TI.MR3] = false; }
                        else if (techIdx != -1) { TechnologyApplicability.Enable(vehicle, year, TI.MR3, settings); }
                    }
                    break;
                case TI.MR3:
                    if (updateTechEnabled)
                    {   
                        if      (!techUsed[i] ) { techArr[TI.MR4] = false; }
                        else if (techIdx != -1) { TechnologyApplicability.Enable(vehicle, year, TI.MR4, settings); }
                    }
                    break;
                case TI.MR4:
                    if (updateTechEnabled)
                    {   
                        if      (!techUsed[i] ) { techArr[TI.MR5] = false; }
                        else if (techIdx != -1) { TechnologyApplicability.Enable(vehicle, year, TI.MR5, settings); }
                    }
                    break;
                case TI.MR5:
                    break;
                case TI.ROLL1:
                    if (updateTechEnabled)
                    {   
                        if      (!techUsed[i] ) { techArr[TI.ROLL2] = false; }
                        else if (techIdx != -1) { TechnologyApplicability.Enable(vehicle, year, TI.ROLL2, settings); }
                    }
                    break;
                case TI.ROLL2:
                    if (updateTechEnabled)
                    {   
                        if      (!techUsed[i] ) { techArr[TI.ROLL3] = false; }
                        else if (techIdx != -1) { TechnologyApplicability.Enable(vehicle, year, TI.ROLL3, settings); }
                    }
                    break;
                case TI.ROLL3:
                    break;
                case TI.LDB:
                case TI.SAX:
                    break;
                case TI.AERO1:
                    if (updateTechEnabled)
                    {   
                        if      (!techUsed[i] ) { techArr[TI.AERO2] = false; }
                        else if (techIdx != -1) { TechnologyApplicability.Enable(vehicle, year, TI.AERO2, settings); }
                    }
                    break;
                case TI.AERO2:
                    break;
            } 
        }
        static void RectifyDependencies_DisableAllEng(bool[] techAE)
        {
            TechnologyApplicability.RectifyDependencies_DisableSohcEng(techAE, true);
            TechnologyApplicability.RectifyDependencies_DisableDohcEng(techAE, false);
            TechnologyApplicability.RectifyDependencies_DisableOhvEng (techAE);
            techAE[TI.LUB1     ] = false;
            techAE[TI.EFR1     ] = false;
            techAE[TI.LUB2_EFR2] = false;
            techAE[TI.TRBDS1_LD] = false;
            techAE[TI.TRBDS1_MD] = false;
            techAE[TI.TRBDS1_SD] = false;
            techAE[TI.TRBDS2_LD] = false;
            techAE[TI.TRBDS2_MD] = false;
            techAE[TI.TRBDS2_SD] = false;
            techAE[TI.CEGR1_LD ] = false;
            techAE[TI.CEGR1_MD ] = false;
            techAE[TI.CEGR1_SD ] = false;
            techAE[TI.CEGR2_LD ] = false;
            techAE[TI.CEGR2_MD ] = false;
            techAE[TI.CEGR2_SD ] = false;
            techAE[TI.ADSL_LD  ] = false;
            techAE[TI.ADSL_MD  ] = false;
            techAE[TI.ADSL_SD  ] = false;
        }
        static void RectifyDependencies_DisableSohcEng(bool[] techAE, bool disableSGDI)
        {
            techAE[TI.CCPS ] = false;
            techAE[TI.DVVLS] = false;
            techAE[TI.DEACS] = false;
            if (disableSGDI) { techAE[TI.SGDI] = false; }
        }
        static void RectifyDependencies_DisableDohcEng(bool[] techAE, bool disableSGDI)
        {
            techAE[TI.ICP  ] = false;
            techAE[TI.DCP  ] = false;
            techAE[TI.DVVLD] = false;
            techAE[TI.CVVL ] = false;
            techAE[TI.DEACD] = false;
            if (disableSGDI) { techAE[TI.SGDI] = false; }
        }
        static void RectifyDependencies_DisableOhvEng(bool[] techAE)
        {
            techAE[TI.DEACO] = false;
            techAE[TI.VVA  ] = false;
            techAE[TI.SGDIO] = false;
        }
        static void RectifyDependencies_DisableDeacEng(bool[] techAE)
        {
            techAE[TI.DEACS] = false;
            techAE[TI.DEACD] = false;
            techAE[TI.DEACO] = false;
        }
        static void RectifyDependencies_DisableAllTrn(bool[] techAE)
        {
            TechnologyApplicability.RectifyDependencies_DisableManualTrn(techAE);
            TechnologyApplicability.RectifyDependencies_DisableAutoTrn  (techAE);
        }
        static void RectifyDependencies_DisableManualTrn(bool[] techAE)
        {
            techAE[TI.MAN6    ] = false;
            techAE[TI.HETRANSM] = false;
        }
        static void RectifyDependencies_DisableAutoTrn(bool[] techAE)
        {
            techAE[TI.IATC   ] = false;
            techAE[TI.NAUTO  ] = false;
            techAE[TI.DCT    ] = false;
            techAE[TI.A8SPD  ] = false;
            techAE[TI.HETRANS] = false;
            techAE[TI.SHFTOPT] = false;
        }
        static void RectifyDependencies_DisableAllElec(bool[] techAE)
        {
            techAE[TI.EPS  ] = false;
            techAE[TI.IACC1] = false;
            techAE[TI.IACC2] = false;
            techAE[TI.MHEV ] = false;
            techAE[TI.ISG  ] = false;
        }
        static void RectifyDependencies_DisableAllHev(bool[] techAE, bool disableEV, bool disableFCV)
        {
            techAE[TI.SHEV1  ] = false;
            techAE[TI.SHEV1_2] = false;
            techAE[TI.SHEV2  ] = false;
            techAE[TI.PHEV1  ] = false;
            techAE[TI.PHEV2  ] = false;
            if (disableEV)
            {
                techAE[TI.EV1] = false;
                techAE[TI.EV2] = false;
                techAE[TI.EV3] = false;
                techAE[TI.EV4] = false;
            }
            if (disableFCV)
            {
                techAE[TI.FCV] = false;
            }
        }
        static void RectifyDependencies_EnableAdvancedTurbo(bool[] techEnabled, bool[] techUsed, Vehicle vehicle, ModelYear year,
            int techIdx, ModelingSettings settings)
        {
            bool[] techAvailable = vehicle.ModelData.TechAvailable;
            bool advancedTurboSatisfied =
                (!techAvailable[TI.MAN6 ] || techUsed[TI.MAN6 ]) &&
                (!techAvailable[TI.A8SPD] || techUsed[TI.A8SPD]);
            if (!advancedTurboSatisfied)
            {   
                techEnabled[TI.TRBDS2_SD] = false;  
                techEnabled[TI.TRBDS2_MD] = false;
                techEnabled[TI.TRBDS2_LD] = false;
                techEnabled[TI.CEGR1_SD ] = false;  
                techEnabled[TI.CEGR1_MD ] = false;
                techEnabled[TI.CEGR1_LD ] = false;
                techEnabled[TI.CEGR2_SD ] = false;  
                techEnabled[TI.CEGR2_MD ] = false;
                techEnabled[TI.CEGR2_LD ] = false;
                techEnabled[TI.ADSL_SD  ] = false;  
                techEnabled[TI.ADSL_MD  ] = false;
                techEnabled[TI.ADSL_LD  ] = false;
            }
            else if (techIdx != -1)
            {   
                TechnologyApplicability.Enable(vehicle, year, TI.TRBDS2_SD, settings);  
                TechnologyApplicability.Enable(vehicle, year, TI.TRBDS2_MD, settings);
                TechnologyApplicability.Enable(vehicle, year, TI.TRBDS2_LD, settings);
                TechnologyApplicability.Enable(vehicle, year, TI.CEGR1_SD , settings);  
                TechnologyApplicability.Enable(vehicle, year, TI.CEGR1_MD , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.CEGR1_LD , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.CEGR2_SD , settings);  
                TechnologyApplicability.Enable(vehicle, year, TI.CEGR2_MD , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.CEGR2_LD , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.ADSL_SD  , settings);  
                TechnologyApplicability.Enable(vehicle, year, TI.ADSL_MD  , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.ADSL_LD  , settings);
            }
        }
        static void RectifyDependencies_EnableHev(bool[] techEnabled, bool[] techUsed, Vehicle vehicle, ModelYear year,
            int techIdx, ModelingSettings settings)
        {
            bool[] techAvailable = vehicle.ModelData.TechAvailable;
            bool hevSatisfied =
                (!techAvailable[TI.LUB1     ] || techUsed[TI.LUB1     ]) &&     
                (!techAvailable[TI.EFR1     ] || techUsed[TI.EFR1     ]) &&     
                (!techAvailable[TI.LUB2_EFR2] || techUsed[TI.LUB2_EFR2]) &&     
                (!techAvailable[TI.TRBDS1_LD] || techUsed[TI.TRBDS1_LD]) &&
                (!techAvailable[TI.TRBDS1_MD] || techUsed[TI.TRBDS1_MD]) &&
                (!techAvailable[TI.TRBDS1_SD] || techUsed[TI.TRBDS1_SD]) &&
                (!techAvailable[TI.TRBDS2_LD] || techUsed[TI.TRBDS2_LD]) &&
                (!techAvailable[TI.TRBDS2_MD] || techUsed[TI.TRBDS2_MD]) &&
                (!techAvailable[TI.TRBDS2_SD] || techUsed[TI.TRBDS2_SD]) &&
                (!techAvailable[TI.CEGR1_LD ] || techUsed[TI.CEGR1_LD ]) &&
                (!techAvailable[TI.CEGR1_MD ] || techUsed[TI.CEGR1_MD ]) &&
                (!techAvailable[TI.CEGR1_SD ] || techUsed[TI.CEGR1_SD ]) &&
                (!techAvailable[TI.CEGR2_LD ] || techUsed[TI.CEGR2_LD ]) &&
                (!techAvailable[TI.CEGR2_MD ] || techUsed[TI.CEGR2_MD ]) &&
                (!techAvailable[TI.CEGR2_SD ] || techUsed[TI.CEGR2_SD ]) &&
                (!techAvailable[TI.IATC     ] || techUsed[TI.IATC     ]) &&
                (!techAvailable[TI.NAUTO    ] || techUsed[TI.NAUTO    ]) &&
                (!techAvailable[TI.DCT      ] || techUsed[TI.DCT      ]) &&
                (!techAvailable[TI.A8SPD    ] || techUsed[TI.A8SPD    ]) &&
                (!techAvailable[TI.HETRANS  ] || techUsed[TI.HETRANS  ]) &&
                (!techAvailable[TI.SHFTOPT  ] || techUsed[TI.SHFTOPT  ]); 
            if (!hevSatisfied || techUsed[TI.ADSL_LD] || techUsed[TI.ADSL_MD] || techUsed[TI.ADSL_SD])
            {   
                TechnologyApplicability.RectifyDependencies_DisableAllHev(techEnabled, true, true);
            }
            else if (techIdx != -1)
            {   
                TechnologyApplicability.Enable(vehicle, year, TI.SHEV1, settings);
                TechnologyApplicability.Enable(vehicle, year, TI.SHEV2, settings);
                TechnologyApplicability.Enable(vehicle, year, TI.PHEV1, settings);
                TechnologyApplicability.Enable(vehicle, year, TI.PHEV2, settings);
                TechnologyApplicability.Enable(vehicle, year, TI.EV1  , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.EV2  , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.EV3  , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.EV4  , settings);
                TechnologyApplicability.Enable(vehicle, year, TI.FCV  , settings);
                if (techEnabled[TI.SHEV1]) { TechnologyApplicability.Enable(vehicle, year, TI.SHEV1_2, settings); }
            }
        }
        #endregion
        #region 
        #region 
        static double GetSynergy(int techIndex, bool[] techUsedOrApplied, bool[] techSuperseded, double[] techSynergies,
            SynergyType[] synergyType, List<Technology> techGroup, ulong techKey)
        {
            double synergy = 0.0;
            for (int i = 0; i < TI.TechnologyCount; i++)
            {
                if (techUsedOrApplied[i]) { synergy += techSynergies[i]; }
            }
            if (techGroup != null && techKey > 0)
            {
                for (int i = 0, techCount = techGroup.Count; i < techCount; i++)
                {   
                    if ((techKey & TechKeys[i]) != 0)
                    {   
                        if (techGroup[i].Index >= techIndex) { break; }
                        synergy += techSynergies[techGroup[i].Index];
                    }
                } 
            }
            return synergy;
        }
        static double GetSynergy(int techIndex, bool[] techApplied, bool[] techSuperseded, double[] techSynergies,
            SynergyType[] synergyType)
        {
            double synergy = 0.0;
            for (int i = 0; i < techIndex; i++)
            {
                if (techApplied[i]) { synergy += techSynergies[i]; }
            }
            return synergy;
        }
        #endregion
        #region 
        public static double GetImprovement(Technology tech, TechnologyClass techClass, bool[] techUsed, bool[] techSuperseded,
            List<Technology> techGroup, ulong techKey, bool ignoreSynergy, out double fcg, out double fcgShare)
        {
            TechnologyAttributesTable.TechnologyAttributes ta = tech.Attributes[techClass];
            double fc = ta.FC;
            fcg = ta.FCg;
            fcgShare = ta.FCgShare;
            if (!ignoreSynergy)
            {
                double synergy = GetSynergy(tech.Index, techUsed, techSuperseded, tech.ImprvSynergies[techClass].Synergies,
                    tech.ImprvSynergies[techClass].Type, techGroup, techKey);
                fc  += synergy;
                fcg += synergy;
            }
            fc  = (fc  < 0) ? 0 : fc;
            fcg = (fcg < 0) ? 0 : fcg;
            return fc;
        }
        #endregion
        #region 
        public static double GetCost(Technology tech, Vehicle veh, Industry data, ModelYear year, int startYear, int endYear)
        {
            double erpCost;
            bool[] committedERPCosts = null;
            return GetCost(tech, veh, data, null, year, 0, startYear, endYear, null, 0, false, out erpCost, ref committedERPCosts);
        }
        public static double GetCost(Technology tech, Vehicle veh, Industry data, ModelingSettings settings, ModelYear year,
            int yearOfInitialApplication, int startYear, int endYear, List<Technology> techGroup, ulong techKey)
        {
            double erpCost;
            bool[] committedERPCosts = null;
            return GetCost(tech, veh, data, settings, year, yearOfInitialApplication, startYear, endYear, techGroup, techKey,
                true, out erpCost, ref committedERPCosts);
        }
        public static double GetCost(Technology tech, Vehicle veh, Industry data, ModelingSettings settings, ModelYear year,
            int yearOfInitialApplication, int startYear, int endYear, List<Technology> techGroup, ulong techKey,
            out double erpCost, ref bool[] committedERPCosts)
        {
            return GetCost(tech, veh, data, settings, year, yearOfInitialApplication, startYear, endYear, techGroup, techKey,
                true, out erpCost, ref committedERPCosts);
        }
        static double GetCost(Technology tech, Vehicle veh, Industry data, ModelingSettings settings, ModelYear year,
            int yearOfInitialApplication, int startYear, int endYear, List<Technology> techGroup, ulong techKey, bool isCF,
            out double erpCost, ref bool[] committedERPCosts)
        {
            Vehicle.CModelData vmd = veh.ModelData;
            double techCost = GetEstimatedCost(tech, veh, year);
            double synergy  = (isCF) ?
                GetSynergy(tech.Index, vmd.TechApplied, vmd.TechSuperseded, tech.CostSynergies[veh.TechnologyClass].Synergies,
                           tech.CostSynergies[veh.TechnologyClass].Type, techGroup, techKey) :
                GetSynergy(tech.Index, vmd.TechApplied, vmd.TechSuperseded, tech.CostSynergies[veh.TechnologyClass].Synergies,
                           tech.CostSynergies[veh.TechnologyClass].Type);
            erpCost = GetPenaltyCost(tech, veh, settings, year, yearOfInitialApplication, isCF, ref committedERPCosts);
            return techCost + synergy + erpCost;
        }
        static double GetEstimatedCost(Technology tech, Vehicle veh, ModelYear year)
        {
            TechnologyAttributesTable.TechnologyAttributes attr = tech.Attributes[veh.TechnologyClass];
            int yrIndex = Math.Min(attr.CostTable.Length - 1, year.Index);
            double cost = attr.CostTable[yrIndex];
            if (TI.IsMaterialSubstitution(tech.Index))
            {   
                if      (attr.DeltaWeightPCT != 0) { cost *= (veh.Description.BaseWeight * Math.Max(0, attr.DeltaWeightPCT)); }
                else if (attr.DeltaWeightLBS != 0) { cost *= (Math.Max(0, attr.DeltaWeightLBS)); }
                else                               { cost  = 0; }
            }
            else if (TI.IsCylinderBased(tech.Index))
            {   
                int cylinders = (veh.Engine == null) ? veh.LegacyEng.Cylinders : veh.Engine.Description.Cylinders;
                cost *= cylinders;
            }
            else if (TI.IsConfigurationBased(tech.Index))
            {   
                string engConfig = (veh.Engine == null) ? veh.LegacyEng.Configuration.ToUpper() : veh.Engine.Description.Configuration.ToUpper();
                int costMult = (engConfig == "R" || engConfig == "I") ? 1 :
                    (engConfig == "F" || engConfig == "H" || engConfig == "V") ? 2 : (engConfig == "W") ? 4 : 0;
                cost *= costMult;
            }
            return cost;
        }
        static double GetPenaltyCost(Technology tech, Vehicle veh, ModelingSettings settings, ModelYear year,
            int yearOfInitialApplication, bool isCF, ref bool[] committedERPCosts)
        {
            double penalty = 0;
            if (isCF)
            {   
                for (int i = 0; i < TechSupersedes2[tech.Index].Length; i++)
                {
                    int sTech = TechSupersedes2[tech.Index][i]; 
                    Vehicle.CModelData vmd = veh.ModelData;
                    bool isCommittedERPCosts = (committedERPCosts != null && committedERPCosts[sTech]);
                    if (!isCommittedERPCosts && vmd.TechApplied[sTech] && vmd.TechSupersededYear[sTech] == 0)
                    {   
                        int techAge = yearOfInitialApplication - vmd.TechAppliedYear[sTech] - 1;
                        penalty += (techAge < 0 || techAge >= tech.ERPCTable.Length) ? 0 : settings.Technologies[sTech].ERPCTable[techAge];
                        if (committedERPCosts != null) { committedERPCosts[sTech] = true; }
                    }
                } 
            }
            else
            {   
                Vehicle.CModelData vmd = veh.ModelData;
                int supersededYear = vmd.TechSupersededYear[tech.Index];
                if (vmd.TechApplied[tech.Index] && supersededYear > 0)
                {   
                    if (year.Year - supersededYear < 5)
                    {
                        int techAge = supersededYear - vmd.TechAppliedYear[tech.Index] - 1;
                        penalty = (techAge < 0 || techAge >= tech.ERPCTable.Length) ? 0 : tech.ERPCTable[techAge];
                    }
                }
            }
            return penalty;
        }
        #endregion
        #region 
        public static double GetMaintenanceCost(Technology tech, Vehicle veh, ModelYear year)
        {
            return GetEstimatedMaintenanceCost(tech, veh, year);
        }
        static double GetEstimatedMaintenanceCost(Technology tech, Vehicle veh, ModelYear year)
        {
            TechnologyAttributesTable.TechnologyAttributes attr = tech.Attributes[veh.TechnologyClass];
            int yrIndex = Math.Min(attr.MaintenanceCostTable.Length - 1, year.Index);
            double cost = attr.MaintenanceCostTable[yrIndex];
            return cost;
        }
        public static double GetRepairCost(Technology tech, Vehicle veh, ModelYear year)
        {
            return GetEstimatedRepairCost(tech, veh, year);
        }
        static double GetEstimatedRepairCost(Technology tech, Vehicle veh, ModelYear year)
        {
            TechnologyAttributesTable.TechnologyAttributes attr = tech.Attributes[veh.TechnologyClass];
            int yrIndex = Math.Min(attr.RepairCostTable.Length - 1, year.Index);
            double cost = attr.RepairCostTable[yrIndex];
            if (TI.IsMaterialSubstitution(tech.Index))
            {   
                if      (attr.DeltaWeightPCT != 0) { cost *= (veh.Description.BaseWeight * Math.Max(0, attr.DeltaWeightPCT)); }
                else if (attr.DeltaWeightLBS != 0) { cost *= (Math.Max(0, attr.DeltaWeightLBS)); }
                else                               { cost  = 0; }
            }
            return cost;
        }
        #endregion
        #endregion
        #region 
        public static List<Technology> GetTechnologyGroup(List<Technology> techs, TechnologyType type)
        {
            List<Technology> newTechs = new List<Technology>(GetTechnologyGroupCount(techs, type));
            for (int i = 0, techCount = techs.Count; i < techCount; i++)
            {
                if (techs[i].Type == type) { newTechs.Add(techs[i]); }
            }
            return newTechs;
        }
        public static int GetTechnologyGroupCount(List<Technology> techs, TechnologyType type)
        {
            int count = 0;
            for (int i = 0, techCount = techs.Count; i < techCount; i++)
            {
                if (techs[i].Type == type) { count++; }
            }
            return count;
        }
        public static bool HasTechnologyGroup(List<Technology> techs, TechnologyType type)
        {
            for (int i = 0, techCount = techs.Count; i < techCount; i++)
            {
                if      (techs[i]      == null) { break; }
                else if (techs[i].Type == type) { return true; }
            }
            return false;
        }
        public static bool HasOther(List<Technology> techs) { return HasTechnologyGroup(techs, TechnologyType.Other); }
        public static bool HasEngine(List<Technology> techs) { return HasTechnologyGroup(techs, TechnologyType.Engine); }
        public static bool HasTransmission(List<Technology> techs) { return HasTechnologyGroup(techs, TechnologyType.Transmission); }
        public static bool HasElectrification(List<Technology> techs) { return HasTechnologyGroup(techs, TechnologyType.Electrification); }
        public static bool HasMR(List<Technology> techs) { return HasTechnologyGroup(techs, TechnologyType.MR); }
        public static bool HasROLL(List<Technology> techs) { return HasTechnologyGroup(techs, TechnologyType.ROLL); }
        public static bool HasDLR(List<Technology> techs) { return HasTechnologyGroup(techs, TechnologyType.DLR); }
        public static bool HasAerodynamics(List<Technology> techs) { return HasTechnologyGroup(techs, TechnologyType.Aerodynamics); }
        #endregion
        public static bool CanBackfill(int techIndex, List<Technology> techs, bool[] techUsed)
        {
            for (int i = 0, count = techs.Count; i < count; i++)
            {   
                if (!CanBackfillHelper(techIndex, techs[i].Index)) { return false; }
            }
            for (int i = 0; i < TI.TechnologyCount; i++)
            {   
                if (techUsed[i] && !CanBackfillHelper(techIndex, i)) { return false; }
            }
            return true;
        }
        public static bool CanBackfill(int techIndex, List<Technology> techGroup, ulong techKey, bool[] techUsed)
        {
            int trueTechIndex = techGroup[techIndex].Index;
            for (int i = 0, techCount = techGroup.Count; i < techCount; i++)
            {   
                if ((techKey & TechKeys[i]) != 0)
                {   
                    if (!CanBackfillHelper(trueTechIndex, techGroup[i].Index)) { return false; }
                }
            }
            for (int i = 0; i < TI.TechnologyCount; i++)
            {   
                if (techUsed[i] && !CanBackfillHelper(trueTechIndex, i)) { return false; }
            }
            return true;
        }
        static bool CanBackfillHelper(int techIndex, int selTechIndex)
        {
            if (techIndex == selTechIndex) { return false; }
            int[] techDisallowBackfill = TechDisallowBackfill[selTechIndex];
            for (int j = 0, disallowCount = techDisallowBackfill.Length; j < disallowCount; j++)
            {
                if (techIndex == techDisallowBackfill[j])
                {   
                    return false;
                }
            }
            return true;
        }
        public static bool IsTechValid(Manufacturer.CModelData mmd, Vehicle.CModelData vmd, RegulatoryClass regClass,
            int techIdx, bool ignorePhaseIn, bool backfilling, Manufacturer.CModelData lMmd, out bool lingering)
        {
            lingering = false;
            if (vmd.TechUsed[techIdx]) { return false; }
            bool canApply = backfilling ||
                (!vmd.TechIgnored[techIdx] && (ignorePhaseIn || !mmd.TechExhausted[techIdx][regClass]));
            if (vmd.TechEnabled[techIdx])
            {   
                return canApply;
            }
            else if (vmd.TechLingering[techIdx])
            {   
                lingering = canApply &&
                    (backfilling || (ignorePhaseIn || !lMmd.TechExhausted[techIdx][regClass]));
                return lingering;
            }
            return false;
        }
        public static void InstallTechnologyOnVehicle(Vehicle veh, Technology tech, ModelYear year, int yearOfInitialApplication,
            ModelingSettings settings, bool backfilling, bool isExhausted)
        {
            Vehicle     .CModelData   vmd = veh.ModelData;
            Vehicle     .CDescription vd  = veh.Description;
            Engine      .CDescription ed  = (veh.Engine       == null) ? null : veh.Engine.Description;
            Transmission.CDescription td  = (veh.Transmission == null) ? null : veh.Transmission.Description;
            TechnologyAttributesTable.TechnologyAttributes attr = tech.Attributes[veh.TechnologyClass];
            vmd.TechUsed       [tech.Index] = true;
            vmd.TechApplied    [tech.Index] = true;
            vmd.TechAppliedYear[tech.Index] = yearOfInitialApplication;
            if (backfilling)
            {
                vmd.TechBackfilled[tech.Index] = true;
                if (isExhausted) { vmd.TechPhaseInBackfilled[tech.Index] = true; }
            }
            switch (tech.Index)
            {
                case TI.LUB1:
                    ed.EngineOilViscosity = "LUB1";
                    break;
                case TI.LUB2_EFR2:
                    ed.EngineOilViscosity = "LUB2";
                    break;
                case TI.DVVLS:
                case TI.CVVL:
                    ed.ValveLift = "CV";
                    break;
                case TI.DVVLD:
                    ed.ValveLift = "SVI";
                    break;
                case TI.TRBDS1_LD:
                case TI.TRBDS1_MD:
                case TI.TRBDS1_SD:
                case TI.TRBDS2_LD:
                case TI.TRBDS2_MD:
                case TI.TRBDS2_SD:
                    ed.Aspiration = "T";
                    break;
                case TI.ADSL_LD:
                case TI.ADSL_MD:
                case TI.ADSL_SD:
                    ed.ValveActuationTiming = "F";
                    ed.ValveLift = "F";
                    ed.Fuel = FuelType.Diesel;
                    ed.FuelSystem = "DI";
                    ed.Aspiration = "T";
                    ed.Cycle = 'D';
                    break;
                case TI.MAN6:
                    td.Type = "M";
                    td.NumForwardGears = 6;
                    break;
                case TI.IATC:
                    td.Type = "A";
                    break;
                case TI.NAUTO:
                    td.Type = "A";
                    td.NumForwardGears = 6;
                    break;
                case TI.DCT:
                    td.Type = "DCT";
                    td.NumForwardGears = 6;
                    break;
                case TI.A8SPD:
                    td.Type = "A";
                    td.NumForwardGears = 8;
                    break;
                case TI.SHEV1:
                case TI.SHEV1_2:
                case TI.SHEV2:
                    vd.HybridType = "SHEV";
                    veh.RemoveTransmission();
                    break;
                case TI.PHEV1:
                case TI.PHEV2:
                    vd.HybridType = "PHEV";
                    veh.Description.ElectricRange = attr.ElectricRange;
                    veh.RemoveTransmission();
                    break;
                case TI.EV1:
                case TI.EV2:
                case TI.EV3:
                case TI.EV4:
                    vd.HybridType = "EV";
                    veh.Description.ElectricRange = attr.ElectricRange;
                    if (ed != null) { veh.Description.ElectricPower = ed.Horsepower; }
                    veh.RemoveTransmission();
                    veh.RemoveEngine();
                    break;
                case TI.FCV:
                    vd.HybridType = "FCV";
                    veh.Description.ElectricRange = attr.ElectricRange;
                    if (ed != null) { veh.Description.ElectricPower = ed.Horsepower; }
                    veh.RemoveTransmission();
                    veh.RemoveEngine();
                    break;
            }
            TechnologyApplicability.Enable(veh, year, tech.Index, settings);
        }
        public static void CheckPhaseIn(ModelingSettings settings, Manufacturer.CModelData mmd, RegulatoryClass regClass,
            Technology tech, ModelYear year, int sYrIndex)
        {
            if (mmd.TechExhausted[tech.Index][regClass]) { return; }
            RCDouble[] techSalesArr = (settings.OperatingModes.PhaseInMode == PhaseInMode.Legacy) ? mmd.TechAppliedSales : mmd.TechUsedSales;
            int[] phaseInPairs = TI.GetSharedPhaseInTechs(tech.Index);
            if (phaseInPairs == null)
            {   
                double techSales = techSalesArr[tech.Index][regClass];
                double mfrSales = mmd.Sales[regClass];
                if (mfrSales > 0 && techSales >= mfrSales * tech.GetPhaseIn(year.Index, sYrIndex, mmd.TechPhaseInOffset[tech.Index][regClass]))
                {   
                    if (tech.Index != -1) { mmd.TechExhausted[tech.Index][regClass] = true; }
                }
            }
            else
            {   
                double techSales = 0D;
                for (int i = 0; i < phaseInPairs.Length; i++)
                {
                    if (phaseInPairs[i] != -1) { techSales += techSalesArr[phaseInPairs[i]][regClass]; }
                }
                double mfrSales = mmd.Sales[regClass];
                if (mfrSales > 0 && techSales >= mfrSales * tech.GetPhaseIn(year.Index, sYrIndex, mmd.TechPhaseInOffset[tech.Index][regClass]))
                {   
                    for (int i = 0; i < phaseInPairs.Length; i++)
                    {
                        if (phaseInPairs[i] != -1) { mmd.TechExhausted[phaseInPairs[i]][regClass] = true; }
                    }
                }
            }
        }
        public static void ComputeMfrAdjustedPhaseIn(Manufacturer mfr, ModelYear year, int sYrIndex, ModelingSettings settings)
        {
            Manufacturer.CModelData mmd = mfr.ModelData;
            RCDouble mfrSales = Standards.GetSales(mfr, year);
            for (int i = 0; i < TI.TechnologyCount; i++)
            {   
                RCDouble techPenRate = (mmd.TechUsedSales[i] - mmd.TechAppliedSales[i]) / mfrSales;
                double[] techPhaseIn = settings.Technologies[i].PhaseIn;
                for (int j = 0; j < RegClasses.Length; j++)
                {
                    RegulatoryClass regClass = RegClasses[j];
                    if (mfrSales[regClass] <= 0) { continue; }
                    for (int k = 0; k < techPhaseIn.Length; k++)
                    {
                        if (techPenRate[regClass] <= techPhaseIn[k])
                        {
                            mmd.TechPhaseInOffset[i][regClass] = Math.Max(mmd.TechPhaseInOffset[i][regClass], k + sYrIndex - year.Index);
                            break;
                        }
                    }
                } 
            } 
        }
        #endregion
        #region 
        static readonly int[][] TechSupersedes;
        static readonly int[][] TechSoftSupersedes;
        static readonly int[][] TechSupersedes2;
        static readonly int[][] TechDisallowBackfill;
        public static readonly ulong[] TechKeys;
        public static TechnologyType[] TechTypes;
        public static int TechTypeCount;
        public static RegulatoryClass[] RegClasses;
        #endregion
    }
}

