using System;
using System.Globalization;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe
{
    [Serializable]
    public class ModelYear : IComparable
    {
        #region 
        static ModelYear()
        {
            ModelYears = new ModelYear[100];
            for (int i = 0; i < ModelYears.Length; i++)
            {
                ModelYears[i] = new ModelYear(ModelYear.MinYear + i);
            }
        }
        public ModelYear(int year)
        {
            this.Year = year;
        }
        #endregion
        #region 
        #region 
        public int CompareTo(object obj)
        {
            int year = 0;
            if      (obj is ModelYear          ) { year = ((ModelYear)obj).Year; }
            else if (Interaction.IsNumeric(obj)) { year = Interaction.GetInt32(obj); }
            else                                 { throw new ArgumentException(ErrorStrings.ObjectIsNotModelYearOrNumeric); }
            return this._year.CompareTo(year);
        }
        #endregion
        #region 
        public override string ToString()
        {
            return this._year.ToString();
        }
        public override int GetHashCode()
        {
            return this._year.GetHashCode();
        }
        public override bool Equals(object obj)
        {
            if (obj == null)
            {
                throw new ArgumentNullException("obj");
            }
            return (this.CompareTo(obj) == 0);
        }
        public bool Equals(ModelYear year)
        {
            return (this._year == year.Year);
        }
        public bool Equals(int year)
        {
            return (this._year == year);
        }
        public bool EqualsIndex(int index)
        {
            return (this._index == index);
        }
        #endregion
        public static int ToIndex(int year)
        {
            if (year < ModelYear.MinYear) { throw new ArgumentOutOfRangeException("year", year, ErrorStrings.ModelYearTooSmall); }
            return year - ModelYear.MinYear;
        }
        public static int FromIndex(int index)
        {
            return index + ModelYear.MinYear;
        }
        public static bool IsValid(int year)
        {
            return (year >= ModelYear.MinYear);
        }
        public static ModelYear NewModelYearFromIndex(int index)
        {
            if (index < 0 || index >= ModelYear.ModelYears.Length)
            {
                return new ModelYear(ModelYear.MinYear + index);
            }
            else
            {
                return ModelYear.ModelYears[index];
            }
        }
        public static ModelYear NewModelYearFromYear(int year)
        {
            return NewModelYearFromIndex(year - ModelYear.MinYear);
        }
        void SetModelYearInternal(int year, int index)
        {
            if (index < 0) { throw new ArgumentOutOfRangeException("year", year, ErrorStrings.ModelYearTooSmall); }
            this._year  = year;
            this._index = index;
        }
        #endregion
        #region 
        public int Index
        {
            get { return this._index; }
            set { this.SetModelYearInternal(value + ModelYear.MinYear, value); }
        }
        public int Year
        {
            get { return this._year; }
            set { this.SetModelYearInternal(value, value - ModelYear.MinYear); }
        }
        #endregion
        #region 
        static readonly ModelYear[] ModelYears;
        public static readonly ModelYear MinModelYear = new ModelYear(ModelYear.MinYear);
        public const int MinYear = 2000;
        int _year;  
        int _index; 
        #endregion
    }
}

