using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.IO;
using Volpe.Cafe.Data;
using Volpe.Cafe.IO;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.Settings
{
    [Serializable]
    public class CreditTradingSettings : ISettings, ICloneable
    {
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        ISettings ISettings.Clone()
        {
            return this.Clone();
        }
        CreditTradingSettings Clone()
        {
            CreditTradingSettings value = new CreditTradingSettings();
            Clone(this, value);
            return value;
        }
        static void Clone(CreditTradingSettings src, CreditTradingSettings dst)
        {
            dst.AllowTrading                    = src.AllowTrading;
            dst.AllowMfrTrading                 = src.AllowMfrTrading;
            dst.AllowFleetTransfers             = src.AllowFleetTransfers;
            dst.AllowCarryForward               = src.AllowCarryForward;
            dst.CarryForwardYears               = src.CarryForwardYears;
            dst.AllowCarryBackward              = src.AllowCarryBackward;
            dst.CarryBackwardYears              = src.CarryBackwardYears;
            dst.AllowTransferOut                = src.AllowTransferOut;
            dst.AllowTransferIn                 = src.AllowTransferIn;
            dst.AlternativeTransferInForMinStnd = src.AlternativeTransferInForMinStnd;
            dst.TransferCaps                    = src.CloneDoubleList(src.TransferCaps);
            dst.CreditAdjustmentFactor          = src.CreditAdjustmentFactor;
            dst.CreditAdjustmentVMT_DPC         = src.CloneDoubleList(src.CreditAdjustmentVMT_DPC);
            dst.CreditAdjustmentVMT_IPC         = src.CloneDoubleList(src.CreditAdjustmentVMT_IPC);
            dst.CreditAdjustmentVMT_LT          = src.CloneDoubleList(src.CreditAdjustmentVMT_LT);
        }
        List<double> CloneDoubleList(List<double> value)
        {
            if (value == null || value.Count == 0)
            {
                return new List<double>();
            }
            else
            {
                int count = value.Count;
                List<double> d = new List<double>(count);
                for (int i = 0; i < count; i++) { d.Add(value[i]); }
                return d;
            }
        }
        #endregion
        #region 
        void ISettings.SetDefaultSettings()
        {
            this.AllowTrading                  = false;
            this.AllowMfrTrading               = false;
            this.AllowFleetTransfers           = true;
            this.AllowCarryForward             = true;
            this.CarryForwardYears             = 5;
            this.AllowCarryBackward            = true;
            this.CarryBackwardYears            = 3;
            this.AllowTransferOut.DomesticAuto = true;
            this.AllowTransferOut.ImportedAuto = true;
            this.AllowTransferOut.LightTruck   = true;
            this.AllowTransferIn.DomesticAuto  = true;
            this.AllowTransferIn.ImportedAuto  = true;
            this.AllowTransferIn.LightTruck    = true;
            this.AlternativeTransferInForMinStnd.DomesticAuto = true;
            this.TransferCaps                  = new List<double>();
        }
        void ISettings.UpdateSettings(ISettings newSettings)
        {
            this.UpdateSettings((CreditTradingSettings)newSettings);
        }
        public void UpdateSettings(CreditTradingSettings newSettings)
        {
            Clone(newSettings, this);
        }
        public void WriteSummaryLog(LogWriter writer)
        {
            if (writer.Summary == null) { return; }
            string outStr = "--- Credit Trading Settings ---" +
                "\n  AllowTrading="        + this.AllowTrading        +
                "\n  AllowMfrTrading="     + this.AllowMfrTrading     +
                "\n  AllowFleetTransfers=" + this.AllowFleetTransfers +
                "\n  AllowCarryForward="   + this.AllowCarryForward   + ((this.AllowCarryForward ) ? " (" + this.CarryForwardYears  + ")" : string.Empty) +
                "\n  AllowCarryBackward="  + this.AllowCarryBackward  + ((this.AllowCarryBackward) ? " (" + this.CarryBackwardYears + ")" : string.Empty) +
                "\n  AllowTransferOut="    + this.AllowTransferOut    +
                "\n  AllowTransferIn="     + this.AllowTransferIn     +
                "\n  AlternativeTransferInForMinStnd=" + this.AlternativeTransferInForMinStnd;
            outStr += this.WriteSummaryLog_WriteDoubleList("\n  TransferCaps=", this.TransferCaps);
            outStr += "\n  CreditAdjustmentFactor=" + CreditAdjustmentFactor;
            outStr += this.WriteSummaryLog_WriteDoubleList("\n  CreditAdjustmentVMT_DPC=", this.CreditAdjustmentVMT_DPC);
            outStr += this.WriteSummaryLog_WriteDoubleList("\n  CreditAdjustmentVMT_IPC=", this.CreditAdjustmentVMT_IPC);
            outStr += this.WriteSummaryLog_WriteDoubleList("\n  CreditAdjustmentVMT_LT=" , this.CreditAdjustmentVMT_LT);
            writer.Summary.WriteLine(outStr + "\n");
        }
        string WriteSummaryLog_WriteDoubleList(string header, List<double> value)
        {
            if (value != null)
            {
                string s = header;
                for (int i = 0; i < value.Count; i++)
                {
                    s += "\n    MY-" + (ModelYear.MinYear + i).ToString() + "=" + value[i].ToString();
                }
                return s;
            }
            return string.Empty;
        }
        #endregion
        internal void Parse(StreamReader sr)
        {
            string line = sr.ReadLine();
            while (line != null)
            {
                string[] tokens = this.Parse_GetTokens(line);
                if (tokens == null || tokens.Length == 0 || tokens[0].Trim() == string.Empty) { line = sr.ReadLine(); continue; }
                string[] subtokens = null;
                int year = 0;
                switch (tokens[0])
                {
                    case "allowtrading"       : this.AllowTrading        = Interaction.GetBool (tokens[1]); break;
                    case "allowmfrtrading"    : this.AllowMfrTrading     = Interaction.GetBool (tokens[1]); break;
                    case "allowfleettransfers": this.AllowFleetTransfers = Interaction.GetBool (tokens[1]); break;
                    case "allowcarryforward"  : this.AllowCarryForward   = Interaction.GetBool (tokens[1]); break;
                    case "carryforwardyears"  : this.CarryForwardYears   = Interaction.GetInt32(tokens[1]); break;
                    case "allowcarrybackward" : this.AllowCarryBackward  = Interaction.GetBool (tokens[1]); break;
                    case "carrybackwardyears" : this.CarryBackwardYears  = Interaction.GetInt32(tokens[1]); break;
                    case "allowtransferout":
                    case "allowtransferin":
                    case "alternativetransferinforminstnd":
                        subtokens = tokens[1].Trim().Split(',');
                        RCBoolean rc = new RCBoolean();
                        for (int i = 0; i < subtokens.Length; i++)
                        {
                            string subtoken = subtokens[i].Trim();
                            if      (subtoken == "dpc") { rc.DomesticAuto = true; }
                            else if (subtoken == "ipc") { rc.ImportedAuto = true; }
                            else if (subtoken == "lt" ) { rc.LightTruck   = true; }
                        }
                        switch (tokens[0])
                        {
                            case "allowtransferout"               : this.AllowTransferOut                = rc; break;
                            case "allowtransferin"                : this.AllowTransferIn                 = rc; break;
                            case "alternativetransferinforminstnd": this.AlternativeTransferInForMinStnd = rc; break;
                        }
                        break;
                    case "transfercap":
                        if (this.TransferCaps == null) { this.TransferCaps = new List<double>(32); }
                        subtokens = tokens[1].Trim().Split(',');
                        double cap = Interaction.GetDouble(subtokens[0]);
                        for (int i = 1; i < subtokens.Length; i++)
                        {
                            year = Interaction.GetInt32(subtokens[i]);
                            if (!ModelYear.IsValid(year)) { throw new ArgumentException("The model year used for the \"TransferCap\" entry is not valid."); }
                            int yrIndex = year - ModelYear.MinYear;
                            if (this.TransferCaps.Count <= yrIndex)
                            {
                                for (int j = this.TransferCaps.Count; j <= yrIndex; j++) { this.TransferCaps.Add(0); }
                            }
                            this.TransferCaps[yrIndex] = cap;
                        }
                        break;
                    case "creditadjustmentfactor": this.CreditAdjustmentFactor = Interaction.GetString(tokens[1]); break;
                    case "creditadjustmentvmt":
                        if (this.CreditAdjustmentVMT_DPC == null) { this.CreditAdjustmentVMT_DPC = new List<double>(32); }
                        if (this.CreditAdjustmentVMT_IPC == null) { this.CreditAdjustmentVMT_IPC = new List<double>(32); }
                        if (this.CreditAdjustmentVMT_LT  == null) { this.CreditAdjustmentVMT_LT  = new List<double>(32); }
                        subtokens = tokens[1].Trim().Split(',');
                        if (subtokens.Length != 4) { throw new ArgumentException("The \"CreditAdjustmentVMT\" entry is not valid."); }
                        year = Interaction.GetInt32(subtokens[0]);
                        if (!ModelYear.IsValid(year)) { throw new ArgumentException("The model year used for the \"CreditAdjustmentVMT\" entry is not valid."); }
                        for (int i = 1; i < subtokens.Length; i++)
                        {
                            int yrIndex = year - ModelYear.MinYear;
                            if (this.CreditAdjustmentVMT_DPC.Count <= yrIndex)
                            {
                                for (int j = this.CreditAdjustmentVMT_DPC.Count; j <= yrIndex; j++)
                                {
                                    this.CreditAdjustmentVMT_DPC.Add(0);
                                    this.CreditAdjustmentVMT_IPC.Add(0);
                                    this.CreditAdjustmentVMT_LT .Add(0);
                                }
                            }
                            this.CreditAdjustmentVMT_DPC[yrIndex] = Interaction.GetDouble(subtokens[1]);
                            this.CreditAdjustmentVMT_IPC[yrIndex] = Interaction.GetDouble(subtokens[2]);
                            this.CreditAdjustmentVMT_LT [yrIndex] = Interaction.GetDouble(subtokens[3]);
                        }
                        break;
                }
                line = sr.ReadLine();
            }
        }
        string[] Parse_GetTokens(string line)
        {
            line = line.Trim();
            if (line == string.Empty || line.StartsWith("#")) { return null; }
            string[] tokens = line.Split(' ', '\t');
            line = "";
            for (int i = 0; i < tokens.Length; i++)
            {
                if (tokens[i].Trim() != string.Empty) { line += tokens[i] + " "; }
            }
            return line.TrimEnd().ToLower().Split(' ');
        }
        #endregion
        #region 
        public bool AllowTrading;
        public bool AllowMfrTrading;
        public bool AllowFleetTransfers;
        public bool AllowCarryForward;
        public int CarryForwardYears;
        public bool AllowCarryBackward;
        public int CarryBackwardYears;
        public RCBoolean AllowTransferOut;
        public RCBoolean AllowTransferIn;
        public RCBoolean AlternativeTransferInForMinStnd;
        public List<double> TransferCaps;
        public string CreditAdjustmentFactor;
        public List<double> CreditAdjustmentVMT_DPC;
        public List<double> CreditAdjustmentVMT_IPC;
        public List<double> CreditAdjustmentVMT_LT;
        #endregion
    }
}

