using System;
using Volpe.Cafe.IO;
namespace Volpe.Cafe.Settings
{
    [Serializable]
    public class OperatingModes : ISettings, ICloneable
    {
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        ISettings ISettings.Clone()
        {
            return this.Clone();
        }
        OperatingModes Clone()
        {
            OperatingModes value = new OperatingModes();
            Clone(this, value);
            return value;
        }
        static void Clone(OperatingModes src, OperatingModes dst)
        {
            dst.ModelingApproach                  = src.ModelingApproach;
            dst.AutoToggleApproach                = src.AutoToggleApproach;
            dst.MergedFleet                       = src.MergedFleet;
            dst.NoFines                           = src.NoFines;
            dst.Overcomply                        = src.Overcomply;
            dst.OvercomplyCostEffectiveOnly       = src.OvercomplyCostEffectiveOnly;
            dst.OvercomplyLookAheadYears          = src.OvercomplyLookAheadYears;
            dst.MultiYearModeling                 = src.MultiYearModeling;
            dst.MultiYearStartYear                = src.MultiYearStartYear;
            dst.DynamicFleetShare                 = src.DynamicFleetShare;
            dst.DFS_InitialPCShare                = src.DFS_InitialPCShare;
            dst.DFS_InitialPCCAFE                 = src.DFS_InitialPCCAFE;
            dst.DFS_InitialLTCAFE                 = src.DFS_InitialLTCAFE;
            dst.DFS_Epsilon                       = src.DFS_Epsilon;
            dst.SplitSharedEngsOrTrns             = src.SplitSharedEngsOrTrns;
            dst.SplitConflictingEngsOrTrns        = src.SplitConflictingEngsOrTrns;
            dst.SkipSharedEngsOrTrns              = src.SkipSharedEngsOrTrns;
            dst.SkipConflictingEngsOrTrns         = src.SkipConflictingEngsOrTrns;
            dst.Backfill                          = src.Backfill;
            dst.PhaseInMode                       = src.PhaseInMode;
            dst.IgnoreYearAvailable               = src.IgnoreYearAvailable;
            dst.IgnorePhaseIn                     = src.IgnorePhaseIn;
            dst.IgnoreRefreshRedesign             = src.IgnoreRefreshRedesign;
            dst.IgnoreWeightInMsApplication       = src.IgnoreWeightInMsApplication;
            dst.FuelPriceEstimates                = src.FuelPriceEstimates;
            dst.CO2Estimates                      = src.CO2Estimates;
        }
        #endregion
        #region 
        void ISettings.SetDefaultSettings()
        {
            this.ModelingApproach                  = ModelingApproach.LowCostFirst;
            this.AutoToggleApproach                = false;
            this.MergedFleet                       = false;
            this.NoFines                           = false;
            this.Overcomply                        = false;
            this.OvercomplyCostEffectiveOnly       = true;
            this.OvercomplyLookAheadYears          = 0;
            this.MultiYearModeling                 = true;
            this.MultiYearStartYear                = 1;
            this.DynamicFleetShare                 = false;
            this.DFS_InitialPCShare                = Volpe.Cafe.Model.DynamicFleetShare.DefaultInitialPCShare;
            this.DFS_InitialPCCAFE                 = Volpe.Cafe.Model.DynamicFleetShare.DefaultInitialPCCAFE;
            this.DFS_InitialLTCAFE                 = Volpe.Cafe.Model.DynamicFleetShare.DefaultInitialLTCAFE;
            this.DFS_Epsilon                       = 0;
            this.SplitSharedEngsOrTrns             = true;
            this.SplitConflictingEngsOrTrns        = true;
            this.SkipSharedEngsOrTrns              = false;
            this.SkipConflictingEngsOrTrns         = false;
            this.Backfill                          = true;
            this.PhaseInMode                       = PhaseInMode.DynamicSlidingPhaseIn;
            this.IgnoreYearAvailable               = false;
            this.IgnorePhaseIn                     = false;
            this.IgnoreRefreshRedesign             = false;
            this.IgnoreWeightInMsApplication       = true;
            this.FuelPriceEstimates                = Estimates.Average;
            this.CO2Estimates                      = CO2Estimates.Average;
        }
        void ISettings.UpdateSettings(ISettings newSettings)
        {
            this.UpdateSettings((OperatingModes)newSettings);
        }
        public void UpdateSettings(OperatingModes newSettings)
        {
            Clone(newSettings, this);
        }
        public void WriteSummaryLog(LogWriter writer)
        {
            if (writer.Summary == null) { return; }
            string outStr = "--- Operating Modes ---" +
                "\n  ModelingApproach="                  + this.ModelingApproach                  +
                "\n  AutoToggleApproach="                + this.AutoToggleApproach                +
                "\n  MergedFleet="                       + this.MergedFleet                       +
                "\n  NoFines="                           + this.NoFines                           +
                "\n  Overcomply="                        + this.Overcomply                        +
                    ((!this.Overcomply) ? "" :
                "\n    OvercomplyCostEffectiveOnly="     + this.OvercomplyCostEffectiveOnly       +
                "\n    OvercomplyLookAheadYears="        + this.OvercomplyLookAheadYears)         +
                "\n  MultiYearModeling="                 + this.MultiYearModeling                 +
                                                           ((this.MultiYearModeling) ? " (" + this.MultiYearStartYear + ")" : string.Empty) +
                "\n  DynamicFleetShare="                 + this.DynamicFleetShare                 +
                    ((!this.DynamicFleetShare) ? "" :
                "\n    DFS_InitialPCShare="              + this.DFS_InitialPCShare                +
                "\n    DFS_InitialPCCAFE="               + this.DFS_InitialPCCAFE                 +
                "\n    DFS_InitialLTCAFE="               + this.DFS_InitialLTCAFE                 +
                "\n    DFS_Epsilon="                     + this.DFS_Epsilon)                      +
                "\n  SplitSharedEngsOrTrns="             + this.SplitSharedEngsOrTrns             +
                "\n  SplitConflictingEngsOrTrns="        + this.SplitConflictingEngsOrTrns        +
                "\n  SkipSharedEngsOrTrns="              + this.SkipSharedEngsOrTrns              +
                "\n  SkipConflictingEngsOrTrns="         + this.SkipConflictingEngsOrTrns         +
                "\n  Backfill="                          + this.Backfill                          +
                "\n  PhaseInMode="                       + this.PhaseInMode                       +
                "\n  IgnoreYearAvailable="               + this.IgnoreYearAvailable               +
                "\n  IgnorePhaseIn="                     + this.IgnorePhaseIn                     +
                "\n  IgnoreRefreshRedesign="             + this.IgnoreRefreshRedesign             +
                "\n  IgnoreWeightInMsApplication="       + this.IgnoreWeightInMsApplication       +
                "\n  FuelPriceEstimates="                + this.FuelPriceEstimates                +
                "\n  CO2Estimates="                      + this.CO2Estimates;
            writer.Summary.WriteLine(outStr + "\n");
        }
        #endregion
        #endregion
        #region 
        #region 
        public ModelingApproach ModelingApproach;
        public bool AutoToggleApproach;
        public bool MergedFleet;
        public bool NoFines;
        public bool Overcomply;
        public bool OvercomplyCostEffectiveOnly;
        public int OvercomplyLookAheadYears;
        public bool MultiYearModeling;
        public int MultiYearStartYear;
        public bool DynamicFleetShare;
        public double DFS_InitialPCShare;
        public double DFS_InitialPCCAFE;
        public double DFS_InitialLTCAFE;
        public double DFS_Epsilon;
        public bool SplitSharedEngsOrTrns;
        public bool SplitConflictingEngsOrTrns; 
        public bool SkipSharedEngsOrTrns;
        public bool SkipConflictingEngsOrTrns;
        #endregion
        #region 
        public bool Backfill;
        public PhaseInMode PhaseInMode;
        #endregion
        #region 
        public bool IgnoreYearAvailable;
        public bool IgnorePhaseIn;
        public bool IgnoreRefreshRedesign;
        public bool IgnoreWeightInMsApplication;
        #endregion
        #region 
        public Estimates FuelPriceEstimates;
        public CO2Estimates CO2Estimates;
        #endregion
        #endregion
    }
}

