using Meta.Numerics.Statistics.Distributions;
using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using Volpe.Cafe.Data;
using Volpe.Cafe.IO;
using Volpe.Cafe.Settings;
using Volpe.Cafe.Utils;
using Volpe.XlLib;
namespace Volpe.Cafe.Settings
{
    [Serializable]
    public sealed class Parameters : ICloneable
    {
        #region 
        [Serializable]
        public sealed class VehicleAgeDataValues
        {
            #region 
            internal VehicleAgeDataValues(int length)
            {
                this._length = length;
                this.SurvivalCars    = new double[length];
                this.SurvivalVans    = new double[length];
                this.SurvivalSUVs    = new double[length];
                this.SurvivalPickups = new double[length];
                this.SurvivalAllLTs  = new double[length];
                this.MilesCars       = new double[length];
                this.MilesVans       = new double[length];
                this.MilesSUVs       = new double[length];
                this.MilesPickups    = new double[length];
                this.MilesAllLTs     = new double[length];
            }
            #endregion
            #region 
            internal VehicleAgeDataValues Clone()
            {
                VehicleAgeDataValues value = new VehicleAgeDataValues(this._length);
                for (int i = 0; i < this._length; i++)
                {
                    value.SurvivalCars   [i] = this.SurvivalCars   [i];
                    value.SurvivalVans   [i] = this.SurvivalVans   [i];
                    value.SurvivalSUVs   [i] = this.SurvivalSUVs   [i];
                    value.SurvivalPickups[i] = this.SurvivalPickups[i];
                    value.SurvivalAllLTs [i] = this.SurvivalAllLTs [i];
                    value.MilesCars      [i] = this.MilesCars      [i];
                    value.MilesVans      [i] = this.MilesVans      [i];
                    value.MilesSUVs      [i] = this.MilesSUVs      [i];
                    value.MilesPickups   [i] = this.MilesPickups   [i];
                    value.MilesAllLTs    [i] = this.MilesAllLTs    [i];
                }
                return value;
            }
            internal void UpdateVMTSchedule(double milesCars0, double milesAllLTs0, double carsCap, double trucksCap)
            {
                this.UpdateVMTSchedule(milesCars0  , this.MilesCars  , carsCap);
                this.UpdateVMTSchedule(milesAllLTs0, this.MilesAllLTs, trucksCap);
                this.MilesVans = this.MilesSUVs = this.MilesPickups = this.MilesAllLTs;
            }
            void UpdateVMTSchedule(double newMiles0, double[] miles, double milesCap)
            {
                double ratio = newMiles0 / miles[0];
                double sum   = newMiles0;
                miles[0]     = newMiles0;
                for (int i = 1; i < miles.Length; i++)
                {
                    miles[i] *= ratio;
                    sum += miles[i];
                    if (sum >= milesCap) { Array.Clear(miles, i + 1, miles.Length - i - 1); break; }
                }
            }
            #endregion
            #region 
            int _length;
            public double[] SurvivalCars;
            public double[] SurvivalVans;
            public double[] SurvivalSUVs;
            public double[] SurvivalPickups;
            public double[] SurvivalAllLTs;
            public double[] MilesCars;
            public double[] MilesVans;
            public double[] MilesSUVs;
            public double[] MilesPickups;
            public double[] MilesAllLTs;
            #endregion
        }
        [Serializable]
        public sealed class ForecastDataValues
        {
            #region 
            [Serializable]
            sealed class FuelValueArray
            {
                internal FuelValueArray(int length)
                {
                    this._gas = new double[length];
                    this._dsl = new double[length];
                    this._e85 = new double[length];
                    this._elc = new double[length];
                    this._hgn = new double[length];
                }
                internal FuelValueArray Clone(int length)
                {
                    FuelValueArray value = new FuelValueArray(length);
                    Array.Copy(this._gas, 0, value._gas, 0, length);
                    Array.Copy(this._dsl, 0, value._dsl, 0, length);
                    Array.Copy(this._e85, 0, value._e85, 0, length);
                    Array.Copy(this._elc, 0, value._elc, 0, length);
                    Array.Copy(this._hgn, 0, value._hgn, 0, length);
                    return value;
                }
                internal double[] this[FuelType fuelType]
                {
                    get
                    {
                        return
                            (fuelType == FuelType.Gasoline   ) ? this._gas :
                            (fuelType == FuelType.Diesel     ) ? this._dsl :
                            (fuelType == FuelType.Ethanol85  ) ? this._e85 :
                            (fuelType == FuelType.Electricity) ? this._elc :
                            (fuelType == FuelType.Hydrogen   ) ? this._hgn : null;
                    }
                }
                double[] _gas, _dsl, _e85, _elc, _hgn;
            }
            #endregion
            #region 
            internal ForecastDataValues()
            {
                int length = MaxYear - MinYear + 1;
                this._randomized = false;
                this._fuelTax    = new FuelValueArray(length);
                this._retailLow  = new FuelValueArray(length);
                this._retailAvg  = new FuelValueArray(length);
                this._retailHigh = new FuelValueArray(length);
                this._preTaxLow  = new FuelValueArray(length);
                this._preTaxAvg  = new FuelValueArray(length);
                this._preTaxHigh = new FuelValueArray(length);
                this._betaInv_Alpha = new FuelValueArray(length);
                this._betaInv_Beta  = new FuelValueArray(length);
            }
            #endregion
            #region 
            internal ForecastDataValues Clone()
            {
                ForecastDataValues value = new ForecastDataValues();
                int length = MaxYear - MinYear + 1;
                value._randomized = this._randomized;
                value._fuelTax    = this._fuelTax   .Clone(length);
                value._retailLow  = this._retailLow .Clone(length);
                value._retailAvg  = this._retailAvg .Clone(length);
                value._retailHigh = this._retailHigh.Clone(length);
                value._preTaxLow  = this._preTaxLow .Clone(length);
                value._preTaxAvg  = this._preTaxAvg .Clone(length);
                value._preTaxHigh = this._preTaxHigh.Clone(length);
                value._betaInv_Alpha = this._betaInv_Alpha.Clone(length);
                value._betaInv_Beta  = this._betaInv_Beta .Clone(length);
                return value;
            }
            internal void SetData(int modelYear, FuelType fuelType, double low, double avg, double high, double tax,
                double betaInvAlpha, double betaInvBeta)
            {
                int index = modelYear - MinYear;
                this._fuelTax   [fuelType][index] = tax;
                this._retailLow [fuelType][index] = low;
                this._retailAvg [fuelType][index] = avg;
                this._retailHigh[fuelType][index] = high;
                this._preTaxLow [fuelType][index] = low  - tax;
                this._preTaxAvg [fuelType][index] = avg  - tax;
                this._preTaxHigh[fuelType][index] = high - tax;
                this._betaInv_Alpha[fuelType][index] = betaInvAlpha;
                this._betaInv_Beta [fuelType][index] = betaInvBeta;
            }
            public double GetRetailFuelPrice(ModelYear year, Estimates priceEstimates, FuelType fuelType)
            {
                double[] fp = this.GetRetailFuelPrice(priceEstimates, fuelType);
                int index = Math.Min(year.Year, MaxYear) - MinYear;
                return fp[index];
            }
            public double[] GetRetailFuelPrice(Estimates priceEstimates, FuelType fuelType)
            {
                return (this._randomized                ) ? this._retailAvg [fuelType] :
                       (priceEstimates == Estimates.Low ) ? this._retailLow [fuelType] :
                       (priceEstimates == Estimates.High) ? this._retailHigh[fuelType] : this._retailAvg[fuelType];
            }
            public double[] GetPreTaxFuelPrice(Estimates priceEstimates, FuelType fuelType)
            {
                return (this._randomized                ) ? this._preTaxAvg [fuelType] :
                       (priceEstimates == Estimates.Low ) ? this._preTaxLow [fuelType] :
                       (priceEstimates == Estimates.High) ? this._preTaxHigh[fuelType] : this._preTaxAvg[fuelType];
            }
            public double[] GetFuelTax(FuelType fuelType)
            {
                return this._fuelTax[fuelType];
            }
            public void RandomizeFuelPrices_BETAINV(double probability)
            {
                this._randomized = true;
                int length = MaxYear - MinYear + 1;
                FuelType[] fuelTypes = new FuelType[] {FuelType.Gasoline, FuelType.Diesel, FuelType.Ethanol85, FuelType.Electricity, FuelType.Hydrogen};
                foreach (FuelType fuelType in fuelTypes)
                {
                    double[] alpha    = this._betaInv_Alpha[fuelType];
                    double[] beta     = this._betaInv_Beta [fuelType];
                    double[] retailFP = this._retailAvg    [fuelType];
                    double[] lowFP    = this._retailLow    [fuelType];
                    double[] highFP   = this._retailHigh   [fuelType];
                    double[] preTaxFP = this._preTaxAvg    [fuelType];
                    double[] fuelTax  = this._fuelTax      [fuelType];
                    for (int i = 0; i < length; i++)
                    {
                        if (alpha[i] != -1 && beta[i] != -1)
                        {
                            BetaDistribution bd = new BetaDistribution(alpha[i], beta[i]);
                            retailFP[i] = lowFP[i] + (highFP[i] - lowFP[i]) * bd.InverseLeftProbability(probability);
                            preTaxFP[i] = retailFP[i] - fuelTax[i];
                        }
                    } 
                } 
            }
            #endregion
            #region 
            public const int MinYear = 1975;
            public const int MaxYear = 2100;
            bool _randomized;
            FuelValueArray _fuelTax;
            FuelValueArray _retailLow;
            FuelValueArray _retailAvg;
            FuelValueArray _retailHigh;
            FuelValueArray _preTaxLow;
            FuelValueArray _preTaxAvg;
            FuelValueArray _preTaxHigh;
            FuelValueArray _betaInv_Alpha;
            FuelValueArray _betaInv_Beta;
            #endregion
        }
        [Serializable]
        public sealed class FuelEconomyDataValues
        {
            #region 
            internal FuelEconomyDataValues()
            {
                int length = MaxYear - MinYear + 1;
                this.CarsFuelEconomy = new FuelValue[length];
                this.CarsFuelShare   = new FuelValue[length];
                this.TrksFuelEconomy = new FuelValue[length];
                this.TrksFuelShare   = new FuelValue[length];
            }
            #endregion
            #region 
            internal FuelEconomyDataValues Clone()
            {
                FuelEconomyDataValues value = new FuelEconomyDataValues();
                int length = MaxYear - MinYear + 1;
                Array.Copy(this.CarsFuelEconomy, value.CarsFuelEconomy, length);
                Array.Copy(this.CarsFuelShare  , value.CarsFuelShare  , length);
                Array.Copy(this.TrksFuelEconomy, value.TrksFuelEconomy, length);
                Array.Copy(this.TrksFuelShare  , value.TrksFuelShare  , length);
                return value;
            }
            #endregion
            #region 
            public const int MinYear = 1975;
            public const int MaxYear = 2064;
            public FuelValue[] CarsFuelEconomy;
            public FuelValue[] CarsFuelShare;
            public FuelValue[] TrksFuelEconomy;
            public FuelValue[] TrksFuelShare;
            #endregion
        }
        [Serializable]
        public sealed class EconomicDataValues
        {
            #region 
            internal EconomicDataValues() { }
            #endregion
            #region 
            internal EconomicDataValues Clone()
            {
                EconomicDataValues value = new EconomicDataValues();
                value.ReboundEffect          = this.ReboundEffect;
                value.DiscountRates          = (double[])Interaction.CloneArray(this.DiscountRates, typeof(double));
                value.PrivateDiscountRate    = this.PrivateDiscountRate;
                value.Kf                     = this.Kf;
                value.TestVsOnRoadGap        = this.TestVsOnRoadGap;
                value.RefuelTime             = this.RefuelTime;
                value.RefuelTankVolume       = this.RefuelTankVolume;
                value.VehicleTravelTimeValue = this.VehicleTravelTimeValue;
                value.MonopsonyCosts         = this.MonopsonyCosts;
                value.PriceShockCosts        = this.PriceShockCosts;
                value.MilitarySecurityCosts  = this.MilitarySecurityCosts;
                value.TotalEconomicCosts     = this.TotalEconomicCosts;
                value.TotalEconomicCostsBBL  = this.TotalEconomicCostsBBL;
                value.CarCongestionCosts     = this.CarCongestionCosts;
                value.CarAccidentCosts       = this.CarAccidentCosts;
                value.CarNoiseCosts          = this.CarNoiseCosts;
                value.TrkCongestionCosts     = this.TrkCongestionCosts;
                value.TrkAccidentCosts       = this.TrkAccidentCosts;
                value.TrkNoiseCosts          = this.TrkNoiseCosts;
                value.CODamageCosts          = this.CODamageCosts;
                value.VOCDamageCosts         = this.VOCDamageCosts;
                value.NOXDamageCosts         = this.NOXDamageCosts;
                value.PMDamageCosts          = this.PMDamageCosts;
                value.SOXDamageCosts         = this.SOXDamageCosts;
                value.CH4DamageCostScalar    = this.CH4DamageCostScalar;
                value.N2ODamageCostScalar    = this.N2ODamageCostScalar;
                value.CO2DamageCostsLow      = (double[])Interaction.CloneArray(this.CO2DamageCostsLow     , typeof(double));
                value.CO2DamageCostsAverage  = (double[])Interaction.CloneArray(this.CO2DamageCostsAverage , typeof(double));
                value.CO2DamageCostsHigh     = (double[])Interaction.CloneArray(this.CO2DamageCostsHigh    , typeof(double));
                value.CO2DamageCostsVeryHigh = (double[])Interaction.CloneArray(this.CO2DamageCostsVeryHigh, typeof(double));
                value.CO2DiscRateLow         = this.CO2DiscRateLow;
                value.CO2DiscRateAverage     = this.CO2DiscRateAverage;
                value.CO2DiscRateHigh        = this.CO2DiscRateHigh;
                value.CO2DiscRateVeryHigh    = this.CO2DiscRateVeryHigh;
                value.VMTBaseYear            = this.VMTBaseYear;
                value.VMTGrowthRateLow       = this.VMTGrowthRateLow;
                value.VMTGrowthRateAverage   = this.VMTGrowthRateAverage;
                value.VMTGrowthRateHigh      = this.VMTGrowthRateHigh;
                value.TaxesAndFees           = this.TaxesAndFees;
                value.Financing              = this.Financing;
                value.Insurance              = this.Insurance;
                value.RelativeValueLoss      = this.RelativeValueLoss;
                value.ResaleValue            = this.ResaleValue;
                return value;
            }
            public void GetCO2Estimates(CO2Estimates co2Estimates, out double[] co2Cost, out double co2DiscRate)
            {
                co2Cost =
                    (co2Estimates == CO2Estimates.Low     ) ? this.CO2DamageCostsLow :
                    (co2Estimates == CO2Estimates.High    ) ? this.CO2DamageCostsHigh :
                    (co2Estimates == CO2Estimates.VeryHigh) ? this.CO2DamageCostsVeryHigh : this.CO2DamageCostsAverage;
                co2DiscRate =
                    (co2Estimates == CO2Estimates.Low     ) ? this.CO2DiscRateLow :
                    (co2Estimates == CO2Estimates.High    ) ? this.CO2DiscRateHigh :
                    (co2Estimates == CO2Estimates.VeryHigh) ? this.CO2DiscRateVeryHigh : this.CO2DiscRateAverage;
            }
            public double GetVMTGrowthFactor(Estimates priceEstimates, int year, int age)
            {
                double growthRate =
                    (priceEstimates == Estimates.High) ? this.VMTGrowthRateHigh :
                    (priceEstimates == Estimates.Low ) ? this.VMTGrowthRateLow  : this.VMTGrowthRateAverage;
                return Math.Pow(1 + growthRate, (year + age) - this.VMTBaseYear);
            }
            #endregion
            #region 
            public double ReboundEffect;
            public double[] DiscountRates;
            public double PrivateDiscountRate;
            public double Kf;
            public FuelValue TestVsOnRoadGap;
            public FuelValue RefuelTime;
            public double RefuelTankVolume;
            public double VehicleTravelTimeValue;
            public double MonopsonyCosts;
            public double PriceShockCosts;
            public double MilitarySecurityCosts;
            public double TotalEconomicCosts;
            public double TotalEconomicCostsBBL;
            public double CarCongestionCosts;
            public double CarAccidentCosts;
            public double CarNoiseCosts;
            public double TrkCongestionCosts;
            public double TrkAccidentCosts;
            public double TrkNoiseCosts;
            public double CODamageCosts;
            public double VOCDamageCosts;
            public double NOXDamageCosts;
            public double PMDamageCosts;
            public double SOXDamageCosts;
            public double CH4DamageCostScalar;
            public double N2ODamageCostScalar;
            public double[] CO2DamageCostsLow;
            public double[] CO2DamageCostsAverage;
            public double[] CO2DamageCostsHigh;
            public double[] CO2DamageCostsVeryHigh;
            public double CO2DiscRateLow;
            public double CO2DiscRateAverage;
            public double CO2DiscRateHigh;
            public double CO2DiscRateVeryHigh;
            public int VMTBaseYear;
            public double VMTGrowthRateLow;
            public double VMTGrowthRateAverage;
            public double VMTGrowthRateHigh;
            public double TaxesAndFees;
            public double Financing;
            public double Insurance;
            public double RelativeValueLoss;
            public double ResaleValue;
            #endregion
        }
        [Serializable]
        public sealed class FuelPropertiesValues
        {
            #region 
            internal FuelPropertiesValues() { }
            #endregion
            #region 
            internal FuelPropertiesValues Clone()
            {
                FuelPropertiesValues value = new FuelPropertiesValues();
                value.ConventionalGasolineShare                          = this.ConventionalGasolineShare;
                value.EnergyDensity                                      = this.EnergyDensity;
                value.MassDensity                                        = this.MassDensity;
                value.CarbonContent                                      = this.CarbonContent;
                value.SOXRate                                            = this.SOXRate;
                value.ShareOfFuelSavingsLeadingToLowerImports            = this.ShareOfFuelSavingsLeadingToLowerImports;
                value.ShareOfFuelSavingsLeadingToReducedDomesticRefining = this.ShareOfFuelSavingsLeadingToReducedDomesticRefining;
                value.ShareOfReducedDomesticRefiningFromDomesticCrude    = this.ShareOfReducedDomesticRefiningFromDomesticCrude;
                value.ShareOfReducedDomesticRefiningFromImportedCrude    = this.ShareOfReducedDomesticRefiningFromImportedCrude;
                return value;
            }
            #endregion
            #region 
            public double ConventionalGasolineShare;
            public FuelValue EnergyDensity;
            public FuelValue MassDensity;
            public FuelValue CarbonContent;
            public FuelValue SOXRate;
            public FuelValue ShareOfFuelSavingsLeadingToLowerImports;
            public FuelValue ShareOfFuelSavingsLeadingToReducedDomesticRefining;
            public FuelValue ShareOfReducedDomesticRefiningFromDomesticCrude;
            public FuelValue ShareOfReducedDomesticRefiningFromImportedCrude;
            #endregion
        }
        [Serializable]
        public sealed class UpstreamEmissionsValues
        {
            #region 
            internal UpstreamEmissionsValues() { }
            #endregion
            #region 
            internal UpstreamEmissionsValues Clone()
            {
                UpstreamEmissionsValues value = new UpstreamEmissionsValues();
                value.CO = this.CO;
                value.VOC = this.VOC;
                value.NOX = this.NOX;
                value.SOX = this.SOX;
                value.PM  = this.PM;
                value.CO2 = this.CO2;
                value.Acetaldehyde = this.Acetaldehyde;
                value.Acrolein     = this.Acrolein;
                value.Benzene      = this.Benzene;
                value.Butadiene    = this.Butadiene;
                value.Formaldehyde = this.Formaldehyde;
                value.CH4          = this.CH4;
                value.DieselPM     = this.DieselPM;
                value.MTBE         = this.MTBE;
                value.N2O          = this.N2O;
                return value;
            }
            #endregion
            #region 
            public FuelValue CO, VOC, NOX, SOX, PM, CO2;
            public FuelValue Acetaldehyde, Acrolein, Benzene, Butadiene, Formaldehyde;
            public FuelValue CH4, DieselPM, MTBE, N2O;
            #endregion
        }
        [Serializable]
        public sealed class SafetyDataValues
        {
            #region 
            [Serializable]
            public sealed class SafetyParameters
            {
                internal SafetyParameters() { }
                internal SafetyParameters Clone()
                {
                    SafetyParameters value = new SafetyParameters();
                    value.Threshold   = this.Threshold;
                    value.SmallEffect = this.SmallEffect;
                    value.SmallBase   = this.SmallBase;
                    value.SmallFMVSS  = this.SmallFMVSS;
                    value.LargeEffect = this.LargeEffect;
                    value.LargeBase   = this.LargeBase;
                    value.LargeFMVSS  = this.LargeFMVSS;
                    return value;
                }
                public double Threshold;
                public double SmallEffect;
                public double SmallBase;
                public double SmallFMVSS;
                public double LargeEffect;
                public double LargeBase;
                public double LargeFMVSS;
            }
            #endregion
            #region 
            internal SafetyDataValues()
            {
                this.PC = new SafetyParameters();
                this.LT = new SafetyParameters();
                this.CM = new SafetyParameters();
            }
            SafetyDataValues(bool b) { }
            #endregion
            #region 
            internal SafetyDataValues Clone()
            {
                SafetyDataValues value = new SafetyDataValues(true);
                value.PC = this.PC.Clone();
                value.LT = this.LT.Clone();
                value.CM = this.CM.Clone();
                value.FatalityCosts    = this.FatalityCosts;
                return value;
            }
            #endregion
            #region 
            public SafetyParameters PC;
            public SafetyParameters LT;
            public SafetyParameters CM;
            public double FatalityCosts;
            #endregion
        }
        [Serializable]
        public sealed class VehicleChoiceValues
        {
            #region 
            internal VehicleChoiceValues() { }
            #endregion
            #region 
            internal VehicleChoiceValues Clone()
            {
                VehicleChoiceValues value = new VehicleChoiceValues();
                value.Mu       = (double[])Interaction.CloneArray(this.Mu      , typeof(double));
                value.Beta     = (double[])Interaction.CloneArray(this.Beta    , typeof(double));
                value.BetaNew  = (double[])Interaction.CloneArray(this.BetaNew , typeof(double));
                return value;
            }
            #endregion
            #region 
            public double[] Mu;
            public double[] Beta;
            public double[] BetaNew;
            #endregion
        }
        [Serializable]
        public sealed class HouseholdDataValues
        {
            #region 
            [Serializable]
            public sealed class Household
            {
                #region 
                internal Household() { }
                #endregion
                #region 
                internal Household Clone()
                {
                    Household value = new Household();
                    value.ID            = this.ID;
                    value.Urban         = this.Urban;
                    value.Rural         = this.Rural;
                    value.HHSize4Plus   = this.HHSize4Plus;
                    value.NumAd2Plus    = this.NumAd2Plus;
                    value.NumWork2Plus  = this.NumWork2Plus;
                    value.YKidLE15      = this.YKidLE15;
                    value.RetiredNK     = this.RetiredNK;
                    value.NonRNK        = this.NonRNK;
                    value.College       = this.College;
                    value.HHIncLT25K    = this.HHIncLT25K;
                    value.HHInc75to100K = this.HHInc75to100K;
                    value.HHIncGE100K   = this.HHIncGE100K;
                    value.Income        = this.Income;
                    value.BuyUsedExp    = this.BuyUsedExp;
                    value.Weights       = (double[])Interaction.CloneArray(this.Weights, typeof(double));
                    return value;
                }
                #endregion
                #region 
                public int ID;
                public int Urban;
                public int Rural;
                public int HHSize4Plus;
                public int NumAd2Plus;
                public int NumWork2Plus;
                public int YKidLE15;
                public int RetiredNK;
                public int NonRNK;
                public int College;
                public int HHIncLT25K;
                public int HHInc75to100K;
                public int HHIncGE100K;
                public double Income;
                public double BuyUsedExp;
                public double[] Weights;
                #endregion
            }
            #endregion
            #region 
            internal HouseholdDataValues()
            {
                this._households = new List<Household>();
            }
            #endregion
            #region 
            internal HouseholdDataValues Clone()
            {
                HouseholdDataValues value = new HouseholdDataValues();
                for (int i = 0, hhCount = this._households.Count; i < hhCount; i++)
                {
                    value._households.Add(this._households[i].Clone());
                }
                return value;
            }
            internal void AddHousehold(Household h)
            {
                this._households.Add(h);
            }
            public Household GetHousehold(int id)
            {
                for (int i = 0, hhCount = this._households.Count; i < hhCount; i++)
                {
                    if (this._households[i].ID == id) { return this._households[i]; }
                }
                return null;
            }
            #endregion
            #region 
            public List<Household> Households { get { return this._households; } }
            #endregion
            #region 
            List<Household> _households;
            #endregion
        }
        #endregion
        #region 
        internal Parameters()
        {
            this._vehicleAgeData    = new VehicleAgeDataValues(Interaction.CY);
            this._forecastData      = new ForecastDataValues();
            this._fuelEconomyData   = new FuelEconomyDataValues();
            this._economicData      = new EconomicDataValues();
            this._fuelProperties    = new FuelPropertiesValues();
            this._upstreamEmissions = new UpstreamEmissionsValues();
            this._safetyData        = new SafetyDataValues();
            this._vehicleChoiceData = new VehicleChoiceValues();
            this._householdData     = new HouseholdDataValues();
        }
        #endregion
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        public Parameters Clone()
        {
            Parameters p = new Parameters();
            p._vehicleAgeData    = this._vehicleAgeData   .Clone();
            p._forecastData      = this._forecastData     .Clone();
            p._fuelEconomyData   = this._fuelEconomyData  .Clone();
            p._economicData      = this._economicData     .Clone();
            p._fuelProperties    = this._fuelProperties   .Clone();
            p._upstreamEmissions = this._upstreamEmissions.Clone();
            p._safetyData        = this._safetyData       .Clone();
            p._vehicleChoiceData = this._vehicleChoiceData.Clone();
            p._householdData     = this._householdData    .Clone();
            return p;
        }
        #endregion
        public double[] GetVehicleSurvivability(Vehicle veh)
        {
            switch (veh.VehicleStyle)
            {
                case VehicleStyle.Convertible:
                case VehicleStyle.Coupe:
                case VehicleStyle.Hatchback:
                case VehicleStyle.Sedan:
                case VehicleStyle.Wagon:
                    return this._vehicleAgeData.SurvivalCars;
                case VehicleStyle.Minivan:
                case VehicleStyle.Van:
                    return this._vehicleAgeData.SurvivalVans;
                case VehicleStyle.SportUtility:
                    return this._vehicleAgeData.SurvivalSUVs;
                case VehicleStyle.Pickup:
                    return this._vehicleAgeData.SurvivalPickups;
                default:
                    return new double[Interaction.CY];
            }
        }
        public double[] GetVehicleMilesDriven(Vehicle veh)
        {
            switch (veh.VehicleStyle)
            {
                case VehicleStyle.Convertible:
                case VehicleStyle.Coupe:
                case VehicleStyle.Hatchback:
                case VehicleStyle.Sedan:
                case VehicleStyle.Wagon:
                    return this._vehicleAgeData.MilesCars;
                case VehicleStyle.Minivan:
                case VehicleStyle.Van:
                    return this._vehicleAgeData.MilesVans;
                case VehicleStyle.SportUtility:
                    return this._vehicleAgeData.MilesSUVs;
                case VehicleStyle.Pickup:
                    return this._vehicleAgeData.MilesPickups;
                default:
                    return new double[Interaction.CY];
            }
        }
        public void UpdateForMonteCarlo(double fuelPriceEstimates, double reboundEffect, double priceShock,
            double milesCars0, double milesAllLTs0)
        {
            this._economicData.ReboundEffect         = reboundEffect;
            this._economicData.PriceShockCosts       = priceShock;
            this._economicData.TotalEconomicCosts    = priceShock;
            this._economicData.TotalEconomicCostsBBL = this._economicData.TotalEconomicCosts * 42;
            this._vehicleAgeData.UpdateVMTSchedule(milesCars0, milesAllLTs0, 300000, 350000);
            this._forecastData.RandomizeFuelPrices_BETAINV(fuelPriceEstimates);
        }
        public void UpdateParametersOverrides(ParametersOverrides overrides)
        {
            if (overrides.OverrideReboundEffect         ) { this._economicData.ReboundEffect          = overrides.ReboundEffect; }
            if (overrides.OverrideDiscountRate          ) { this._economicData.DiscountRates          = new double[] {overrides.DiscountRate}; }
            if (overrides.OverrideVehicleTravelTimeValue) { this._economicData.VehicleTravelTimeValue = overrides.VehicleTravelTimeValue; }
            if (overrides.OverrideMilitarySecurity      ) { this._economicData.MilitarySecurityCosts  = overrides.MilitarySecurity; }
        }
        #endregion
        #region 
        public Parameters.VehicleAgeDataValues VehicleAgeData { get { return this._vehicleAgeData; } }
        public Parameters.ForecastDataValues ForecastData { get { return this._forecastData; } }
        public Parameters.FuelEconomyDataValues FuelEconomyData { get { return this._fuelEconomyData; } }
        public Parameters.EconomicDataValues EconomicData { get { return this._economicData; } }
        public Parameters.FuelPropertiesValues FuelProperties { get { return this._fuelProperties; } }
        public Parameters.UpstreamEmissionsValues UpstreamEmissions { get { return this._upstreamEmissions; } }
        public Parameters.SafetyDataValues SafetyData { get { return this._safetyData; } }
        public Parameters.VehicleChoiceValues VehicleChoiceData { get { return this._vehicleChoiceData; } }
        public Parameters.HouseholdDataValues HouseholdData { get { return this._householdData; } }
        #endregion
        #region 
        internal Parameters.VehicleAgeDataValues    _vehicleAgeData;
        internal Parameters.ForecastDataValues      _forecastData;
        internal Parameters.FuelEconomyDataValues   _fuelEconomyData;
        internal Parameters.EconomicDataValues      _economicData;
        internal Parameters.FuelPropertiesValues    _fuelProperties;
        internal Parameters.UpstreamEmissionsValues _upstreamEmissions;
        internal Parameters.SafetyDataValues        _safetyData;
        internal Parameters.VehicleChoiceValues     _vehicleChoiceData;
        internal Parameters.HouseholdDataValues     _householdData;
        #endregion
    }
}

