using System;
using Volpe.Cafe.IO;
namespace Volpe.Cafe.Settings
{
    [Serializable]
    public class ParametersOverrides : ISettings, ICloneable
	{
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        ISettings ISettings.Clone()
        {
            return this.Clone();
        }
        ParametersOverrides Clone()
        {
            ParametersOverrides value = new ParametersOverrides();
            Clone(this, value);
            return value;
        }
        static void Clone(ParametersOverrides src, ParametersOverrides dst)
        {
            dst.OverrideFuelPriceEstimates     = src.OverrideFuelPriceEstimates;
            dst.OverrideCO2Estimates           = src.OverrideCO2Estimates;
            dst.OverrideReboundEffect          = src.OverrideReboundEffect;
            dst.OverrideDiscountRate           = src.OverrideDiscountRate;
            dst.OverrideVehicleTravelTimeValue = src.OverrideVehicleTravelTimeValue;
            dst.OverrideMilitarySecurity       = src.OverrideMilitarySecurity;
            dst.ScaleConsumerBenefits          = src.ScaleConsumerBenefits;
            dst.FuelPriceEstimates             = src.FuelPriceEstimates;
            dst.CO2Estimates                   = src.CO2Estimates;
            dst.ReboundEffect                  = src.ReboundEffect;
            dst.DiscountRate                   = src.DiscountRate;
            dst.VehicleTravelTimeValue         = src.VehicleTravelTimeValue;
            dst.MilitarySecurity               = src.MilitarySecurity;
            dst.ConsumerBenefits               = src.ConsumerBenefits;
        }
        #endregion
        #region 
        void ISettings.SetDefaultSettings()
        {
            this.OverrideFuelPriceEstimates     = false;
            this.OverrideReboundEffect          = false;
            this.OverrideDiscountRate           = false;
            this.OverrideVehicleTravelTimeValue = false;
            this.OverrideMilitarySecurity       = false;
            this.OverrideCO2Estimates           = false;
            this.ScaleConsumerBenefits          = false;
            this.FuelPriceEstimates             = Estimates.Average;
            this.ReboundEffect                  = -0.10D;
            this.DiscountRate                   =  0.03D;
            this.VehicleTravelTimeValue         = 24.64D;
            this.MilitarySecurity               =  0.00D;
            this.CO2Estimates                   = CO2Estimates.Average;
            this.ConsumerBenefits               =  0.25D;
        }
        void ISettings.UpdateSettings(ISettings newSettings)
        {
            this.UpdateSettings((ParametersOverrides)newSettings);
        }
        public void UpdateSettings(ParametersOverrides newSettings)
        {
            Clone(newSettings, this);
        }
        public void WriteSummaryLog(LogWriter writer)
        {
            if (writer.Summary == null) { return; }
            string outStr = "--- Parameters Overrides ---" +
                "\n  OverrideFuelPriceEstimates="     + this.OverrideFuelPriceEstimates     + ((this.OverrideFuelPriceEstimates    ) ? " (" + this.FuelPriceEstimates     + ")" : string.Empty) +
                "\n  OverrideCO2Estimates="           + this.OverrideCO2Estimates           + ((this.OverrideCO2Estimates          ) ? " (" + this.CO2Estimates           + ")" : string.Empty) +
                "\n  OverrideReboundEffect="          + this.OverrideReboundEffect          + ((this.OverrideReboundEffect         ) ? " (" + this.ReboundEffect          + ")" : string.Empty) +
                "\n  OverrideDiscountRate="           + this.OverrideDiscountRate           + ((this.OverrideDiscountRate          ) ? " (" + this.DiscountRate           + ")" : string.Empty) +
                "\n  OverrideVehicleTravelTimeValue=" + this.OverrideVehicleTravelTimeValue + ((this.OverrideVehicleTravelTimeValue) ? " (" + this.VehicleTravelTimeValue + ")" : string.Empty) +
                "\n  OverrideMilitarySecurity="       + this.OverrideMilitarySecurity       + ((this.OverrideMilitarySecurity      ) ? " (" + this.MilitarySecurity       + ")" : string.Empty) +
                "\n  ScaleConsumerBenefits="          + this.ScaleConsumerBenefits          + ((this.ScaleConsumerBenefits         ) ? " (" + this.ConsumerBenefits       + ")" : string.Empty);
            writer.Summary.WriteLine(outStr + "\n");
        }
        #endregion
        #endregion
        #region 
        public bool OverrideFuelPriceEstimates;
        public bool OverrideCO2Estimates;
        public bool OverrideReboundEffect;
        public bool OverrideDiscountRate;
        public bool OverrideVehicleTravelTimeValue;
        public bool OverrideMilitarySecurity;
        public bool ScaleConsumerBenefits;
        public Estimates FuelPriceEstimates;
        public CO2Estimates CO2Estimates;
        public double ReboundEffect;
        public double DiscountRate;
        public double VehicleTravelTimeValue;
        public double MilitarySecurity;
        public double ConsumerBenefits;
        #endregion
    }
}

