using System;
using System.Runtime.Serialization;
using Volpe.Cafe.IO;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.Settings
{
    [Serializable]
    public sealed class Scenario : ICloneable
    {
        #region 
        [Serializable]
        public struct ScenarioOptions
        {
            internal ScenarioOptions(bool preferredAlternative, string regulatoryMerging)
            {
                this.PreferredAlternative = preferredAlternative;
                this.RegulatoryMerging    = regulatoryMerging;
            }
            public readonly bool PreferredAlternative;
            public readonly string RegulatoryMerging;
        }
        #endregion
        #region 
        Scenario() { }
        internal Scenario(int index)
        {
            this._index = index;
            this._isBaseline = (index == 0);
        }
        #endregion
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        public Scenario Clone()
        {
            Scenario sn = new Scenario();
            this.CloneTo(sn);
            return sn;
        }
        void CloneTo(Scenario sn)
        {
            sn._index       = this._index;
            sn._isBaseline  = this._isBaseline;
            sn._description = this._description;
            sn._options     = this._options;
            sn._multiFuelCAFE  = (int   [ ])Interaction.CloneArray(this._multiFuelCAFE , typeof(int   ));
            sn._pcFunctionType = (int   [ ])Interaction.CloneArray(this._pcFunctionType, typeof(int   ));
            sn._pcCoefficients = (double[,])Interaction.CloneArray(this._pcCoefficients, typeof(double));
            sn._pcMinStndMpg   = (double[ ])Interaction.CloneArray(this._pcMinStndMpg  , typeof(double));
            sn._pcMinStndPct   = (double[ ])Interaction.CloneArray(this._pcMinStndPct  , typeof(double));
            sn._ltFunctionType = (int   [ ])Interaction.CloneArray(this._ltFunctionType, typeof(int   ));
            sn._ltCoefficients = (double[,])Interaction.CloneArray(this._ltCoefficients, typeof(double));
            sn._ltMinStndMpg   = (double[ ])Interaction.CloneArray(this._ltMinStndMpg  , typeof(double));
            sn._ltMinStndPct   = (double[ ])Interaction.CloneArray(this._ltMinStndPct  , typeof(double));
            sn._includeAC      = (bool  [ ])Interaction.CloneArray(this._includeAC     , typeof(bool  ));
            sn._pcACAdj        = (double[ ])Interaction.CloneArray(this._pcACAdj       , typeof(double));
            sn._pcACCost       = (double[ ])Interaction.CloneArray(this._pcACCost      , typeof(double));
            sn._ltACAdj        = (double[ ])Interaction.CloneArray(this._ltACAdj       , typeof(double));
            sn._ltACCost       = (double[ ])Interaction.CloneArray(this._ltACCost      , typeof(double));
            sn._pcOffCycleCap  = (double[ ])Interaction.CloneArray(this._pcOffCycleCap , typeof(double));
            sn._ltOffCycleCap  = (double[ ])Interaction.CloneArray(this._ltOffCycleCap , typeof(double));
            sn._taxCreditPHEV  = (double[ ])Interaction.CloneArray(this._taxCreditPHEV , typeof(double));
            sn._taxCreditEV    = (double[ ])Interaction.CloneArray(this._taxCreditEV   , typeof(double));
        }
        #endregion
        public override string ToString()
        {
            return this._index + ". " + Interaction.GetTitleCase(this._description);
        }
        public string ToString(int descrLength)
        {
            int len = this._description.Length;
            bool outOfBounds = (descrLength < len);
            descrLength = (outOfBounds) ? descrLength - 3 : len;
            string descr = Interaction.GetTitleCase(this._description.Substring(0, descrLength));
            if (outOfBounds) { descr += " ..."; }
            return this._index + ". " + descr;
        }
        #endregion
        #region 
        public int Index { get { return this._index; } }
        public bool IsBaseline { get { return this._isBaseline; } }
        public string Description { get { return this._description; } }
        public ScenarioOptions Options { get { return this._options; } }
        public int[] MultiFuelCAFE { get { return this._multiFuelCAFE; } }
        public int[] AutoFunctionType { get { return this._pcFunctionType; } }
        public double[,] AutoCoefficients { get { return this._pcCoefficients; } }
        public double[] AutoMinStndMpg { get { return this._pcMinStndMpg; } }
        public double[] AutoMinStndPct { get { return this._pcMinStndPct; } }
        public int[] LtFunctionType { get { return this._ltFunctionType; } }
        public double[,] LtCoefficients { get { return this._ltCoefficients; } }
        public double[] LtMinStndMpg { get { return this._ltMinStndMpg; } }
        public double[] LtMinStndPct { get { return this._ltMinStndPct; } }
        public bool[] IncludeAC { get { return this._includeAC; } }
        public double[] PCACAdj { get { return this._pcACAdj; } }
        public double[] PCACCost { get { return this._pcACCost; } }
        public double[] LTACAdj { get { return this._ltACAdj; } }
        public double[] LTACCost { get { return this._ltACCost; } }
        public double[] PCOffCycleCap { get { return this._pcOffCycleCap; } }
        public double[] LTOffCycleCap { get { return this._ltOffCycleCap; } }
        public double[] TaxCreditPHEV { get { return this._taxCreditPHEV; } }
        public double[] TaxCreditEV { get { return this._taxCreditEV; } }
        #endregion
        #region 
        int                      _index;
        bool                     _isBaseline;
        internal string          _description;
        internal ScenarioOptions _options;
        internal int[] _multiFuelCAFE;
        internal int   [ ] _pcFunctionType;
        internal double[,] _pcCoefficients;
        internal double[ ] _pcMinStndMpg;
        internal double[ ] _pcMinStndPct;
        internal int   [ ] _ltFunctionType;
        internal double[,] _ltCoefficients;
        internal double[ ] _ltMinStndMpg;
        internal double[ ] _ltMinStndPct;
        internal bool  [ ] _includeAC;
        internal double[ ] _pcACAdj;
        internal double[ ] _pcACCost;
        internal double[ ] _ltACAdj;
        internal double[ ] _ltACCost;
        internal double[ ] _pcOffCycleCap;
        internal double[ ] _ltOffCycleCap;
        internal double[ ] _taxCreditPHEV;
        internal double[ ] _taxCreditEV;
        #endregion
    }
}

