using System;
using Volpe.Cafe.IO;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.Settings
{
    [Serializable]
    public sealed class TailpipeEmissions : ICloneable
    {
        #region 
        [Serializable]
        public sealed class RateValues
        {
            #region 
            internal RateValues()
            {
                this.LDV  = new double[Interaction.CY];
                this.LDT1 = new double[Interaction.CY];
                this.LDT2 = new double[Interaction.CY];
                this.HDV  = new double[Interaction.CY];
            }
            #endregion
            #region 
            internal void CopyTo(RateValues value)
            {
                Array.Copy(this.LDV , value.LDV , Interaction.CY);
                Array.Copy(this.LDT1, value.LDT1, Interaction.CY);
                Array.Copy(this.LDT2, value.LDT2, Interaction.CY);
                Array.Copy(this.HDV , value.HDV , Interaction.CY);
            }
            #endregion
            #region 
            public double[] this[Mobile6Class value]
            {
                get
                {
                    return
                        (value == Mobile6Class.LDV ) ? this.LDV  :
                        (value == Mobile6Class.LDT1) ? this.LDT1 :
                        (value == Mobile6Class.LDT2) ? this.LDT2 :
                        (value == Mobile6Class.HDV ) ? this.HDV  : null;
                }
            }
            #endregion
            #region 
            public double[] LDV;
            public double[] LDT1;
            public double[] LDT2;
            public double[] HDV;
            #endregion
        }
        [Serializable]
        public sealed class Emissions
        {
            #region 
            internal Emissions()
            {
                this.Gasoline    = new RateValues();
                this.GasolineRfg = new RateValues();
                this.Diesel      = new RateValues();
                this.Ethanol85   = new RateValues();
            }
            #endregion
            #region 
            internal void CopyTo(Emissions value)
            {
                this.Gasoline   .CopyTo(value.Gasoline   );
                this.GasolineRfg.CopyTo(value.GasolineRfg);
                this.Diesel     .CopyTo(value.Diesel     );
                this.Ethanol85  .CopyTo(value.Ethanol85  );
            }
            #endregion
            #region 
            public RateValues Gasoline;
            public RateValues GasolineRfg;
            public RateValues Diesel;
            public RateValues Ethanol85;
            #endregion
        }
        #endregion
        #region 
        internal TailpipeEmissions()
        {
            this._co  = new Emissions();
            this._voc = new Emissions();
            this._nox = new Emissions();
            this._pm  = new Emissions();
            this._ch4 = new Emissions();
            this._n2o = new Emissions();
        }
        #endregion
        #region 
        #region 
        object ICloneable.Clone()
        {
            return this.Clone();
        }
        public TailpipeEmissions Clone()
        {
            TailpipeEmissions value = new TailpipeEmissions();
            this._co .CopyTo(value._co );
            this._voc.CopyTo(value._voc);
            this._nox.CopyTo(value._nox);
            this._pm .CopyTo(value._pm );
            this._ch4.CopyTo(value._ch4);
            this._n2o.CopyTo(value._n2o);
            return value;
        }
        #endregion
        #endregion
        #region 
        public TailpipeEmissions.Emissions CO { get { return this._co; } }
        public TailpipeEmissions.Emissions VOC { get { return this._voc; } }
        public TailpipeEmissions.Emissions NOX { get { return this._nox; } }
        public TailpipeEmissions.Emissions PM { get { return this._pm; } }
        public TailpipeEmissions.Emissions CH4 { get { return this._ch4; } }
        public TailpipeEmissions.Emissions N2O { get { return this._n2o; } }
        #endregion
        #region 
        internal TailpipeEmissions.Emissions _co, _voc, _nox, _pm, _ch4, _n2o;
        #endregion
    }
}

