using System;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.Settings
{
    [Serializable]
    public sealed class TechnologyAttributesTable
    {
        #region 
        [Serializable]
        public sealed class TechnologyAttributes
        {
            #region 
            internal TechnologyAttributes()
            {
            }
            #endregion
            #region 
            internal TechnologyAttributes Clone()
            {
                TechnologyAttributes ta = new TechnologyAttributes();
                ta.TechnologyClass  = this.TechnologyClass;
                ta.Applicable       = this.Applicable;
                ta.YearAvailable    = this.YearAvailable;
                ta.YearRetired      = this.YearRetired;
                ta.ElectricRange    = this.ElectricRange;
                ta.DeltaWeightPCT   = this.DeltaWeightPCT;
                ta.DeltaWeightLBS   = this.DeltaWeightLBS;
                ta.LossOfValue      = this.LossOfValue;
                ta.FC               = this.FC;
                ta.FCg              = this.FCg;
                ta.FCgShare         = this.FCgShare;
                ta.OffCycleCreditPC = this.OffCycleCreditPC;
                ta.OffCycleCreditLT = this.OffCycleCreditLT;
                ta.CostTable            = (double[])Interaction.CloneArray(this.CostTable           , typeof(double));
                ta.MaintenanceCostTable = (double[])Interaction.CloneArray(this.MaintenanceCostTable, typeof(double));
                ta.RepairCostTable      = (double[])Interaction.CloneArray(this.RepairCostTable     , typeof(double));
                return ta;
            }
            #endregion
            #region 
            public TechnologyClass TechnologyClass;
            public bool Applicable;
            public int YearAvailable;
            public int YearRetired;
            public double ElectricRange;
            public double DeltaWeightPCT;
            public double DeltaWeightLBS;
            public double LossOfValue;
            public double FC;
            public double FCg;
            public double FCgShare;
            public double OffCycleCreditPC;
            public double OffCycleCreditLT;
            public double[] CostTable;
            public double[] MaintenanceCostTable;
            public double[] RepairCostTable;
            #endregion
        }
        #endregion
        #region 
        internal TechnologyAttributesTable()
        {
            this._attributes = new TechnologyAttributes[(int)TechnologyClass.Max];
            for (int i = 0; i < this._attributes.Length; i++)
            {
                this._attributes[i] = new TechnologyAttributes();
            }
        }
        #endregion
        #region 
        internal TechnologyAttributesTable Clone()
        {
            TechnologyAttributesTable table = new TechnologyAttributesTable();
            for (int i = 0; i < this._attributes.Length; i++)
            {
                table._attributes[i] = this._attributes[i].Clone();
            }
            return table;
        }
        public TechnologyAttributesTable.TechnologyAttributes[] ToArray()
        {
            return this._attributes;
        }
        #endregion
        #region 
        public TechnologyAttributesTable.TechnologyAttributes this[TechnologyClass techClass]
        {
            get
            {
                int index = (int)techClass - 1;
                if (index < 0 || index >= (int)TechnologyClass.Max) { throw new ArgumentOutOfRangeException("techClass", ErrorStrings.TechClassNotValid); }
                return this._attributes[index];
            }
            set
            {
                int index = (int)techClass - 1;
                if (index < 0 || index >= (int)TechnologyClass.Max) { throw new ArgumentOutOfRangeException("techClass", ErrorStrings.TechClassNotValid); }
                this._attributes[index] = value;
            }
        }
        #endregion
        #region 
        TechnologyAttributesTable.TechnologyAttributes[] _attributes;
        #endregion
    }
}

