using System;
using Volpe.Cafe.Utils;
namespace Volpe.Cafe.Settings
{
    [Serializable]
    public sealed class TechnologySynergiesTable
    {
        #region 
        [Serializable]
        public sealed class TechnologySynergies
        {
            #region 
            internal TechnologySynergies()
            {
                this._synergies = new double[TechnologyIndexes.TechnologyCount];
                this._type = new SynergyType[TechnologyIndexes.TechnologyCount];
            }
            #endregion
            #region 
            internal TechnologySynergies Clone()
            {
                TechnologySynergies ts = new TechnologySynergies();
                ts._synergies = (double     [])Interaction.CloneArray(this._synergies, typeof(double));
                ts._type      = (SynergyType[])Interaction.CloneArray(this._type, typeof(SynergyType));
                return ts;
            }
            internal void SetSynergy(int index, double synergy, SynergyType type)
            {
                if (index >= 0 && index < TechnologyIndexes.TechnologyCount)
                {
                    this._synergies[index] = synergy;
                    this._type[index] = type;
                }
            }
            #endregion
            #region 
            public double[] Synergies { get { return this._synergies; } }
            public SynergyType[] Type { get { return this._type; } }
            #endregion
            #region 
            double     [] _synergies;
            SynergyType[] _type;
            #endregion
        }
        #endregion
        #region 
        internal TechnologySynergiesTable()
        {
            this._synergies = new TechnologySynergies[(int)TechnologyClass.Max];
            for (int i = 0; i < this._synergies.Length; i++)
            {
                this._synergies[i] = new TechnologySynergies();
            }
        }
        #endregion
        #region 
        internal TechnologySynergiesTable Clone()
        {
            TechnologySynergiesTable table = new TechnologySynergiesTable();
            for (int i = 0; i < this._synergies.Length; i++)
            {
                table._synergies[i] = this._synergies[i].Clone();
            }
            return table;
        }
        public TechnologySynergiesTable.TechnologySynergies[] ToArray()
        {
            return this._synergies;
        }
        #endregion
        #region 
        public TechnologySynergiesTable.TechnologySynergies this[TechnologyClass techClass]
        {
            get
            {
                int index = (int)techClass - 1;
                if (index < 0 || index >= (int)TechnologyClass.Max) { throw new ArgumentOutOfRangeException("techClass", ErrorStrings.TechClassNotValid); }
                return this._synergies[index];
            }
            set
            {
                int index = (int)techClass - 1;
                if (index < 0 || index >= (int)TechnologyClass.Max) { throw new ArgumentOutOfRangeException("techClass", ErrorStrings.TechClassNotValid); }
                this._synergies[index] = value;
            }
        }
        #endregion
        #region 
        TechnologySynergiesTable.TechnologySynergies[] _synergies;
        #endregion
    }
}

